Ext.define('documents.Api', {singleton:true, getCmpByMime:function(mime) {
  mime = mime === undefined ? '' : mime;
  return documents.DocumentTypes.getComponentByMime(mime);
}, deleteTag:function(id) {
  return documents.Support.deleteById(documents.Support.buildTagsUrl(id), 'DELETE_TAG', 'TAG_WILL_BE_DELETED');
}, getUploadWindow:function(moduleId, defaultFileProperties, width, height) {
  defaultFileProperties = defaultFileProperties === undefined ? {} : defaultFileProperties;
  width = width === undefined ? 0.5 : width;
  height = height === undefined ? 0.8 : height;
  return documents.Support.getWindowCmp(documents.Support.TR('ADD_DOCUMENTS'), 'fa-upload', 0.5, 0.8, {xtype:'bpcDocsCreate', moduleId:moduleId, defaultFileProperties:Ext.apply({}, defaultFileProperties)}, {listeners:{boxready:function(w) {
    var cmp = w.down('bpcDocsCreate');
    if (cmp) {
      cmp.onOwnerCtBoxReady();
    }
  }}});
}, getDownloadUrl:function(documentId) {
  return document.location.origin + document.location.pathname + documents.Support.getDownloadPath(documentId).substring(1);
}, createTag:function(params) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('tags', params), undefined, {foo:'bar'}, 'POST');
}, createDocument:function(params, data) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('document', params), undefined, data, 'POST');
}, updateDocument:function(documentId, data) {
  return documents.Support.newRequest(documents.Support.buildDocumentUrl(documentId), undefined, data, 'PUT');
}, getDocument:function(params) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('document', params), undefined, undefined, 'GET');
}, getDocuments:function(params) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('documents', params), undefined, undefined, 'GET');
}, getBpcUUID:function() {
  return documents.Support.newRequest('/cxf/bpc-core/status', undefined, undefined, 'GET').then(function(json) {
    var $jscomp$destructuring$var0 = json;
    var serverUUID = $jscomp$destructuring$var0.serverUUID;
    return serverUUID;
  });
}, clearTrashBin:function(instanceId) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('document', {instanceId:instanceId, emptyBin:true}), undefined, undefined, 'DELETE');
}, clearInstanceData:function(instanceId, deployTargetInstanceId) {
  if (deployTargetInstanceId) {
    return documents.Support.newRequest(documents.Support.basePath + 'list?' + Ext.Object.toQueryString({instanceId:instanceId, action:'emptyInstance', targetInstanceId:deployTargetInstanceId}), undefined, undefined, 'GET');
  }
  return documents.Support.newRequest(documents.Support.basePath + 'document?' + Ext.Object.toQueryString({instanceId:instanceId}), undefined, undefined, 'DELETE');
}, updateTag:function(params) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('tags', params), undefined, {foo:'bar'}, 'PUT');
}, removeFromCache:function(id) {
  if (documents.Support.cached(id)) {
    delete documents.Cache.docs[id];
  }
}, getTags:function(params) {
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('tags', params), undefined, undefined, 'GET').then(function(json) {
    if (json && json.total > 0) {
      return json.data;
    }
  });
}, testInstance:function(moduleId) {
  return documents.Support.newRequest(documents.Support.basePath + 'test?' + Ext.Object.toQueryString({instanceId:moduleId}), undefined, undefined, 'GET');
}, isDocumentsAdmin:function() {
  return BpcCommon.Api.hasUserRole(documents.Support.adminRole);
}, setupInstance:function(moduleId, action) {
  if (documents.Api.isDocumentsAdmin()) {
    var params = {instanceId:moduleId, action:action};
    return documents.Support.newRequest(documents.Support.getDocumentsUrl('configure', params), undefined, undefined, 'GET');
  }
}, instance:{restoreInstanceData:function(moduleId) {
  if (documents.Api.isDocumentsAdmin()) {
    var params = {instanceId:moduleId, mode:'restoreFromDataBase'};
    return documents.Support.newRequest(documents.Support.getDocumentsUrl('deploy', params), undefined, undefined, 'GET');
  }
}, migrateInstanceDataToDb:function(moduleId) {
  if (documents.Api.isDocumentsAdmin()) {
    var params = {instanceId:moduleId, mode:'migrateAllToDatabase'};
    return documents.Support.newRequest(documents.Support.getDocumentsUrl('deploy', params), undefined, undefined, 'GET');
  }
}, deleteInstanceData:function(moduleId) {
  if (documents.Api.isDocumentsAdmin()) {
    var params = {instanceId:moduleId};
    return documents.Support.newRequest(documents.Support.getDocumentsUrl('document', params), undefined, undefined, 'DELETE');
  }
}}, sendToInubit:function(documentId, path, moduleId) {
  documents.Support.getDocument(documentId, moduleId, true).then(function(json) {
    documents.Support.newRequest(path, {moduleId:moduleId}, json || {}, 'POST').then(function(success) {
    }, function(response) {
      BpcCommon.Api.showNotification('ERROR!');
    });
  });
}, getStats:function(moduleId) {
  var params = {start:0, limit:0, instanceId:moduleId};
  return documents.Support.newRequest(documents.Support.getDocumentsUrl('documents', params), undefined, undefined).then(function(json) {
    json = json || {};
    return function() {
      var $jscomp$destructuring$var1 = json;
      var $jscomp$destructuring$var2 = $jscomp$destructuring$var1;
      total = $jscomp$destructuring$var2.total;
      tags = $jscomp$destructuring$var2.tags;
      types = $jscomp$destructuring$var2.types;
      return $jscomp$destructuring$var1;
    }();
  });
}});
Ext.define('documents.Cache', {singleton:true, docs:{}, tags:{}, getTags:function(moduleId) {
  if (this.tags[moduleId]) {
    return this.tags[moduleId]['tags'] || [];
  }
}, tagsArrayToString:function(value, tags) {
  value = value === undefined ? [] : value;
  return value.map(function(id, i) {
    id = id || '';
    var match = tags.filter(function(i) {
      return i.id === id.trim();
    });
    return '\x3cdiv style\x3d"' + documents.Support.getTagBoxStyle(false, match.length ? match[0].color : undefined) + (i === 0 ? 'margin-left: auto;' : '') + ' margin-bottom: 0.1em;"\x3e ' + (match.length ? match[0].name : id) + '\x3c/div\x3e';
  }).join('');
}, getInlineTags:function(value, record, tags) {
  var tagsValue = record.get('tags') || [];
  if (Ext.isString(tagsValue)) {
    tagsValue = tagsValue.length === 0 ? [] : [tagsValue];
  }
  if (tags.length === 0) {
    tags = documents.Cache.getTags(record.get('instanceId')) || [];
  }
  var tagSpans = tagsValue.map(function(id, i) {
    id = id || '';
    var match = tags.filter(function(i) {
      return i.id === id.trim();
    });
    return '\x3cdiv style\x3d"' + documents.Support.getTagBoxStyle(false, match.length ? match[0].color : undefined) + (i === 0 ? 'margin-left: auto;' : '') + ' margin-right: 0.3em;"\x3e ' + (match.length ? match[0].name : id) + '\x3c/div\x3e';
  });
  return '\x3cdiv style\x3d"width: 100%;display: flex"\x3e\x3cdiv\x3e' + (Ext.String.htmlEncode(value) || '') + '\x3c/div\x3e' + tagSpans.join('') + '\x3c/div\x3e';
}});
Ext.define('documents.DocumentTypes', {singleton:true, getComponentByMime:function(mimeStr) {
  mimeStr = mimeStr === undefined ? '' : mimeStr;
  var mimeMap = {'application/zip':this.map.zip, 'message/rfc822':this.map.mail};
  return mimeMap.hasOwnProperty(mimeStr) ? mimeMap[mimeStr] : undefined;
}, map:{mail:'documentsMail', folder:'documentsFolder', zip:'docsZip'}, mail:function(documentObject) {
  return {xtype:'documentsMail', documentObject:documentObject};
}, folder:function(documentObject) {
  return {xtype:'documentsFolder', documentObject:documentObject};
}, zip:function(documentObject) {
  return {xtype:'docsZip', documentObject:documentObject};
}, error:function(documentObject, obj) {
  obj = obj === undefined ? {errorMessage:'not set'} : obj;
  var status = [];
  if (documentObject && documentObject.record) {
    status = documentObject.record.get('status') || [];
  }
  return Ext.apply({xtype:'documentsFileError', documentObject:documentObject, status:status}, obj);
}, pdf:function(documentObject) {
  var pdfAsArray = documents.Support.convertDataURIToBinary(documentObject.dataUri);
  if (pdfAsArray) {
    return {xtype:'PdfViewerPdfPanel', width:600, height:500, disableTextLayer:true, pageScale:1.25, src:pdfAsArray};
  }
  return undefined;
}, video:function(documentObject) {
  var statusButton = Ext.create({xtype:'button', text:'Status: -', hoverCls:'', setStatus:function(status) {
    var states = {play:{key:'PLAYER_PLAY', iconCls:'fa-play-circle'}, playing:{key:'PLAYER_PLAYING', iconCls:'fa-play-circle'}, pause:{key:'PLAYER_PAUSED', iconCls:'fa-pause-circle'}, stopped:{key:'PLAYER_STOPPED', iconCls:'fa-stop-circle'}};
    this.setText('Status: ' + documents.Support.TR(states[status].key));
    this.setIconCls('Status: ' + documents.Support.icon(states[status].iconCls));
  }});
  var dynId = Ext.id();
  var mediaContainer = Ext.create({xtype:'container', hidden:true, flex:1, html:'\x3cdiv class\x3d"mediaWrapper" style\x3d"width: 100%;height: 100%"\x3e\x3cvideo id\x3d"' + dynId + '" height\x3d"100%" width\x3d"100%" autobuffer\x3d"autobuffer"\x3e\x3csource src\x3d"' + documentObject.dataUri + '" /\x3e\x3c/video\x3e\x3c/div\x3e'});
  return {xtype:'panel', layout:{type:'vbox', align:'stretch'}, items:[{xtype:'container', layout:'center', itemId:'player', flex:1, items:[{xtype:'button', iconCls:documents.Support.icon('play-circle'), scale:'large', hoverCLs:'', style:'border:0!important', ui:'secondary', handler:function(b) {
    var audio = document.querySelector('#' + dynId);
    if (audio) {
      if (audio.paused) {
        audio.play();
        statusButton.setStatus('playing');
        b.setIconCls(documents.Support.icon('fa-headphones'));
      } else {
        audio.pause();
        statusButton.setStatus('pause');
        b.setIconCls(documents.Support.icon('fa-pause-circle'));
      }
    }
    mediaContainer.setVisible(true);
    b.up('#player').hide();
  }}]}, mediaContainer], bbar:[statusButton]};
}, audio:function(documentObject) {
  return {xtype:'documentsMedia', documentObject:documentObject};
}, image:function(documentObject) {
  return {xtype:'container', layout:'fit', style:'background-color:#d4d4d4; display: flex;align-items: center;justify-content: center;', canvasId:undefined, getImageAsObject:function() {
    var mime = 'image/jpeg';
    var dataUri = this.getCanvas().toDataURL(mime, 1);
    var base64string = dataUri.split(',')[1];
    return {mime:mime, base64string:base64string};
  }, getCanvas:function() {
    return document.getElementById(this.canvasId);
  }, drawImageToCanvas:function(cmp) {
    var canvas = cmp.getCanvas();
    var ctx = canvas.getContext('2d');
    var img = new Image;
    var w = cmp.getWidth();
    var h = cmp.getHeight();
    img.onload = function() {
      var imageRatio = img.width / img.height, ctxHeight, ctxWidth;
      if (imageRatio <= 1) {
        ctxHeight = h * 0.9;
        ctxWidth = ctxHeight * imageRatio;
      } else {
        ctxWidth = w * 0.9;
        ctxHeight = ctxWidth / imageRatio;
      }
      canvas.height = ctxHeight;
      canvas.width = ctxWidth;
      ctx.drawImage(img, 0, 0, ctxWidth, ctxHeight);
    };
    img.src = documentObject.dataUri;
  }, listeners:{boxready:function(cmp) {
    cmp.canvasId = Ext.id(undefined, 'image-canvas');
    cmp.setHtml('\x3ccanvas id\x3d"' + cmp.canvasId + '"\x3e\x3c/canvas\x3e');
    cmp.drawImageToCanvas(cmp);
  }, resize:function(cmp) {
    cmp.drawImageToCanvas(cmp);
  }}};
}, docx:function(documentObject) {
  return {xtype:'panel', itemId:'docxViewer', wordFile:documents.Support.convertDataURIToBinary(documentObject.dataUri), listeners:{boxready:function(cmp) {
    var id = Ext.id(undefined, 'docx-div');
    cmp.setHtml('\x3cdiv id\x3d"' + id + '"\x3e\x3c/div\x3e');
    var container = document.getElementById(id);
    docx.renderAsync(cmp.wordFile, container, null, {debug:true}).then(function(x) {
      console.log(x);
    });
  }}};
}, xlsx:function(documentObject) {
  return {xtype:'panel', itemId:'xlsxViewer', fileBinary:documents.Support.convertDataURIToBinary(documentObject.dataUri), layout:'fit', listeners:{boxready:function(cmp) {
    cmp.add({xtype:'container', layout:'center', style:'display: flex;align-items: center;justify-content: center;opacity:0.7;', html:'\x3cdiv style\x3d"text-align: center;"\x3e' + BpcCommon.util.Icons.getSpanForIconCls('fa-file-excel', '#000', 50) + '\x3cbr\x3e\x3cbr\x3e' + documents.Support.TR('TYPE_SPREADSHEET') + '\x3c/div\x3e'});
  }}};
}, json:function(documentObject) {
}, xml:function(documentObject) {
}, html:function(documentObject) {
  documentObject = documentObject === undefined ? {} : documentObject;
  var base64_utf8 = documents.Support.convertDataURIToBinary(documentObject.dataUri, true);
  var value = base64_utf8.replace(/\s/g, '');
  value = decodeURIComponent(escape(window.atob(base64_utf8)));
  return {xtype:'docsTextEditor', mime:documentObject.mime, value:value};
}, text:function(documentObject) {
  documentObject = documentObject === undefined ? {} : documentObject;
  var value = documents.Support.convertDataURIToBinary(documentObject.dataUri, true);
  if (documentObject.mime && documentObject.mime.indexOf('xml') !== -1) {
    value = documents.Support.prettyXml(value);
  } else {
    if (documentObject.mime && documentObject.mime.indexOf('edifact') !== -1) {
      value = value.split("'").map(function(i) {
        return i.trim();
      }).join("'\r\n").trim();
    }
  }
  return {xtype:'docsTextEditor', mime:documentObject.mime, value:value};
}});
Ext.define('documents.Support', {singleton:true, baseModuleId:'documents', basePath:'/cxf/bpc-documents/', resourcesBasePath:'/bpc-fe-documents/', defaultInputDateFormat:'C', defaultDateFormat:'d.m.Y H:i:s', defaultDateFormatShort:'d.m. H:i', constructor:function() {
  BpcCommon.Api.onReady(this.bootSearch.bind(this));
}, getUrl:function() {
}, getDocumentsUrl:function(path, params) {
  params = params === undefined ? {} : params;
  return documents.Support.basePath + path + '?' + Ext.Object.toQueryString(params);
}, isBpcVersion:function(targetVersion) {
  targetVersion = targetVersion === undefined ? 4 : targetVersion;
  try {
    var $jscomp$destructuring$var3 = Ext.manifest;
    var packages = $jscomp$destructuring$var3.packages;
    packages = packages || {};
    var common = packages.BpcCommon || {};
    var $jscomp$destructuring$var4 = common;
    var version = $jscomp$destructuring$var4.version;
    version = version || '';
    var parts = version.split('.');
    return parseInt(parts[0]) === targetVersion;
  } catch (e) {
    return false;
  }
}, getDownloadPath:function(documentId) {
  return this.buildDocumentUrl(documentId) + '\x26download\x3dtrue';
}, downloadFile:function(documentId) {
  if (documentId) {
    return BpcCommon.Util.downloadFile(this.getDownloadPath(documentId));
  }
  return false;
}, icon:function(fontAwesomeName) {
  return BpcCommon.util.Icons.getBpcIconCls(fontAwesomeName);
}, libLoaded:function(libSpecifier) {
  return libSpecifier && libSpecifier in window && window[libSpecifier];
}, getWindowCmp:function(title, iconCls, widthMultiplier, heightMultiplier, cmp, customWindowCfg) {
  var sizes = {};
  if (widthMultiplier !== null && heightMultiplier !== null) {
    widthMultiplier = widthMultiplier || 0.5;
    heightMultiplier = heightMultiplier || 0.5;
    sizes = {width:document.body.clientWidth * widthMultiplier, height:document.body.clientHeight * heightMultiplier, minWidth:document.body.clientWidth * widthMultiplier * 0.5, minHeight:document.body.clientHeight * heightMultiplier * 0.5};
  }
  var windowCfg = Ext.apply({iconCls:documents.Support.icon(iconCls || 'fa-file'), title:title || documents.Support.TR('LABEL_DOCUMENT'), layout:'fit', closeAction:'destroy', items:cmp ? [cmp] : []}, Ext.isDefined(customWindowCfg) ? customWindowCfg : {});
  windowCfg = Ext.apply(windowCfg, sizes);
  var win = Ext.create('Ext.window.Window', windowCfg);
  win.show();
  return win;
}, getBaseGridConfigColumns:function(filter) {
  filter = filter === undefined ? [] : filter;
  return [this.getSimpleColumnConfig('id', 180), {dataIndex:'filename', flex:2}, {dataIndex:'documentTitle', flex:1}, this.getSimpleColumnConfig('mime', 120), this.getSimpleColumnConfig('modified', 150, function(v) {
    return documents.Support.dateFormatter(v);
  }), this.getSimpleColumnConfig('created', 150, function(v) {
    return documents.Support.dateFormatter(v);
  })].filter(function(c) {
    return filter.length ? filter.indexOf(c.dataIndex) !== -1 : true;
  }).map(function(c) {
    if (c.dataIndex) {
      c.text = documents.Support.TR('DOC_' + c.dataIndex.toUpperCase());
    }
    return c;
  });
}, getSimpleColumnConfig:function(dataIndex, width, renderer) {
  var col = {dataIndex:dataIndex};
  if (width) {
    col.width = width;
  }
  if (renderer) {
    col.renderer = renderer;
  }
  return col;
}, getBaseGridTagsConfigColumns:function(filter) {
  filter = filter === undefined ? [] : filter;
  return [{dataIndex:'name', width:210, renderer:function(val, m, r) {
    return '\x3cdiv style\x3d"width:200px;' + documents.Support.getTagBoxStyle(false, r.get('color')) + '"\x3e' + val + '\x3c/div\x3e';
  }}, {dataIndex:'color', width:40, hidden:true, renderer:function(v) {
    return '\x3cdiv style\x3d"width:100%;height:100%;background-color:' + v + ';"\x3e\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x3c/div\x3e';
  }}, {dataIndex:'iconCls', width:40, renderer:function(v) {
    return v && v.length > 0 ? BpcCommon.util.Icons.getSpanForIconCls(v, '#000') : '';
  }}, this.getSimpleColumnConfig('id', 180), {dataIndex:'slug', width:80, hidden:true}, {dataIndex:'creator', text:documents.Support.TR('CREATOR'), flex:1}, this.getSimpleColumnConfig('modified', 150, function(v) {
    return documents.Support.dateFormatter(v);
  }), this.getSimpleColumnConfig('created', 150, function(v) {
    return documents.Support.dateFormatter(v);
  })].filter(function(c) {
    return filter.length ? filter.indexOf(c.dataIndex) !== -1 : true;
  }).map(function(c) {
    if (c.dataIndex) {
      c.text = documents.Support.TR('TAG_' + c.dataIndex.toUpperCase());
    }
    return c;
  });
}, renderDate:function(record, key) {
  return Ext.Date.format(Ext.Date.parse(record.get(key), documents.Support.defaultInputDateFormat), documents.Support.defaultDateFormat) || '';
}, getBackendUrl:function(proxyId) {
  try {
    return BpcCommon.Api.getBackendUrl(proxyId);
  } catch (e) {
    return undefined;
  }
}, getButtonTextWithBr:function(labelKey) {
  var translation = documents.Support.TR(labelKey) || '';
  return translation.replaceAll(' ', '\x3cbr\x3e');
}, getDocumentFromRecord:function(record) {
  if (record) {
    var base64file = record.get('base64string');
    var mime = record.get('mime') || '';
    var dataUri = base64file ? 'data:' + mime + ';base64,' + base64file : undefined;
    var tags = record.get('tags') || [];
    tags = Ext.isString(tags) ? [tags] : tags;
    return {dataUri:dataUri, mime:mime, filename:record.get('filename') || '', documentTitle:record.get('documentTitle') || '', tags:tags, creator:record.get('creator') || '', created:documents.Support.dateFormatter(record.get('created')), modified:documents.Support.dateFormatter(record.get('modified')), id:record.get('id'), record:record, iconCls:documents.Support.getIconClsFromRecord(record)};
  }
}, isFolderRecord:function(record) {
  return record && record.get('mime') === 'inode/directory';
}, openDetailEditWindow:function(record, moduleId) {
  var titleTextKey = this.isFolderRecord(record) ? 'EDIT_DOCUMENT_FOLDER' : 'EDIT_DOCUMENT_FILE';
  return documents.Support.getWindowCmp(documents.Support.TR(titleTextKey), 'fa-edit', null, null, {xtype:'bpcDocsEdit', moduleId:moduleId, record:record}, {width:document.body.clientWidth * 0.3});
}, openDetailWindow:function(record, moduleId) {
  var id = record.get('id');
  if (id) {
    var fileDetailWrapper = Ext.create({xtype:'container', flex:4, layout:'fit', items:[]});
    moduleId = moduleId || record.get('instanceId');
    var config = BpcCommon.Api.getSetting(moduleId, 'previewAreaRight') || {};
    var $jscomp$destructuring$var5 = config;
    var buttonsHidden = $jscomp$destructuring$var5.buttonsHidden;
    var changeLogHidden = $jscomp$destructuring$var5.changeLogHidden;
    buttonsHidden = buttonsHidden || false;
    changeLogHidden = changeLogHidden || false;
    var metaGrid = Ext.create({xtype:'bpcDocsMetaGrid', moduleId:moduleId});
    metaGrid.resetInit();
    metaGrid.updateWithRecord(record);
    var historyGrid = Ext.create({xtype:'bpcDocsFileHistoryGrid', header:false, flex:1, hidden:changeLogHidden});
    historyGrid.changeDocumentId(record.getId());
    var detailContainer = Ext.create({xtype:'container', layout:{type:'hbox', align:'stretch'}, items:[fileDetailWrapper, {xtype:'container', flex:2, hidden:changeLogHidden, layout:{type:'vbox', align:'stretch'}, items:[historyGrid, metaGrid]}]});
    var window$8 = documents.Support.getWindowCmp(record.get('documentTitle'), 'fa-file', 0.8, 0.8, detailContainer);
    documents.Support.getDocument(id, moduleId).then(function(fullDocumentObject) {
      var cmp = documents.Support.getViewerComponentByDocument(fullDocumentObject);
      if (cmp) {
        fileDetailWrapper.add(cmp);
      }
    });
  }
}, getDocumentIdsFromColumns:function(assetColumns, r) {
  var delimiter = ',';
  assetColumns = assetColumns || [];
  var obj = {};
  var assetIdStoreData = Ext.Array.map(assetColumns, function(column) {
    var rawValues = r.get(column) || '', meta = {}, trimmed = [];
    if (rawValues) {
      if (rawValues.startsWith('[') || rawValues.startsWith('{')) {
        var json = Ext.decode(rawValues, true);
        if (json != null) {
          if (Ext.isArray(json)) {
            Ext.Array.each(json, function(item) {
              if (item) {
                if (Ext.isString(item)) {
                  var t = item.split(delimiter);
                  t.map(function(v) {
                    trimmed.push(v);
                  });
                } else {
                  if (Ext.isNumber(item)) {
                    item = String(item);
                    var t = item.split(delimiter);
                    t.map(function(v) {
                      trimmed.push(v);
                    });
                  } else {
                    if (Ext.isObject(item)) {
                      Ext.iterate(item, function(key, val) {
                        if (key && val) {
                          i = val;
                          if (Ext.isString(i)) {
                            var t = i.split(delimiter);
                            t.map(function(v) {
                              trimmed.push(v);
                              if (!meta.hasOwnProperty(v)) {
                                meta[v] = [];
                              } else {
                                meta[v].push(key);
                              }
                            });
                          } else {
                            if (Ext.isNumber(i)) {
                              i = String(i);
                              var t = i.split(delimiter);
                              t.map(function(v) {
                                trimmed.push(v);
                                if (!meta.hasOwnProperty(v)) {
                                  meta[v] = [];
                                } else {
                                  meta[v].push(key);
                                }
                              });
                            } else {
                            }
                          }
                        }
                      });
                    } else {
                      if (Ext.isArray(item)) {
                      } else {
                      }
                    }
                  }
                }
              }
            });
          } else {
            if (Ext.isObject(json)) {
              Ext.iterate(json, function(key, val) {
                if (key && val) {
                  item = val;
                  if (Ext.isString(item)) {
                    var t = item.split(delimiter);
                    t.map(function(v) {
                      trimmed.push(v);
                      if (!meta.hasOwnProperty(v)) {
                        meta[v] = [];
                      } else {
                        meta[v].push(key);
                      }
                    });
                  } else {
                    if (Ext.isNumber(item)) {
                      item = String(item);
                      var t = item.split(delimiter);
                      t.map(function(v) {
                        trimmed.push(v);
                        if (!meta.hasOwnProperty(v)) {
                          meta[v] = [];
                        } else {
                          meta[v].push(key);
                        }
                      });
                    } else {
                    }
                  }
                }
              });
            }
          }
        }
      } else {
        trimmed = rawValues.split(delimiter);
      }
    }
    trimmed = trimmed.map(function(val) {
      return val.trim();
    });
    obj[column] = trimmed;
    return {key:column, values:rawValues, trimmed:trimmed, meta:meta, count:trimmed.length};
  });
  return obj;
}, getViewerComponentByDocument:function(obj) {
  obj = obj === undefined ? {} : obj;
  var $jscomp$destructuring$var6 = obj;
  var mime = $jscomp$destructuring$var6.mime;
  if (mime) {
    if (obj.dataUri || mime === 'inode/directory') {
      var mapping = BpcCommon.Api.getSetting('documents', 'frontend_mime_cmp_mapping') || {};
      var mimeComponentKeyMap = {'application/pdf':'pdf', 'application/zip':'zip', 'application/json':'text', 'application/javascript':'text', 'application/edifact':'text', 'text/css':'text', 'inode/directory':'folder'};
      mimeComponentKeyMap = Ext.apply(mimeComponentKeyMap, mapping);
      var componentKey = mimeComponentKeyMap[mime];
      if (componentKey) {
        return documents.DocumentTypes[componentKey](obj);
      }
      if (mime.startsWith('image')) {
        return documents.DocumentTypes.image(obj);
      } else {
        if (mime.startsWith('application') && obj.mime.indexOf('word') !== -1) {
          return documents.DocumentTypes.docx(obj);
        } else {
          if (mime.startsWith('application') && obj.mime.indexOf('spreadsheet') !== -1) {
            return documents.DocumentTypes.xlsx(obj);
          } else {
            if (obj.mime === 'text/html') {
              return documents.DocumentTypes.html(obj);
            } else {
              if (mime.startsWith('text/') || obj.mime === 'text/html') {
                return documents.DocumentTypes.text(obj);
              } else {
                if (mime.startsWith('audio/')) {
                  return documents.DocumentTypes.audio(obj);
                } else {
                  if (mime.startsWith('video/')) {
                    return documents.DocumentTypes.video(obj);
                  } else {
                    if (mime.startsWith('message/rfc822')) {
                      return documents.DocumentTypes.mail(obj);
                    } else {
                      return documents.DocumentTypes.error(obj, {errorMessage:documents.Support.TR('MIME_NOT_SUPPORTED')});
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    return documents.DocumentTypes.error(obj, {errorMessage:documents.Support.TR('MSG_DATA_URI_NOT_DEFINED')});
  }
  return documents.DocumentTypes.error(obj, {errorMessage:documents.Support.TR('MSG_TYPE_NOT_SET')});
}, getDragNDropUrl:function(moduleId) {
  var tokenObj = BpcCommon.Api.getCsrfToken();
  if (moduleId) {
    tokenObj.instanceId = moduleId;
  }
  return documents.Support.basePath + 'upload?' + Ext.Object.toQueryString(tokenObj);
}, createAndShowMessageBox:function(titleKey, messageKey, iconCls, components) {
  iconCls = iconCls === undefined ? 'fa-exclamation-triangle' : iconCls;
  components = components === undefined ? [] : components;
  return new Promise(function(resolve) {
    var m = Ext.create({xtype:'messagebox', closeAction:'destroy'});
    if (components.length) {
      m.add(components);
    }
    m.show({title:documents.Support.TR(titleKey), message:documents.Support.TR(messageKey), buttons:Ext.Msg.OKCANCEL, iconCls:documents.Support.icon(iconCls), fn:function(btn) {
      resolve(btn === 'ok');
    }});
  });
}, fileSizeOk:function(size, moduleId) {
  size = size / 1000;
  return size / 1000 <= BpcCommon.Api.getSetting(moduleId, 'upload_maxFileSize');
}, deleteById:function(url, title, message, cmps) {
  var $jscomp$this = this;
  title = title || 'DELETE_DOCUMENT';
  message = message || 'FILE_WILl_BE_DELETED';
  return new Promise(function(rs, rj) {
    $jscomp$this.createAndShowMessageBox(title, message, undefined, cmps).then(function(succesBool) {
      if (succesBool) {
        $jscomp$this.newRequest(url, undefined, undefined, 'DELETE').then(function(statusCode) {
          rs(statusCode);
        }, function(statusCode) {
          rj(statusCode);
        });
        return;
      }
      rs(false);
    });
  });
}, getWorkflowConnectorPath:function(moduleId) {
  var connectionId = BpcCommon.Api.getSetting(moduleId, 'connectionId');
  if (connectionId) {
    var connectionIdBasePath = documents.Support.getBackendUrl(connectionId);
    if (connectionIdBasePath) {
      var path = BpcCommon.Api.getSetting(moduleId, 'function_externalDocumentHandlingPath');
      if (path) {
        return connectionIdBasePath += path;
      }
    }
  }
}, libsLoaded:false, libsLoading:false, fileLoadPromises:[], bootSearch:function() {
  var $jscomp$this = this;
  return new Promise(function(rs) {
    if ($jscomp$this.libsLoaded) {
      rs(true);
      return;
    }
    if (!$jscomp$this.libsLoaded && !$jscomp$this.libsLoading) {
      $jscomp$this.libsLoading = true;
      var baseUrl = $jscomp$this.resourcesBasePath + 'resources/';
      var files = ['libs.js', 'jszip.js', 'qrcode.js', 'proto/html2canvas.min.js'].map(function(path) {
        return $jscomp$this.loadFile(baseUrl + path);
      });
      Ext.Promise.all(files).then(function(success) {
        $jscomp$this.libsLoaded = true;
        $jscomp$this.libsLoading = false;
        rs(true);
      });
    }
  });
}, getIconClsFromRecord:function(r, key) {
  key = key === undefined ? 'mime' : key;
  var mimeString = documents.Support.getMimeTypeFromRecord(r, key);
  var mimeArray = mimeString.split('/');
  var iconCls = 'file';
  var mimeTypes = {word:'file-word', archive:'file-archive', zip:'file-archive', jar:'file-archive', java:'file-archive', csv:'file-alt', pdf:'file-pdf', json:'file-code', powerpoint:'file-powerpoint', image:'file-image', excel:'file-excel', audio:'file-audio', video:'file-video', rfc822:'envelope', directory:'folder', inode:'folder'};
  if (mimeArray.length) {
    if (mimeArray.length === 1) {
    } else {
      if (mimeArray.length === 2) {
        if (mimeArray[0] in mimeTypes) {
          iconCls = mimeTypes[mimeArray[0]];
        } else {
          if (mimeArray[1] in mimeTypes) {
            iconCls = mimeTypes[mimeArray[1]];
          }
        }
      }
    }
  }
  if (r.get('deleted') === true) {
    iconCls = 'fa-trash';
  }
  return documents.Support.icon(iconCls);
}, TR:function(key, om) {
  return this.getTranslation(key, om);
}, getTranslation:function(key, otherModuleString) {
  key = key === undefined ? '' : key;
  key = key.toUpperCase();
  var starter = 'DOCUMENTS';
  if (Ext.isDefined(otherModuleString)) {
    starter = otherModuleString.toUpperCase();
  }
  starter += '_';
  if (!Ext.String.startsWith(key, starter)) {
    key = starter + key;
  }
  var translation = BpcCommon.Api.getTranslation(key);
  if (key === translation) {
    this.translations.missing[key] = key;
  }
  return translation;
}, translations:{missing:{}}, getMissingTranslations:function() {
  return this.translations.missing;
}, createHeadline:function(key, iconHtml) {
  return '\x3ch3\x3e' + (iconHtml ? iconHtml + ' ' : '') + documents.Support.TR('LABEL_' + key.toUpperCase()) + '\x3c/h3\x3e';
}, buildDataUriFromRecord:function(r) {
  var base64file = r.get('base64string');
  return base64file ? documents.Utils.base64ToDataUrl(base64file, documents.Support.getMimeTypeFromRecord(r)) : undefined;
}, convertDataURIToBinary:function(dataURI, rawOnly) {
  rawOnly = rawOnly === undefined ? false : rawOnly;
  try {
    var marker = ';base64,';
    var markerPosition = dataURI.indexOf(marker) + marker.length;
    var encodedFile = dataURI.substring(markerPosition);
    var raw = window.atob(encodedFile);
    if (rawOnly) {
      return raw;
    }
    var rawLength = raw.length;
    var array = new Uint8Array(new ArrayBuffer(rawLength));
    for (var i = 0; i < rawLength; i++) {
      array[i] = raw.charCodeAt(i);
    }
    return array;
  } catch (e) {
    return undefined;
  }
}, objectToHtmlTable:function(obj, keys, trPrefix) {
  keys = keys === undefined ? [] : keys;
  trPrefix = trPrefix === undefined ? '' : trPrefix;
  obj = obj || {};
  var html = '\x3ctable style\x3d"width: 300px"\x3e';
  Object.keys(obj).filter(function(k) {
    return keys.length ? keys.indexOf(k) !== -1 : true;
  }).map(function(k) {
    html += '\x3ctr\x3e';
    html += '\x3ctd style\x3d"width: 120px"\x3e' + (trPrefix.length ? documents.Support.TR(trPrefix + k) : k) + '\x3c/td\x3e';
    html += '\x3ctd style\x3d"width: 180px"\x3e' + obj[k] + '\x3c/td\x3e';
    html += '\x3c/tr\x3e';
  });
  return html + '\x3c/table\x3e';
}, getTagBoxStyle:function(inActive, color) {
  var bgColor = color || this.getRandomVirtimoColors()[0], textColor = 'bisque', borderColor = '#000';
  if (inActive) {
    bgColor = '#eee';
    textColor = 'darkgrey';
    borderColor = 'darkgrey';
  }
  return 'background-color:' + bgColor + ';color:' + textColor + ';font-size:14px;border:1px; border-style: solid; border-color: ' + borderColor + '; padding-left:0.15em; padding-right: 0.15em;';
}, getIsListenerPath:function(moduleId) {
  var proxyId = BpcCommon.Api.getSetting(moduleId, 'connectionId');
  if (proxyId) {
    var listener = BpcCommon.Api.getSetting(moduleId, 'connectionListener') || '';
    var basePath = BpcCommon.Api.getSetting(moduleId, 'connectionBasePath') || '';
    if (listener.length) {
      return (documents.Support.getBackendUrl(proxyId) || '-1') + basePath + listener;
    }
  }
}, getProxyBaseConfig:function(endpoint, moduleId) {
  endpoint = endpoint === undefined ? 'documents' : endpoint;
  return {type:'ajax', url:documents.Support.basePath + endpoint, reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:moduleId}};
}, newRequest:function(url, params, data, method, timeout) {
  params = params === undefined ? {} : params;
  data = data === undefined ? null : data;
  method = method === undefined ? 'GET' : method;
  timeout = timeout === undefined ? 30000 : timeout;
  return new Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, params:params, timeout:timeout, method:method, jsonData:data, headers:{'Content-Type':'application/json'}, success:function(response) {
      resolve(response.responseText ? Ext.decode(response.responseText, true) : {});
    }, failure:function(response) {
      reject(response);
    }});
  });
}, getSpanForIconCls:function(iconCls, color, size, spin) {
  spin = spin === undefined ? false : spin;
  var legacy = BpcCommon.util.Icons.isLegacyIconCls(iconCls);
  iconCls = BpcCommon.util.Icons.getBpcIconCls(iconCls);
  size = size || BpcCommon.util.Icons.defaultFontSize;
  size += BpcCommon.util.Icons.defaultFontUnit;
  color = color || BpcCommon.util.Icons.fallbackIconColor;
  var marginRight = BpcCommon.util.Icons.marginRight;
  var style = (spin ? 'display:inline-block;' : '') + 'color:' + color + ';' + (legacy ? '' : 'font-size:' + size + ';') + 'margin-right:' + marginRight + ';';
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"' + style + '"\x3e\x3c/span\x3e';
}, dateFormatter:function(utcString, output) {
  return Ext.Date.format(Ext.Date.parse(utcString, documents.Support.defaultInputDateFormat), output || documents.Support.defaultDateFormat);
}, getMimeTypeFromRecord:function(r, key) {
  key = key === undefined ? 'mime' : key;
  return r ? r.get(key) || '' : undefined;
}, getTotalBox:function(icon) {
  icon = icon === undefined ? 'fa-database' : icon;
  return Ext.create({xtype:'button', ui:'secondary', style:{border:0, backgroundColor:'transparent'}, overCls:'', focusCls:'', iconCls:documents.Support.icon(icon), setTitle:function(count) {
    count = count === undefined ? 0 : count;
    this.setText(documents.Support.TR('TXT_TOTAL') + ': ' + count);
  }});
}, getHtmlLinksFromRecord:function(r) {
  if (r) {
    var linksArray = r.get('links');
    if (linksArray && Ext.isArray(linksArray)) {
      var htmlLinks = Ext.Array.map(linksArray, function(linkContext) {
        var html = '';
        if (linkContext && Ext.isObject(linkContext)) {
          Ext.iterate(linkContext, function(k, v) {
            if (k && v && k === 'instanceId') {
              var name = BpcCommon.Api.getSetting(v, 'module_name');
              html += name || 'module_name not found for ' + v;
            }
          });
        }
        return html;
      });
      Ext.log({dump:htmlLinks});
    }
    return htmlLinks;
  }
  return undefined;
}, getDocumentBasePath:function() {
  return this.basePath + 'document';
}, buildDocumentUrl:function(documentId) {
  return this.getDocumentBasePath() + '?id\x3d' + documentId;
}, buildDocumentUrlFromParams:function(params) {
  return this.getDocumentBasePath() + '?' + Ext.Object.toQueryString(params);
}, tagsUrl:function() {
  return this.basePath + 'tags';
}, buildTagsUrl:function(id) {
  return this.tagsUrl() + '?id\x3d' + id;
}, documentResponseToDocumentRecord:function(json) {
  return documents.Support.getDocumentFromRecord(Ext.data.Model.loadData(json || {}));
}, cached:function(documentId) {
  if (documentId && documents.Cache.docs[documentId]) {
    return documents.Cache.docs[documentId];
  }
}, canRead:function(moduleId) {
  return BpcCommon.Api.hasUserAnyRole([documents.Support.adminRole, 'documents_' + moduleId + '_reader']);
}, adminRole:'documents-administrator', canWrite:function(moduleId) {
  return BpcCommon.Api.hasUserAnyRole([documents.Support.adminRole, 'documents_' + moduleId + '_writer']);
}, isAdmin:function() {
  return documents.Api.isDocumentsAdmin();
}, getDocument:function(documentId, moduleId, force) {
  var $jscomp$this = this;
  force = force === undefined ? false : force;
  return new Promise(function(resolve, reject) {
    var params = {instanceId:moduleId, id:documentId};
    $jscomp$this.newRequest($jscomp$this.buildDocumentUrlFromParams(params)).then(function(json) {
      json ? resolve($jscomp$this.documentResponseToDocumentRecord(json)) : reject('No Json data found');
    }, function(response) {
      reject(response);
    });
  });
}, bytesToKb:function(size, asInt) {
  asInt = asInt === undefined ? false : asInt;
  size = size / 1000;
  return asInt ? Math.round(size) : size;
}, loadFile:function(url) {
  return new Promise(function(resolve, reject) {
    Ext.Loader.loadScript({url:url, onLoad:function() {
      resolve(true);
    }, onError:function() {
      resolve(false);
    }});
  });
}, getResponseObject:function(r) {
  if ('responseText' in r && Ext.isDefined(r.responseText)) {
    return Ext.decode(r.responseText, true);
  } else {
    if ('responseJson' in r && r.responseJson) {
      return r.responseJson;
    }
  }
}, copyToClipBoard:function(text, extCmp) {
  if (navigator.clipboard && window.isSecureContext) {
    return navigator.clipboard.writeText(text);
  } else {
    extCmp = extCmp || BpcCommon.Api.getActiveModuleCmp();
    return new Promise(function(resolve) {
      var myFluffyTextarea = document.createElement('textarea');
      myFluffyTextarea.innerHTML = text;
      var parentElement = document.getElementById(extCmp.getId());
      parentElement.appendChild(myFluffyTextarea);
      myFluffyTextarea.select();
      document.execCommand('copy');
      parentElement.removeChild(myFluffyTextarea);
      resolve();
    });
  }
}, getRandomVirtimoColors:function() {
  var shuffle = function shuffleArray(array) {
    for (var i = array.length - 1; i > 0; i--) {
      var j = Math.floor(Math.random() * (i + 1));
      var temp = array[i];
      array[i] = array[j];
      array[j] = temp;
    }
    return array;
  };
  return shuffle(['#46C5BE', '#B5E0E5', '#E6DEDA', '#B2AEB0', '#FF935D', '#F23C4D', '#F7BE57', '#BCB936', '#86CDD4', '#338891', '#FF6E25', '#B80C1C', '#A09A9D', '#D1C3BB', '#B2AEB0']);
}, prettyXml:function(xml, tab, nl) {
  tab = tab === undefined ? '\t' : tab;
  nl = nl === undefined ? '\n' : nl;
  var formatted = '', indent = '';
  var nodes = xml.slice(1, -1).split(/>\s*</);
  if (nodes[0][0] === '?') {
    formatted += '\x3c' + nodes.shift() + '\x3e' + nl;
  }
  for (var i$9 = 0; i$9 < nodes.length; i$9++) {
    var node = nodes[i$9];
    if (node[0] === '/') {
      indent = indent.slice(tab.length);
    }
    formatted += indent + '\x3c' + node + '\x3e' + nl;
    if (node[0] !== '/' && node[node.length - 1] !== '/' && node.indexOf('\x3c/') === -1) {
      indent += tab;
    }
  }
  return formatted;
}});
Ext.define('documents.components.LoadingPanel', {extend:'Ext.panel.Panel', alias:'widget.docsMessagePanel', layout:'fit', initComponent:function() {
  this.messagePanel = Ext.create({xtype:'container', width:'400', height:'150', html:''});
  this.setMessage();
  this.items = [{xtype:'container', layout:'center', items:[this.messagePanel]}];
  this.callParent(arguments);
}, setMessage:function(message, icon, color, size) {
  message = message === undefined ? '' : message;
  icon = icon === undefined ? 'fa-file-exclamation' : icon;
  color = color === undefined ? '#000' : color;
  size = size === undefined ? 40 : size;
  var html = '\x3cdiv style\x3d"display: flex; flex-direction: column; justify-content: center; justify-items: center; align-items: center; padding: 30px;"\x3e' + '\x3cdiv style\x3d"padding: 10px;"\x3e' + BpcCommon.util.Icons.getSpanForIconCls(documents.Support.icon(icon), color, size) + '\x3c/div\x3e' + '\x3cdiv style\x3d"padding: 10px;"\x3e' + message + '\x3c/div\x3e' + '\x3c/div\x3e';
  this.messagePanel.update(html);
}});
Ext.define('documents.components.Tag', {extend:'Ext.container.Container', alias:'widget.bpcDocsTag', layout:{type:'hbox', align:'stretch'}, margin:'0 1 0 1', initComponent:function() {
  var $jscomp$this = this;
  var baseButton = {xtype:'button', overCls:'', focusCls:'', handleMouseEvents:false, text:this._tagName || '-1', flex:1};
  if (this._tagCfg && this._tagCfg.color && this._tagCfg.color.length) {
    baseButton.style = 'background-color:' + this._tagCfg.color + ';';
  }
  if (this._tagCfg && this._tagCfg.iconCls && this._tagCfg.iconCls.length) {
    baseButton.iconCls = documents.Support.icon(this._tagCfg.iconCls);
  }
  var tagDisplay = Ext.create(baseButton);
  this.items = [tagDisplay, {xtype:'button', tooltip:documents.Support.TR('REMOVE'), iconCls:documents.Support.icon('fa-times'), handler:function(b) {
    $jscomp$this.destroy();
  }}];
  this.callParent();
}, getTagId:function() {
  return this._tagId;
}});
Ext.define('documents.components.bpc.mixins.BpcPluginMixin', {getValueFromPluginConfiguration:function(key, fromContext) {
  if (this.pluginConfiguration && Ext.isObject(this.pluginConfiguration)) {
    if (!fromContext) {
      var $jscomp$destructuring$var7 = this.pluginConfiguration;
      var configuration = $jscomp$destructuring$var7.configuration;
      configuration = configuration || {};
      if (key in configuration && configuration[key]) {
        return configuration[key];
      }
    } else {
      if (fromContext) {
        if (documents.util.BpcSupport.versionStartsWith('4.1')) {
          var supportMap = {moduleId:'baseModuleId', instanceId:'moduleId'};
          if (supportMap.hasOwnProperty(key || '')) {
            key = supportMap[key];
          }
        }
        var $jscomp$destructuring$var8 = this.pluginConfiguration;
        var context = $jscomp$destructuring$var8.context;
        context = context || {};
        if (key in context && context[key]) {
          return context[key];
        }
      }
    }
  }
}, countInBraces:false, baseTitle:undefined, updateTitle:function(title, count) {
  count = count === undefined ? 0 : count;
  title = title || this.baseTitle || '';
  title = title + (this.countInBraces ? ' ( ' + count + ' )' : '');
  this.setTitle(title);
  if (this.onTitleChange) {
    try {
      this.onTitleChange(title, this);
    } catch (e) {
    }
  }
}, onTitleChange:function(title, plugin) {
}, getMonitorActionPanel:function() {
  var w = this.up('window');
  if (w) {
    var actionPanel = w.down('actionPanel');
    if (actionPanel) {
      this.monitorActionPanel = actionPanel;
      return actionPanel;
    }
  }
}, pluginRefresh:function() {
  this.loadData();
}, pluginLoad:function() {
  this.loadData();
}, getActionButtonCfg:function(text, iconCls, tooltip, actionMethod, actionCfg) {
  var $jscomp$this = this;
  return {xtype:'button', iconCls:documents.Support.icon(iconCls), text:documents.Support.TR(text), action:actionCfg, tooltip:documents.Support.TR(tooltip), handler:actionMethod ? actionMethod : function(b) {
    if ($jscomp$this.executeAction) {
      $jscomp$this.executeAction(b);
    }
  }};
}, addActionsToMonitor:function(actionButtons) {
  var $jscomp$this = this;
  if (this.monitorActionPanel) {
    this.actionsButtons = actionButtons.map(function(btnCfg) {
      return $jscomp$this.monitorActionPanel.add(btnCfg);
    });
  }
}, isMonitor:function() {
  var baseModuleId = this.getValueFromPluginConfiguration('moduleId', true);
  return baseModuleId && baseModuleId === 'monitor';
}});
Ext.define('documents.components.bpc.mixins.Component', {handleModuleRights:function() {
  this.canRead = documents.Support.canRead(this.moduleId);
  this.canWrite = documents.Support.canWrite(this.moduleId);
  if (this.canWrite) {
    this.canRead = true;
  }
  this.isAdmin = documents.Support.isAdmin(this.moduleId);
  if (this.isAdmin) {
    this.canWrite = true;
    this.canRead = true;
  }
}, TR:function(k) {
  return documents.Support.TR(k);
}, faIcon:function(cls) {
  return documents.Support.icon(cls);
}});
Ext.define('documents.components.bpc.monitorJump.Functions', {alias:'widget.bpcDocsJumpFunctions', singleton:true, jumpParamFn:function(record, column) {
  if (column && 'dataIndex' in column && column.dataIndex) {
    return {documentId:record ? record.get(column.dataIndex) || null : null};
  }
  return undefined;
}});
Ext.define('documents.components.editor.Editor', {extend:'Ext.panel.Panel', alias:'widget.docsTextEditor', editorTheme:'chrome', editorLanguage:'xml', editor:undefined, editorMode:'inline', mime:undefined, value:undefined, layout:'fit', readOnly:true, copyEnabled:true, initComponent:function() {
  var $jscomp$this = this;
  this.editorId = Ext.id();
  this.items = [this.getEditorWrapper()];
  this.listeners = {afterrender:this.initEditor.bind(this), beforedestroy:this.destroyEditor.bind(this), resize:this.onResize.bind(this)};
  if (this.copyEnabled) {
    this.tools = [{iconCls:documents.Support.icon('fa-copy'), tooltip:documents.Support.TR('ACTION_COPY'), handler:function(event, toolEl, panelHeader) {
      if (documents.Support.copyToClipBoard($jscomp$this.getValue())) {
        BpcCommon.Api.showNotification(documents.Support.TR('GENERIC_COPY_SUCCESS'));
      }
    }}];
  }
  this.callParent(arguments);
}, getEditorWrapper:function() {
  return Ext.create({xtype:'container', html:'\x3cdiv style\x3d"height: 100%;width: 100%"\x3e\x3cdiv style\x3d"height: 100%;width: 100%" id\x3d"' + this.editorId + '"\x3e\x3c/div\x3e\x3c/div\x3e'});
}, initEditor:function() {
  var $jscomp$this = this;
  if (!window.ace) {
    lazyLoad.Ace().then(function() {
      $jscomp$this.initEditor();
    });
    return;
  }
  this.editor = ace.edit(this.editorId);
  this.editor.setTheme('ace/theme/' + this.editorTheme);
  if (this.mime) {
    this.setMime(this.mime);
  } else {
    this.setAceMode(this.editorLanguage);
  }
  this.editor.setOptions({enableBasicAutocompletion:true, enableLiveAutocompletion:true, showLineNumbers:true, showPrintMargin:false});
  this.setReadOnly(this.readOnly);
  this.editor.on('change', this.onEditorChange.bind(this));
  if (this.value) {
    this.setValue(this.value);
  }
}, setMime:function(mime) {
  mime = mime === undefined ? '' : mime;
  var map = {'application/json':'json', 'application/javascript':'javascript', 'application/ecmascript':'javascript', 'application/edifact':'edifact', 'text/css':'css', 'text/html':'html', 'text/xml':'xml'};
  var editorLanguage = map[mime] || (mime.indexOf('xml') !== -1 ? 'xml' : undefined);
  if (editorLanguage) {
    this.setAceMode(editorLanguage);
  }
}, setAceMode:function(editorLanguage) {
  this.editor.getSession().setMode('ace/mode/' + editorLanguage);
}, setReadOnly:function(readOnly) {
  this.editor.setReadOnly(readOnly);
}, onEditorChange:function() {
  this.fireEvent('aceEditorValueChange', {isEmpty:(this.editor.getValue() || '').length === 0});
}, onResize:function() {
  if (this.editor) {
    this.editor.resize();
  }
}, getValue:function(base64) {
  base64 = base64 === undefined ? false : base64;
  if (this.editor) {
    var value = this.editor.getValue();
    if (this.editorLanguage === 'edifact') {
      return value.split('\n').join('').trim();
    }
    return base64 ? btoa(value) : value;
  }
}, setValue:function(value) {
  if (this.editor) {
    this.editor.setValue(value, -1);
  }
}, destroyEditor:function() {
  if (this.editor) {
    this.editor.destroy.bind(this.editor);
  }
}});
Ext.define('documents.components.field.File', {extend:'Ext.form.field.File', alias:['widget.docsFileUpload'], commitedUpload:false, moduleId:undefined, uploadGrid:undefined, buttonText:documents.Support.TR('COMPONENTS_FILE_BUTTON_TEXT'), validateFileSize:true, initComponent:function() {
  var $jscomp$this = this;
  this.name = 'file';
  this.listeners = {change:function(fileField, value, eOpts) {
    var ok = true, file;
    try {
      if (value && value.length) {
        file = fileField.fileInputEl.dom.files[0];
        ok = $jscomp$this.validateFileSize ? $jscomp$this.checkFile(file) : true;
      } else {
        return;
      }
    } catch (e) {
      ok = false;
    }
    if (ok) {
      fileField.setRawValue(value.replace(/(^.*(\\|\/))?/, ''));
      if (!$jscomp$this.commitedUpload) {
        $jscomp$this.uploadGrid.addFileToStore(file);
        fileField.reset();
      }
    } else {
      BpcCommon.Api.showNotification(documents.Support.TR('FILE_TOO_LARGE'));
      fileField.reset();
    }
  }};
  this.callParent(arguments);
}, checkFile:function(file) {
  file = file === undefined ? {} : file;
  return Ext.isDefined(file.size) && documents.Support.fileSizeOk(file.size, this.moduleId);
}, performRequest:function(fileToSubmit, r) {
  var xhr = new XMLHttpRequest;
  var fd = new FormData;
  xhr.open('POST', this.getUploadUrl(), true);
  fd.append('file', fileToSubmit);
  xhr.onreadystatechange = function() {
    if (xhr.readyState === 4 && xhr.status === 200) {
      if (Ext.decode(xhr.responseText).success) {
        this.reset();
      }
    } else {
      if (xhr.readyState == 4 && xhr.status == 404) {
      }
    }
  }.bind(this);
  xhr.send(fd);
}, getUploadUrl:function() {
  return documents.Support.getDragNDropUrl(this.moduleId) + (this.documentId ? '\x26id\x3d' + this.documentId : '');
}, doUploadFile:function() {
  var data = new FormData;
  data.append('file', this.fileInputEl.dom.files[0]);
  return fetch(this.getUploadUrl(), {method:'POST', body:data});
}});
Ext.define('documents.components.field.Text', {extend:'Ext.form.field.Text', alias:['widget.documentsTextField'], emptyText:' - ', anchor:'90%', trKey:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.listeners = {afterrender:function() {
    $jscomp$this.getEl().on('dblclick', function() {
      var v = $jscomp$this.getValue();
      if (v) {
        if (documents.Support.copyToClipBoard(v)) {
          BpcCommon.Api.showNotification(v + ' ' + documents.Support.TR('HAS_BEEN_COPIED'));
        }
      }
    });
  }, change:function(f, newVal) {
    var t = $jscomp$this.getTrigger('reset');
    if (t) {
      t.setVisible(newVal && newVal.length > 0);
    }
  }};
  if (this.trKey) {
    this.fieldLabel = documents.Support.TR(this.trKey);
  }
  this.callParent(arguments);
  this.setTriggers({reset:{cls:documents.Support.icon('fa-times'), hidden:(this.getValue() || '').length === 0, handler:function() {
    $jscomp$this.setValue(undefined);
  }}});
}});
Ext.define('documents.components.field.bpcconfig.Storage', {extend:'Ext.form.field.ComboBox', alias:['widget.bpcconfig.docsStorage', 'widget.docsStorage'], viewModel:{data:{}}, displayField:'key', valueField:'value', bind:{value:'{setting.value}'}, setting:undefined, moduleId:undefined, forceSelection:true, initComponent:function() {
  this.store = Ext.create('Ext.data.Store', {data:[{key:'STORAGE_MIXED_DB', value:documents.Utils.DATABASE_ADAPTER_ID}, {key:'STORAGE_ES', value:documents.Utils.FAST_STORAGE_ADAPTER_ID()}, {key:'STORAGE_REMOTE_INUBIT', value:'inubit'}].map(function(o) {
    o.key = documents.Support.TR(o.key);
    return o;
  })});
  if (!this.setting) {
    this.setting = {value:this.getValueViaApi()};
  }
  this.callParent(arguments);
  this.getViewModel().set('setting', this.setting);
}, getValueViaApi:function() {
  return BpcCommon.Api.getSetting(this.moduleId, 'documentStorage');
}});
Ext.define('documents.components.grid.FileHistoryGrid', {extend:'Ext.grid.Panel', alias:'widget.bpcDocsFileHistoryGrid', plugins:[{ptype:'rowwidget', widget:{xtype:'container', layout:'fit', items:[]}, onWidgetAttach:function(plugin, cmp, record) {
  cmp.removeAll();
  var changed = record.get('changes') || {oldValues:{}, newValues:{}};
  var allKeys = Ext.Array.union(Object.keys(changed.oldValues), Object.keys(changed.newValues));
  var storeData = allKeys.map(function(k) {
    var storeObj = {};
    storeObj.key = k;
    storeObj.oldValue = changed.oldValues[k] || '-1';
    storeObj.newValue = changed.newValues[k] || '-1';
    return storeObj;
  });
  cmp.add({xtype:'grid', store:Ext.create('Ext.data.Store', {data:storeData}), header:false, columns:[{dataIndex:'key', text:documents.Support.TR('KEY'), flex:1}, {dataIndex:'oldValue', text:documents.Support.TR('VALUE_OLD'), flex:2}, {dataIndex:'newValue', text:documents.Support.TR('VALUE_NEW'), flex:2}]});
}}], viewConfig:{emptyText:documents.Support.TR('NO_HISTORY_FOUND'), enableTextSelection:true}, style:'border: 1px solid #eee;', initComponent:function() {
  var $jscomp$this = this;
  this.totalChangesBtn = documents.Support.getTotalBox();
  this.store = Ext.create('Ext.data.Store', {proxy:{type:'ajax', url:documents.Support.basePath + 'document', reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{id:this.documentId, history:true}, listeners:{exception:function(p, r) {
  }}}, listeners:{load:function(s, records, success) {
    $jscomp$this.totalChangesBtn.setTitle(success ? s.getTotalCount() : 0);
  }}});
  this.columns = [{dataIndex:'activityType', width:35, renderer:function(v) {
    var activities = {create:{iconCls:'fa-plus-circle'}, 'import':{iconCls:'fa-file-import'}, update:{iconCls:'fa-edit'}, fileChanged:{iconCls:'fa-file'}, download:{iconCls:'fa-download'}, 'delete':{iconCls:'fa-trash'}, deleteFlag:{iconCls:'fa-ban'}, restore:{iconCls:'fa-undo'}, unknown:{iconCls:'fa-circle-question'}};
    var target = activities[v] || activities['unknown'];
    return BpcCommon.util.Icons.getSpanForIconCls(target.iconCls, '#000');
  }}, {dataIndex:'created', width:110, renderer:function(v) {
    return documents.Support.dateFormatter(v, 'd.m. H:i');
  }}, {dataIndex:'changes', flex:2, renderer:function(v) {
    if (v) {
      return Ext.Array.union(Object.keys(v.oldValues || {}), Object.keys(v.newValues || {})).join(', ');
    }
    return '';
  }}, {dataIndex:'creator', flex:1}].map(function(c) {
    c.text = documents.Support.TR('DOCUMENTS_DOC_' + c.dataIndex);
    return c;
  });
  this.title = documents.Support.TR('CHANGES'), this.iconCls = documents.Support.icon('fa-history'), this.bbar = [this.totalChangesBtn, {xtype:'tbfill'}];
  this.callParent(arguments);
}, changeDocumentId:function(documentId) {
  this.documentId = documentId;
  var proxy = this.store.getProxy();
  var p = proxy.getExtraParams();
  p.id = this.documentId;
  proxy.setExtraParams(p);
  this.store.load();
}});
Ext.define('documents.components.grid.MetaGrid', {extend:'Ext.grid.Panel', alias:'widget.bpcDocsMetaGrid', hideHeaders:true, viewConfig:{markDirty:false, style:'background-color: #e2e2e2;', enableTextSelection:true}, style:'border: 1px solid #eee;', moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.columns = [{dataIndex:'key', flex:2, renderer:function(k) {
    return '\x3cstrong\x3e' + documents.Support.TR('DOC_' + k.toUpperCase()).toUpperCase() + '\x3c/strong\x3e';
  }}, {dataIndex:'value', flex:4, renderer:function(v, m, r) {
    if (r.get('key') === 'tags') {
      var tagsValue = r.get('value') || [];
      return tagsValue.length ? documents.Utils.ui.getInlineTags(tagsValue, $jscomp$this.moduleId) : '';
    }
    return v || '';
  }}];
  this.store = Ext.create('Ext.data.Store', {data:[]});
  this.listeners = {rowdblclick:function(table, record, element, rowIndex) {
    documents.Utils.copy.clipboard(record, 'value', 'Has been added to your clipboard', table);
  }};
  this.callParent(arguments);
}, resetInit:function() {
  this.store.removeAll();
  var visibleMetaData = BpcCommon.Api.getSetting(this.moduleId, 'visibleMetaData') || ['documentTitle', 'filename', 'description', 'tags', 'mime', 'id', 'slug', 'creator', 'created', 'modified', 'bytes', 'url'];
  this.store.add(visibleMetaData.map(function(name) {
    return {key:name, value:undefined};
  }));
}, updateWithRecord:function(rec) {
  this.store.each(function(r) {
    var k = r.get('key');
    var v = rec.get(k);
    if (['created', 'modified'].indexOf(k) !== -1) {
      v = documents.Support.dateFormatter(v);
    }
    if (k === 'bytes') {
      v = documents.Support.bytesToKb(v) + ' KB';
    }
    if (k === 'url') {
      v = documents.Api.getDownloadUrl(rec.get('id'));
    }
    if (k === 'tags') {
      v = v || [];
    }
    r.set('value', v || ' - ');
  });
}});
Ext.define('documents.components.grid.TagsGrid', {extend:'Ext.grid.Panel', alias:'widget.bpcDocsTagsGrid', selModel:{mode:'MULTI', allowDeselect:false}, resetForms:function() {
  this.previewBtn.changeColor('#fff');
  this.iconClsField.setValue(undefined);
  this.idField.setValue(undefined);
  this.tagNameField.setValue(undefined);
  this.getSelectionModel().deselectAll();
}, initComponent:function() {
  var $jscomp$this = this;
  this.iconCls = BpcCommon.util.Icons.getBpcIconCls('fa-tags');
  this.title = documents.Support.TR('TAGS');
  this.viewConfig = {emptyText:documents.Support.TR('EMPTYTEXT_GRID'), enableTextSelection:true};
  this.store = Ext.create('Ext.data.BufferedStore', {proxy:{type:'ajax', url:documents.Support.tagsUrl(), reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:this.moduleId}}, autoLoad:true, listeners:{prefetch:function(s, records, success) {
    if (success) {
      $jscomp$this.totalBox.setTitle(success ? s.getTotalCount() : '0');
    }
  }}});
  this.tagNameField = Ext.create({xtype:'textfield', flex:1, allowBlank:false, minLength:2, emptyText:documents.Support.TR('PLEASE_ENTER'), fieldLabel:documents.Support.TR('TAG_NAME'), listeners:{change:function(f, newVal) {
    return $jscomp$this.toggleSend();
  }}});
  this.iconClsField = Ext.create({xtype:'textfield', flex:1, allowBlank:true, emptyText:documents.Support.TR('PLEASE_ENTER'), fieldLabel:documents.Support.TR('TAG_ICONCLS'), listeners:{change:function(f, newVal) {
    return $jscomp$this.toggleSend();
  }}});
  this.idField = Ext.create({xtype:'textfield', width:200, readOnly:true, emptyText:documents.Support.TR('ID'), listeners:{change:function(f, newVal) {
    return $jscomp$this.toggleSend();
  }}});
  this.previewBtn = Ext.create({xtype:'button', width:150, changeColor:function(c) {
    this.setStyle({backgroundColor:c, color:c});
    this.color = c;
  }, handler:function(b) {
    return $jscomp$this.copyValue(b.color);
  }});
  var colors = documents.Support.getRandomVirtimoColors() || [];
  this.previewBtn.changeColor(colors.length ? colors[0] : '#fff');
  this.createEditBtn = Ext.create({xtype:'button', tagMode:'create', ui:'primary', text:documents.Support.TR('ACTION_CREATE'), iconCls:documents.Support.icon('fa-plus'), handler:function(b) {
    var name = $jscomp$this.tagNameField.getValue();
    var tag = {instanceId:$jscomp$this.moduleId, name:name, iconCls:$jscomp$this.iconClsField.getValue(), color:$jscomp$this.previewBtn.color};
    if (($jscomp$this.idField.getValue() || '').length === 0) {
      documents.Api.createTag(tag).then(function(success) {
        $jscomp$this.store.load();
        $jscomp$this.resetForms();
      });
    } else {
      tag.id = $jscomp$this.idField.getValue();
      documents.Api.updateTag(tag).then(function(success) {
        $jscomp$this.store.load();
        $jscomp$this.resetForms();
      });
    }
  }});
  this.resetBtn = Ext.create({xtype:'button', text:documents.Support.TR('RESET'), iconCls:documents.Support.icon('fa-times'), handler:this.resetForms.bind(this), ui:'secondary'});
  this.deleteBtn = Ext.create({xtype:'button', text:documents.Support.TR('ACTION_DELETE'), iconCls:documents.Support.icon('fa-trash'), ui:'secondary', handler:function(b) {
    documents.Support.deleteById(documents.Support.buildTagsUrl($jscomp$this.getSelection().map(function(r) {
      return r.get('id');
    })), 'DELETE_TAG', 'TAG_WILL_BE_DELETED').then(function(success) {
      $jscomp$this.store.load();
      $jscomp$this.resetForms();
    });
  }});
  this.topContainer = Ext.create({xtype:'fieldset', flex:1, title:documents.Support.TR('TAG_MANAGEMENT'), layout:{type:'fit'}, items:[{xtype:'panel', header:false, flex:1, layout:{type:'vbox', align:'stretch'}, items:[this.tagNameField, this.iconClsField, {xtype:'fieldcontainer', layout:'fit', fieldLabel:documents.Support.TR('COLOR'), items:[{xtype:'toolbar', items:colors.map(function(color) {
    return {xtype:'button', width:30, margin:'2 2 2 2', style:'background-color:' + color + ';color:' + color + ';', handler:function(b) {
      $jscomp$this.previewBtn.changeColor(color);
    }};
  })}]}], bbar:[this.createEditBtn, this.deleteBtn, this.resetBtn, {xtype:'tbfill'}, this.previewBtn, this.idField]}]});
  this.tbar = [this.topContainer];
  this.listeners = {select:function(rowModel, record, index, eopts) {
    $jscomp$this.previewBtn.changeColor(record.get('color') || '#fff');
    $jscomp$this.idField.setValue(record.get('id'));
    $jscomp$this.tagNameField.setValue(record.get('name'));
    $jscomp$this.iconClsField.setValue(record.get('iconCls'));
  }, rowdblclick:function(table, r, element, rowIndex) {
    $jscomp$this.copyValue(r.get('id'));
  }};
  this.columns = documents.Support.getBaseGridTagsConfigColumns().concat([{xtype:'actioncolumn', width:60, items:[{iconCls:documents.Support.icon('fa-clone'), tooltip:documents.Support.TR('CLONE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    var $jscomp$destructuring$var9 = r.getData() || {};
    var name = $jscomp$destructuring$var9.name;
    var color = $jscomp$destructuring$var9.color;
    var iconCls = $jscomp$destructuring$var9.iconCls;
    var instanceId = $jscomp$destructuring$var9.instanceId;
    var id = $jscomp$destructuring$var9.id;
    documents.Api.createTag({name:name, color:color, iconCls:iconCls, instanceId:instanceId, id:id}).then(function(success) {
      if (success) {
        $jscomp$this.store.load();
      }
    });
  }}, {iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('ACTION_DELETE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Api.deleteTag(r.get('id')).then(function(success) {
      if (success) {
        $jscomp$this.store.load();
      }
    });
  }}]}]);
  this.totalBox = documents.Support.getTotalBox();
  this.bbar = [this.totalBox, {xtype:'tbfill'}, {iconCls:documents.Support.icon('fa-refresh'), text:documents.Support.TR('REFRESH'), ui:'secondary', style:{border:0}, handler:function() {
    return $jscomp$this.store.load();
  }}];
  this.callParent(arguments);
  this.toggleSend();
  this.store.load();
}, copyValue:function(v) {
  if (v) {
    if (documents.Support.copyToClipBoard(v)) {
      BpcCommon.Api.showNotification(v + ' has been added to your clipboard');
    }
  }
}, toggleSend:function() {
  var validatedFields = [this.idField, this.tagNameField, this.iconClsField].map(function(f) {
    return f.validate();
  });
  this.createEditBtn.setDisabled(validatedFields.indexOf(false) !== -1);
  var idEmpty = (this.idField.getValue() || '').length === 0;
  this.deleteBtn.setDisabled(idEmpty);
  this.createEditBtn.setIconCls(BpcCommon.util.Icons.getBpcIconCls(idEmpty ? 'fa-plus' : 'fa-save'));
  this.createEditBtn.setText(documents.Support.TR(documents.Support.TR(idEmpty ? 'ACTION_CREATE' : 'ACTION_EDIT')));
}});
Ext.define('documents.components.grid.Upload', {extend:'Ext.grid.Panel', alias:'widget.bpcDocsUploadGridPanel', defaultFileProperties:undefined, customFields:[], requiredFields:[], uploadMain:undefined, setPredefinedValues:function(valueFrom) {
  valueFrom = valueFrom === undefined ? {} : valueFrom;
  var defaultValue = '';
  try {
    var $jscomp$destructuring$var10 = valueFrom;
    var source = $jscomp$destructuring$var10.source;
    var key = $jscomp$destructuring$var10.key;
    var target = $jscomp$destructuring$var10.target;
    if (source === 'globalViewModel') {
      var globalVm = BpcCommon.Api.getGlobalViewModel();
      var parts = key.split('.');
      var temp = globalVm.data || {};
      parts.map(function(part) {
        temp = temp[part];
      });
      var $jscomp$destructuring$var11 = target;
      var property = $jscomp$destructuring$var11.property;
      var type = $jscomp$destructuring$var11.type;
      if (type === 'record') {
        var v = temp.get(property);
        if (v) {
          defaultValue = v;
        }
      }
    }
  } catch (e) {
    defaultValue = 'FILL_ME';
  }
  return defaultValue;
}, notReadyKey:'FILL_ME', initComponent:function() {
  var $jscomp$this = this;
  this.tags = [];
  this.viewConfig = {emptyText:documents.Support.TR('DROP_OR_ADD_FILES_HERE'), deferEmptyText:false};
  this.requiredFields = BpcCommon.Api.getSetting(this.moduleId, 'upload_requiredFields') || [];
  this.selModel = 'rowmodel';
  this.plugins = {rowediting:{clicksToEdit:1, startEdit:function(record, columnHeader) {
    var me = this, editor = me.getEditor(), context;
    if (Ext.isEmpty(columnHeader)) {
      columnHeader = me.grid.getTopLevelVisibleColumnManager().getHeaderAtIndex(0);
    }
    if (editor.beforeEdit() !== false) {
      context = me.getEditingContext(record, columnHeader, true);
      if (context && context.record.get('status') !== 'updated') {
        if (context && me.beforeEdit(context) !== false && me.fireEvent('beforeedit', me, context) !== false && !context.cancel) {
          me.context = context;
          if (me.lockingPartner) {
            me.lockingPartner.cancelEdit();
          }
          editor.startEdit(context.record, context.column, context);
          me.editing = true;
          return true;
        }
      } else {
        documents.Utils.toast(documents.Support.TR('EDIT_NOT_POSSIBLE'), documents.Support.TR('FILE_ALREADY_UPLOADED'), undefined, me.grid);
      }
    }
    return false;
  }}};
  var tagStore = Ext.create('Ext.data.Store', {data:[]});
  var storeModelFields = ['documentTitle', 'description', 'size', 'file', 'status', 'name', 'tags', 'mime', 'documentObject', 'isRaw'];
  this.customFields = [];
  this.customFieldDefaultsMap = {};
  this.missingDefaults = {};
  this.readyToUpload = true;
  var customFieldDefinitions = BpcCommon.Api.getSetting(this.moduleId, 'customFields') || {};
  var customFieldColumns = Object.keys(customFieldDefinitions).map(function(dataIndex) {
    var fieldDefinition = customFieldDefinitions[dataIndex] || {};
    var textDefinitions = fieldDefinition['text'] || {};
    var valueFrom = fieldDefinition['valueFrom'];
    if (valueFrom) {
      var defaultValue = $jscomp$this.setPredefinedValues(valueFrom);
      if (defaultValue && defaultValue === $jscomp$this.notReadyKey && $jscomp$this.readyToUpload) {
        $jscomp$this.readyToUpload = false;
        var params = {};
        params[dataIndex] = valueFrom || {};
        $jscomp$this.missingDefaults = params;
      }
      $jscomp$this.customFieldDefaultsMap[dataIndex] = defaultValue;
    }
    var languageShort = BpcCommon.Api.getSetting('_core', 'language');
    var text = textDefinitions[languageShort] || documents.Support.TR('CUSTOM_FIELD_' + dataIndex);
    storeModelFields.push(dataIndex);
    $jscomp$this.customFields.push(dataIndex);
    return {dataIndex:dataIndex, editor:{xtype:'textfield'}, flex:2, text:text};
  });
  this.store = Ext.create('Ext.data.Store', {fields:storeModelFields});
  this.columns = [{dataIndex:'status', width:35, renderer:documents.Utils.renderer.uploadStateColor}, {dataIndex:'status', width:35, renderer:documents.Utils.renderer.uploadStateIcon}, {dataIndex:'mime', sortable:false, width:35, renderer:documents.Utils.renderer.documentIcon}, {text:documents.Support.TR('DOC_DOCUMENTTITLE'), editor:'textfield', dataIndex:'documentTitle', flex:1}, {text:documents.Support.TR('DOC_DESCRIPTION'), dataIndex:'description', editor:'textfield', flex:2}, {text:documents.Support.TR('DOC_TAGS'), 
  dataIndex:'tags', editor:{xtype:'tagfield', store:tagStore, displayField:'tagName', valueField:'tags', queryMode:'local', filterPickList:true}, flex:1, renderer:documents.Utils.renderer.tags.bind(this)}].concat(customFieldColumns).concat([{dataIndex:'size', renderer:documents.Utils.renderer.bytesToKb}, {xtype:'actioncolumn', width:30, items:[{iconCls:documents.Support.icon('fa-times'), tooltip:documents.Support.TR('REMOVE_FROM_LIST'), handler:function(tv, rI, cI, item, e, r, row) {
    $jscomp$this.store.remove(r);
  }}]}]);
  this.listeners = {afterrender:function(c) {
    this.target = new Ext.drag.Target({element:c.body, listeners:{scope:c, drop:this.onDrop}});
  }};
  var padding = BpcCommon.Api.getSetting(this.moduleId, 'uploadHeaderPadding') || 5;
  padding = (padding + ' ').repeat(4).trim();
  this.dockedItems = [{xtype:'toolbar', dock:'top', padding:padding, items:[{xtype:'docsFileUpload', validateFileSize:false, fieldLabel:documents.Support.TR('DOC_FILE'), uploadGrid:this, emptyText:documents.Support.TR('EMPTYTEXT_FILE_FIELD'), moduleId:this.moduleId, flex:1}]}];
  this.callParent(arguments);
  documents.Api.getTags({instanceId:this.moduleId}).then(function(json) {
    if (json) {
      $jscomp$this.tags = json;
      json.map(function(obj) {
        tagStore.add({tags:obj.id, tagName:obj.name});
      });
    }
  });
  if (!this.readyToUpload) {
    try {
      var components = Ext.ComponentQuery.query('bpcDocsMain');
      var vm = undefined;
      if (components.length && components.length === 1) {
        vm = components[0].getViewModel();
      }
      if (vm) {
        var win = Ext.create({xtype:'theGatewayOrgSelectionPopup', viewModel:{parent:vm}});
        win.show();
        win.toFront();
        win.on({destroyable:true, close:function(w) {
          var bpcDocsCreate = $jscomp$this.up('bpcDocsCreate');
          if (bpcDocsCreate) {
            bpcDocsCreate.doShowWindow();
          }
          $jscomp$this.updatePredefinedDefaults();
        }});
      }
    } catch (e) {
    }
  }
}, updatePredefinedDefaults:function() {
  var $jscomp$this = this;
  Ext.defer(function() {
    Object.keys($jscomp$this.missingDefaults).map(function(dataIndex) {
      var valueFrom = $jscomp$this.missingDefaults[dataIndex] || {};
      var defaultValue = $jscomp$this.setPredefinedValues(valueFrom);
      if (defaultValue !== $jscomp$this.notReadyKey) {
        $jscomp$this.customFieldDefaultsMap[dataIndex] = defaultValue;
      } else {
        Ext.toast('UNEXPECTED_ERROR');
      }
    });
  }, 300);
}, hasUploads:function() {
  var $jscomp$this = this;
  var hasUploads = false;
  this.store.each(function(r) {
    if (r.get('status') === 'added') {
      if (!hasUploads) {
        if ($jscomp$this.requiredFields.length) {
          var fieldsDefined = $jscomp$this.requiredFields.map(function(dataIndex) {
            return (r.get(dataIndex) || '').length > 0;
          });
          if (fieldsDefined.indexOf(false) === -1) {
            hasUploads = true;
          } else {
            r.set('status', 'incomplete');
          }
        } else {
          hasUploads = true;
        }
      }
    }
  });
  return hasUploads;
}, addFolderItem:function() {
  this.store.add({name:'NEW FOLDER NAME', mime:'inode/directory', documentTitle:'NEW FOLDER NAME', description:'', tags:[], size:0, status:'added'});
}, addRawItem:function(item) {
  item = item === undefined ? {} : item;
  this.store.add(Ext.applyIf(item, {name:'NEW DOCUMENT', isRaw:true, documentTitle:'NEW FILE NAME', description:'', tags:[], status:'added'}));
}, uploadAll:function() {
  var $jscomp$this = this;
  var recordsToUpload = [];
  if (this.hasUploads()) {
    this.store.each(function(r) {
      if (r.get('status') === 'added') {
        if ($jscomp$this.requiredFields.length) {
          var fieldsDefined = $jscomp$this.requiredFields.map(function(dataIndex) {
            return (r.get(dataIndex) || '').length > 0;
          });
          if (fieldsDefined.indexOf(false) === -1) {
            recordsToUpload.push(r);
          } else {
            r.set('status', 'incomplete');
          }
        } else {
          recordsToUpload.push(r);
        }
      }
    });
  }
  if (recordsToUpload.length) {
    var uploads = recordsToUpload.map(function(r) {
      return $jscomp$this.uploadFile(r);
    });
    return Promise.all(uploads).then(function(array) {
      return {uploads:array.length};
    });
  }
  return Promise.resolve({uploads:0});
}, uploadFile:function(r) {
  var mime = r.get('mime');
  var isRaw = r.get('isRaw');
  if (mime === 'inode/directory' || isRaw) {
    var obj = r.getData();
    var $jscomp$destructuring$var12 = obj;
    var documentTitle = $jscomp$destructuring$var12.documentTitle;
    var description = $jscomp$destructuring$var12.description;
    var tags$10 = $jscomp$destructuring$var12.tags;
    var documentObject = $jscomp$destructuring$var12.documentObject;
    documentObject = documentObject || {};
    documents.Api.createDocument({instanceId:this.moduleId}, Ext.apply(documentObject, {documentTitle:documentTitle, description:description, tags:tags$10, mime:mime})).then(function() {
      r.set('status', 'updated');
    });
    return;
  }
  return this.performRequest(r.get('file'), r);
}, checkFile:function(file) {
  file = file === undefined ? {} : file;
  return Ext.isDefined(file.size) && documents.Support.fileSizeOk(file.size, this.moduleId);
}, addFileToStore:function(file) {
  var status = this.checkFile(file);
  status = status ? 'added' : 'notvalid';
  var fileObject = Ext.apply({file:file, name:file.name, mime:file.type, documentTitle:file.name, description:'', tags:[], size:file.size, status:status}, this.customFieldDefaultsMap || {});
  if (this.requiredFields.length) {
    var reqMeet = this.requiredFields.map(function(cfg) {
      var $jscomp$destructuring$var13 = cfg;
      var dataIndex = $jscomp$destructuring$var13.dataIndex;
      dataIndex = dataIndex || '';
      return (fileObject[dataIndex] || '').length > 0;
    });
    fileObject.status = reqMeet.indexOf(false) ? 'incomplete' : 'added';
  }
  this.store.add(fileObject);
}, performRequest:function(fileToSubmit, r) {
  var $jscomp$this = this;
  return new Promise(function(resolve) {
    var xhr = new XMLHttpRequest;
    var fd = new FormData;
    xhr.open('POST', documents.Support.getDragNDropUrl($jscomp$this.moduleId) + ($jscomp$this.documentId ? '\x26id\x3d' + $jscomp$this.documentId : ''), true);
    fd.append('file', fileToSubmit);
    xhr.onreadystatechange = function() {
      if (xhr.readyState === 4 && xhr.status === 200) {
        if (Ext.decode(xhr.responseText).success) {
          $jscomp$this.handleUploadSuccess(Ext.decode(xhr.responseText), r);
          resolve(true);
        } else {
          r.set('status', 'error');
        }
        resolve(false);
      } else {
        if (xhr.readyState == 4 && xhr.status == 404) {
          resolve(false);
        }
      }
    };
    xhr.send(fd);
  });
}, payloadFromRecord:function(r, properties) {
  properties = properties === undefined ? [] : properties;
  var payload = Ext.apply({}, this.defaultFileProperties);
  properties.map(function(cfg) {
    var $jscomp$destructuring$var14 = cfg;
    var dataIndex = $jscomp$destructuring$var14.dataIndex;
    var defaultValue = $jscomp$destructuring$var14.defaultValue;
    var value = r.get(dataIndex) || defaultValue;
    if (value.length) {
      payload[dataIndex] = value;
    }
  });
  var meta = {};
  this.customFields.map(function(dataIndex) {
    meta[dataIndex] = r.get(dataIndex);
  });
  if (Object.keys(meta).length) {
    payload.meta = meta;
  }
  return Ext.Object.isEmpty(payload) ? undefined : payload;
}, handleUploadSuccess:function(json, r) {
  var uploadState = json.success ? 'uploaded' : 'error';
  r.set('status', uploadState);
  if (json.success) {
    var props = [{dataIndex:'description', defaultValue:''}, {dataIndex:'documentTitle', defaultValue:''}, {dataIndex:'tags', defaultValue:[]}];
    var data = this.payloadFromRecord(r, props);
    if (data) {
      this.updateDocument(json.id, data, r);
      return;
    }
  }
}, updateDocument:function(documentId, data, r) {
  var error = false;
  return documents.Support.newRequest(documents.Support.buildDocumentUrl(documentId), undefined, data, 'PUT').then(function(json) {
    var state = 'updated';
    r.set('status', state);
    return state;
  })['catch'](function(error) {
    error = true;
  }).then(function(state) {
    return {error:error, state:state};
  });
}, onDrop:function(target, event) {
  var $jscomp$this = this;
  var files = Ext.Array.from(event.dataTransfer.files);
  files.map(function(file) {
    file.status = $jscomp$this.checkFile(file) ? 'added' : 'incomplete';
    $jscomp$this.addFileToStore(file);
  });
}});
Ext.define('documents.components.grid.grid', {extend:'Ext.grid.Panel', alias:'widget.bpcDocsGrid', style:'border: 1px solid #eee;', widgetContext:undefined, pluginContext:undefined, pluginMainView:undefined, attachedDocs:[], selModel:{mode:'MULTI', allowDeselect:false}, openIn:undefined, moduleId:undefined, title:undefined, tagFilter:undefined, tagsExclusive:undefined, pluginConfig:undefined, documentsBaseTemplate:{}, header:false, tags:[], canRead:false, canWrite:false, isAdmin:false, docsWindowOpened:false, 
docsRefreshNeeded:false, initComponent:function() {
  var $jscomp$this = this;
  this.canRead = documents.Support.canRead(this.moduleId);
  this.canWrite = documents.Support.canWrite(this.moduleId);
  if (this.canWrite) {
    this.canRead = true;
  }
  this.isAdmin = documents.Support.isAdmin(this.moduleId);
  if (this.isAdmin) {
    this.canWrite = true;
    this.canRead = true;
  }
  this.documentStorage = BpcCommon.Api.getSetting(this.moduleId, 'documentStorage');
  this.iconCls = documents.Support.icon('fa-hdd');
  if (this.pluginConfig) {
    this.pluginContext = true;
    this.bbar = null;
  }
  if (this.pluginContext || this.widgetContext) {
    this.dockedItems = null;
  }
  if (this.hideGridHeader) {
    this.hideHeaders = true;
  }
  this.listeners = {beforedrop:function(node, data, overModel, dropPosition, dropHandlers) {
    if (data && data.records && data.records.length && overModel && overModel.get('mime') === 'inode/directory') {
      var folderId = overModel.get('id');
      data.records.map(function(r) {
        documents.Api.updateDocument(r.get('id'), {directoryId:folderId});
      });
    }
    dropHandlers.cancelDrop();
  }, rowDblClick:function(tableView, record, index, eopts) {
    return $jscomp$this.open(undefined, 'window', record);
  }, beforedestroy:this.onBeforeDestroy.bind(this), selectionChange:this.onSelectionChange.bind(this), rowcontextmenu:function(tv, rec, tr, rowIndex, e, eOpts) {
    e.preventDefault();
    var listenerPath = documents.Support.getIsListenerPath($jscomp$this.moduleId);
    var tags = documents.Cache.getTags($jscomp$this.moduleId);
    var tagsMenuItemCfg = {xtype:'menuitem', iconCls:documents.Support.icon('fa-tags'), text:documents.Support.TR('TAGS'), menu:[], disabled:tags.length === 0};
    if (tags.length === 0) {
      delete tagsMenuItemCfg.menu;
    }
    var tagMenuItem = Ext.create(tagsMenuItemCfg);
    if (tags.length) {
      tagMenuItem.getMenu().add(tags.map(function(t) {
        var menuitem = {iconCls:documents.Support.icon(t.record.get('iconCls')), text:t.name, tagId:t.id, documentRecord:rec, handler:function(m) {
          var selection = $jscomp$this.getSelection();
          if (selection.length === 0) {
            selection = [m.documentRecord];
          }
          var requests = selection.map(function(r) {
            var tags = r.get('tags') || [];
            if (tags.indexOf(m.tagId) === -1) {
              tags.push(m.tagId);
            } else {
              tags = tags.filter(function(t) {
                return t !== m.tagId;
              });
            }
            return {documentId:r.getId(), payload:{tags:tags}};
          }).map(function(obj) {
            return documents.Api.updateDocument(obj.documentId, obj.payload);
          });
          Promise.all(requests).then(function(arr) {
            $jscomp$this.loadData();
          });
        }};
        if (t.color && t.color.length) {
          menuitem.style = 'background-color:' + t.color + ';';
        }
        return menuitem;
      }));
    }
    var instanceStore = BpcCommon.Api.getModuleInstances('documents');
    var menuItems = [];
    instanceStore.each(function(r) {
      var $jscomp$destructuring$var15 = r.getData() || {};
      var moduleId = $jscomp$destructuring$var15.moduleId;
      var iconCls = BpcCommon.Api.getSetting(moduleId, 'module_iconCls');
      iconCls = documents.Support.icon(iconCls);
      if (BpcCommon.Api.getSetting(moduleId, 'documentStorage') === documents.Utils.FAST_STORAGE_ADAPTER_ID()) {
        menuItems.push({moduleId:moduleId, iconCls:iconCls});
      }
    });
    var m = Ext.create('Ext.menu.Menu', {width:200, margin:'0 0 10 0', floating:true, items:[{iconCls:documents.Support.icon('fa-edit'), text:documents.Support.TR('EDIT_DOCUMENT') + ' - ' + rec.get('documentTitle'), handler:function() {
      $jscomp$this.handleOpenWindow(documents.Support.openDetailEditWindow(rec, $jscomp$this.moduleId));
    }}, tagMenuItem, {iconCls:documents.Support.icon('fa-sort'), text:documents.Support.TR('ACTION_SORT'), menu:[{iconCls:documents.Support.icon('fa-columns'), text:documents.Support.TR('DOC_DOCUMENTTITLE'), key:'documentTitle'}, {iconCls:documents.Support.icon('fa-columns'), text:documents.Support.TR('DOC_MODIFIED'), key:'modified'}, {iconCls:documents.Support.icon('fa-columns'), text:documents.Support.TR('DOC_CREATOR'), key:'creator'}, {iconCls:documents.Support.icon('fa-columns'), text:documents.Support.TR('DOC_BYTES'), 
    key:'bytes'}].map(function(item) {
      item.menu = [{text:'ASC', iconCls:documents.Support.icon('fa-caret-up')}, {text:'DESC', iconCls:documents.Support.icon('fa-caret-down')}].map(function(i) {
        i.key = item.key;
        i.handler = function(m) {
          $jscomp$this.store.sort(m.key, m.text);
        };
        return i;
      });
      return item;
    })}, {iconCls:documents.Support.icon('fa-filter'), text:documents.Support.TR('ACTION_REMOVE_FILTER'), disabled:Ext.Object.isEmpty($jscomp$this.getFilters()), hidden:$jscomp$this.pluginContext, handler:function(m) {
      m.setDisabled(true);
      $jscomp$this.removeAllFilters();
    }}, {iconCls:documents.Support.icon('fa-external-link'), text:documents.Support.TR('ACTION_LINK_TO_APP'), disabled:true, handler:function() {
      var instances = BpcCommon.Api.getModuleInstances('monitor');
      var items = [];
      if (instances) {
        instances.each(function(r) {
          items.push({text:r.getId()});
        });
      }
      documents.Support.getWindowCmp(documents.Support.TR('LINK_TO'), 'fa-external-link', 0.8, 0.8, {xtype:'container', layout:{type:'hbox', align:'stretch'}, items:[{flex:1, xtype:'toolbar', layout:{type:'vbox', align:'stretch'}, items:items}, {flex:5, xtype:'panel', tbar:[{xtype:'textfield', flex:1, emptyText:'SEARCH'}], bbar:[{}, {xtype:'tbfill'}, {text:documents.Support.TR('LINK')}]}]});
    }}, {iconCls:documents.Support.icon('fa-share-square'), text:documents.Support.TR('ACTION_SHARE'), disabled:true}, {iconCls:documents.Support.icon('fa-paper-plane'), text:documents.Support.TR('ACTION_SEND_TO'), menu:[{iconCls:documents.Support.icon('fa-solar-system'), text:documents.Support.TR('INUBIT'), path:listenerPath, disabled:!Ext.isDefined(listenerPath), handler:function(menuItem) {
      documents.Api.sendToInubit(rec.get('id'), menuItem.path, $jscomp$this.moduleId);
    }}]}, {iconCls:documents.Support.icon('fa-arrows'), text:documents.Support.TR('ACTION_MOVE'), disabled:menuItems.length === 0, menu:menuItems.map(function(o) {
      o.text = BpcCommon.Api.getSetting(o.moduleId, 'module_name');
      o.handler = function(m) {
        documents.Support.newRequest(documents.Support.buildDocumentUrl(rec.get('id')), undefined, {instanceId:m.moduleId}, 'PUT').then(function(json) {
          return $jscomp$this.store.load();
        })['catch'](function(error) {
        });
      };
      return o;
    })}, {iconCls:documents.Support.icon('fa-undo'), text:documents.Support.TR('ACTION_RESTORE'), hidden:!($jscomp$this.isAdmin && BpcCommon.Api.getSetting($jscomp$this.moduleId, 'keepDeletedDocuments')), disabled:rec.get('deleted') !== true, handler:function() {
      documents.Support.newRequest(documents.Support.buildDocumentUrl(rec.get('id')), undefined, {deleted:false}, 'PUT').then(function(json) {
        $jscomp$this.store.load();
      })['catch'](function(error) {
      });
    }}]});
    m.showAt(e.getXY());
  }};
  var storeConfig = {proxy:{type:'ajax', url:documents.Support.basePath + 'documents', reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:this.moduleId}, listeners:{exception:function(p, r) {
    var emptyText = 'UNEXPECTED_ERROR';
    if (r && r.status) {
      if (r.status === 403) {
        emptyText = 'PERMISSION_DENIED';
      }
      if (r.status === 400) {
        emptyText = 'WRONG_REQUEST';
      }
    }
    $jscomp$this.updateEmptyText(emptyText);
  }}}, autoLoad:false, listeners:{prefetch:function(s, records, success, operation, eOpts) {
    if (success) {
      $jscomp$this.updateEmptyText();
      $jscomp$this.totalBox.setTitle(s.getTotalCount());
    }
  }}};
  if (this.documentStorage === 'inubit') {
    storeConfig.pageSize = 500;
  }
  this.store = Ext.create('Ext.data.BufferedStore', storeConfig);
  this.selectionBox = Ext.create({xtype:'button', ui:'secondary', style:{border:0, backgroundColor:'transparent'}, overCls:'', focusCls:'', setTitle:function(count) {
    count = count === undefined ? 0 : count;
    this.setVisible(count > 0);
    this.setText(documents.Support.TR('TXT_SELECTED') + ': ' + count);
  }});
  this.ninjaButton = Ext.create({xtype:'button', iconCls:documents.Support.icon('fa-user-ninja'), hidden:true, includeDeleted:false, handler:function(b) {
    var p = $jscomp$this.store.getProxy().getExtraParams() || {};
    p.includeDeleted = !b.includeDeleted;
    $jscomp$this.store.getProxy().setExtraParams(p);
    b.setStyle({backgroundColor:p.includeDeleted ? '#86b4b4' : 'transparent'});
    b.includeDeleted = !b.includeDeleted;
    $jscomp$this.store.load();
  }});
  this.ninjaButton.setVisible();
  this.totalBox = documents.Support.getTotalBox();
  if (!this.hideBottomBar) {
    this.bbar = [this.totalBox, this.selectionBox, {xtype:'tbfill'}, this.ninjaButton, {hidden:this.pluginContext ? !this.pluginConfig.showUploadButton : true, iconCls:documents.Support.icon('fa-upload'), text:documents.Support.TR('UPLOAD'), handler:function(b) {
      $jscomp$this.createDocument();
    }}];
  }
  var mainArea = BpcCommon.Api.getSetting(this.moduleId, 'mainArea') || {};
  var $jscomp$destructuring$var16 = mainArea;
  var backgroundColor = $jscomp$destructuring$var16.backgroundColor;
  var viewConfig = {emptyText:documents.Support.TR('EMPTYTEXT_GRID'), pluginContext:this.pluginContext, enableTextSelection:true};
  if (backgroundColor && backgroundColor.length) {
    viewConfig.style = 'background-color: ' + backgroundColor + ';';
  }
  if (this.documentStorage === documents.Utils.FAST_STORAGE_ADAPTER_ID()) {
    viewConfig.plugins = [{ptype:'gridviewdragdrop', dragText:'Drag and drop to reorganize'}];
  }
  this.viewConfig = viewConfig;
  this.showTagsInline = BpcCommon.Api.getSetting(this.moduleId, 'showTagsInline');
  var configuredColumns = this.widgetColumns;
  if (this.pluginContext) {
    configuredColumns = this.pluginConfig.columns;
  } else {
    configuredColumns = BpcCommon.Api.getSetting(this.moduleId, 'columns');
  }
  var gridActions = this.widgetGridActions || BpcCommon.Api.getSetting(this.moduleId, 'defaultGridActions');
  this.columns = this.getGridColumns(configuredColumns).concat([this.getActionColumn(gridActions)]);
  if (!this.pluginContext && !this.hideSearchBar) {
    this.tbar = [{xtype:'button', text:documents.Support.TR('OR'), mode:'or', hidden:true, handler:function(b) {
      var before = b.mode;
      b.mode = b.mode === 'or' ? 'and' : 'or';
      b.setText(documents.Support.TR(b.mode.toUpperCase()));
      b.setIconCls(documents.Support.icon(b.mode));
    }}, {xtype:'tagfield', flex:1, store:Ext.create('Ext.data.Store', {data:[]}), displayField:'tagName', forceSelection:false, createNewOnEnter:true, valueField:'id', queryMode:'local', emptyText:documents.Support.TR('EMPTYTEXT_DOCUMENTS_SEARCH'), listeners:{change:function(cmp, newValue, oldValue, eOpts) {
      var params = $jscomp$this.getFilters();
      var key = 'query';
      if (newValue && newValue.length) {
        params[key] = newValue.map(function(i) {
          return i.trim();
        }).join(',');
      } else {
        if (key in params) {
          delete params[key];
        }
      }
      $jscomp$this.changeFilters(params, true);
    }}}];
  }
  this.callParent(arguments);
  this.eventReferences = Ext.on({destroyable:true, documentDeleted:this.onDocumentsChanged.bind(this), documentUpdated:this.onDocumentsChanged.bind(this), documentUploaded:this.onDocumentsChanged.bind(this), documentCreated:this.onDocumentsChanged.bind(this)});
  if (!this.pluginContext) {
    this.store.load();
  }
}, getGridColumns:function(columnDataIndexes) {
  var $jscomp$this = this;
  var showImpersonator = BpcCommon.Api.getSetting('documents', 'upload_storeImpersonator');
  var impersonatorText = BpcCommon.Api.getSetting(this.moduleId, 'impersonatorText') || '';
  var baseColumns = [{dataIndex:'icon', width:this.pluginContext ? 50 : 30, filter:false}, {dataIndex:'documentTitle', flex:2, renderer:function(value, meta, record) {
    meta.tdAttr = 'data-qtip\x3d"' + documents.Utils.getDocumentRecordTooltip(record) + '"';
    if ($jscomp$this.showTagsInline) {
      return documents.Cache.getInlineTags(value, record, $jscomp$this.tags);
    }
    return '\x3cdiv style\x3d"width: 100%;display: flex"\x3e\x3cdiv style\x3d"' + (record.get('deleted') === true ? 'text-decoration: line-through;' : '') + '"\x3e' + (Ext.String.htmlEncode(value) || '') + '\x3c/div\x3e\x3c/div\x3e';
  }}, {dataIndex:'description', flex:1, text:documents.Support.TR('DOC_DESCRIPTION'), hidden:!this.pluginContext}, {dataIndex:'tags', text:documents.Support.TR('DOC_TAGS'), hidden:this.showTagsInline, renderer:documents.Utils.renderer.tags.bind(this)}, {dataIndex:'creator', text:documents.Support.TR('DOC_CREATOR'), width:this.widgetContext ? 100 : 150, hidden:this.widgetContext, renderer:function(v, m, r) {
    if (showImpersonator) {
      var imp = r.get('impersonator');
      return imp ? v + ' ' + (impersonatorText.length ? impersonatorText : imp) : v;
    }
    return v;
  }}, {dataIndex:'bytes', align:'end', text:documents.Support.TR('DOC_BYTES'), width:100, hidden:this.widgetContext, renderer:documents.Utils.renderer.bytesToKb}, {dataIndex:'modified', text:documents.Support.TR('DOC_MODIFIED'), width:this.widgetContext ? 100 : 120, hidden:this.widgetContext, renderer:function(v) {
    return documents.Support.dateFormatter(v, 'd.m.Y H:i');
  }}, {dataIndex:'created', text:documents.Support.TR('DOC_CREATED'), width:this.widgetContext ? 100 : 120, hidden:this.widgetContext, renderer:function(v) {
    return documents.Support.dateFormatter(v, 'd.m.Y H:i');
  }}, {dataIndex:'links', sortable:false, text:BpcCommon.util.Icons.getSpanForIconCls('fa-external-link', '#000'), width:35, filter:false, hidden:this.widgetContext === true || this.pluginContext === true, renderer:function(value, meta, record) {
    if (value && Ext.isArray(value) && value.length) {
      meta.tdAttr = 'data-qtip\x3d"' + Ext.htmlEncode(documents.Support.getHtmlLinksFromRecord(record).join('\x3cbr/\x3e')) + '"';
      return String(value.length);
    }
    return '0';
  }}];
  var columns = columnDataIndexes || ['icon', 'documentTitle', 'tags', 'creator', 'bytes', 'modified', 'links'];
  columns = columns.map(function(dataIndex) {
    var match = baseColumns.filter(function(c) {
      return c.dataIndex === dataIndex;
    });
    var targetColumn = match.length ? match[0] : {};
    return Ext.applyIf(targetColumn, documents.Utils.columns.getBaseColumnForDataIndex(dataIndex));
  });
  var customFieldDefinitions = BpcCommon.Api.getSetting(this.moduleId, 'customFields') || {};
  var customFieldColumns = Object.keys(customFieldDefinitions).map(function(dataIndex) {
    var textDefinitions = customFieldDefinitions[dataIndex]['text'] || {};
    var languageShort = BpcCommon.Api.getSetting('_core', 'language');
    var text = textDefinitions[languageShort] || documents.Support.TR('CUSTOM_FIELD_' + dataIndex);
    var dataIndexInMeta = dataIndex;
    dataIndex = 'meta';
    return {dataIndex:dataIndex, flex:2, text:text, renderer:function(v, m, r) {
      return (v || {})[dataIndexInMeta];
    }};
  });
  var cols = columns.concat(customFieldColumns);
  var showColumnFilters = false;
  return showColumnFilters ? cols.map(function(c) {
    c.menuDisabled = true;
    if (c.hasOwnProperty('filter') && c.filter === false) {
      return c;
    }
    c.items = [{xtype:'textfield', width:'100%', fieldFilterProperty:c.dataIndex, listeners:{change:function(f, newVal) {
      if (newVal && newVal.length) {
        Ext.toast('IMPL FILTER. ' + f.fieldFilterProperty);
        var filter = {field:f.fieldFilterProperty, value:newVal};
        $jscomp$this.addStoreFilter(filter);
      }
    }}}];
    return c;
  }) : cols;
}, addStoreFilter:function(filter) {
  var params = this.getFilters();
  params.fieldFilter = JSON.stringify([filter]);
  this.changeFilters(params, true);
}, getActionColumn:function(gridActions) {
  var $jscomp$this = this;
  var baseActions = [{iconCls:documents.Support.icon('fa-info-square'), tooltip:documents.Support.TR('ACTION_OPEN_DETAILS'), actionKey:'detail', handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.open(undefined, 'window', r);
  }}, {iconCls:documents.Support.icon('fa-external-link'), tooltip:documents.Support.TR('TXT_OPENINMODULE'), hidden:this.pluginContext, actionKey:'link', isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return (rec.get('links') || []).length === 0;
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    var links = r.get('links') || [];
    var m = Ext.create('Ext.menu.Menu', {margin:'0 0 10 0', floating:true, items:links.map(function(linkItem) {
      var $jscomp$destructuring$var17 = linkItem;
      var instanceId = $jscomp$destructuring$var17.instanceId;
      var referenceValue = $jscomp$destructuring$var17.referenceValue;
      return {text:referenceValue + ' ' + BpcCommon.Api.getSetting(instanceId, 'module_name'), iconCls:documents.Support.icon(BpcCommon.Api.getSetting(instanceId, 'module_iconCls')), linkItem:linkItem, handler:function(m) {
        return documents.util.ModuleJump.jumpByItem(m.linkItem);
      }};
    })});
    m.showAt(e.getXY());
  }}, {iconCls:documents.Support.icon('fa-clone'), tooltip:documents.Support.TR('ACTION_COPY_LINK'), actionKey:'clone', isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return rec.get('mime') === 'inode/directory';
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Utils.copy.clipboard(r, 'id', 'URL has been copied', grid);
  }}, {iconCls:documents.Support.icon('fa-download'), tooltip:documents.Support.TR('ACTION_DOWNLOAD'), actionKey:'download', isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return rec.get('mime') === 'inode/directory';
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Support.downloadFile(r.get('id'));
  }}, {iconCls:documents.Support.icon('fa-edit'), tooltip:documents.Support.TR('ACTION_EDIT'), actionKey:'edit', isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return rec.get('mime') === 'inode/directory';
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.handleOpenWindow(documents.Support.openDetailEditWindow(r, $jscomp$this.moduleId));
  }}, {iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('ACTION_DELETE'), actionKey:'delete', hidden:!this.canWrite || !this.canWrite && this.widgetContext === true, isActionDisabled:function(view, row, column, item, record) {
    if (!view.pluginContext) {
      return false;
    }
    var g = view.up('grid');
    var id = record.get('id');
    return !(id && g.attachedDocs.indexOf(id) === -1);
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    if (r) {
      $jscomp$this.deleteDocument(r.get('id'), undefined, undefined, [r]);
    }
  }}];
  var actionKeys = gridActions || ['link', 'clone', 'download', 'delete'];
  var actionItems = actionKeys.map(function(k) {
    return baseActions.filter(function(action) {
      return action.actionKey === k;
    })[0];
  }).filter(function(item) {
    return !item.hidden;
  });
  return {xtype:'actioncolumn', hidden:this.widgetGridActions && this.widgetGridActions.length === 0, width:actionItems.length * 30, items:actionItems};
}, openInPreview:function(record) {
  var main = this.moduleInstance;
  if (record) {
    var isDirectory = record.get('isDirectory');
    if (!isDirectory) {
      var id = record.get('id');
      if (id && main.rightPanel.documentId !== id) {
        main.rightPanel.loadId(id);
      }
      return;
    }
  }
  main.rightPanel.updateActiveDocumentCmp();
}, openInTab:function(record) {
  var win = tableView.up('window');
  if (win) {
    var id = record.get('id');
    if (id) {
      documents.Support.getDocument(id, this.moduleId).then(function(fullDocumentObject) {
        var cmp = documents.Support.getViewerComponentByDocument(fullDocumentObject);
        if (cmp) {
          var g = tableView.up('grid');
        }
      }, function(error) {
        BpcCommon.Api.showNotification('Sorry, please be patient, something went wrong. Sorry for any inconveniences.');
      });
      return;
    }
  }
}, open:function(id, openIn, record) {
  openIn = openIn === undefined ? 'window' : openIn;
  if (openIn === 'tab') {
    this.openInPreview(record);
  } else {
    if (openIn === 'window') {
      documents.Support.openDetailWindow(record, this.moduleId);
    } else {
      if (this.pluginContext && this.openIn === 'tab') {
        this.openInTab(record);
        return;
      }
    }
  }
}, onSelectionChange:function(selModel, records) {
  if (!this.widgetContext && !this.pluginContext) {
    this.open(undefined, 'tab', records.length === 1 ? records[0] : undefined);
  }
  this.selectionBox.setTitle(records.length);
}, loadData:function() {
  if (this.docsRefreshNeeded) {
    this.docsRefreshNeeded = false;
  }
  this.store.load();
}, deleteRecords:function(records) {
  records = records || this.getSelection();
  if (records.length) {
    var id = records.map(function(r) {
      return r.get('id');
    });
    this.deleteDocument(id, 'DELETE_DOCUMENTS', 'FILES_WILL_BE_DELETED', records);
  }
}, changeFilters:function(p, load) {
  p = p === undefined ? {} : p;
  if (Ext.Object.isEmpty(p)) {
    p.instanceId = this.moduleId;
  }
  this.store.getProxy().setExtraParams(p);
  if (load) {
    this.loadData();
  }
}, getFilters:function() {
  return this.store.getProxy().getExtraParams() || {};
}, removeAllFilters:function() {
  this.changeFilters({}, true);
  this.onFilterRemoved();
}, onFilterRemoved:function() {
}, scopeGrid:function(pluginConfig) {
  this.addToBaseTemplate(pluginConfig);
}, addToBaseTemplate:function(pluginConfig) {
  this.documentsBaseTemplate = {links:[pluginConfig.filter]};
}, onBeforeDestroy:function() {
  if (this.eventReferences) {
    this.eventReferences.destroy();
  }
}, onDocumentsChanged:function(data) {
  data = data === undefined ? {} : data;
  var $jscomp$destructuring$var18 = data;
  var properties = $jscomp$destructuring$var18.properties;
  properties = properties || {};
  if (properties.instanceId && properties.instanceId === this.moduleId) {
    if (!this.docsWindowOpened) {
      this.loadData();
    } else {
      this.docsRefreshNeeded = true;
    }
  }
}, updateTags:function(tags) {
  tags = tags === undefined ? [] : tags;
  this.tags = tags;
  this.getView().refresh();
}, updateEmptyText:function(txt) {
  txt = txt || 'No Documents found';
  var v = this.down('tableview');
  v.setEmptyText(txt);
  v.refresh();
}, deleteDocument:function(id, title, message, records) {
  var $jscomp$this = this;
  if (id) {
    var cmps = [{xtype:'container', padding:'0 10 0 10', layout:{type:'vbox'}, items:records.map(function(r) {
      return {xtype:'checkbox', readOnly:true, boxLabel:r.get('documentTitle'), checked:true};
    })}];
    documents.Support.deleteById(documents.Support.buildDocumentUrl(id), title, message, cmps).then(function(success) {
      if (Ext.isDefined(success) && success !== false) {
        $jscomp$this.store.load({callback:function() {
          this.selectItem();
        }.bind($jscomp$this)});
      }
    }, function(errorCode) {
      BpcCommon.Api.showNotification('Error :' + errorCode);
    });
  }
}, selectItem:function() {
  if (this.store.getTotalCount() > 0) {
    this.setSelection(this.store.getAt(0));
  } else {
    this.tryResetRightPanel();
  }
}, tryResetRightPanel:function() {
  if (this.moduleInstance && this.moduleInstance.rightPanel) {
    this.moduleInstance.rightPanel.resetPanel();
  }
}, handleOpenWindow:function(w) {
  var $jscomp$this = this;
  this.docsWindowOpened = true;
  w.on('close', function(w) {
    $jscomp$this.docsWindowOpened = false;
    if (w.fileUpdated || $jscomp$this.docsRefreshNeeded) {
      $jscomp$this.loadData();
      if ($jscomp$this.moduleInstance && $jscomp$this.moduleInstance.rightPanel) {
        $jscomp$this.moduleInstance.rightPanel.refreshGui();
      }
    }
  });
}, createDocument:function() {
  var w = documents.Api.getUploadWindow(this.moduleId, this.documentsBaseTemplate);
  this.handleOpenWindow(w);
}});
Ext.define('documents.components.moduleConfigurationComponents.Administration', {extend:'BpcCommon.component.panel.moduleConfigurationComponents.Wrapper', alias:'widget.docsAdministration', initComponent:function() {
  this.callParent(arguments);
  this.setIconCls(BpcCommon.util.Icons.getBpcIconCls('fa-cog'));
  this.setTitle(documents.Support.TR('Administration'));
  this.saveButton.setVisible(false);
  this.resetButton.setVisible(false);
}, cache:{}, onSelect:function(grid, record) {
  this.settingsWrapper.removeAll();
  var moduleId = record.get('moduleId');
  this.instanceFileAdmin = Ext.create({xtype:'tabpanel', items:[{xtype:'bpcDocsTagsGrid'}, {xtype:'docsConfig'}, {xtype:'docsDeploy'}, {xtype:'docsImport'}, {xtype:'docsExport'}, {xtype:'docsTrash'}, {xtype:'documentsDsgvo'}, {xtype:'documentsAdminWebhooks'}].map(function(o) {
    o.moduleId = moduleId;
    return o;
  })});
  this.instanceFileAdmin.setActiveTab(0);
  this.settingsWrapper.add(this.instanceFileAdmin);
}});
Ext.define('documents.components.moduleConfigurationComponents.Config', {extend:'Ext.panel.Panel', alias:'widget.docsConfig', layout:'fit', moduleId:undefined, getDocsButton:function(primary, iconCls, textKey, cfg) {
  iconCls = iconCls === undefined ? '' : iconCls;
  textKey = textKey === undefined ? '' : textKey;
  cfg = cfg === undefined ? {} : cfg;
  return Ext.create(documents.Utils.ui.simpleButton(primary, iconCls, textKey, cfg));
}, spacer:function() {
  return {xtype:'tbspacer', width:3};
}, initComponent:function() {
  var $jscomp$this = this;
  this.title = documents.Support.TR('CONFIGURATION');
  this.iconCls = documents.Support.icon('fa-database');
  this.hidden = BpcCommon.Api.getSetting(this.moduleId, 'documentStorage') !== 'db';
  this.connectButton = this.getDocsButton(true, 'fa-play', 'INSTANCE_TEST', {handler:function(b) {
    var dbProxyId = BpcCommon.Api.getSetting($jscomp$this.moduleId, 'databaseConnectionId');
    if (dbProxyId && dbProxyId.length > 0) {
      $jscomp$this.connectAndLoad();
    } else {
      $jscomp$this.notificationContainer.update('check setting: databaseConnectionId');
    }
  }});
  this.migrateAll = this.getDocsButton(false, 'fa-arrow-up', 'MIGRATE_TO_DB', {disabled:true, handler:function(b) {
    documents.Support.createAndShowMessageBox('WARNING_RESTORE_FROM_DB', 'DATA_WILL_BE_MIGRATED_TO_DB').then(function(confirmed) {
      if (confirmed) {
        documents.Api.instance.migrateInstanceDataToDb($jscomp$this.moduleId);
      }
    });
  }});
  this.restoreFromDBButton = this.getDocsButton(false, 'fa-arrow-down', 'RESTORE_FROM_DB', {disabled:true, handler:function(b) {
    documents.Support.createAndShowMessageBox('WARNING_RESTORE_FROM_DB', 'DATA_WILL_BE_RESTORED_FROM_DB').then(function(confirmed) {
      if (confirmed) {
        documents.Api.instance.restoreInstanceData($jscomp$this.moduleId);
      }
    });
  }});
  this.createButton = this.getDocsButton(false, 'fa-plus', 'TABLE_CREATE', {disabled:true, handler:function(b) {
    $jscomp$this.setup('createTable').then(function() {
      $jscomp$this.connectAndLoad();
    });
  }});
  this.emptyButton = this.getDocsButton(false, 'fa-times', 'TABLE_EMPTY', {disabled:true, handler:function(b) {
    $jscomp$this.setup('emptyTable');
  }});
  this.deleteButton = this.getDocsButton(false, 'fa-trash', 'TABLE_DELETE', {disabled:true, handler:function(b) {
    $jscomp$this.setup('deleteTable').then(function() {
      $jscomp$this.connectAndLoad();
    });
  }});
  this.deleteEsButton = this.getDocsButton(false, 'fa-trash', 'ELASTIC_DELETE', {handler:function(b) {
    documents.Support.createAndShowMessageBox('WARNING_ELASTIC_DATA_WILL_BE_DELETED', 'INSTANCE_DATA_WILL_BE_DELETED').then(function(confirmed) {
      if (confirmed) {
        documents.Api.instance.deleteInstanceData($jscomp$this.moduleId);
      }
    });
  }});
  this.notificationContainer = Ext.create({xtype:'container', flex:1});
  this.testGrid = Ext.create({xtype:'grid', features:[{ftype:'grouping'}], viewConfig:{enableTextSelection:true, emptyText:documents.Support.TR('NOT_STARTED')}, store:Ext.create('Ext.data.Store', {data:[], groupField:'group'}), columns:[{dataIndex:'key', flex:2}, {dataIndex:'value', flex:3}, {dataIndex:'msg', flex:2}], dockedItems:[{xtype:'toolbar', dock:'top', padding:'3 3 3 3', items:[this.createButton, this.spacer(), this.emptyButton, this.spacer(), this.deleteButton, {xtype:'tbfill'}, this.deleteEsButton]}, 
  {xtype:'toolbar', dock:'bottom', padding:'3 3 3 3', items:[this.connectButton, this.notificationContainer, {xtype:'tbfill'}, this.migrateAll, this.spacer(), this.restoreFromDBButton]}]});
  this.items = [this.testGrid];
  this.callParent(arguments);
  this.connectAndLoad();
}, connectAndLoad:function() {
  var $jscomp$this = this;
  return documents.Api.testInstance(this.moduleId).then(function(json) {
    $jscomp$this.notificationContainer.update('');
    var storeData = [];
    var $jscomp$destructuring$var19 = json;
    var test = $jscomp$destructuring$var19.test;
    test = test || {};
    var $jscomp$destructuring$var20 = test;
    var driver = $jscomp$destructuring$var20.driver;
    var columns = $jscomp$destructuring$var20.columns;
    var table = $jscomp$destructuring$var20.table;
    var elastic = $jscomp$destructuring$var20.elastic;
    driver = driver || {};
    table = table || {};
    elastic = elastic || {};
    Object.keys(driver).map(function(key) {
      storeData.push({group:'driver', key:key, value:driver[key] || '-1'});
    });
    Object.keys(table).map(function(key) {
      storeData.push({group:'table', key:key, value:String(table[key])});
    });
    Object.keys(elastic).map(function(key) {
      storeData.push({group:documents.Utils.FAST_STORAGE_ADAPTER_ID(), key:key, value:String(elastic[key])});
    });
    columns = columns || [];
    columns.map(function(col) {
      storeData.push({group:'columns', key:col.COLUMN_NAME || '-1', value:col.DATA_TYPE || '-1'});
    });
    $jscomp$this.testGrid.store.removeAll();
    $jscomp$this.testGrid.store.add(storeData);
    $jscomp$this.createButton.setDisabled(test.testSuccess);
    $jscomp$this.emptyButton.setDisabled(!test.testSuccess);
    $jscomp$this.deleteButton.setDisabled(!test.testSuccess);
    $jscomp$this.migrateAll.setDisabled(!test.testSuccess);
    $jscomp$this.restoreFromDBButton.setDisabled(!test.testSuccess);
  });
}, setup:function(action) {
  return documents.Api.setupInstance(this.moduleId, action);
}});
Ext.define('documents.components.moduleConfigurationComponents.Deploy', {extend:'Ext.panel.Panel', alias:'widget.docsDeploy', layout:'fit', moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.title = documents.Support.TR('DEPLOY');
  this.iconCls = documents.Support.icon('fa-code-branch');
  this.connectButton = Ext.create({xtype:'button', text:'Connect', ui:'primary', iconCls:documents.Support.icon('fa-plug'), disabled:true, handler:function(b) {
    return $jscomp$this.connectAndLoad();
  }});
  this.deployTargetCombo = Ext.create({xtype:'combo', fieldLabel:documents.Support.TR('DEPLOY_TARGET'), store:Ext.create('Ext.data.Store', {data:this.getInstanceStoreData()}), queryMode:'local', listeners:{change:function(f, newVal) {
    return $jscomp$this.enableConnectButton();
  }}, valueField:'moduleId', displayField:'instanceName'});
  this.deployButton = this.createDeployButton();
  this.moveToDbButton = this.createMoveDbButton();
  this.moveAllToDbButton = this.createMoveDbButton(true);
  this.sourceGrid = Ext.create({xtype:'docsDeployGrid', deployMain:this, moduleId:this.moduleId, gridType:'source'});
  this.targetGrid = Ext.create({xtype:'docsDeployGrid', deployMain:this, moduleId:this.moduleId, gridType:'target'});
  this.deployPanel = Ext.create({xtype:'panel', header:false, layout:{type:'hbox', align:'stretch'}, items:[this.sourceGrid, this.targetGrid]});
  this.items = [this.deployPanel];
  this.dockedItems = [{xtype:'toolbar', dock:'top', padding:'3 3 3 3', items:[{xtype:'tbfill'}, this.deployTargetCombo, this.connectButton, {xtype:'tbfill'}]}];
  this.callParent(arguments);
}, getSelectionDocumentsAndTags:function() {
  var tags = [];
  var documentIds = this.sourceGrid.getSelection().map(function(r) {
    tags = tags.concat(r.get('tags') || []);
    return r.get('id');
  });
  tags = Ext.Array.unique(tags);
  return {tags:tags, documentIds:documentIds};
}, doDeploy:function(documentIds, w, mode) {
  var $jscomp$async$this = this;
  var index;
  var success, rStatus, id, start, end;
  var json;
  var $jscomp$destructuring$var21;
  var rStatus;
  var e;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    switch($jscomp$generator$context.nextAddress) {
      case 1:
        index = 0;
      case 2:
        if (!(index < documentIds.length)) {
          $jscomp$generator$context.jumpTo(0);
          break;
        }
        success = true, rStatus = -1, id = documentIds[index];
        $jscomp$generator$context.setCatchFinallyBlocks(5);
        w.updateItem(id, 'fa-file');
        start = new Date;
        return $jscomp$generator$context.yield($jscomp$async$this.deploy(id, false, mode), 7);
      case 7:
        json = $jscomp$generator$context.yieldResult;
        end = new Date;
        $jscomp$destructuring$var21 = json || {};
        rStatus = $jscomp$destructuring$var21.rStatus;
        success = rStatus === 200;
        return $jscomp$generator$context.yield(new Promise(function(res) {
          return setTimeout(res, 300);
        }), 8);
      case 8:
        $jscomp$generator$context.leaveTryBlock(6);
        break;
      case 5:
        e = $jscomp$generator$context.enterCatchBlock();
        success = false;
      case 6:
        w.addToGrid(id, {success:success, status:rStatus, duration:Ext.Date.diff(start, end, 's'), type:'doc'});
        index++;
        $jscomp$generator$context.jumpTo(2);
        break;
    }
  });
}, createMoveDbButton:function(all) {
  var $jscomp$this = this;
  all = all === undefined ? false : all;
  return Ext.create({xtype:'button', hidden:BpcCommon.Api.getSetting(this.moduleId, 'documentStorage') !== 'db', deployMode:all ? 'migrateAllToDatabase' : 'migrateToDatabase', disabled:!all, ui:'primary', text:documents.Support.TR(all ? 'MOVE_DB' : 'MOVE_DB_ALL'), iconCls:documents.Support.icon('fa-file-export'), handler:function(b) {
    b.setDisabled(true);
    if (b.deployMode === 'migrateToDatabase') {
      var w = Ext.create({xtype:'docsDeployWindow', mode:b.deployMode, deployMain:$jscomp$this});
      w.updateContainer('fa-files', false);
      w.show();
      w.doDeploy().bind(w);
    } else {
      $jscomp$this.deploy(null, false, b.deployMode).then(function(obj) {
        var w = Ext.create({xtype:'docsDeployWindow', mode:'displayOnly', deployMain:$jscomp$this});
        w.updateContainer('fa-files', false);
        var $jscomp$destructuring$var22 = obj;
        var documents = $jscomp$destructuring$var22.documents;
        documents = documents || {};
        Object.keys(documents).map(function(id) {
          w.addToGrid(id, {success:documents[id], status:documents[id] ? 200 : 500, duration:-1, documentTitle:-1});
        });
        w.closeButton.setDisabled(false);
        w.show();
        b.setDisabled(false);
      });
    }
  }});
}, createDeployButton:function() {
  var $jscomp$this = this;
  return Ext.create({xtype:'button', disabled:true, ui:'primary', deployMode:'instanceData', text:documents.Support.TR('DEPLOY_SELECTION'), iconCls:documents.Support.icon('fa-file-export'), handler:function(b) {
    b.setDisabled(true);
    var w = Ext.create({xtype:'docsDeployWindow', mode:b.deployMode, deployMain:$jscomp$this});
    w.updateContainer('fa-files', false);
    w.show();
  }});
}, connectAndLoad:function() {
  this.sourceGrid.load();
  var targetProxyId = this.deployTargetCombo.getValue();
  if (targetProxyId) {
    var p = this.targetGrid.store.getProxy();
    p.url = documents.Support.basePath + 'list';
    var params = p.getExtraParams() || {};
    params.instanceId = this.moduleId;
    params.targetInstanceId = targetProxyId;
    p.setExtraParams(params);
    this.targetGrid.load();
  }
}, getGridBase:function() {
  var refreshTool = Ext.create({xtype:'tool', iconCls:documents.Support.icon('fa-info'), tooltip:documents.Support.TR('DEPLOY_INFO')});
  var totalBox = documents.Support.getTotalBox();
  return {xtype:'grid', refreshTool:refreshTool, totalBox:totalBox, flex:1, viewConfig:{deferEmptyText:false, emptyText:documents.Support.TR('STORE_NOT_LOADED'), enableTextSelection:true}, padding:'5 5 5 5', style:'border: 0px !important', refreshTitle:function(num) {
    num = num === undefined ? 0 : num;
    this.setTitle(documents.Support.TR(this.baseTitleKey) + ' ( ' + num + ' )');
    this.totalBox.setTitle(num);
  }, tools:[refreshTool], columns:documents.Support.getBaseGridConfigColumns(['id', 'documentTitle']), bbar:[totalBox]};
}, getInstanceStoreData:function(moduleId, instanceType) {
  instanceType = instanceType === undefined ? 'deployment_target' : instanceType;
  moduleId = documents.Support.isBpcVersion(4) ? 'backendconnection' : 'httpproxy';
  var moduleStore = BpcCommon.Api.getModuleInstances(moduleId);
  var instanceSimpleArray = [];
  moduleStore.each(function(r) {
    if (r && r.get('instanceType') === instanceType) {
      var moduleId$11 = r.get('moduleId'), instanceName = r.get('instanceName');
      instanceSimpleArray.push({moduleId:moduleId$11, instanceName:instanceName});
    }
  });
  return instanceSimpleArray;
}, enableConnectButton:function() {
  this.connectButton.setDisabled(!this.deployTargetCombo.getValue());
}, onDeploySuccess:function(cmp) {
  var g = cmp.isSource ? this.targetGrid : this.sourceGrid;
  g.load();
}, getBaseParams:function(id) {
  return {documentId:id, targetInstanceId:this.deployTargetCombo.getValue(), instanceId:this.moduleId};
}, deploy:function(id, reload, type) {
  var $jscomp$this = this;
  reload = reload === undefined ? true : reload;
  type = type === undefined ? 'doc' : type;
  var params = this.getBaseParams(id);
  if (type === 'tag') {
    delete params.documentId;
    params.tagId = id;
  }
  params.mode = type;
  var timeout = type === 'doc' ? 60000 : 30000;
  if (type === 'migrateAllToDatabase') {
    var total$12 = this.sourceGrid.store.getTotalCount();
    timeout = total$12 * 15000;
  }
  return documents.Support.newRequest(documents.Support.basePath + 'deploy', params, undefined, undefined, timeout).then(function(success) {
    if (reload) {
      $jscomp$this.targetGrid.load();
    }
    return success;
  }, function(error) {
    BpcCommon.Api.showNotification('DEPLOY ERROR');
  });
}});
Ext.define('documents.components.moduleConfigurationComponents.Deployment.DeployGrid', {extend:'Ext.grid.Panel', alias:'widget.docsDeployGrid', padding:'5 5 5 5', style:'border: 0px !important', gridType:undefined, moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.viewConfig = {deferEmptyText:false, emptyText:documents.Support.TR('STORE_NOT_LOADED'), enableTextSelection:true};
  this.store = Ext.create('Ext.data.Store', {pageSize:500, proxy:{type:'ajax', url:documents.Support.basePath + 'documents', reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:this.moduleId}, listeners:{exception:function(p, r) {
    var $jscomp$destructuring$var23 = r || {};
    var status = $jscomp$destructuring$var23.status;
    var statusText = $jscomp$destructuring$var23.statusText;
    $jscomp$this.refreshTitle((status || '-1') + ' - ' + (statusText || '-1'));
    var json = documents.Support.getResponseObject(r);
    var $jscomp$destructuring$var24 = json || {};
    var error = $jscomp$destructuring$var24.error;
    var $jscomp$destructuring$var25 = error || {};
    var message = $jscomp$destructuring$var25.message;
    if (message) {
      $jscomp$this.view.setEmptyText(message || documents.Support.TR('ERROR'));
    }
  }}}, autoLoad:false, listeners:{load:function(s, records, success, operation, eOpts) {
    if (success) {
      var total$13 = s.getTotalCount();
      $jscomp$this.view.setEmptyText(documents.Support.TR('NO_DATA'));
      $jscomp$this.refreshTitle(total$13);
    }
  }}});
  var baseTitleKey = this.gridType.toUpperCase(), isSource = baseTitleKey === 'SOURCE', iconCls = isSource ? 'fa-dot-circle' : 'fa-bullseye';
  this.isSource = isSource;
  var sourceGridCfg = Ext.apply({}, {baseTitleKey:baseTitleKey, iconCls:documents.Support.icon(iconCls)}, this.getGridBase());
  if (isSource) {
    sourceGridCfg.selModel = {selType:'checkboxmodel'};
    sourceGridCfg.listeners = {selectionchange:function(selModel, records) {
      $jscomp$this.deployMain.moveToDbButton.setDisabled(records.length === 0);
      $jscomp$this.deployMain.deployButton.setDisabled(records.length === 0);
    }};
    var sourceActions = [{iconCls:documents.Support.icon('fa-file-export'), tooltip:documents.Support.TR('DEPLOY_SINGLE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
      $jscomp$this.deploy(r.get('id'));
    }}, {iconCls:documents.Support.icon('fa-database'), tooltip:documents.Support.TR('MIGRATE_TO_DATABASE'), hidden:BpcCommon.Api.getSetting(this.moduleId, 'documentStorage') !== 'db', handler:function(grid, rowIndex, colIndex, item, e, r) {
      $jscomp$this.deploy(r.get('id'), false, 'migrateToDatabase');
    }}];
    sourceGridCfg.columns.push({xtype:'actioncolumn', width:sourceActions.length * 30, items:sourceActions});
    sourceGridCfg.bbar.push({xtype:'tbfill'});
    sourceGridCfg.bbar.push(this.deployMain.moveAllToDbButton, this.deployMain.moveToDbButton, this.deployMain.deployButton);
  } else {
    sourceGridCfg.columns.push({xtype:'actioncolumn', width:50, items:[{iconCls:documents.Support.icon('fa-file-import'), tooltip:documents.Support.TR('PULL_SINGLE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
      $jscomp$this.pullFromTarget(r.get('id'));
    }}]});
  }
  Ext.apply(this, sourceGridCfg);
  this.callParent(arguments);
  this.refreshTitle();
}, load:function() {
  this.store.load();
}, getGridBase:function() {
  var $jscomp$this = this;
  var infoTool = Ext.create({xtype:'tool', iconCls:documents.Support.icon('fa-info'), tooltip:documents.Support.TR('DEPLOY_INFO')});
  var refreshTool = Ext.create({xtype:'tool', iconCls:documents.Support.icon('fa-sync'), tooltip:documents.Support.TR('REFRESH'), handler:function() {
    $jscomp$this.load();
  }});
  var deleteTool = Ext.create({xtype:'tool', iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('DELETE_ALL'), handler:function() {
    documents.Support.createAndShowMessageBox('ACTION_CLEAR_INSTANCE', 'WARNING_INSTANCE_WILL_BE_CLEARED', 'fa-trash').then(function(confirm) {
      if (confirm) {
        var targetInstance = $jscomp$this.isSource ? undefined : $jscomp$this.deployMain.deployTargetCombo.getValue();
        documents.Api.clearInstanceData($jscomp$this.moduleId, targetInstance).then(function() {
          Ext.defer(function() {
            $jscomp$this.store.load();
          }, 500);
        });
      }
    });
  }});
  var totalBox = documents.Support.getTotalBox();
  return {refreshTool:refreshTool, totalBox:totalBox, flex:1, refreshTitle:function(num) {
    num = num === undefined ? 0 : num;
    this.setTitle(documents.Support.TR(this.baseTitleKey) + ' ( ' + num + ' )');
    this.totalBox.setTitle(num);
  }, tools:[deleteTool, infoTool, refreshTool], columns:documents.Support.getBaseGridConfigColumns(['id', 'documentTitle']), bbar:[totalBox]};
}, pullFromTarget:function(id, reload, type) {
  var $jscomp$this = this;
  reload = reload === undefined ? true : reload;
  type = type === undefined ? 'doc' : type;
  var params = this.getBaseParams(id);
  if (type === 'tag') {
    delete params.documentId;
    params.tagId = id;
  }
  params.mode = type;
  return documents.Support.newRequest(documents.Support.basePath + 'pull', params, undefined, undefined, type === 'doc' ? 60000 : 30000).then(function(success) {
    if (reload) {
      $jscomp$this.deployMain.onDeploySuccess($jscomp$this);
    }
    return success;
  }, function(error) {
    BpcCommon.Api.showNotification('PULL_ERROR');
  });
}, getBaseParams:function(id) {
  return {documentId:id, targetInstanceId:this.deployMain.deployTargetCombo.getValue(), instanceId:this.moduleId};
}, deploy:function(id, reload, type) {
  var $jscomp$this = this;
  reload = reload === undefined ? true : reload;
  type = type === undefined ? 'doc' : type;
  var params = this.getBaseParams(id);
  if (type === 'tag') {
    delete params.documentId;
    params.tagId = id;
  }
  params.mode = type;
  return documents.Support.newRequest(documents.Support.basePath + 'deploy', params, undefined, undefined, type === 'doc' ? 60000 : 30000).then(function(success) {
    if (reload) {
      $jscomp$this.deployMain.onDeploySuccess($jscomp$this);
    }
    return success;
  }, function(error) {
    BpcCommon.Api.showNotification('DEPLOY ERROR');
  });
}});
Ext.define('documents.components.moduleConfigurationComponents.Dsgvo', {extend:'Ext.panel.Panel', alias:'widget.documentsDsgvo', mixins:['documents.components.bpc.mixins.Component'], layout:{type:'vbox', align:'stretch'}, moduleId:undefined, onFieldChange:function(f) {
  this.changeDataButton.setDisabled(!((this.usernameField.getValue() || '').length && (this.usernameNewField.getValue() || '').length));
}, initComponent:function() {
  var $jscomp$this = this;
  this.title = this.TR('DSGVO');
  this.iconCls = this.faIcon('fa-user-check');
  this.usernameField = Ext.create({xtype:'textfield', emptyText:this.TR('EMPTYTEXT_USERNAME'), fieldLabel:this.TR('CREATOR'), listeners:{change:this.onFieldChange.bind(this)}});
  this.usernameNewField = Ext.create({xtype:'textfield', emptyText:this.TR('EMPTYTEXT_USERNAME'), fieldLabel:this.TR('CREATOR_NEW'), listeners:{change:this.onFieldChange.bind(this)}});
  this.changeDataButton = Ext.create(documents.Utils.ui.simpleButton(false, 'fa-edit', 'CHANGE_DATA', {disabled:true, handler:function() {
    $jscomp$this.setLoading($jscomp$this.TR('UPDATING_DATA'));
    $jscomp$this.changeCreator().then(function(json) {
      $jscomp$this.setLoading(false);
      var $jscomp$destructuring$var26 = json;
      var after = $jscomp$destructuring$var26.after;
      var before = $jscomp$destructuring$var26.before;
      var afterCreatorName = $jscomp$destructuring$var26.afterCreatorName;
      var beforeCreatorName = $jscomp$destructuring$var26.beforeCreatorName;
      after = after || {};
      before = before || {};
      var result = {};
      ['documents', 'tags', 'historyEntries'].map(function(property) {
        result[property] = after[property] === before[property];
      });
      $jscomp$this.removeAll();
      $jscomp$this.add({xtype:'panel', flex:1, layout:'center', items:[{xtype:'container', width:document.body.clientWidth * 0.4, layout:{type:'vbox', align:'stretch'}, tbar:[{xtype:'textfield', fieldLabel:'BEFORE', readOnly:true, value:beforeCreatorName}, {xtype:'textfield', fieldLabel:'AFTER', readOnly:true, value:afterCreatorName}], items:[{xtype:'checkbox', readOnly:true, boxLabel:'DOCUMENTS', value:result.documents}, {xtype:'checkbox', readOnly:true, boxLabel:'TAGS', value:result.tags}, {xtype:'checkbox', 
      readOnly:true, boxLabel:'HISTORYENTRIES', value:result.historyEntries}, {xtype:'checkbox', readOnly:true, boxLabel:'OVERALL', value:result.documents && result.tags && result.historyEntries}, {xtype:'textarea', grow:true, value:JSON.stringify(json, undefined, 4)}]}]});
    });
  }}));
  this.dockedItems = [{xtype:'toolbar', dock:'top', padding:'5 5 5 5', items:[{xtype:'tbfill'}, this.usernameField, this.usernameNewField, this.changeDataButton, {xtype:'tbfill'}]}];
  this.items = [];
  this.callParent(arguments);
}, changeCreator:function() {
  return documents.Support.newRequest(documents.Support.basePath + 'action?' + Ext.Object.toQueryString({instanceId:this.moduleId, mode:'dsgvoChange', query:this.usernameField.getValue() || '', targetValue:this.usernameNewField.getValue() || ''}), undefined, undefined, 'put');
}});
Ext.define('documents.components.moduleConfigurationComponents.DsgvoGrid', {extend:'Ext.grid.Panel', alias:'widget.documentsDsgvoGrid', layout:'fit', moduleId:undefined, _type:'documents', queryPrefix:'!!X-DSGVO', updateTitle:function(count) {
  count = count === undefined ? -1 : count;
  this.setTitle(this._type.toUpperCase() + ' - ' + count);
}, initComponent:function() {
  var $jscomp$this = this;
  var types = ['history', 'documents', 'tags'];
  var urlSuffix = 'documents';
  var extraParams = {instanceId:this.moduleId, query:this.queryPrefix};
  if (types.indexOf(this._type) !== -1) {
    urlSuffix = this._type;
    this.iconCls = documents.Support.icon(urlSuffix === 'tags' ? 'fa-tags' : urlSuffix === 'history' ? 'fa-database' : 'fa-file');
    if (urlSuffix === 'history') {
      urlSuffix = 'documents';
      extraParams.history = true;
    }
  }
  this.store = Ext.create('Ext.data.BufferedStore', {proxy:{type:'ajax', url:documents.Support.basePath + urlSuffix, reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:extraParams}, listeners:{beforeprefetch:function(s) {
    var p = s.getProxy().getExtraParams() || {};
    if (p.query && p.query.length && !p.query.startsWith($jscomp$this.queryPrefix)) {
      p.query = $jscomp$this.queryPrefix + '-' + p.query;
    } else {
      p.query = $jscomp$this.queryPrefix;
    }
    s.getProxy().setExtraParams(p);
  }, prefetch:function(s, records, success, operation, eOpts) {
    var total = undefined;
    if (success) {
      total = s.getTotalCount();
      $jscomp$this.updateTitle(total);
      $jscomp$this.totalBox.setTitle(total);
    }
  }}, autoLoad:false});
  this.columns = documents.Support.getBaseGridConfigColumns().concat([{xtype:'actioncolumn', width:60, items:[{iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('ACTION_DELETE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    if (urlSuffix === 'tags') {
      documents.Api.deleteTag(r.get('id'));
      return;
    }
    documents.Support.deleteById(documents.Support.buildDocumentUrl(r.get('id'))).then(function(success) {
      if (success) {
        $jscomp$this.store.load();
      }
    });
  }}]}]);
  this.totalBox = documents.Support.getTotalBox();
  this.bbar = [this.totalBox, {xtype:'tbfill'}, {ui:'secondary', style:{border:0}, iconCls:documents.Support.icon('fa-sync'), text:documents.Support.TR('ACTION_REFRESH'), handler:function() {
    return $jscomp$this.store.load();
  }}];
  this.callParent(arguments);
  this.updateTitle();
}, loadData:function(params) {
  params = params === undefined ? {} : params;
  var beforeParams = this.store.getProxy().getExtraParams() || {};
  this.store.getProxy().setExtraParams(Ext.applyIf(params, beforeParams));
  this.store.load();
}});
Ext.define('documents.components.moduleConfigurationComponents.Export', {extend:'Ext.grid.Panel', alias:'widget.docsExport', layout:'fit', moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.title = documents.Support.TR('EXPORT');
  this.iconCls = documents.Support.icon('fa-file-export');
  this.listeners = {afterrender:function(c) {
    c.store.load();
  }, rowDblClick:function(tableView, record, index, eopts) {
    var docId = record.get('id');
    documents.Support.getDocument(docId, $jscomp$this.moduleId, true).then(function(fullDocumentObject) {
      var data = fullDocumentObject.record.getData();
      if (data.hasOwnProperty('status')) {
        delete data.status;
      }
      Ext.create('Ext.window.Window', {title:'Hello', height:document.body.clientHeight * 0.8, width:document.body.clientWidth * 0.8, layout:'fit', items:[{xtype:'docsTextEditor', editorLanguage:'json', value:JSON.stringify(data, undefined, 4)}]}).show();
    });
  }};
  this.store = Ext.create('Ext.data.BufferedStore', {proxy:{type:'ajax', url:documents.Support.basePath + 'documents', reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:this.moduleId}, listeners:{exception:function(p, r) {
    var emptyText = 'UNEXPECTED_ERROR';
    if (r && r.status) {
      if (r.status === 403) {
        emptyText = 'PERMISSION_DENIED';
      }
      if (r.status === 400) {
        emptyText = 'WRONG_REQUEST';
      }
    }
    $jscomp$this.updateEmptyText(emptyText);
  }}}, listeners:{prefetch:function(s, records, success, operation, eOpts) {
    var total = undefined;
    if (success) {
      total = s.getTotalCount();
      $jscomp$this.totalBox.setTitle(total);
    }
  }}, autoLoad:false});
  this.columns = documents.Support.getBaseGridConfigColumns().concat([{xtype:'actioncolumn', width:60, items:[{iconCls:documents.Support.icon('fa-copy'), tooltip:documents.Support.TR('COPY_AS_JSON'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Support.createAndShowMessageBox('FILE_RESTORE', 'FILE_WILL_BE_RESTORED').then(function(confirm) {
      if (confirm) {
        documents.Support.newRequest(documents.Support.buildDocumentUrl(r.get('id')), undefined, {deleted:false}, 'PUT').then(function(json) {
          $jscomp$this.store.load();
        })['catch'](function(error) {
        });
      }
    });
  }}, {iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('ACTION_DELETE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Support.deleteById(documents.Support.buildDocumentUrl(r.get('id'))).then(function(success) {
      if (success) {
        $jscomp$this.store.load();
      }
    });
  }}]}]);
  this.totalBox = documents.Support.getTotalBox();
  this.bbar = [this.totalBox, {xtype:'tbfill'}, {ui:'secondary', style:{border:0}, iconCls:documents.Support.icon('fa-sync'), text:documents.Support.TR('ACTION_REFRESH'), handler:function() {
    return $jscomp$this.store.load();
  }}];
  this.callParent(arguments);
}});
Ext.define('documents.components.moduleConfigurationComponents.Import', {extend:'Ext.panel.Panel', alias:'widget.docsImport', layout:{type:'hbox', align:'stretch'}, moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.title = documents.Support.TR('IMPORT');
  this.iconCls = documents.Support.icon('fa-file-import');
  this.editor = Ext.create({xtype:'docsTextEditor', flex:8, editorLanguage:'json', readOnly:false});
  var props = ['creator', 'created', 'mime', 'description', 'filename', 'instanceId', 'bytes', 'organizations', 'modified', 'id', 'documentTitle', 'slug', 'base64string'].sort();
  var map = {ok:'#46C5BE', other:'#FF935D'};
  this.checkGrid = Ext.create({xtype:'grid', flex:1, style:{border:0}, title:documents.Support.TR('DOC_PROPERTIES'), store:Ext.create('Ext.data.Store', {data:props.map(function(prop) {
    return {property:prop, id:prop, r:'', msg:'', flag:''};
  })}), columns:[{dataIndex:'property', flex:1, renderer:function(v, m, r) {
    var color = map.other;
    var f = r.get('flag') || '';
    if (map[f]) {
      color = map[f];
    }
    m.tdStyle = 'background-color:' + color + ';';
    return v;
  }}]});
  this.items = [this.editor, this.checkGrid];
  var cfg = documents.Utils.ui.getButtonCfg(true, 'fa-file-import', 'IMPORT');
  this.importBtn = Ext.create(Ext.apply(cfg, {disabled:true, handler:function() {
    var jsonString = $jscomp$this.editor.getValue();
    var json = JSON.parse(jsonString);
    json.instanceId = $jscomp$this.moduleId;
    documents.Api.createDocument({instanceId:$jscomp$this.moduleId, rawImport:true}, json);
  }}));
  this.bbar = [{xtype:'tbfill'}, this.importBtn];
  this.callParent(arguments);
  this.editor.on('aceEditorValueChange', function(params) {
    params = params || {};
    var jsonString = $jscomp$this.editor.getValue();
    var error = false;
    var json = undefined;
    try {
      json = JSON.parse(jsonString) || {};
      if (json && Ext.isObject(json)) {
        var keys = Object.keys(json);
        $jscomp$this.checkGrid.store.each(function(r) {
          r.set('flag', keys.indexOf(r.get('id')) === -1 ? 'missing' : 'ok');
        });
      }
    } catch (e) {
      error = true;
    }
    $jscomp$this.importBtn.setDisabled(params.isEmpty || error);
  });
}});
Ext.define('documents.components.moduleConfigurationComponents.TrashBin', {extend:'Ext.grid.Panel', alias:'widget.docsTrash', layout:'fit', moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  var canDeleteKeepedDocs = BpcCommon.Api.getSetting(this.moduleId, 'keepDeletedDocuments') && documents.Api.isDocumentsAdmin();
  this.hidden = !canDeleteKeepedDocs;
  this.title = documents.Support.TR('BIN');
  this.iconCls = documents.Support.icon('fa-trash');
  this.store = Ext.create('Ext.data.BufferedStore', {proxy:Ext.apply({}, documents.Support.getProxyBaseConfig('documents', this.moduleId), {listeners:{exception:function(p, r) {
    var emptyText = 'UNEXPECTED_ERROR';
    if (r && r.status) {
      if (r.status === 403) {
        emptyText = 'PERMISSION_DENIED';
      }
      if (r.status === 400) {
        emptyText = 'WRONG_REQUEST';
      }
    }
    $jscomp$this.updateEmptyText(emptyText);
  }}}), listeners:{prefetch:function(s, records, success, operation, eOpts) {
    var total = undefined;
    if (success) {
      total = s.getTotalCount();
      $jscomp$this.totalBox.setTitle(total);
    }
    $jscomp$this.emptyBinBtn.setDisabled(!total || total === 0);
  }}, autoLoad:true});
  var p = this.store.getProxy().getExtraParams() || {};
  p.deletedOnly = true;
  this.store.getProxy().setExtraParams(p);
  this.store.load();
  this.viewConfig = {enableTextSelection:true, emptyText:documents.Support.TR('NO_DATA')};
  this.columns = documents.Support.getBaseGridConfigColumns().concat([{xtype:'actioncolumn', width:60, items:[{iconCls:documents.Support.icon('fa-undo'), tooltip:documents.Support.TR('RESTORE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Support.createAndShowMessageBox('FILE_RESTORE', 'FILE_WILL_BE_RESTORED').then(function(confirm) {
      if (confirm) {
        documents.Support.newRequest(documents.Support.buildDocumentUrl(r.get('id')), undefined, {deleted:false}, 'PUT').then(function(json) {
          $jscomp$this.store.load();
        })['catch'](function(error) {
        });
      }
    });
  }}, {iconCls:documents.Support.icon('fa-trash'), tooltip:documents.Support.TR('ACTION_DELETE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    documents.Support.deleteById(documents.Support.buildDocumentUrl(r.get('id'))).then(function(success) {
      if (success) {
        $jscomp$this.store.load();
      }
    });
  }}]}]);
  this.emptyBinBtn = Ext.create({xtype:'button', ui:'primary', iconCls:documents.Support.icon('fa-trash'), text:documents.Support.TR('BIN_EMPTY'), handler:function() {
    documents.Support.createAndShowMessageBox('BIN_EMPTY', 'BIN_EMPTIED').then(function(confirm) {
      if (confirm) {
        documents.Api.clearTrashBin($jscomp$this.moduleId).then(function() {
          Ext.defer(function() {
            $jscomp$this.store.load();
          }, 500);
        });
      }
    });
  }});
  this.totalBox = documents.Support.getTotalBox();
  this.bbar = [this.totalBox, {xtype:'tbfill'}, this.emptyBinBtn, {iconCls:documents.Support.icon('fa-sync'), text:documents.Support.TR('ACTION_REFRESH'), ui:'secondary', handler:function() {
    return $jscomp$this.store.load();
  }}];
  this.callParent(arguments);
}, updateEmptyText:function(txt) {
  txt = txt || documents.Support.TR('NO_DOCUMENTS_FOUND');
  var v = this.down('tableview');
  if (v) {
    v.setEmptyText(txt);
    v.refresh();
  }
}});
Ext.define('documents.components.type.FileBase', {extend:'Ext.panel.Panel', layout:{type:'vbox', align:'stretch'}, padding:'10 10 10 10', getMinimalDocObject:function(mime, base64str) {
  var minimal = documents.Utils.getMinimalDocObject(mime, base64str) || {};
  minimal.sourceDocumentObject = this.getBaseDocumentObject();
  return minimal;
}, getBaseDocumentObject:function() {
  return this.documentObject && this.documentObject.sourceDocumentObject ? this.documentObject.sourceDocumentObject : this.documentObject;
}, documentModuleId:undefined, handleDocumentModuleId:function() {
  if (this.documentObject.record) {
    this.documentModuleId = this.documentObject.record.get('instanceId');
  } else {
    if (this.documentObject.sourceDocumentObject) {
      this.documentModuleId = this.documentObject.sourceDocumentObject.record.get('instanceId');
    }
  }
}});
Ext.define('documents.components.type.Error', {extend:'documents.components.type.FileBase', alias:'widget.documentsFileError', layout:'fit', errorMessage:'', iconCls:undefined, status:[], initComponent:function() {
  var $jscomp$this = this;
  this.messagePanel = Ext.create({xtype:'docsMessagePanel', html:this.errorMessage || ''});
  this.messagePanel.setMessage(this.errorMessage, this.iconCls);
  this.items = [this.messagePanel];
  if (this.status && this.status.length) {
    this.bbar = [{xtype:'tbfill'}, {iconCls:documents.Support.icon('fa-exclamation-circle'), handler:function() {
      documents.Support.getWindowCmp('Warnings', 'fa-exclamation-circle', undefined, undefined, {xtype:'grid', store:Ext.create('Ext.data.Store', {fields:['type', 'message'], data:$jscomp$this.status || []}), columns:[{text:'type', dataIndex:'type'}, {text:'message', dataIndex:'message', flex:1}]});
    }}];
  }
  this.callParent(arguments);
}});
Ext.define('documents.components.type.Folder', {extend:'documents.components.type.FileBase', alias:'widget.documentsFolder', documentObject:undefined, initComponent:function() {
  this.attachmentGrid = Ext.create({xtype:'grid', store:Ext.create('Ext.data.Store', {data:[]}), flex:1, viewConfig:{emptyText:'No files found', enableTextSelection:true}, columns:[{dataIndex:'icon', width:30}, {dataIndex:'documentTitle', text:documents.Support.TR('DOC_DOCUMENTTITLE'), flex:1, renderer:function(value, meta, record) {
    meta.tdAttr = 'data-qtip\x3d"' + documents.Utils.getDocumentRecordTooltip(record) + '"';
    return documents.Cache.getInlineTags(value, record, []);
  }}].map(function(c) {
    c = Ext.applyIf(c, documents.Utils.columns.getBaseColumnForDataIndex(c.dataIndex));
    c.menuDisabled = true;
    c.sortable = false;
    return c;
  }).concat([{xtype:'actioncolumn', width:35, items:[{iconCls:documents.Support.icon('fa-folder-open'), tooltip:documents.Support.TR('OPEN'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    return documents.Support.openDetailWindow(r);
  }}]}])});
  this.items = [this.attachmentGrid];
  this.callParent(arguments);
  if (this.documentObject) {
    if (this.documentObject.documentTitle) {
      this.attachmentGrid.setIconCls(documents.Support.icon('fa-folder'));
      this.attachmentGrid.setTitle(this.documentObject.documentTitle);
    }
    this.loadFile();
  }
}, loadFile:function() {
  var $jscomp$this = this;
  try {
    var instanceId = this.documentObject.record.get('instanceId');
    var path = '/' + this.documentObject.id;
    documents.Api.getDocuments({instanceId:instanceId, path:path}).then(function(json) {
      json = json || {};
      if (json.total) {
        $jscomp$this.attachmentGrid.store.add(json.data);
      }
    });
  } catch (e) {
  }
}});
Ext.define('documents.components.type.Mail', {extend:'documents.components.type.FileBase', alias:'widget.documentsMail', documentsContext:'documents', mainArrayBuffer:undefined, initComponent:function() {
  var $jscomp$this = this;
  var labelWidth = 120;
  this.headerGrid = Ext.create({xtype:'grid', store:Ext.create('Ext.data.Store', {data:[]}), viewConfig:{enableTextSelection:true}, listeners:{rowdblclick:function(table, r) {
    return $jscomp$this.copyValue(r.get('value'));
  }}, columns:[{dataIndex:'key', text:documents.Support.TR('header')}, {dataIndex:'value', text:documents.Support.TR('value'), flex:1}].concat([{xtype:'actioncolumn', width:30, items:[{iconCls:documents.Support.icon('fa-copy'), tooltip:documents.Support.TR('CLONE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.copyValue(r.get('value'));
  }}]}])});
  this.fieldset = Ext.create({xtype:'fieldset', collapsible:true, collapsed:true, layout:'fit', items:[this.headerGrid], scrollable:true, updateTitle:function(count) {
    count = count === undefined ? 0 : count;
    this.setTitle(documents.Support.TR('header') + '  (' + count + ')');
  }});
  this.fieldset.updateTitle();
  this.fromField = Ext.create({xtype:'textfield', flex:1, readOnly:true});
  this.toField = Ext.create({xtype:'textfield', fieldLabel:documents.Support.TR('MAIL_TO'), labelWidth:labelWidth, readOnly:true});
  this.subjextField = Ext.create({xtype:'textfield', fieldLabel:documents.Support.TR('MAIL_SUBJECT'), labelWidth:labelWidth, emptyText:documents.Support.TR('NO_DATA'), readOnly:true});
  this.dateField = Ext.create({xtype:'datefield', format:'d.m.Y H:i:s', readOnly:true});
  this.topFieldContainer = Ext.create({xtype:'fieldcontainer', fieldLabel:documents.Support.TR('MAIL_FROM'), labelWidth:labelWidth, layout:{type:'hbox', align:'stretch'}, items:[this.fromField, this.dateField]});
  this.textBodyField = Ext.create({xtype:'textarea', fieldLabel:documents.Support.TR('MAIL_BODY'), labelWidth:labelWidth, flex:1, minHeight:120, readOnly:true});
  this.attachmentGrid = Ext.create({xtype:'grid', store:Ext.create('Ext.data.Store', {data:[]}), maxHeight:100, header:false, hideHeaders:true, viewConfig:{emptyText:'No attachments found', enableTextSelection:true}, columns:[{dataIndex:'filename', flex:1}, {dataIndex:'mimeType'}, {xtype:'actioncolumn', width:60, items:[{iconCls:documents.Support.icon('fa-folder-open'), tooltip:documents.Support.TR('OPEN'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    var base64str = documents.Utils.arrayBufferToBase64(r.get('arrayBuffer'));
    var p = documents.Support.getViewerComponentByDocument($jscomp$this.getMinimalDocObject(r.get('mimeType'), base64str));
    documents.Utils.openFileWindow(p, r.get('filename'));
  }}, {iconCls:documents.Support.icon('fa-save'), tooltip:documents.Support.TR(this.documentsContext !== 'documents' ? 'DOWNLOAD' : 'STORE_AS_DOCUMENT'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    if ($jscomp$this.documentsContext !== 'documents') {
      var b = new Blob([r.get('arrayBuffer')], {type:r.get('mimeType')});
      var a = window.document.createElement('a');
      a.href = window.URL.createObjectURL(b, {type:r.get('mimeType')});
      a.download = r.get('filename');
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
      return;
    }
    $jscomp$this.createDocumentFromAttachmentRecord(r);
  }}]}]});
  this.botttomFieldContainer = Ext.create({xtype:'fieldcontainer', fieldLabel:documents.Support.TR('MAIL_ATTACHMENTS'), labelWidth:labelWidth, layout:{type:'fit'}, items:[this.attachmentGrid]});
  this.items = [{xtype:'fieldset', flex:1, padding:'20 20 20 20', layout:{type:'vbox', align:'stretch'}, scrollable:true, items:[this.fieldset, this.topFieldContainer, this.toField, this.subjextField, this.textBodyField, this.botttomFieldContainer]}];
  this.callParent(arguments);
  this.handleDocumentModuleId();
  this.loadFile();
}, copyValue:function(v) {
  if (v) {
    if (documents.Support.copyToClipBoard(v)) {
      BpcCommon.Api.showNotification(v + ' has been added to your clipboard');
    }
  }
}, createDocumentFromAttachmentRecord:function(r) {
  var mimeType = r.get('mimeType'), base64string = documents.Utils.arrayBufferToBase64(r.get('arrayBuffer'));
  documents.Api.createDocument({instanceId:this.documentModuleId}, {filename:r.get('filename'), description:'extracted from mail', documentTitle:r.get('filename'), mime:mimeType, base64string:base64string});
}, loadFile:function() {
  var $jscomp$this = this;
  var text = documents.Support.convertDataURIToBinary(this.documentObject.dataUri);
  var mailObj;
  try {
    var mime = Ext.create('documents.parser.mime');
    mime.parse(text).then(function(mailObj) {
      var $jscomp$destructuring$var27 = mailObj;
      var text = $jscomp$destructuring$var27.text;
      var from = $jscomp$destructuring$var27.from;
      var to = $jscomp$destructuring$var27.to;
      var date = $jscomp$destructuring$var27.date;
      var subject = $jscomp$destructuring$var27.subject;
      var headers = $jscomp$destructuring$var27.headers;
      var attachments = $jscomp$destructuring$var27.attachments;
      $jscomp$this.textBodyField.setValue(text || '-1');
      $jscomp$this.dateField.setValue(Ext.Date.parse(date, 'c'));
      $jscomp$this.subjextField.setValue(subject || '-1');
      $jscomp$this.fromField.setValue(from.name + ' \x3c' + from.address + '\x3e' || '-1');
      $jscomp$this.toField.setValue((to.length ? to[0].name + ' \x3c' + to[0].address + '\x3e ' : '-1') || '-1');
      if (headers) {
        $jscomp$this.headerGrid.store.add(headers);
        if (headers.length) {
          $jscomp$this.fieldset.updateTitle(headers.length);
        }
      }
      if (attachments && attachments.length) {
        $jscomp$this.attachmentGrid.store.add(attachments.map(function(o) {
          return {filename:o.filename, mimeType:o.mimeType, arrayBuffer:o.content};
        }));
      }
    });
  } catch (e) {
  }
}});
Ext.define('documents.components.type.Media', {extend:'documents.components.type.FileBase', alias:'widget.documentsMedia', documentObject:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.statusButton = Ext.create({xtype:'button', text:'Status: -', hoverCls:'', main:this, setStatus:function(status) {
    var states = {play:{key:'PLAYER_PLAY', iconCls:'fa-play-circle'}, playing:{key:'PLAYER_PLAYING', iconCls:'fa-headphones'}, pause:{key:'PLAYER_PAUSED', iconCls:'fa-pause-circle'}, stopped:{key:'PLAYER_STOPPED', iconCls:'fa-stop-circle'}, ended:{key:'PLAYER_ENDED', iconCls:'fa-stop-circle'}};
    this.setText('Status: ' + documents.Support.TR(states[status].key));
    this.setIconCls('Status: ' + documents.Support.icon(states[status].iconCls));
    this.main.playStateButton.setIconCls(documents.Support.icon(states[status].iconCls));
  }});
  var id = Ext.id();
  var playerWrapper = Ext.create({xtype:'container', hidden:true, html:'\x3cdiv style\x3d"display: none;"\x3e\x3caudio id\x3d"' + id + '" preload\x3d"auto"\x3e\x3csource src\x3d"' + this.documentObject.dataUri + '" /\x3e\x3c/audio\x3e\x3c/div\x3e'});
  this.playStateButton = Ext.create({xtype:'button', iconCls:documents.Support.icon('play-circle'), scale:'large', hoverCLs:'', style:'border:0!important', onPlay:function() {
  }, onPause:function() {
  }, onEnded:function() {
  }, ui:'secondary', handler:function(b) {
    var media = document.getElementById(id);
    b.mediaElem = media;
    if (b.mediaElem) {
      if (b.mediaElem.paused || b.mediaElem.ended) {
        b.mediaElem.play();
      } else {
        b.mediaElem.pause();
      }
    }
  }});
  this.items = [{xtype:'container', layout:'center', flex:1, items:[this.playStateButton], listeners:{afterrender:function(container) {
    Ext.defer(function() {
      $jscomp$this.playStateButton.handler($jscomp$this.playStateButton);
    }, 300);
    var media = document.getElementById(id);
    media.addEventListener('ended', function(event) {
      $jscomp$this.statusButton.setStatus('ended');
    });
    media.addEventListener('playing', function(event) {
      $jscomp$this.statusButton.setStatus('playing');
    });
    media.addEventListener('pause', function(event) {
      $jscomp$this.statusButton.setStatus('pause');
    });
  }}}, playerWrapper];
  this.bbar = [this.statusButton];
  this.callParent(arguments);
}});
Ext.define('documents.view.widget.Main', {extend:'Ext.panel.Panel', alias:'widget.bpcDocsWidgetMain', mixins:['BpcCommon.mixin.Widget'], statics:{WIDGET_NAME:'Documents', WIDGET_DESCRIPTION:'Schnellzugriff auf Ihre Dokumente', WIDGET_ICON_CLS:'x-fal fa-cloud'}, widgetConfigSettingMap:{moduleId:{settingType:'moduleInstance'}, hideSearchBar:{settingType:'boolean'}, actions:{settingType:'json'}, hideBottomBar:{settingType:'boolean'}, hideGridHeader:{settingType:'boolean'}, columns:{settingType:'json'}}, 
config:{widgetConfiguration:{hideSearchBar:false, hideBottomBar:false, hideGridHeader:false, actions:null, columns:null}}, header:false, layout:'card', activeItem:0, title:false, moduleId:undefined, initComponent:function() {
  this.initWidgetSettings();
  this.callParent(arguments);
  this.bootWidget();
  this.settingPanel = this.add({xtype:'bpcSettingPanel', settings:this.settingStore, buttons:[{text:documents.Support.TR('ACTION_OK').toUpperCase(), ui:'primary', handler:this.configureWidget.bind(this)}]});
}, bootWidget:function() {
  var $jscomp$this = this;
  var wrapper = Ext.create({xtype:'container', layout:'fit', items:[]});
  documents.Api.getTags({instanceId:this.moduleId}).then(function(tags) {
    $jscomp$this.docsGrid = wrapper.add({xtype:'bpcDocsGrid', moduleId:$jscomp$this.moduleId, widgetContext:true, hideSearchBar:$jscomp$this.widgetConfiguration.hideSearchBar, hideBottomBar:$jscomp$this.widgetConfiguration.hideBottomBar, hideGridHeader:$jscomp$this.widgetConfiguration.hideGridHeader, widgetGridActions:$jscomp$this.widgetConfiguration.actions, widgetColumns:$jscomp$this.widgetConfiguration.columns, tags:tags});
  });
  this.wrapper = wrapper;
  this.add(this.wrapper);
  return this.wrapper;
}, refreshWidget:function() {
  if (this.docsGrid) {
    this.docsGrid.loadData();
  }
}, configureWidget:function() {
  this.getLayout().setActiveItem(this.settingPanel.isVisible(true) ? this.wrapper : this.settingPanel);
}});
Ext.define('documents.BpcInterface', {extend:'BpcCommon.BpcInterface', requires:['documents.Support', 'documents.components.*', 'documents.view.widget.Main'], config:{moduleConfigurationComponents:['docsAdministration'], requiredModules:[], widgets:['documents.view.widget.Main'], version:'4.0.16-SNAPSHOT', build:'v.4.0.15-5-g891184a-2023-03-28T09:49:17+0200'}});
Ext.define('documents.Mime', {singleton:true, map:{123:'application/vnd.lotus-1-2-3', '1km':'application/vnd.1000minds.decision-model+xml', '3dml':'text/vnd.in3d.3dml', '3ds':'image/x-3ds', '3g2':'video/3gpp2', '3gp':'video/3gpp', '3gpp':'video/3gpp', '3mf':'model/3mf', '7z':'application/x-7z-compressed', 'aab':'application/x-authorware-bin', 'aac':'audio/x-aac', 'aam':'application/x-authorware-map', 'aas':'application/x-authorware-seg', 'abw':'application/x-abiword', 'ac':'application/vnd.nokia.n-gage.ac+xml', 
'acc':'application/vnd.americandynamics.acc', 'ace':'application/x-ace-compressed', 'acu':'application/vnd.acucobol', 'acutc':'application/vnd.acucorp', 'adp':'audio/adpcm', 'aep':'application/vnd.audiograph', 'afm':'application/x-font-type1', 'afp':'application/vnd.ibm.modcap', 'ahead':'application/vnd.ahead.space', 'ai':'application/postscript', 'aif':'audio/x-aiff', 'aifc':'audio/x-aiff', 'aiff':'audio/x-aiff', 'air':'application/vnd.adobe.air-application-installer-package+zip', 'ait':'application/vnd.dvb.ait', 
'ami':'application/vnd.amiga.ami', 'amr':'audio/amr', 'apk':'application/vnd.android.package-archive', 'apng':'image/apng', 'appcache':'text/cache-manifest', 'application':'application/x-ms-application', 'apr':'application/vnd.lotus-approach', 'arc':'application/x-freearc', 'arj':'application/x-arj', 'asc':'application/pgp-signature', 'asf':'video/x-ms-asf', 'asm':'text/x-asm', 'aso':'application/vnd.accpac.simply.aso', 'asx':'video/x-ms-asf', 'atc':'application/vnd.acucorp', 'atom':'application/atom+xml', 
'atomcat':'application/atomcat+xml', 'atomdeleted':'application/atomdeleted+xml', 'atomsvc':'application/atomsvc+xml', 'atx':'application/vnd.antix.game-component', 'au':'audio/basic', 'avi':'video/x-msvideo', 'avif':'image/avif', 'aw':'application/applixware', 'azf':'application/vnd.airzip.filesecure.azf', 'azs':'application/vnd.airzip.filesecure.azs', 'azv':'image/vnd.airzip.accelerator.azv', 'azw':'application/vnd.amazon.ebook', 'b16':'image/vnd.pco.b16', 'bat':'application/x-msdownload', 'bcpio':'application/x-bcpio', 
'bdf':'application/x-font-bdf', 'bdm':'application/vnd.syncml.dm+wbxml', 'bdoc':'application/x-bdoc', 'bed':'application/vnd.realvnc.bed', 'bh2':'application/vnd.fujitsu.oasysprs', 'bin':'application/octet-stream', 'blb':'application/x-blorb', 'blorb':'application/x-blorb', 'bmi':'application/vnd.bmi', 'bmml':'application/vnd.balsamiq.bmml+xml', 'bmp':'image/x-ms-bmp', 'book':'application/vnd.framemaker', 'box':'application/vnd.previewsystems.box', 'boz':'application/x-bzip2', 'bpk':'application/octet-stream', 
'bsp':'model/vnd.valve.source.compiled-map', 'btif':'image/prs.btif', 'buffer':'application/octet-stream', 'bz':'application/x-bzip', 'bz2':'application/x-bzip2', 'c':'text/x-c', 'c11amc':'application/vnd.cluetrust.cartomobile-config', 'c11amz':'application/vnd.cluetrust.cartomobile-config-pkg', 'c4d':'application/vnd.clonk.c4group', 'c4f':'application/vnd.clonk.c4group', 'c4g':'application/vnd.clonk.c4group', 'c4p':'application/vnd.clonk.c4group', 'c4u':'application/vnd.clonk.c4group', 'cab':'application/vnd.ms-cab-compressed', 
'caf':'audio/x-caf', 'cap':'application/vnd.tcpdump.pcap', 'car':'application/vnd.curl.car', 'cat':'application/vnd.ms-pki.seccat', 'cb7':'application/x-cbr', 'cba':'application/x-cbr', 'cbr':'application/x-cbr', 'cbt':'application/x-cbr', 'cbz':'application/x-cbr', 'cc':'text/x-c', 'cco':'application/x-cocoa', 'cct':'application/x-director', 'ccxml':'application/ccxml+xml', 'cdbcmsg':'application/vnd.contact.cmsg', 'cdf':'application/x-netcdf', 'cdfx':'application/cdfx+xml', 'cdkey':'application/vnd.mediastation.cdkey', 
'cdmia':'application/cdmi-capability', 'cdmic':'application/cdmi-container', 'cdmid':'application/cdmi-domain', 'cdmio':'application/cdmi-object', 'cdmiq':'application/cdmi-queue', 'cdx':'chemical/x-cdx', 'cdxml':'application/vnd.chemdraw+xml', 'cdy':'application/vnd.cinderella', 'cer':'application/pkix-cert', 'cfs':'application/x-cfs-compressed', 'cgm':'image/cgm', 'chat':'application/x-chat', 'chm':'application/vnd.ms-htmlhelp', 'chrt':'application/vnd.kde.kchart', 'cif':'chemical/x-cif', 'cii':'application/vnd.anser-web-certificate-issue-initiation', 
'cil':'application/vnd.ms-artgalry', 'cjs':'application/node', 'cla':'application/vnd.claymore', 'class':'application/java-vm', 'clkk':'application/vnd.crick.clicker.keyboard', 'clkp':'application/vnd.crick.clicker.palette', 'clkt':'application/vnd.crick.clicker.template', 'clkw':'application/vnd.crick.clicker.wordbank', 'clkx':'application/vnd.crick.clicker', 'clp':'application/x-msclip', 'cmc':'application/vnd.cosmocaller', 'cmdf':'chemical/x-cmdf', 'cml':'chemical/x-cml', 'cmp':'application/vnd.yellowriver-custom-menu', 
'cmx':'image/x-cmx', 'cod':'application/vnd.rim.cod', 'coffee':'text/coffeescript', 'com':'application/x-msdownload', 'conf':'text/plain', 'cpio':'application/x-cpio', 'cpp':'text/x-c', 'cpt':'application/mac-compactpro', 'crd':'application/x-mscardfile', 'crl':'application/pkix-crl', 'crt':'application/x-x509-ca-cert', 'crx':'application/x-chrome-extension', 'cryptonote':'application/vnd.rig.cryptonote', 'csh':'application/x-csh', 'csl':'application/vnd.citationstyles.style+xml', 'csml':'chemical/x-csml', 
'csp':'application/vnd.commonspace', 'css':'text/css', 'cst':'application/x-director', 'csv':'text/csv', 'cu':'application/cu-seeme', 'curl':'text/vnd.curl', 'cww':'application/prs.cww', 'cxt':'application/x-director', 'cxx':'text/x-c', 'dae':'model/vnd.collada+xml', 'daf':'application/vnd.mobius.daf', 'dart':'application/vnd.dart', 'dataless':'application/vnd.fdsn.seed', 'davmount':'application/davmount+xml', 'dbf':'application/vnd.dbf', 'dbk':'application/docbook+xml', 'dcr':'application/x-director', 
'dcurl':'text/vnd.curl.dcurl', 'dd2':'application/vnd.oma.dd2+xml', 'ddd':'application/vnd.fujixerox.ddd', 'ddf':'application/vnd.syncml.dmddf+xml', 'dds':'image/vnd.ms-dds', 'deb':'application/x-debian-package', 'def':'text/plain', 'deploy':'application/octet-stream', 'der':'application/x-x509-ca-cert', 'dfac':'application/vnd.dreamfactory', 'dgc':'application/x-dgc-compressed', 'dic':'text/x-c', 'dir':'application/x-director', 'dis':'application/vnd.mobius.dis', 'disposition-notification':'message/disposition-notification', 
'dist':'application/octet-stream', 'distz':'application/octet-stream', 'djv':'image/vnd.djvu', 'djvu':'image/vnd.djvu', 'dll':'application/x-msdownload', 'dmg':'application/x-apple-diskimage', 'dmp':'application/vnd.tcpdump.pcap', 'dms':'application/octet-stream', 'dna':'application/vnd.dna', 'doc':'application/msword', 'docm':'application/vnd.ms-word.document.macroenabled.12', 'docx':'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'dot':'application/msword', 'dotm':'application/vnd.ms-word.template.macroenabled.12', 
'dotx':'application/vnd.openxmlformats-officedocument.wordprocessingml.template', 'dp':'application/vnd.osgi.dp', 'dpg':'application/vnd.dpgraph', 'dra':'audio/vnd.dra', 'drle':'image/dicom-rle', 'dsc':'text/prs.lines.tag', 'dssc':'application/dssc+der', 'dtb':'application/x-dtbook+xml', 'dtd':'application/xml-dtd', 'dts':'audio/vnd.dts', 'dtshd':'audio/vnd.dts.hd', 'dump':'application/octet-stream', 'dvb':'video/vnd.dvb.file', 'dvi':'application/x-dvi', 'dwd':'application/atsc-dwd+xml', 'dwf':'model/vnd.dwf', 
'dwg':'image/vnd.dwg', 'dxf':'image/vnd.dxf', 'dxp':'application/vnd.spotfire.dxp', 'dxr':'application/x-director', 'ear':'application/java-archive', 'ecelp4800':'audio/vnd.nuera.ecelp4800', 'ecelp7470':'audio/vnd.nuera.ecelp7470', 'ecelp9600':'audio/vnd.nuera.ecelp9600', 'ecma':'application/ecmascript', 'edm':'application/vnd.novadigm.edm', 'edx':'application/vnd.novadigm.edx', 'efif':'application/vnd.picsel', 'ei6':'application/vnd.pg.osasli', 'elc':'application/octet-stream', 'emf':'image/emf', 
'eml':'message/rfc822', 'emma':'application/emma+xml', 'emotionml':'application/emotionml+xml', 'emz':'application/x-msmetafile', 'eol':'audio/vnd.digital-winds', 'eot':'application/vnd.ms-fontobject', 'eps':'application/postscript', 'epub':'application/epub+zip', 'es':'application/ecmascript', 'es3':'application/vnd.eszigno3+xml', 'esa':'application/vnd.osgi.subsystem', 'esf':'application/vnd.epson.esf', 'et3':'application/vnd.eszigno3+xml', 'etx':'text/x-setext', 'eva':'application/x-eva', 'evy':'application/x-envoy', 
'exe':'application/x-msdownload', 'exi':'application/exi', 'exr':'image/aces', 'ext':'application/vnd.novadigm.ext', 'ez':'application/andrew-inset', 'ez2':'application/vnd.ezpix-album', 'ez3':'application/vnd.ezpix-package', 'f':'text/x-fortran', 'f4v':'video/x-f4v', 'f77':'text/x-fortran', 'f90':'text/x-fortran', 'fbs':'image/vnd.fastbidsheet', 'fcdt':'application/vnd.adobe.formscentral.fcdt', 'fcs':'application/vnd.isac.fcs', 'fdf':'application/vnd.fdf', 'fdt':'application/fdt+xml', 'fe_launch':'application/vnd.denovo.fcselayout-link', 
'fg5':'application/vnd.fujitsu.oasysgp', 'fgd':'application/x-director', 'fh':'image/x-freehand', 'fh4':'image/x-freehand', 'fh5':'image/x-freehand', 'fh7':'image/x-freehand', 'fhc':'image/x-freehand', 'fig':'application/x-xfig', 'fits':'image/fits', 'flac':'audio/x-flac', 'fli':'video/x-fli', 'flo':'application/vnd.micrografx.flo', 'flv':'video/x-flv', 'flw':'application/vnd.kde.kivio', 'flx':'text/vnd.fmi.flexstor', 'fly':'text/vnd.fly', 'fm':'application/vnd.framemaker', 'fnc':'application/vnd.frogans.fnc', 
'fo':'application/vnd.software602.filler.form+xml', 'for':'text/x-fortran', 'fpx':'image/vnd.fpx', 'frame':'application/vnd.framemaker', 'fsc':'application/vnd.fsc.weblaunch', 'fst':'image/vnd.fst', 'ftc':'application/vnd.fluxtime.clip', 'fti':'application/vnd.anser-web-funds-transfer-initiation', 'fvt':'video/vnd.fvt', 'fxp':'application/vnd.adobe.fxp', 'fxpl':'application/vnd.adobe.fxp', 'fzs':'application/vnd.fuzzysheet', 'g2w':'application/vnd.geoplan', 'g3':'image/g3fax', 'g3w':'application/vnd.geospace', 
'gac':'application/vnd.groove-account', 'gam':'application/x-tads', 'gbr':'application/rpki-ghostbusters', 'gca':'application/x-gca-compressed', 'gdl':'model/vnd.gdl', 'gdoc':'application/vnd.google-apps.document', 'geo':'application/vnd.dynageo', 'geojson':'application/geo+json', 'gex':'application/vnd.geometry-explorer', 'ggb':'application/vnd.geogebra.file', 'ggt':'application/vnd.geogebra.tool', 'ghf':'application/vnd.groove-help', 'gif':'image/gif', 'gim':'application/vnd.groove-identity-message', 
'glb':'model/gltf-binary', 'gltf':'model/gltf+json', 'gml':'application/gml+xml', 'gmx':'application/vnd.gmx', 'gnumeric':'application/x-gnumeric', 'gph':'application/vnd.flographit', 'gpx':'application/gpx+xml', 'gqf':'application/vnd.grafeq', 'gqs':'application/vnd.grafeq', 'gram':'application/srgs', 'gramps':'application/x-gramps-xml', 'gre':'application/vnd.geometry-explorer', 'grv':'application/vnd.groove-injector', 'grxml':'application/srgs+xml', 'gsf':'application/x-font-ghostscript', 'gsheet':'application/vnd.google-apps.spreadsheet', 
'gslides':'application/vnd.google-apps.presentation', 'gtar':'application/x-gtar', 'gtm':'application/vnd.groove-tool-message', 'gtw':'model/vnd.gtw', 'gv':'text/vnd.graphviz', 'gxf':'application/gxf', 'gxt':'application/vnd.geonext', 'gz':'application/gzip', 'h':'text/x-c', 'h261':'video/h261', 'h263':'video/h263', 'h264':'video/h264', 'hal':'application/vnd.hal+xml', 'hbci':'application/vnd.hbci', 'hbs':'text/x-handlebars-template', 'hdd':'application/x-virtualbox-hdd', 'hdf':'application/x-hdf', 
'heic':'image/heic', 'heics':'image/heic-sequence', 'heif':'image/heif', 'heifs':'image/heif-sequence', 'hej2':'image/hej2k', 'held':'application/atsc-held+xml', 'hh':'text/x-c', 'hjson':'application/hjson', 'hlp':'application/winhlp', 'hpgl':'application/vnd.hp-hpgl', 'hpid':'application/vnd.hp-hpid', 'hps':'application/vnd.hp-hps', 'hqx':'application/mac-binhex40', 'hsj2':'image/hsj2', 'htc':'text/x-component', 'htke':'application/vnd.kenameaapp', 'htm':'text/html', 'html':'text/html', 'hvd':'application/vnd.yamaha.hv-dic', 
'hvp':'application/vnd.yamaha.hv-voice', 'hvs':'application/vnd.yamaha.hv-script', 'i2g':'application/vnd.intergeo', 'icc':'application/vnd.iccprofile', 'ice':'x-conference/x-cooltalk', 'icm':'application/vnd.iccprofile', 'ico':'image/x-icon', 'ics':'text/calendar', 'ief':'image/ief', 'ifb':'text/calendar', 'ifm':'application/vnd.shana.informed.formdata', 'iges':'model/iges', 'igl':'application/vnd.igloader', 'igm':'application/vnd.insors.igm', 'igs':'model/iges', 'igx':'application/vnd.micrografx.igx', 
'iif':'application/vnd.shana.informed.interchange', 'img':'application/octet-stream', 'imp':'application/vnd.accpac.simply.imp', 'ims':'application/vnd.ms-ims', 'in':'text/plain', 'ini':'text/plain', 'ink':'application/inkml+xml', 'inkml':'application/inkml+xml', 'install':'application/x-install-instructions', 'iota':'application/vnd.astraea-software.iota', 'ipfix':'application/ipfix', 'ipk':'application/vnd.shana.informed.package', 'irm':'application/vnd.ibm.rights-management', 'irp':'application/vnd.irepository.package+xml', 
'iso':'application/x-iso9660-image', 'itp':'application/vnd.shana.informed.formtemplate', 'its':'application/its+xml', 'ivp':'application/vnd.immervision-ivp', 'ivu':'application/vnd.immervision-ivu', 'jad':'text/vnd.sun.j2me.app-descriptor', 'jade':'text/jade', 'jam':'application/vnd.jam', 'jar':'application/java-archive', 'jardiff':'application/x-java-archive-diff', 'java':'text/x-java-source', 'jhc':'image/jphc', 'jisp':'application/vnd.jisp', 'jls':'image/jls', 'jlt':'application/vnd.hp-jlyt', 
'jng':'image/x-jng', 'jnlp':'application/x-java-jnlp-file', 'joda':'application/vnd.joost.joda-archive', 'jp2':'image/jp2', 'jpe':'image/jpeg', 'jpeg':'image/jpeg', 'jpf':'image/jpx', 'jpg':'image/jpeg', 'jpg2':'image/jp2', 'jpgm':'video/jpm', 'jpgv':'video/jpeg', 'jph':'image/jph', 'jpm':'video/jpm', 'jpx':'image/jpx', 'js':'application/javascript', 'json':'application/json', 'json5':'application/json5', 'jsonld':'application/ld+json', 'jsonml':'application/jsonml+json', 'jsx':'text/jsx', 'jxr':'image/jxr', 
'jxra':'image/jxra', 'jxrs':'image/jxrs', 'jxs':'image/jxs', 'jxsc':'image/jxsc', 'jxsi':'image/jxsi', 'jxss':'image/jxss', 'kar':'audio/midi', 'karbon':'application/vnd.kde.karbon', 'kdbx':'application/x-keepass2', 'key':'application/vnd.apple.keynote', 'kfo':'application/vnd.kde.kformula', 'kia':'application/vnd.kidspiration', 'kml':'application/vnd.google-earth.kml+xml', 'kmz':'application/vnd.google-earth.kmz', 'kne':'application/vnd.kinar', 'knp':'application/vnd.kinar', 'kon':'application/vnd.kde.kontour', 
'kpr':'application/vnd.kde.kpresenter', 'kpt':'application/vnd.kde.kpresenter', 'kpxx':'application/vnd.ds-keypoint', 'ksp':'application/vnd.kde.kspread', 'ktr':'application/vnd.kahootz', 'ktx':'image/ktx', 'ktx2':'image/ktx2', 'ktz':'application/vnd.kahootz', 'kwd':'application/vnd.kde.kword', 'kwt':'application/vnd.kde.kword', 'lasxml':'application/vnd.las.las+xml', 'latex':'application/x-latex', 'lbd':'application/vnd.llamagraphics.life-balance.desktop', 'lbe':'application/vnd.llamagraphics.life-balance.exchange+xml', 
'les':'application/vnd.hhe.lesson-player', 'less':'text/less', 'lgr':'application/lgr+xml', 'lha':'application/x-lzh-compressed', 'link66':'application/vnd.route66.link66+xml', 'list':'text/plain', 'list3820':'application/vnd.ibm.modcap', 'listafp':'application/vnd.ibm.modcap', 'litcoffee':'text/coffeescript', 'lnk':'application/x-ms-shortcut', 'log':'text/plain', 'lostxml':'application/lost+xml', 'lrf':'application/octet-stream', 'lrm':'application/vnd.ms-lrm', 'ltf':'application/vnd.frogans.ltf', 
'lua':'text/x-lua', 'luac':'application/x-lua-bytecode', 'lvp':'audio/vnd.lucent.voice', 'lwp':'application/vnd.lotus-wordpro', 'lzh':'application/x-lzh-compressed', 'm13':'application/x-msmediaview', 'm14':'application/x-msmediaview', 'm1v':'video/mpeg', 'm21':'application/mp21', 'm2a':'audio/mpeg', 'm2v':'video/mpeg', 'm3a':'audio/mpeg', 'm3u':'audio/x-mpegurl', 'm3u8':'application/vnd.apple.mpegurl', 'm4a':'audio/x-m4a', 'm4p':'application/mp4', 'm4s':'video/iso.segment', 'm4u':'video/vnd.mpegurl', 
'm4v':'video/x-m4v', 'ma':'application/mathematica', 'mads':'application/mads+xml', 'maei':'application/mmt-aei+xml', 'mag':'application/vnd.ecowin.chart', 'maker':'application/vnd.framemaker', 'man':'text/troff', 'manifest':'text/cache-manifest', 'map':'application/json', 'mar':'application/octet-stream', 'markdown':'text/markdown', 'mathml':'application/mathml+xml', 'mb':'application/mathematica', 'mbk':'application/vnd.mobius.mbk', 'mbox':'application/mbox', 'mc1':'application/vnd.medcalcdata', 
'mcd':'application/vnd.mcd', 'mcurl':'text/vnd.curl.mcurl', 'md':'text/markdown', 'mdb':'application/x-msaccess', 'mdi':'image/vnd.ms-modi', 'mdx':'text/mdx', 'me':'text/troff', 'mesh':'model/mesh', 'meta4':'application/metalink4+xml', 'metalink':'application/metalink+xml', 'mets':'application/mets+xml', 'mfm':'application/vnd.mfmp', 'mft':'application/rpki-manifest', 'mgp':'application/vnd.osgeo.mapguide.package', 'mgz':'application/vnd.proteus.magazine', 'mid':'audio/midi', 'midi':'audio/midi', 
'mie':'application/x-mie', 'mif':'application/vnd.mif', 'mime':'message/rfc822', 'mj2':'video/mj2', 'mjp2':'video/mj2', 'mjs':'application/javascript', 'mk3d':'video/x-matroska', 'mka':'audio/x-matroska', 'mkd':'text/x-markdown', 'mks':'video/x-matroska', 'mkv':'video/x-matroska', 'mlp':'application/vnd.dolby.mlp', 'mmd':'application/vnd.chipnuts.karaoke-mmd', 'mmf':'application/vnd.smaf', 'mml':'text/mathml', 'mmr':'image/vnd.fujixerox.edmics-mmr', 'mng':'video/x-mng', 'mny':'application/x-msmoney', 
'mobi':'application/x-mobipocket-ebook', 'mods':'application/mods+xml', 'mov':'video/quicktime', 'movie':'video/x-sgi-movie', 'mp2':'audio/mpeg', 'mp21':'application/mp21', 'mp2a':'audio/mpeg', 'mp3':'audio/mpeg', 'mp4':'video/mp4', 'mp4a':'audio/mp4', 'mp4s':'application/mp4', 'mp4v':'video/mp4', 'mpc':'application/vnd.mophun.certificate', 'mpd':'application/dash+xml', 'mpe':'video/mpeg', 'mpeg':'video/mpeg', 'mpg':'video/mpeg', 'mpg4':'video/mp4', 'mpga':'audio/mpeg', 'mpkg':'application/vnd.apple.installer+xml', 
'mpm':'application/vnd.blueice.multipass', 'mpn':'application/vnd.mophun.application', 'mpp':'application/vnd.ms-project', 'mpt':'application/vnd.ms-project', 'mpy':'application/vnd.ibm.minipay', 'mqy':'application/vnd.mobius.mqy', 'mrc':'application/marc', 'mrcx':'application/marcxml+xml', 'ms':'text/troff', 'mscml':'application/mediaservercontrol+xml', 'mseed':'application/vnd.fdsn.mseed', 'mseq':'application/vnd.mseq', 'msf':'application/vnd.epson.msf', 'msg':'application/vnd.ms-outlook', 'msh':'model/mesh', 
'msi':'application/x-msdownload', 'msl':'application/vnd.mobius.msl', 'msm':'application/octet-stream', 'msp':'application/octet-stream', 'msty':'application/vnd.muvee.style', 'mtl':'model/mtl', 'mts':'model/vnd.mts', 'mus':'application/vnd.musician', 'musd':'application/mmt-usd+xml', 'musicxml':'application/vnd.recordare.musicxml+xml', 'mvb':'application/x-msmediaview', 'mwf':'application/vnd.mfer', 'mxf':'application/mxf', 'mxl':'application/vnd.recordare.musicxml', 'mxmf':'audio/mobile-xmf', 'mxml':'application/xv+xml', 
'mxs':'application/vnd.triscape.mxs', 'mxu':'video/vnd.mpegurl', 'n-gage':'application/vnd.nokia.n-gage.symbian.install', 'n3':'text/n3', 'nb':'application/mathematica', 'nbp':'application/vnd.wolfram.player', 'nc':'application/x-netcdf', 'ncx':'application/x-dtbncx+xml', 'nfo':'text/x-nfo', 'ngdat':'application/vnd.nokia.n-gage.data', 'nitf':'application/vnd.nitf', 'nlu':'application/vnd.neurolanguage.nlu', 'nml':'application/vnd.enliven', 'nnd':'application/vnd.noblenet-directory', 'nns':'application/vnd.noblenet-sealer', 
'nnw':'application/vnd.noblenet-web', 'npx':'image/vnd.net-fpx', 'nq':'application/n-quads', 'nsc':'application/x-conference', 'nsf':'application/vnd.lotus-notes', 'nt':'application/n-triples', 'ntf':'application/vnd.nitf', 'numbers':'application/vnd.apple.numbers', 'nzb':'application/x-nzb', 'oa2':'application/vnd.fujitsu.oasys2', 'oa3':'application/vnd.fujitsu.oasys3', 'oas':'application/vnd.fujitsu.oasys', 'obd':'application/x-msbinder', 'obgx':'application/vnd.openblox.game+xml', 'obj':'model/obj', 
'oda':'application/oda', 'odb':'application/vnd.oasis.opendocument.database', 'odc':'application/vnd.oasis.opendocument.chart', 'odf':'application/vnd.oasis.opendocument.formula', 'odft':'application/vnd.oasis.opendocument.formula-template', 'odg':'application/vnd.oasis.opendocument.graphics', 'odi':'application/vnd.oasis.opendocument.image', 'odm':'application/vnd.oasis.opendocument.text-master', 'odp':'application/vnd.oasis.opendocument.presentation', 'ods':'application/vnd.oasis.opendocument.spreadsheet', 
'odt':'application/vnd.oasis.opendocument.text', 'oga':'audio/ogg', 'ogex':'model/vnd.opengex', 'ogg':'audio/ogg', 'ogv':'video/ogg', 'ogx':'application/ogg', 'omdoc':'application/omdoc+xml', 'onepkg':'application/onenote', 'onetmp':'application/onenote', 'onetoc':'application/onenote', 'onetoc2':'application/onenote', 'opf':'application/oebps-package+xml', 'opml':'text/x-opml', 'oprc':'application/vnd.palm', 'opus':'audio/ogg', 'org':'text/x-org', 'osf':'application/vnd.yamaha.openscoreformat', 
'osfpvg':'application/vnd.yamaha.openscoreformat.osfpvg+xml', 'osm':'application/vnd.openstreetmap.data+xml', 'otc':'application/vnd.oasis.opendocument.chart-template', 'otf':'font/otf', 'otg':'application/vnd.oasis.opendocument.graphics-template', 'oth':'application/vnd.oasis.opendocument.text-web', 'oti':'application/vnd.oasis.opendocument.image-template', 'otp':'application/vnd.oasis.opendocument.presentation-template', 'ots':'application/vnd.oasis.opendocument.spreadsheet-template', 'ott':'application/vnd.oasis.opendocument.text-template', 
'ova':'application/x-virtualbox-ova', 'ovf':'application/x-virtualbox-ovf', 'owl':'application/rdf+xml', 'oxps':'application/oxps', 'oxt':'application/vnd.openofficeorg.extension', 'p':'text/x-pascal', 'p10':'application/pkcs10', 'p12':'application/x-pkcs12', 'p7b':'application/x-pkcs7-certificates', 'p7c':'application/pkcs7-mime', 'p7m':'application/pkcs7-mime', 'p7r':'application/x-pkcs7-certreqresp', 'p7s':'application/pkcs7-signature', 'p8':'application/pkcs8', 'pac':'application/x-ns-proxy-autoconfig', 
'pages':'application/vnd.apple.pages', 'pas':'text/x-pascal', 'paw':'application/vnd.pawaafile', 'pbd':'application/vnd.powerbuilder6', 'pbm':'image/x-portable-bitmap', 'pcap':'application/vnd.tcpdump.pcap', 'pcf':'application/x-font-pcf', 'pcl':'application/vnd.hp-pcl', 'pclxl':'application/vnd.hp-pclxl', 'pct':'image/x-pict', 'pcurl':'application/vnd.curl.pcurl', 'pcx':'image/x-pcx', 'pdb':'application/x-pilot', 'pde':'text/x-processing', 'pdf':'application/pdf', 'pem':'application/x-x509-ca-cert', 
'pfa':'application/x-font-type1', 'pfb':'application/x-font-type1', 'pfm':'application/x-font-type1', 'pfr':'application/font-tdpfr', 'pfx':'application/x-pkcs12', 'pgm':'image/x-portable-graymap', 'pgn':'application/x-chess-pgn', 'pgp':'application/pgp-encrypted', 'php':'application/x-httpd-php', 'pic':'image/x-pict', 'pkg':'application/octet-stream', 'pki':'application/pkixcmp', 'pkipath':'application/pkix-pkipath', 'pkpass':'application/vnd.apple.pkpass', 'pl':'application/x-perl', 'plb':'application/vnd.3gpp.pic-bw-large', 
'plc':'application/vnd.mobius.plc', 'plf':'application/vnd.pocketlearn', 'pls':'application/pls+xml', 'pm':'application/x-perl', 'pml':'application/vnd.ctc-posml', 'png':'image/png', 'pnm':'image/x-portable-anymap', 'portpkg':'application/vnd.macports.portpkg', 'pot':'application/vnd.ms-powerpoint', 'potm':'application/vnd.ms-powerpoint.template.macroenabled.12', 'potx':'application/vnd.openxmlformats-officedocument.presentationml.template', 'ppam':'application/vnd.ms-powerpoint.addin.macroenabled.12', 
'ppd':'application/vnd.cups-ppd', 'ppm':'image/x-portable-pixmap', 'pps':'application/vnd.ms-powerpoint', 'ppsm':'application/vnd.ms-powerpoint.slideshow.macroenabled.12', 'ppsx':'application/vnd.openxmlformats-officedocument.presentationml.slideshow', 'ppt':'application/vnd.ms-powerpoint', 'pptm':'application/vnd.ms-powerpoint.presentation.macroenabled.12', 'pptx':'application/vnd.openxmlformats-officedocument.presentationml.presentation', 'pqa':'application/vnd.palm', 'prc':'application/x-pilot', 
'pre':'application/vnd.lotus-freelance', 'prf':'application/pics-rules', 'provx':'application/provenance+xml', 'ps':'application/postscript', 'psb':'application/vnd.3gpp.pic-bw-small', 'psd':'image/vnd.adobe.photoshop', 'psf':'application/x-font-linux-psf', 'pskcxml':'application/pskc+xml', 'pti':'image/prs.pti', 'ptid':'application/vnd.pvi.ptid1', 'pub':'application/x-mspublisher', 'pvb':'application/vnd.3gpp.pic-bw-var', 'pwn':'application/vnd.3m.post-it-notes', 'pya':'audio/vnd.ms-playready.media.pya', 
'pyv':'video/vnd.ms-playready.media.pyv', 'qam':'application/vnd.epson.quickanime', 'qbo':'application/vnd.intu.qbo', 'qfx':'application/vnd.intu.qfx', 'qps':'application/vnd.publishare-delta-tree', 'qt':'video/quicktime', 'qwd':'application/vnd.quark.quarkxpress', 'qwt':'application/vnd.quark.quarkxpress', 'qxb':'application/vnd.quark.quarkxpress', 'qxd':'application/vnd.quark.quarkxpress', 'qxl':'application/vnd.quark.quarkxpress', 'qxt':'application/vnd.quark.quarkxpress', 'ra':'audio/x-realaudio', 
'ram':'audio/x-pn-realaudio', 'raml':'application/raml+yaml', 'rapd':'application/route-apd+xml', 'rar':'application/x-rar-compressed', 'ras':'image/x-cmu-raster', 'rcprofile':'application/vnd.ipunplugged.rcprofile', 'rdf':'application/rdf+xml', 'rdz':'application/vnd.data-vision.rdz', 'relo':'application/p2p-overlay+xml', 'rep':'application/vnd.businessobjects', 'res':'application/x-dtbresource+xml', 'rgb':'image/x-rgb', 'rif':'application/reginfo+xml', 'rip':'audio/vnd.rip', 'ris':'application/x-research-info-systems', 
'rl':'application/resource-lists+xml', 'rlc':'image/vnd.fujixerox.edmics-rlc', 'rld':'application/resource-lists-diff+xml', 'rm':'application/vnd.rn-realmedia', 'rmi':'audio/midi', 'rmp':'audio/x-pn-realaudio-plugin', 'rms':'application/vnd.jcp.javame.midlet-rms', 'rmvb':'application/vnd.rn-realmedia-vbr', 'rnc':'application/relax-ng-compact-syntax', 'rng':'application/xml', 'roa':'application/rpki-roa', 'roff':'text/troff', 'rp9':'application/vnd.cloanto.rp9', 'rpm':'application/x-redhat-package-manager', 
'rpss':'application/vnd.nokia.radio-presets', 'rpst':'application/vnd.nokia.radio-preset', 'rq':'application/sparql-query', 'rs':'application/rls-services+xml', 'rsat':'application/atsc-rsat+xml', 'rsd':'application/rsd+xml', 'rsheet':'application/urc-ressheet+xml', 'rss':'application/rss+xml', 'rtf':'text/rtf', 'rtx':'text/richtext', 'run':'application/x-makeself', 'rusd':'application/route-usd+xml', 's':'text/x-asm', 's3m':'audio/s3m', 'saf':'application/vnd.yamaha.smaf-audio', 'sass':'text/x-sass', 
'sbml':'application/sbml+xml', 'sc':'application/vnd.ibm.secure-container', 'scd':'application/x-msschedule', 'scm':'application/vnd.lotus-screencam', 'scq':'application/scvp-cv-request', 'scs':'application/scvp-cv-response', 'scss':'text/x-scss', 'scurl':'text/vnd.curl.scurl', 'sda':'application/vnd.stardivision.draw', 'sdc':'application/vnd.stardivision.calc', 'sdd':'application/vnd.stardivision.impress', 'sdkd':'application/vnd.solent.sdkm+xml', 'sdkm':'application/vnd.solent.sdkm+xml', 'sdp':'application/sdp', 
'sdw':'application/vnd.stardivision.writer', 'sea':'application/x-sea', 'see':'application/vnd.seemail', 'seed':'application/vnd.fdsn.seed', 'sema':'application/vnd.sema', 'semd':'application/vnd.semd', 'semf':'application/vnd.semf', 'senmlx':'application/senml+xml', 'sensmlx':'application/sensml+xml', 'ser':'application/java-serialized-object', 'setpay':'application/set-payment-initiation', 'setreg':'application/set-registration-initiation', 'sfd-hdstx':'application/vnd.hydrostatix.sof-data', 'sfs':'application/vnd.spotfire.sfs', 
'sfv':'text/x-sfv', 'sgi':'image/sgi', 'sgl':'application/vnd.stardivision.writer-global', 'sgm':'text/sgml', 'sgml':'text/sgml', 'sh':'application/x-sh', 'shar':'application/x-shar', 'shex':'text/shex', 'shf':'application/shf+xml', 'shtml':'text/html', 'sid':'image/x-mrsid-image', 'sieve':'application/sieve', 'sig':'application/pgp-signature', 'sil':'audio/silk', 'silo':'model/mesh', 'sis':'application/vnd.symbian.install', 'sisx':'application/vnd.symbian.install', 'sit':'application/x-stuffit', 
'sitx':'application/x-stuffitx', 'siv':'application/sieve', 'skd':'application/vnd.koan', 'skm':'application/vnd.koan', 'skp':'application/vnd.koan', 'skt':'application/vnd.koan', 'sldm':'application/vnd.ms-powerpoint.slide.macroenabled.12', 'sldx':'application/vnd.openxmlformats-officedocument.presentationml.slide', 'slim':'text/slim', 'slm':'text/slim', 'sls':'application/route-s-tsid+xml', 'slt':'application/vnd.epson.salt', 'sm':'application/vnd.stepmania.stepchart', 'smf':'application/vnd.stardivision.math', 
'smi':'application/smil+xml', 'smil':'application/smil+xml', 'smv':'video/x-smv', 'smzip':'application/vnd.stepmania.package', 'snd':'audio/basic', 'snf':'application/x-font-snf', 'so':'application/octet-stream', 'spc':'application/x-pkcs7-certificates', 'spdx':'text/spdx', 'spf':'application/vnd.yamaha.smaf-phrase', 'spl':'application/x-futuresplash', 'spot':'text/vnd.in3d.spot', 'spp':'application/scvp-vp-response', 'spq':'application/scvp-vp-request', 'spx':'audio/ogg', 'sql':'application/x-sql', 
'src':'application/x-wais-source', 'srt':'application/x-subrip', 'sru':'application/sru+xml', 'srx':'application/sparql-results+xml', 'ssdl':'application/ssdl+xml', 'sse':'application/vnd.kodak-descriptor', 'ssf':'application/vnd.epson.ssf', 'ssml':'application/ssml+xml', 'st':'application/vnd.sailingtracker.track', 'stc':'application/vnd.sun.xml.calc.template', 'std':'application/vnd.sun.xml.draw.template', 'stf':'application/vnd.wt.stf', 'sti':'application/vnd.sun.xml.impress.template', 'stk':'application/hyperstudio', 
'stl':'model/stl', 'str':'application/vnd.pg.format', 'stw':'application/vnd.sun.xml.writer.template', 'styl':'text/stylus', 'stylus':'text/stylus', 'sub':'text/vnd.dvb.subtitle', 'sus':'application/vnd.sus-calendar', 'susp':'application/vnd.sus-calendar', 'sv4cpio':'application/x-sv4cpio', 'sv4crc':'application/x-sv4crc', 'svc':'application/vnd.dvb.service', 'svd':'application/vnd.svd', 'svg':'image/svg+xml', 'svgz':'image/svg+xml', 'swa':'application/x-director', 'swf':'application/x-shockwave-flash', 
'swi':'application/vnd.aristanetworks.swi', 'swidtag':'application/swid+xml', 'sxc':'application/vnd.sun.xml.calc', 'sxd':'application/vnd.sun.xml.draw', 'sxg':'application/vnd.sun.xml.writer.global', 'sxi':'application/vnd.sun.xml.impress', 'sxm':'application/vnd.sun.xml.math', 'sxw':'application/vnd.sun.xml.writer', 't':'text/troff', 't3':'application/x-t3vm-image', 't38':'image/t38', 'taglet':'application/vnd.mynfc', 'tao':'application/vnd.tao.intent-module-archive', 'tap':'image/vnd.tencent.tap', 
'tar':'application/x-tar', 'tcap':'application/vnd.3gpp2.tcap', 'tcl':'application/x-tcl', 'td':'application/urc-targetdesc+xml', 'teacher':'application/vnd.smart.teacher', 'tei':'application/tei+xml', 'teicorpus':'application/tei+xml', 'tex':'application/x-tex', 'texi':'application/x-texinfo', 'texinfo':'application/x-texinfo', 'text':'text/plain', 'tfi':'application/thraud+xml', 'tfm':'application/x-tex-tfm', 'tfx':'image/tiff-fx', 'tga':'image/x-tga', 'thmx':'application/vnd.ms-officetheme', 'tif':'image/tiff', 
'tiff':'image/tiff', 'tk':'application/x-tcl', 'tmo':'application/vnd.tmobile-livetv', 'toml':'application/toml', 'torrent':'application/x-bittorrent', 'tpl':'application/vnd.groove-tool-template', 'tpt':'application/vnd.trid.tpt', 'tr':'text/troff', 'tra':'application/vnd.trueapp', 'trm':'application/x-msterminal', 'ts':'video/mp2t', 'tsd':'application/timestamped-data', 'tsv':'text/tab-separated-values', 'ttc':'font/collection', 'ttf':'font/ttf', 'ttl':'text/turtle', 'ttml':'application/ttml+xml', 
'twd':'application/vnd.simtech-mindmapper', 'twds':'application/vnd.simtech-mindmapper', 'txd':'application/vnd.genomatix.tuxedo', 'txf':'application/vnd.mobius.txf', 'txt':'text/plain', 'u32':'application/x-authorware-bin', 'u8dsn':'message/global-delivery-status', 'u8hdr':'message/global-headers', 'u8mdn':'message/global-disposition-notification', 'u8msg':'message/global', 'ubj':'application/ubjson', 'udeb':'application/x-debian-package', 'ufd':'application/vnd.ufdl', 'ufdl':'application/vnd.ufdl', 
'ulx':'application/x-glulx', 'umj':'application/vnd.umajin', 'unityweb':'application/vnd.unity', 'uoml':'application/vnd.uoml+xml', 'uri':'text/uri-list', 'uris':'text/uri-list', 'urls':'text/uri-list', 'usdz':'model/vnd.usdz+zip', 'ustar':'application/x-ustar', 'utz':'application/vnd.uiq.theme', 'uu':'text/x-uuencode', 'uva':'audio/vnd.dece.audio', 'uvd':'application/vnd.dece.data', 'uvf':'application/vnd.dece.data', 'uvg':'image/vnd.dece.graphic', 'uvh':'video/vnd.dece.hd', 'uvi':'image/vnd.dece.graphic', 
'uvm':'video/vnd.dece.mobile', 'uvp':'video/vnd.dece.pd', 'uvs':'video/vnd.dece.sd', 'uvt':'application/vnd.dece.ttml+xml', 'uvu':'video/vnd.uvvu.mp4', 'uvv':'video/vnd.dece.video', 'uvva':'audio/vnd.dece.audio', 'uvvd':'application/vnd.dece.data', 'uvvf':'application/vnd.dece.data', 'uvvg':'image/vnd.dece.graphic', 'uvvh':'video/vnd.dece.hd', 'uvvi':'image/vnd.dece.graphic', 'uvvm':'video/vnd.dece.mobile', 'uvvp':'video/vnd.dece.pd', 'uvvs':'video/vnd.dece.sd', 'uvvt':'application/vnd.dece.ttml+xml', 
'uvvu':'video/vnd.uvvu.mp4', 'uvvv':'video/vnd.dece.video', 'uvvx':'application/vnd.dece.unspecified', 'uvvz':'application/vnd.dece.zip', 'uvx':'application/vnd.dece.unspecified', 'uvz':'application/vnd.dece.zip', 'vbox':'application/x-virtualbox-vbox', 'vbox-extpack':'application/x-virtualbox-vbox-extpack', 'vcard':'text/vcard', 'vcd':'application/x-cdlink', 'vcf':'text/x-vcard', 'vcg':'application/vnd.groove-vcard', 'vcs':'text/x-vcalendar', 'vcx':'application/vnd.vcx', 'vdi':'application/x-virtualbox-vdi', 
'vhd':'application/x-virtualbox-vhd', 'vis':'application/vnd.visionary', 'viv':'video/vnd.vivo', 'vmdk':'application/x-virtualbox-vmdk', 'vob':'video/x-ms-vob', 'vor':'application/vnd.stardivision.writer', 'vox':'application/x-authorware-bin', 'vrml':'model/vrml', 'vsd':'application/vnd.visio', 'vsf':'application/vnd.vsf', 'vss':'application/vnd.visio', 'vst':'application/vnd.visio', 'vsw':'application/vnd.visio', 'vtf':'image/vnd.valve.source.texture', 'vtt':'text/vtt', 'vtu':'model/vnd.vtu', 'vxml':'application/voicexml+xml', 
'w3d':'application/x-director', 'wad':'application/x-doom', 'wadl':'application/vnd.sun.wadl+xml', 'war':'application/java-archive', 'wasm':'application/wasm', 'wav':'audio/x-wav', 'wax':'audio/x-ms-wax', 'wbmp':'image/vnd.wap.wbmp', 'wbs':'application/vnd.criticaltools.wbs+xml', 'wbxml':'application/vnd.wap.wbxml', 'wcm':'application/vnd.ms-works', 'wdb':'application/vnd.ms-works', 'wdp':'image/vnd.ms-photo', 'weba':'audio/webm', 'webapp':'application/x-web-app-manifest+json', 'webm':'video/webm', 
'webmanifest':'application/manifest+json', 'webp':'image/webp', 'wg':'application/vnd.pmi.widget', 'wgt':'application/widget', 'wks':'application/vnd.ms-works', 'wm':'video/x-ms-wm', 'wma':'audio/x-ms-wma', 'wmd':'application/x-ms-wmd', 'wmf':'image/wmf', 'wml':'text/vnd.wap.wml', 'wmlc':'application/vnd.wap.wmlc', 'wmls':'text/vnd.wap.wmlscript', 'wmlsc':'application/vnd.wap.wmlscriptc', 'wmv':'video/x-ms-wmv', 'wmx':'video/x-ms-wmx', 'wmz':'application/x-msmetafile', 'woff':'font/woff', 'woff2':'font/woff2', 
'wpd':'application/vnd.wordperfect', 'wpl':'application/vnd.ms-wpl', 'wps':'application/vnd.ms-works', 'wqd':'application/vnd.wqd', 'wri':'application/x-mswrite', 'wrl':'model/vrml', 'wsc':'message/vnd.wfa.wsc', 'wsdl':'application/wsdl+xml', 'wspolicy':'application/wspolicy+xml', 'wtb':'application/vnd.webturbo', 'wvx':'video/x-ms-wvx', 'x32':'application/x-authorware-bin', 'x3d':'model/x3d+xml', 'x3db':'model/x3d+fastinfoset', 'x3dbz':'model/x3d+binary', 'x3dv':'model/x3d-vrml', 'x3dvz':'model/x3d+vrml', 
'x3dz':'model/x3d+xml', 'x_b':'model/vnd.parasolid.transmit.binary', 'x_t':'model/vnd.parasolid.transmit.text', 'xaml':'application/xaml+xml', 'xap':'application/x-silverlight-app', 'xar':'application/vnd.xara', 'xav':'application/xcap-att+xml', 'xbap':'application/x-ms-xbap', 'xbd':'application/vnd.fujixerox.docuworks.binder', 'xbm':'image/x-xbitmap', 'xca':'application/xcap-caps+xml', 'xcs':'application/calendar+xml', 'xdf':'application/xcap-diff+xml', 'xdm':'application/vnd.syncml.dm+xml', 'xdp':'application/vnd.adobe.xdp+xml', 
'xdssc':'application/dssc+xml', 'xdw':'application/vnd.fujixerox.docuworks', 'xel':'application/xcap-el+xml', 'xenc':'application/xenc+xml', 'xer':'application/patch-ops-error+xml', 'xfdf':'application/vnd.adobe.xfdf', 'xfdl':'application/vnd.xfdl', 'xht':'application/xhtml+xml', 'xhtml':'application/xhtml+xml', 'xhvml':'application/xv+xml', 'xif':'image/vnd.xiff', 'xla':'application/vnd.ms-excel', 'xlam':'application/vnd.ms-excel.addin.macroenabled.12', 'xlc':'application/vnd.ms-excel', 'xlf':'application/xliff+xml', 
'xlm':'application/vnd.ms-excel', 'xls':'application/vnd.ms-excel', 'xlsb':'application/vnd.ms-excel.sheet.binary.macroenabled.12', 'xlsm':'application/vnd.ms-excel.sheet.macroenabled.12', 'xlsx':'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet', 'xlt':'application/vnd.ms-excel', 'xltm':'application/vnd.ms-excel.template.macroenabled.12', 'xltx':'application/vnd.openxmlformats-officedocument.spreadsheetml.template', 'xlw':'application/vnd.ms-excel', 'xm':'audio/xm', 'xml':'text/xml', 
'xns':'application/xcap-ns+xml', 'xo':'application/vnd.olpc-sugar', 'xop':'application/xop+xml', 'xpi':'application/x-xpinstall', 'xpl':'application/xproc+xml', 'xpm':'image/x-xpixmap', 'xpr':'application/vnd.is-xpr', 'xps':'application/vnd.ms-xpsdocument', 'xpw':'application/vnd.intercon.formnet', 'xpx':'application/vnd.intercon.formnet', 'xsd':'application/xml', 'xsl':'application/xslt+xml', 'xslt':'application/xslt+xml', 'xsm':'application/vnd.syncml+xml', 'xspf':'application/xspf+xml', 'xul':'application/vnd.mozilla.xul+xml', 
'xvm':'application/xv+xml', 'xvml':'application/xv+xml', 'xwd':'image/x-xwindowdump', 'xyz':'chemical/x-xyz', 'xz':'application/x-xz', 'yaml':'text/yaml', 'yang':'application/yang', 'yin':'application/yin+xml', 'yml':'text/yaml', 'ymp':'text/x-suse-ymp', 'z1':'application/x-zmachine', 'z2':'application/x-zmachine', 'z3':'application/x-zmachine', 'z4':'application/x-zmachine', 'z5':'application/x-zmachine', 'z6':'application/x-zmachine', 'z7':'application/x-zmachine', 'z8':'application/x-zmachine', 
'zaz':'application/vnd.zzazz.deck+xml', 'zip':'application/zip', 'zir':'application/vnd.zul', 'zirz':'application/vnd.zul', 'zmm':'application/vnd.handheld-entertainment+xml'}});
Ext.define('documents.Utils', {singleton:true, DATABASE_ADAPTER_ID:'db', FAST_STORAGE_ADAPTER_ID:function() {
  return documents.util.BpcSupport.versionStartsWith('4.1') ? 'fastStorage' : 'elastic';
}, arrayBufferToBase64:function(arrayBuffer) {
  var base64 = '';
  var encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
  var bytes = new Uint8Array(arrayBuffer);
  var byteLength = bytes.byteLength;
  var byteRemainder = byteLength % 3;
  var mainLength = byteLength - byteRemainder;
  var a, b, c, d;
  var chunk;
  for (var i = 0; i < mainLength; i = i + 3) {
    chunk = bytes[i] << 16 | bytes[i + 1] << 8 | bytes[i + 2];
    a = (chunk & 16515072) >> 18;
    b = (chunk & 258048) >> 12;
    c = (chunk & 4032) >> 6;
    d = chunk & 63;
    base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d];
  }
  if (byteRemainder == 1) {
    chunk = bytes[mainLength];
    a = (chunk & 252) >> 2;
    b = (chunk & 3) << 4;
    base64 += encodings[a] + encodings[b] + '\x3d\x3d';
  } else {
    if (byteRemainder == 2) {
      chunk = bytes[mainLength] << 8 | bytes[mainLength + 1];
      a = (chunk & 64512) >> 10;
      b = (chunk & 1008) >> 4;
      c = (chunk & 15) << 2;
      base64 += encodings[a] + encodings[b] + encodings[c] + '\x3d';
    }
  }
  return base64;
}, base64ToDataUrl:function(base64String, mime) {
  return 'data:' + mime + ';base64,' + base64String;
}, getMinimalDocObject:function(mime, base64str) {
  return {mime:mime, dataUri:documents.Utils.base64ToDataUrl(base64str, mime)};
}, toast:function(title, html, width, cmp) {
  width = width === undefined ? 200 : width;
  var toastConfig = {title:title, html:html, width:width};
  if (cmp) {
    toastConfig.anchor = cmp;
  }
  Ext.toast(toastConfig);
}, openFileWindow:function(p, title) {
  Ext.create('Ext.window.Window', {title:title, iconCls:documents.Support.icon('fa-paperclip'), height:document.body.clientHeight * 0.8, width:document.body.clientWidth * 0.5, layout:'fit', items:[p]}).show();
}, rendererData:{rendererMap:{added:{iconCls:'fa-plus', tooltip:'FILE_ADDED', color:'blue'}, uploaded:{iconCls:'fa-upload', tooltip:'FILE_UPLOADED', color:'green'}, updated:{iconCls:'fa-check', tooltip:'FILE_UPDATED', color:'green'}, error:{iconCls:'fa-exclamation-circle', tooltip:'FILE_ERROR', color:'red'}, incomplete:{iconCls:'fa-exclamation-triangle', color:'orange', tooltip:'FILE_INCOMPLETE'}, unexpected:{iconCls:'fa-exclamation-circle', color:'red', tooltip:'FILE_UNEXPECTED'}, notvalid:{iconCls:'fa-exclamation-circle', 
color:'pink', tooltip:'FILE_NOT_VALID'}}}, getUploadItem:function(v) {
  v = v === undefined ? '' : v;
  var $jscomp$destructuring$var28 = documents.Utils.rendererData;
  var rendererMap = $jscomp$destructuring$var28.rendererMap;
  rendererMap = rendererMap || {};
  var item = rendererMap[v] || {};
  if (!item) {
    item = rendererMap['unexpected'];
  }
  return item;
}, renderer:{bytesToKb:function(v, m, r) {
  return Ext.isDefined(v) ? documents.Support.bytesToKb(v, true) + ' KB' : '';
}, tags:function(v, m, r) {
  v = v || [];
  if (Ext.isString(v)) {
    v = v.length === 0 ? [] : [v];
  }
  return documents.Cache.tagsArrayToString(v, this.tags || []);
}, documentIcon:function(value, meta, record, rowI, colI, store, view) {
  meta.tdAttr = 'data-qtip\x3d"' + documents.Utils.getDocumentRecordTooltip(record) + '"';
  return BpcCommon.util.Icons.getSpanForIconCls(documents.Support.getIconClsFromRecord(record), 'black');
}, uploadStateColor:function(v, m, r) {
  var item = documents.Utils.getUploadItem(v);
  m.tdStyle = 'background-color:' + item.color + ';';
  m.tdAttr = 'data-qtip\x3d"' + documents.Support.TR(item.tooltip) + '"';
  return '';
}, uploadStateIcon:function(v, m, r) {
  var item = documents.Utils.getUploadItem(v);
  if (item.tooltip && item.tooltip.length) {
    m.tdAttr = 'data-qtip\x3d"' + documents.Support.TR(item.tooltip) + '"';
  }
  return Ext.htmlDecode(documents.Support.getSpanForIconCls(item.iconCls, '#000'));
}}, getDocumentRecordTooltip:function(r, keys) {
  keys = keys === undefined ? [] : keys;
  var data = r.getData(), keysToShow = keys.length ? keys : ['documentTitle', 'filename', 'downloadCount', 'creator', 'created', 'mime'];
  keysToShow.map(function(k) {
    if (k === 'created') {
      data[k] = documents.Support.dateFormatter(data[k]);
    }
  });
  return Ext.htmlEncode(documents.Support.objectToHtmlTable(data, keysToShow, 'DOC_'));
}, columns:{getBaseColumnForDataIndex:function(dataIndex) {
  dataIndex = dataIndex === undefined ? '' : dataIndex;
  var baseMap = {icon:{sortable:false, renderer:documents.Utils.renderer.documentIcon}, documentTitle:{text:documents.Support.TR('DOC_DOCUMENTTITLE')}};
  var target = baseMap[dataIndex];
  return target ? Ext.apply({}, target) : {};
}}, copy:{clipboard:function(rec, key, text) {
  if (documents.Support.copyToClipBoard(documents.Api.getDownloadUrl(rec.get(key)))) {
    BpcCommon.Api.showNotification(text);
  }
}}, ui:{baseButtonCfg:function(ui) {
  ui = ui === undefined ? 'primary' : ui;
  return {xtype:'button', scale:'medium', iconAlign:'top', ui:ui};
}, getButtonCfg:function(primary, iconCls, textKey) {
  primary = primary === undefined ? true : primary;
  var base = Ext.apply({}, this.baseButtonCfg(primary ? 'primary' : 'secondary'));
  return Ext.apply(base, {iconCls:documents.Support.icon(iconCls), text:documents.Support.TR(textKey)});
}, simpleButton:function(primary, iconCls, textLangkeyKey, config) {
  return Ext.apply(this.getButtonCfg(primary, iconCls, textLangkeyKey), config);
}, getInlineTags:function(tagsValue, moduleId) {
  var $jscomp$this = this;
  tagsValue = tagsValue === undefined ? [] : tagsValue;
  var tagsSource = documents.Cache.getTags(moduleId);
  var tagSpans = tagsValue.map(function(id, i) {
    id = id || '';
    return $jscomp$this.getTagSpan(tagsSource.filter(function(i) {
      return i.id === id.trim();
    }), id);
  });
  return tagSpans.length ? '\x3cdiv style\x3d"width: 100%;display: flex"\x3e' + tagSpans.join('') + '\x3c/div\x3e' : '';
}, getTagSpan:function(match, id) {
  match = match === undefined ? [] : match;
  return '\x3cdiv style\x3d"' + documents.Support.getTagBoxStyle(false, match.length ? match[0].color : undefined) + ' margin-right: 0.3em;"\x3e ' + (match.length ? match[0].name : id) + '\x3c/div\x3e';
}}});
Ext.define('PdfViewer.view.combo.scaleCombo', {extend:'Ext.form.field.ComboBox', xtype:'PdfViewerScaleCombo', requires:['Ext.data.ArrayStore'], editable:false, keyNavEnabled:true, selectOnFocus:false, submitValue:false, isFormField:false, autoSelect:true, store:new Ext.data.SimpleStore({id:0, fields:['scale', 'text'], data:[[0.5, '50%'], [0.75, '75%'], [1, '100%'], [1.25, '125%'], [1.5, '150%'], [2, '200%']]}), valueField:'scale', displayField:'text', mode:'local', listeners:{change:function(cb, 
newVal) {
  var record = cb.store.findRecord('scale', newVal);
  if (record === null) {
    cb.store.add({scale:newVal, text:Math.round(newVal * 100).toString() + '%'});
  }
}}});
Ext.define('PdfViewer.view.field.pageNumber', {extend:'Ext.form.field.Number', xtype:'pdfviewer_pagenumber', allowDecimals:false, minValue:1, hideTrigger:true, enableKeyEvents:true, keyNavEnabled:false, selectOnFocus:true, submitValue:false, isFormField:false});
Ext.define('PdfViewer.view.panel.PDFController', {extend:'Ext.app.ViewController', alias:'controller.PdfViewerController', requires:[], container:null, pageScale:null, zoomFactor:null, minScale:null, maxScale:null, control:{'#':{boxready:'onBoxReady', onSetSrc:'onSetSrc', onUnset:'onUnset', onGetDocument:'onGetDocument', afterrender:'onAfterrender'}}, init:function(view) {
  var me = this;
  me.zoomFactor = view.zoomFactor;
  me.minScale = view.minScale;
  me.maxScale = view.maxScale;
  me.pageScale = view.pageScale;
  PDFJS.workerSrc = documents.Support.resourcesBasePath + 'resources/worker/pdf.worker.js';
}, onSetSrc:function(src) {
  var view = this.getView();
  view.src = src;
  view.currentPage = 1;
  this.showLoaderMask(view);
  this.getDocument(src);
}, onUnset:function() {
  if (this.container !== null) {
    this.container.innerHTML = '';
    this.setToolbarProperties(0, true);
  }
}, onGetDocument:function() {
  this.getDocument();
}, onBoxReady:function() {
  var view = this.getView();
  if (view.showLoadMaskOnInit) {
    this.showLoaderMask(view);
  }
}, onAfterrender:function(view) {
  var me = this;
  me.container = view.getPdfEl();
  me.container.mozOpaque = true;
  if (!!view.src) {
    me.getDocument(view.src);
  }
}, getDocument:function(src) {
  var me = this;
  var view = me.getView();
  PDFJS.getDocument(src).then(function(pdfDoc) {
    view.pdfDoc = pdfDoc;
    me.renderDocument();
  });
}, renderDocument:function() {
  try {
    var view = this.getView(), isEmpty;
    isEmpty = view.pdfDoc.numPages === 0;
    view.currentPage = view.currentPage || (isEmpty ? 0 : 1);
    this.renderPage(view.currentPage);
  } catch (error) {
    Ext.log({level:'warning'}, "PDF: Can't render: " + error.message);
  }
}, renderPage:function(page) {
  var me = this, view = me.getView(), isEmpty, pageCount, currPage, afterText;
  if (view.isRendering) {
    return;
  }
  view.isRendering = true;
  view.currentPage = page;
  currPage = view.currentPage;
  pageCount = view.pdfDoc.numPages;
  Ext.suspendLayouts();
  this.setToolbarProperties(currPage);
  this.container.innerHTML = '';
  var numPages = view.showPerPage ? page : pageCount;
  var idx = view.showPerPage ? page : 1;
  for (var i = idx; i <= numPages; i++) {
    view.pdfDoc.getPage(i).then(function(page) {
      var viewport = page.getViewport(view.pageScale);
      var div = document.createElement('div');
      var id = view.getId() + '-page-' + (page.pageIndex + 1);
      div.setAttribute('id', id);
      div.setAttribute('style', 'position: relative');
      me.container.appendChild(div);
      var canvas = document.createElement('canvas');
      div.appendChild(canvas);
      var context = canvas.getContext('2d');
      canvas.height = viewport.height;
      canvas.width = viewport.width;
      context.save();
      context.fillStyle = 'rgb(255, 255, 255)';
      context.fillRect(0, 0, canvas.width, canvas.height);
      context.restore();
      var renderContext = {canvasContext:context, viewport:viewport};
      page.render(renderContext).then(function() {
        if (view.disableTextLayer === false) {
          return page.getTextContent();
        }
      }).then(function(textContent) {
        if (textContent) {
          var textLayerDiv = document.createElement('div');
          textLayerDiv.setAttribute('class', 'textLayer');
          div.appendChild(textLayerDiv);
          PDFJS.renderTextLayer({textContent:textContent, container:textLayerDiv, viewport:viewport, pageIndex:page.pageIndex, textDivs:[]});
        }
      });
      Ext.resumeLayouts(true);
      view.isRendering = false;
      if (view.loader) {
        view.loader.destroy();
      }
      if (view.rendered) {
        me.updateBorder(view.body, id, 'bordered', 'add');
        view.scrollTo(0, 0);
        view.fireEvent('change', view, {current:view.currentPage, total:view.pdfDoc.numPages});
      }
    });
  }
}, moveFirst:function() {
  var view = this.getView();
  if (view.fireEvent('beforechange', view, 1) !== false) {
    this.renderThis(1);
  }
}, movePrevious:function() {
  var view = this.getView(), prev = view.currentPage - 1;
  if (prev > 0) {
    if (view.fireEvent('beforechange', view, prev) !== false) {
      this.renderThis(prev);
    }
  }
}, moveNext:function() {
  var view = this.getView(), total = view.pdfDoc.numPages, next = view.currentPage + 1;
  if (next <= total) {
    if (view.fireEvent('beforechange', view, next) !== false) {
      this.renderThis(next);
    }
  }
}, moveLast:function() {
  var me = this.getView(), last = me.pdfDoc.numPages;
  if (me.fireEvent('beforechange', me, last) !== false) {
    this.renderThis(last);
  }
}, onPagingBlur:function(e) {
  var curPage = this.getView().currentPage;
  this.lookup('inputItem').setValue(curPage);
}, onPagingKeyDown:function(field, e) {
  var me = this.getView(), k = e.getKey(), increment = e.shiftKey ? 10 : 1, pageNum, total = me.pdfDoc.numPages;
  if (k === e.RETURN) {
    e.stopEvent();
    pageNum = this.readPageFromInput();
    if (pageNum !== false) {
      pageNum = Math.min(Math.max(1, pageNum), total);
      if (me.fireEvent('beforechange', me, pageNum) !== false) {
        this.renderThis(pageNum);
      }
    }
  } else {
    if (Ext.Array.indexOf([e.HOME, e.END], k) > 0) {
      e.stopEvent();
      pageNum = k == e.HOME ? 1 : total;
      field.setValue(pageNum);
    } else {
      if (Ext.Array.indexOf([e.UP, e.PAGE_UP, e.DOWN, e.PAGE_DOWN], k) > 0) {
        e.stopEvent();
        pageNum = this.readPageFromInput();
        if (pageNum) {
          if (k === e.DOWN || k === e.PAGE_DOWN) {
            increment *= -1;
          }
          pageNum += increment;
          if (pageNum >= 1 && pageNum <= total) {
            field.setValue(pageNum);
          }
        }
      }
    }
  }
}, onPagingFocus:function() {
  this.lookup('inputItem').select();
}, onScaleBlur:function(e) {
  var me = this.getView();
  this.lookup('scaleCombo').setValue(me.pageScale);
}, onBtnZoomInClicked:function(b) {
  var me = this;
  me.pageScale += me.zoomFactor;
  if (me.pageScale > me.maxScale) {
    me.pageScale = me.maxScale;
  }
  this.lookup('scaleCombo').setValue(me.pageScale);
}, onBtnZoomOutClicked:function(b) {
  var me = this;
  me.pageScale -= me.zoomFactor;
  if (me.pageScale < me.minScale) {
    me.pageScale = me.minScale;
  }
  this.lookup('scaleCombo').setValue(me.pageScale);
}, onScaleChange:function(combo, newValue) {
  var me = this;
  var view = me.getView();
  view.pageScale = me.pageScale = newValue;
  me.renderPage(view.currentPage);
}, readPageFromInput:function() {
  var view = this.getView(), v = this.lookup('inputItem').getValue(), pageNum = parseInt(v, 10);
  if (!v || isNaN(pageNum)) {
    this.lookup('inputItem').setValue(view.currentPage);
    return false;
  }
  return pageNum;
}, privates:{setToolbarProperties:function(page, reset) {
  var me = this, view = me.getView(), currPage = page, pageCount = view.pdfDoc.numPages, isEmpty = pageCount === 0, afterText = Ext.String.format(view.afterPageText, isNaN(pageCount) ? 1 : pageCount);
  if (typeof reset !== 'undefined') {
    isEmpty = true;
  }
  view.currentPage = isEmpty ? '' : currPage;
  me.lookup('afterTextItem').setText(afterText);
  me.lookup('inputItem').setDisabled(isEmpty).setValue(currPage);
  me.lookup('first').setDisabled(currPage === 1 || isEmpty);
  me.lookup('prev').setDisabled(currPage === 1 || isEmpty);
  me.lookup('next').setDisabled(currPage === pageCount || isEmpty);
  me.lookup('last').setDisabled(currPage === pageCount || isEmpty);
  me.lookup('scaleCombo').setDisabled(isEmpty).setValue(view.pageScale);
}, renderThis:function(page) {
  var me = this;
  var view = me.getView();
  if (view.showPerPage) {
    me.renderPage(page);
  } else {
    me.setToolbarProperties(page);
    var el = me.container.childNodes.item(page - 1);
    view.scrollTo(0, el.offsetTop, true);
  }
}, showLoaderMask:function(view) {
  view.loader = new Ext.LoadMask({msg:view.loadingMessage, target:view});
  view.loader.show();
}, updateBorder:function(el, id, addCls, action) {
  var elem = el.getById(id);
  var canvas = elem.dom.firstChild;
  var hasClass = canvas.hasAttribute('class');
  if (action === 'add') {
    if (hasClass) {
      canvas.className += ' ' + addCls;
    } else {
      canvas.setAttribute('class', addCls);
    }
  } else {
    if (hasClass) {
      canvas.className = canvas.className.replace(new RegExp('(?:^|s)' + addCls + '(?!S)'), '');
      if (canvas.className.length === 0) {
        canvas.removeAttribute('class');
      }
    }
  }
}}});
Ext.define('PdfViewer.view.panel.PDF', {extend:'Ext.panel.Panel', controller:'PdfViewerController', xtype:'PdfViewerPdfPanel', requires:['PdfViewer.view.combo.scaleCombo', 'PdfViewer.view.field.pageNumber', 'PdfViewer.view.panel.PDFController'], extraBaseCls:Ext.baseCSSPrefix + 'pdf', extraBodyCls:Ext.baseCSSPrefix + 'pdf-body', autoScroll:true, disableTextLayer:false, showPerPage:false, pageScale:1, pageBorders:true, src:'', disableWorker:false, loadingMessage:'Loading PDF, please wait...', beforePageText:'Page', 
afterPageText:'of {0}', firstText:'First Page', prevText:'Previous Page', nextText:'Next Page', lastText:'Last Page', inputItemWidth:60, scaleWidth:100, showLoadMaskOnInit:null, zoomFactor:0.1, minScale:0.5, maxScale:4, initComponent:function() {
  var me = this, dockedItems;
  dockedItems = [{dock:'bottom', reference:'PagingToolbar', xtype:'toolbar', items:[{reference:'first', tooltip:me.firstText, overflowText:me.firstText, iconCls:'ext ext-double-chevron-left', disabled:true, handler:'moveFirst'}, {reference:'prev', tooltip:me.prevText, overflowText:me.prevText, iconCls:'ext ext-chevron-left', disabled:true, handler:'movePrevious'}, '-', me.beforePageText, {xtype:'pdfviewer_pagenumber', reference:'inputItem', name:'inputItem', cls:Ext.baseCSSPrefix + 'tbar-page-number', 
  width:me.inputItemWidth, margins:'-1 2 3 2', disabled:true, listeners:{keydown:'onPagingKeyDown', blur:'onPagingBlur'}}, {xtype:'tbtext', reference:'afterTextItem', text:Ext.String.format(me.afterPageText, 1), margins:'0 5 0 0'}, '-', {reference:'next', tooltip:me.nextText, overflowText:me.nextText, iconCls:'ext ext-chevron-right', disabled:true, handler:'moveNext'}, {reference:'last', tooltip:me.lastText, overflowText:me.lastText, iconCls:'ext ext-double-chevron-right', disabled:true, handler:'moveLast'}, 
  '-\x3e', {xtype:'button', iconCls:documents.Support.icon('fa-search-plus'), tooltip:'Zoom in', handler:'onBtnZoomInClicked'}, {reference:'scaleCombo', xtype:'PdfViewerScaleCombo', disabled:true, width:me.scaleWidth, listeners:{change:'onScaleChange', blur:'onScaleBlur'}}, {xtype:'button', tooltip:'Zoom out', iconCls:documents.Support.icon('fa-search-minus'), handler:'onBtnZoomOutClicked'}]}];
  if (typeof me.dockedItems !== 'undefined') {
    dockedItems = dockedItems.concat(me.dockedItems);
  }
  Ext.apply(me, {html:'\x3cdiv class\x3d"canvasWrapper"\x3e\x3c/div\x3e', dockedItems:dockedItems});
  me.bodyCls = me.bodyCls || '';
  me.bodyCls += ' ' + me.extraBodyCls;
  me.cls = me.cls || '';
  me.cls += ' ' + me.extraBaseCls;
  me.callParent(arguments);
  if (me.disableWorker) {
    PDFJS.disableWorker = true;
  }
}, getPdfEl:function() {
  return this.el.dom.getElementsByClassName('canvasWrapper')[0];
}, setSrc:function(src) {
  this.fireEvent('onSetSrc', src);
}, unset:function() {
  this.fireEvent('onUnset');
}});
Ext.define('documents.types.Zip', {extend:'documents.components.type.FileBase', alias:'widget.docsZip', layout:'fit', initComponent:function() {
  var $jscomp$this = this;
  this.grid = Ext.create({xtype:'grid', store:Ext.create('Ext.data.Store', {data:[]}), listeners:{rowdblclick:function(v, r) {
    $jscomp$this.openFile(r);
  }}, columns:[{dataIndex:'dir', width:35, text:' ', renderer:function(v) {
    return Ext.isDefined(v) ? documents.Support.getSpanForIconCls(v === true ? 'fa-folder' : 'fa-file', 'black') : '';
  }}, {dataIndex:'name', flex:1}, {dataIndex:'comment', flex:1, hidden:true}, {dataIndex:'date', xtype:'datecolumn', format:'d.m.Y H:i:s', width:200}, {dataIndex:'dosPermissions', hidden:true}, {dataIndex:'unixPermissions', hidden:true}, {xtype:'actioncolumn', width:60, text:' ', items:[{iconCls:documents.Support.icon('fa-folder-open'), tooltip:documents.Support.TR('OPEN'), isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return rec.get('dir');
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.openFile(r);
  }}, {iconCls:documents.Support.icon('fa-save'), tooltip:documents.Support.TR('STORE_AS_DOCUMENT'), isActionDisabled:function(view, rowIndex, colIndex, item, rec) {
    return rec.get('dir');
  }, handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.createDocumentFromAttachmentRecord(r);
  }}]}].map(function(c) {
    c.text = c.text || documents.Support.TR('FILE_' + c.dataIndex.toUpperCase());
    c.menuDisabled = true;
    return c;
  })});
  this.items = [this.grid];
  this.callParent(arguments);
  this.handleDocumentModuleId();
  this.loadFile();
}, getBase64FromZippedRecord:function(r) {
  if (!r.get('dir')) {
    var z = r.get('_sourceZip');
    return z.file(r.get('name')).async('base64');
  }
  return Promise.resolve(false);
}, openFile:function(r) {
  var $jscomp$this = this;
  var fileName = (r.get('name') || '').trim();
  var mime = this.getMimeFromFileName(fileName);
  if (mime) {
    this.getBase64FromZippedRecord(r).then(function(base64) {
      if (base64) {
        var fileViewer = documents.Support.getViewerComponentByDocument($jscomp$this.getMinimalDocObject(mime, base64));
        documents.Utils.openFileWindow(fileViewer, fileName);
      } else {
        Ext.toast('TODO ... Dir');
      }
    });
  }
}, getMimeFromFileName:function(str) {
  str = str === undefined ? '' : str;
  var fileName = (str || '').trim();
  var parts = fileName.split('.');
  if (parts.length > 1) {
    var ending = parts[parts.length - 1];
    return documents.Mime.map[ending];
  }
  return BpcCommon.Api.getSetting('documents', 'mime_fallback');
}, createDocumentFromAttachmentRecord:function(r) {
  var $jscomp$this = this;
  var mimeType = this.getMimeFromFileName(r.get('name'));
  if (mimeType) {
    this.getBase64FromZippedRecord(r).then(function(base64) {
      documents.Api.createDocument({instanceId:$jscomp$this.documentModuleId}, {filename:mimeType, description:'extracted from zip', documentTitle:r.get('name'), mime:mimeType, base64string:base64});
    });
  }
}, zipToStoreArray:function(zip) {
  if (zip && zip.files) {
    return Object.keys(zip.files).map(function(name) {
      var file = zip.files[name];
      file._sourceZip = zip;
      return file;
    });
  }
}, loadFile:function() {
  var $jscomp$this = this;
  if (window.JSZip) {
    var zip = new JSZip;
    zip.loadAsync(this.documentObject.record.get('base64string'), {base64:true}).then(function(zipParsed) {
      var s = $jscomp$this.zipToStoreArray(zipParsed) || [];
      $jscomp$this.grid.store.add(s);
    });
  } else {
  }
}});
Ext.define('documents.moduleConfigurationComponents.Deployment.DeployWindow', {extend:'Ext.window.Window', alias:'widget.docsDeployWindow', title:documents.Support.TR('DEPLOY_DIALOG'), layout:'fit', constrain:true, initComponent:function() {
  this.height = document.body.clientHeight * 0.7;
  this.width = document.body.clientWidth * 0.4;
  this.updateContainerCmp = Ext.create({xtype:'container', flex:1, html:this.getUpdateHtml('fa-file')});
  this.closeButton = Ext.create({xtype:'button', disabled:true, text:documents.Support.TR('CLOSE'), iconCls:documents.Support.icon('fa-times'), handler:function(b) {
    b.up('window').close();
  }});
  this.progressButton = Ext.create({xtype:'button', iconCls:documents.Support.icon('fa-info')});
  this.itemButtom = Ext.create({xtype:'button'});
  this.successGrid = Ext.create({xtype:'grid', flex:2, store:Ext.create('Ext.data.Store', {data:[]}), columns:[{dataIndex:'success', width:35, renderer:function(v) {
    return BpcCommon.util.Icons.getSpanForIconCls(v ? 'fa-check-circle' : 'fa-warning', v ? 'green' : 'orange');
  }}, {dataIndex:'status', width:100}, {dataIndex:'duration', width:100}, {dataIndex:'id', flex:1}, {dataIndex:'documentTitle', flex:1}]});
  var deployPanel = Ext.create({xtype:'panel', layout:{type:'vbox', align:'stretch'}, items:[this.updateContainerCmp, this.successGrid], bbar:[this.progressButton, {xtype:'tbfill'}, this.itemButtom, this.closeButton]});
  this.items = [deployPanel];
  this.callParent(arguments);
  if (this.mode === 'instanceData') {
    this.doDeployData();
  }
}, doDeployData:function() {
  var $jscomp$async$this = this;
  var $jscomp$destructuring$var29;
  var documentIds;
  var tags;
  var index;
  var success, rStatus, id, start, end;
  var json;
  var $jscomp$destructuring$var30;
  var rStatus;
  var e;
  var index$14;
  var id;
  var json$15;
  var $jscomp$destructuring$var31;
  var tStatus;
  var e$16;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    switch($jscomp$generator$context.nextAddress) {
      case 1:
        $jscomp$destructuring$var29 = $jscomp$async$this.deployMain.getSelectionDocumentsAndTags();
        documentIds = $jscomp$destructuring$var29.documentIds;
        tags = $jscomp$destructuring$var29.tags;
        $jscomp$async$this.updateContainer('fa-cog fa-spin', true);
        $jscomp$async$this.updateProgress(documents.Support.TR('DEPLOY_DOCS'));
        index = 0;
      case 2:
        if (!(index < documentIds.length)) {
          $jscomp$generator$context.jumpTo(4);
          break;
        }
        success = true, rStatus = -1, id = documentIds[index];
        $jscomp$generator$context.setCatchFinallyBlocks(5);
        $jscomp$async$this.updateItem(id, 'fa-file');
        start = new Date;
        return $jscomp$generator$context.yield($jscomp$async$this.deployMain.deploy(id, false), 7);
      case 7:
        json = $jscomp$generator$context.yieldResult;
        end = new Date;
        $jscomp$destructuring$var30 = json || {};
        rStatus = $jscomp$destructuring$var30.rStatus;
        success = rStatus === 200;
        return $jscomp$generator$context.yield(new Promise(function(res) {
          return setTimeout(res, 300);
        }), 8);
      case 8:
        $jscomp$generator$context.leaveTryBlock(6);
        break;
      case 5:
        e = $jscomp$generator$context.enterCatchBlock();
        success = false;
      case 6:
        $jscomp$async$this.addToGrid(id, {success:success, status:rStatus, duration:Ext.Date.diff(start, end, 's'), type:'doc'});
        index++;
        $jscomp$generator$context.jumpTo(2);
        break;
      case 4:
        $jscomp$async$this.updateProgress(documents.Support.TR('DEPLOY_TAGS'));
        index$14 = 0;
      case 9:
        if (!(index$14 < tags.length)) {
          $jscomp$generator$context.jumpTo(11);
          break;
        }
        id = tags[index$14];
        $jscomp$generator$context.setCatchFinallyBlocks(12);
        $jscomp$async$this.updateItem(id, 'fa-tag');
        return $jscomp$generator$context.yield($jscomp$async$this.deployMain.deploy(id, false, 'tag'), 14);
      case 14:
        json$15 = $jscomp$generator$context.yieldResult;
        $jscomp$destructuring$var31 = json$15 || {};
        tStatus = $jscomp$destructuring$var31.tStatus;
        return $jscomp$generator$context.yield(new Promise(function(res) {
          return setTimeout(res, 50);
        }), 15);
      case 15:
        $jscomp$generator$context.leaveTryBlock(10);
        break;
      case 12:
        e$16 = $jscomp$generator$context.enterCatchBlock();
      case 10:
        index$14++;
        $jscomp$generator$context.jumpTo(9);
        break;
      case 11:
        $jscomp$async$this.updateProgress('');
        $jscomp$async$this.updateItem(documents.Support.TR('DEPLOY_FINISHED'));
        $jscomp$async$this.updateContainer('fa-check-circle', undefined, 'aquamarine');
        $jscomp$async$this.deployMain.targetGrid.store.load();
        $jscomp$async$this.closeButton.setDisabled(false);
        $jscomp$generator$context.jumpToEnd();
    }
  });
}, doDeploy:function() {
  var $jscomp$async$this = this;
  var $jscomp$destructuring$var32;
  var documentIds;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$generator$context.nextAddress == 1) {
      $jscomp$destructuring$var32 = $jscomp$async$this.deployMain.getSelectionDocumentsAndTags();
      documentIds = $jscomp$destructuring$var32.documentIds;
      $jscomp$async$this.updateContainer('fa-cog fa-spin', true);
      $jscomp$async$this.updateProgress(documents.Support.TR('MOVE_DB_DOCS'));
      documentIds = documentIds || [];
      return $jscomp$generator$context.yield($jscomp$async$this.deployMain.doDeploy(documentIds, $jscomp$async$this, $jscomp$async$this.mode), 2);
    }
    $jscomp$async$this.updateProgress('');
    $jscomp$async$this.updateItem(documents.Support.TR('MOVE_DB_FINISHED'));
    $jscomp$async$this.onDeployFinish();
    $jscomp$async$this.closeButton.setDisabled(false);
    $jscomp$generator$context.jumpToEnd();
  });
}, onDeployFinish:function() {
  this.updateContainer('fa-check-circle', undefined, 'aquamarine');
}, getUpdateHtml:function(iconCls, spin, bgcolor) {
  spin = spin === undefined ? false : spin;
  bgcolor = bgcolor === undefined ? 'gainsboro' : bgcolor;
  return '\x3cdiv style\x3d"width: 100%;height:100%; display: flex; flex-direction: column; background-color: ' + bgcolor + '; justify-items: center; align-items: center; justify-content: center;"\x3e\x3cdiv\x3e' + documents.Support.getSpanForIconCls(iconCls, 'black', 48, spin) + '\x3c/div\x3e\x3c/div\x3e';
}, updateProgress:function(text) {
  text = text === undefined ? '' : text;
  this.progressButton.setText(text);
}, addToGrid:function(id, data) {
  data = data === undefined ? {} : data;
  this.successGrid.store.add(Ext.apply({}, {id:id}, data));
}, updateItem:function(item, iconCls) {
  item = item === undefined ? '' : item;
  this.itemButtom.setIconCls(iconCls ? documents.Support.icon(iconCls) : null);
  this.itemButtom.setText(item);
}, updateContainer:function(iconCls, spin, bgcolor) {
  this.updateContainerCmp.setHtml(this.getUpdateHtml(iconCls, spin, bgcolor));
}});
Ext.define('documents.view.admin.webhooks', {extend:'Ext.grid.Panel', alias:'widget.documentsAdminWebhooks', initComponent:function() {
  var $jscomp$this = this;
  this.iconCls = documents.Support.icon('fa-siren');
  this.title = documents.Support.TR('WEBHOOKS');
  this.totalBox = Ext.create({xtype:'box', setTitle:function(count) {
    count = count === undefined ? 0 : count;
    this.update('\x26nbsp;\x26nbsp; ' + documents.Support.TR('TXT_TOTAL') + ': ' + count);
  }, html:''});
  var url = documents.Support.basePath + 'hooks?' + Ext.Object.toQueryString({instanceId:this.moduleId});
  this.store = Ext.create('Ext.data.Store', {proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'data', totalProperty:'total'}}, autoLoad:false, listeners:{load:function(s, records, success, operation, eOpts) {
    if ($jscomp$this.totalBox) {
      $jscomp$this.totalBox.setTitle(success ? s.getTotalCount() : '0');
    }
  }}});
  this.listeners = {rowDblClick:function(tableView, record, index, eopts) {
    return $jscomp$this.openHookRecord(record);
  }};
  this.columns = [{dataIndex:'responseCode', width:50}, {dataIndex:'artefactId', hidden:false, width:200}, {dataIndex:'instanceId', width:150}, {dataIndex:'hookId', width:100}, {dataIndex:'request', flex:1, renderer:function(v) {
    return JSON.stringify(v || {}, undefined, 4);
  }}, {dataIndex:'responseBody', flex:1}, {dataIndex:'created', width:150, renderer:function(v) {
    return documents.Support.dateFormatter(v, 'd.m.Y H:i');
  }}, {dataIndex:'creator'}].map(function(i) {
    i.text = documents.Support.TR('HOOKS_' + i.dataIndex);
    return i;
  });
  this.bbar = [this.totalBox, {xtype:'tbfill'}, {iconCls:documents.Support.icon('fa-refresh'), handler:function() {
    $jscomp$this.store.load();
  }}];
  this.callParent(arguments);
  this.store.load();
}, openHookRecord:function(record) {
  var recordData = record.data || {};
  var $jscomp$destructuring$var33 = recordData;
  var responseCode = $jscomp$destructuring$var33.responseCode;
  var artefactId = $jscomp$destructuring$var33.artefactId;
  var hookId = $jscomp$destructuring$var33.hookId;
  var request = $jscomp$destructuring$var33.request;
  var responseBody = $jscomp$destructuring$var33.responseBody;
  var created = $jscomp$destructuring$var33.created;
  var creator = $jscomp$destructuring$var33.creator;
  var items = [{xtype:'form', layout:'anchor', defaults:{anchor:'100%'}, scrollable:'y', items:[{xtype:'textfield', value:responseCode, name:'responseCode'}, {xtype:'textfield', value:artefactId, name:'artefactId'}, {xtype:'textfield', value:hookId, name:'hookId'}, {xtype:'textarea', value:JSON.stringify(request || {}, undefined, 4), name:'request'}, {xtype:'textarea', value:responseBody, name:'responseBody'}, {xtype:'textfield', value:created, name:'created'}, {xtype:'textfield', value:creator, 
  name:'creator'}].map(function(f) {
    f.labelAlign = 'top';
    f.fieldLabel = documents.Support.TR('HOOKS_' + f.name);
    return f;
  })}];
  Ext.create('Ext.window.Window', {title:documents.Support.TR('HOOKS_REQUEST'), height:document.body.clientHeight * 0.8, width:document.body.clientWidth * 0.5, layout:'fit', padding:'10 10 10 10', items:items, bbar:[{xtype:'tbfill'}, {xtype:'button', ui:'secondary', text:documents.Support.TR('CLOSE'), handler:function(b) {
    var w = b.up('window');
    if (w) {
      w.close();
    }
  }}]}).show();
}});
Ext.define('documents.parser.answerDecoder', {constructor:function(opts) {
  opts = opts || {};
  this.decoder = opts.decoder || new TextDecoder;
  this.maxChunkSize = 100 * 1024;
  this.remainder = '';
  this.chunks = [];
  this.callParent();
}, decodeQPBytes:function(encodedBytes) {
  var buf = new ArrayBuffer(encodedBytes.length);
  var dataView = new DataView(buf);
  for (var i$17 = 0, len = encodedBytes.length; i$17 < len; i$17++) {
    dataView.setUint8(i$17, parseInt(encodedBytes[i$17], 16));
  }
  return buf;
}, decodeChunks:function(str) {
  str = str.replace(/=\r?\n/g, '');
  var list = str.split(/(?==)/);
  var encodedBytes = [];
  for (var $jscomp$iter$0 = $jscomp.makeIterator(list), $jscomp$key$part = $jscomp$iter$0.next(); !$jscomp$key$part.done; $jscomp$key$part = $jscomp$iter$0.next()) {
    var part = $jscomp$key$part.value;
    {
      if (part.charAt(0) !== '\x3d') {
        if (encodedBytes.length) {
          this.chunks.push(this.decodeQPBytes(encodedBytes));
          encodedBytes = [];
        }
        this.chunks.push(part);
        continue;
      }
      if (part.length === 3) {
        encodedBytes.push(part.substr(1));
        continue;
      }
      if (part.length > 3) {
        encodedBytes.push(part.substr(1, 2));
        this.chunks.push(this.decodeQPBytes(encodedBytes));
        encodedBytes = [];
        part = part.substr(3);
        this.chunks.push(part);
      }
    }
  }
  if (encodedBytes.length) {
    this.chunks.push(this.decodeQPBytes(encodedBytes));
  }
}, update:function(buffer) {
  var str = this.decoder.decode(buffer) + '\n';
  str = this.remainder + str;
  if (str.length < this.maxChunkSize) {
    this.remainder = str;
    return;
  }
  this.remainder = '';
  var partialEnding = str.match(/=[a-fA-F0-9]?$/);
  if (partialEnding) {
    if (partialEnding.index === 0) {
      this.remainder = str;
      return;
    }
    this.remainder = str.substr(partialEnding.index);
    str = str.substr(0, partialEnding.index);
  }
  this.decodeChunks(str);
}, finalize:function() {
  var $jscomp$async$this = this;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$async$this.remainder.length) {
      $jscomp$async$this.decodeChunks($jscomp$async$this.remainder);
      $jscomp$async$this.remainder = '';
    }
    return $jscomp$generator$context['return'](documents.parser.util.blobToArrayBuffer(new Blob($jscomp$async$this.chunks, {type:'application/octet-stream'})));
  });
}});
Ext.define('documents.parser.base64Decoder', {constructor:function(opts) {
  opts = opts || {};
  this.decoder = opts.decoder || new TextDecoder;
  this.maxChunkSize = 100 * 1024;
  this.chunks = [];
  this.remainder = '';
  this.callParent();
}, update:function(buffer) {
  var str = this.decoder.decode(buffer);
  if (/[^a-zA-Z0-9+\/]/.test(str)) {
    str = str.replace(/[^a-zA-Z0-9+\/]+/g, '');
  }
  this.remainder += str;
  if (this.remainder.length >= this.maxChunkSize) {
    var allowedBytes = Math.floor(this.remainder.length / 4) * 4;
    var base64Str;
    if (allowedBytes === this.remainder.length) {
      base64Str = this.remainder;
      this.remainder = '';
    } else {
      base64Str = this.remainder.substr(0, allowedBytes);
      this.remainder = this.remainder.substr(allowedBytes);
    }
    if (base64Str.length) {
      this.chunks.push(documents.parser.util.decodeBase64(base64Str));
    }
  }
}, finalize:function() {
  var $jscomp$async$this = this;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$async$this.remainder && !/^=+$/.test($jscomp$async$this.remainder)) {
      $jscomp$async$this.chunks.push(documents.parser.util.decodeBase64($jscomp$async$this.remainder));
    }
    return $jscomp$generator$context['return'](documents.parser.util.blobToArrayBuffer(new Blob($jscomp$async$this.chunks, {type:'application/octet-stream'})));
  });
}});
Ext.define('documents.parser.chunkDecoder', {constructor:function(opts) {
  this.chunks = [];
  this.callParent(arguments);
}, update:function(line) {
  this.chunks.push(line);
  this.chunks.push('\n');
}, finalize:function() {
  var $jscomp$async$this = this;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    return $jscomp$generator$context['return'](documents.parser.util.blobToArrayBuffer(new Blob($jscomp$async$this.chunks, {type:'application/octet-stream'})));
  });
}});
Ext.define('documents.parser.util', {singleton:true, decoders:undefined, getDecoder:function(charset) {
  if (!this.decoders) {
    this.decoders = new Map;
  }
  var decoders = this.decoders;
  charset = charset || 'utf8';
  if (decoders.has(charset)) {
    return decoders.get(charset);
  }
  var decoder;
  try {
    decoder = new TextDecoder(charset);
  } catch (err) {
    if (charset === 'utf8') {
      throw err;
    }
    return this.getDecoder();
  }
  decoders.set(charset, decoder);
  return decoder;
}, decodeURIComponentWithCharset:function(encodedStr, charset) {
  charset = charset || 'utf-8';
  var encodedBytes = [];
  for (var i$18 = 0; i$18 < encodedStr.length; i$18++) {
    var c = encodedStr.charAt(i$18);
    if (c === '%' && /^[a-f0-9]{2}/i.test(encodedStr.substr(i$18 + 1, 2))) {
      var byte = encodedStr.substr(i$18 + 1, 2);
      i$18 += 2;
      encodedBytes.push(parseInt(byte, 16));
    } else {
      if (c.charCodeAt(0) > 126) {
        c = textEncoder.encode(c);
        for (var j = 0; j < c.length; j++) {
          encodedBytes.push(c[j]);
        }
      } else {
        encodedBytes.push(c.charCodeAt(0));
      }
    }
  }
  var byteStr = new ArrayBuffer(encodedBytes.length);
  var dataView = new DataView(byteStr);
  for (var i$19 = 0, len = encodedBytes.length; i$19 < len; i$19++) {
    dataView.setUint8(i$19, encodedBytes[i$19]);
  }
  return this.getDecoder(charset).decode(byteStr);
}, decodeParameterValueContinuations:function(header) {
  var $jscomp$this = this;
  var paramKeys = new Map;
  Object.keys(header.params).forEach(function(key) {
    var match = key.match(/\*((\d+)\*?)?$/);
    if (!match) {
      return;
    }
    var actualKey = key.substr(0, match.index).toLowerCase();
    var nr = Number(match[2]) || 0;
    var paramVal;
    if (!paramKeys.has(actualKey)) {
      paramVal = {charset:false, values:[]};
      paramKeys.set(actualKey, paramVal);
    } else {
      paramVal = paramKeys.get(actualKey);
    }
    var value = header.params[key];
    if (nr === 0 && match[0].charAt(match[0].length - 1) === '*' && (match = value.match(/^([^']*)'[^']*'(.*)$/))) {
      paramVal.charset = match[1] || 'utf-8';
      value = match[2];
    }
    paramVal.values.push({nr:nr, value:value});
    delete header.params[key];
  });
  paramKeys.forEach(function(paramVal, key) {
    header.params[key] = $jscomp$this.decodeURIComponentWithCharset(paramVal.values.sort(function(a, b) {
      return a.nr - b.nr;
    }).map(function(a) {
      return a.value;
    }).join(''), paramVal.charset);
  });
}, _handleAddress:function(tokens) {
  var token;
  var isGroup = false;
  var state = 'text';
  var address;
  var addresses = [];
  var data = {address:[], comment:[], group:[], text:[]};
  var i;
  var len;
  for (i = 0, len = tokens.length; i < len; i++) {
    token = tokens[i];
    if (token.type === 'operator') {
      switch(token.value) {
        case '\x3c':
          state = 'address';
          break;
        case '(':
          state = 'comment';
          break;
        case ':':
          state = 'group';
          isGroup = true;
          break;
        default:
          state = 'text';
      }
    } else {
      if (token.value) {
        if (state === 'address') {
          token.value = token.value.replace(/^[^<]*<\s*/, '');
        }
        data[state].push(token.value);
      }
    }
  }
  if (!data.text.length && data.comment.length) {
    data.text = data.comment;
    data.comment = [];
  }
  if (isGroup) {
    data.text = data.text.join(' ');
    addresses.push({name:this.decodeWords(data.text || address && address.name), group:data.group.length ? this.addressParser(data.group.join(',')) : []});
  } else {
    if (!data.address.length && data.text.length) {
      for (i = data.text.length - 1; i >= 0; i--) {
        if (data.text[i].match(/^[^@\s]+@[^@\s]+$/)) {
          data.address = data.text.splice(i, 1);
          break;
        }
      }
      var _regexHandler = function(address) {
        if (!data.address.length) {
          data.address = [address.trim()];
          return ' ';
        } else {
          return address;
        }
      };
      if (!data.address.length) {
        for (i = data.text.length - 1; i >= 0; i--) {
          data.text[i] = data.text[i].replace(/\s*\b[^@\s]+@[^\s]+\b\s*/, _regexHandler).trim();
          if (data.address.length) {
            break;
          }
        }
      }
    }
    if (!data.text.length && data.comment.length) {
      data.text = data.comment;
      data.comment = [];
    }
    if (data.address.length > 1) {
      data.text = data.text.concat(data.address.splice(1));
    }
    data.text = data.text.join(' ');
    data.address = data.address.join(' ');
    if (!data.address && isGroup) {
      return [];
    } else {
      address = {address:data.address || data.text || '', name:this.decodeWords(data.text || data.address || '')};
      if (address.address === address.name) {
        if ((address.address || '').match(/@/)) {
          address.name = '';
        } else {
          address.address = '';
        }
      }
      addresses.push(address);
    }
  }
  return addresses;
}, base64Chars:'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/', decodeBase64:function(base64) {
  var bufferLength = Math.ceil(base64.length / 4) * 3;
  var len = base64.length;
  var base64Lookup = new Uint8Array(256);
  for (var i = 0; i < this.base64Chars.length; i++) {
    base64Lookup[this.base64Chars.charCodeAt(i)] = i;
  }
  var p = 0;
  if (base64.length % 4 === 3) {
    bufferLength--;
  } else {
    if (base64.length % 4 === 2) {
      bufferLength -= 2;
    } else {
      if (base64[base64.length - 1] === '\x3d') {
        bufferLength--;
        if (base64[base64.length - 2] === '\x3d') {
          bufferLength--;
        }
      }
    }
  }
  var arrayBuffer = new ArrayBuffer(bufferLength);
  var bytes = new Uint8Array(arrayBuffer);
  for (var i$20 = 0; i$20 < len; i$20 += 4) {
    var encoded1 = base64Lookup[base64.charCodeAt(i$20)];
    var encoded2 = base64Lookup[base64.charCodeAt(i$20 + 1)];
    var encoded3 = base64Lookup[base64.charCodeAt(i$20 + 2)];
    var encoded4 = base64Lookup[base64.charCodeAt(i$20 + 3)];
    bytes[p++] = encoded1 << 2 | encoded2 >> 4;
    bytes[p++] = (encoded2 & 15) << 4 | encoded3 >> 2;
    bytes[p++] = (encoded3 & 3) << 6 | encoded4 & 63;
  }
  return arrayBuffer;
}, formatTextAddresses:function(addresses) {
  var parts = [];
  var processAddress = function(address, partCounter) {
    if (partCounter) {
      parts.push(', ');
    }
    if (address.group) {
      var groupStart = address.name + ':';
      var groupEnd = ';';
      parts.push(groupStart);
      address.group.forEach(processAddress);
      parts.push(groupEnd);
    } else {
      parts.push(documents.parser.util.formatTextAddress(address));
    }
  };
  addresses.forEach(processAddress);
  return parts.join('');
}, formatTextHeader:function(message) {
  var rows = [];
  if (message.from) {
    rows.push({key:'From', val:documents.parser.util.formatTextAddress(message.from)});
  }
  if (message.subject) {
    rows.push({key:'Subject', val:message.subject});
  }
  if (message.date) {
    var dateOptions = {year:'numeric', month:'numeric', day:'numeric', hour:'numeric', minute:'numeric', second:'numeric', hour12:false};
    var dateStr = typeof Intl === 'undefined' ? message.date : (new Intl.DateTimeFormat('default', dateOptions)).format(new Date(message.date));
    rows.push({key:'Date', val:dateStr});
  }
  if (message.to && message.to.length) {
    rows.push({key:'To', val:documents.parser.util.formatTextAddresses(message.to)});
  }
  if (message.cc && message.cc.length) {
    rows.push({key:'Cc', val:documents.parser.util.formatTextAddresses(message.cc)});
  }
  if (message.bcc && message.bcc.length) {
    rows.push({key:'Bcc', val:documents.parser.util.formatTextAddresses(message.bcc)});
  }
  var maxKeyLength = rows.map(function(r) {
    return r.key.length;
  }).reduce(function(acc, cur) {
    return cur > acc ? cur : acc;
  }, 0);
  rows = rows.flatMap(function(row) {
    var sepLen = maxKeyLength - row.key.length;
    var prefix = row.key + ': ' + ' '.repeat(sepLen);
    var emptyPrefix = ' '.repeat(row.key.length + 1) + ' ' + ' '.repeat(sepLen);
    var foldedLines = foldLines(row.val, 80, true).split(/\r?\n/).map(function(line) {
      return line.trim();
    });
    return foldedLines.map(function(line, i) {
      return '' + (i ? emptyPrefix : prefix) + line;
    });
  });
  console.log(rows);
  console.log(rows.map(function(r) {
    return r.length;
  }));
  var maxLineLength = rows.map(function(r) {
    return r.length;
  }).reduce(function(acc, cur) {
    return cur > acc ? cur : acc;
  }, 0);
  console.log(maxLineLength);
  var lineMarker = '-'.repeat(maxLineLength);
  return '\n' + lineMarker + '\n' + rows.join('\n') + '\n' + lineMarker + '\n';
}, formatTextAddress:function(address) {
  return [].concat(address.name || []).concat(address.name ? '\x3c' + address.address + '\x3e' : address.address).join(' ');
}, getHex:function(c) {
  if (c >= 48 && c <= 57 || c >= 97 && c <= 102 || c >= 65 && c <= 70) {
    return String.fromCharCode(c);
  }
  return false;
}, decodeWord:function(charset, encoding, str) {
  var splitPos = charset.indexOf('*');
  if (splitPos >= 0) {
    charset = charset.substr(0, splitPos);
  }
  encoding = encoding.toUpperCase();
  var byteStr;
  var textEncoder = new TextEncoder;
  if (encoding === 'Q') {
    str = str.replace(/=\s+([0-9a-fA-F])/g, '\x3d$1').replace(/[_\s]/g, ' ');
    var buf = textEncoder.encode(str);
    var encodedBytes = [];
    for (var i$21 = 0, len = buf.length; i$21 < len; i$21++) {
      var c = buf[i$21];
      if (i$21 <= len - 2 && c === 61) {
        var c1 = this.getHex(buf[i$21 + 1]);
        var c2 = this.getHex(buf[i$21 + 2]);
        if (c1 && c2) {
          var c$22 = parseInt(c1 + c2, 16);
          encodedBytes.push(c$22);
          i$21 += 2;
          continue;
        }
      }
      encodedBytes.push(c);
    }
    byteStr = new ArrayBuffer(encodedBytes.length);
    var dataView = new DataView(byteStr);
    for (var i$23 = 0, len = encodedBytes.length; i$23 < len; i$23++) {
      dataView.setUint8(i$23, encodedBytes[i$23]);
    }
  } else {
    if (encoding === 'B') {
      byteStr = this.decodeBase64(str.replace(/[^a-zA-Z0-9\+\/=]+/g, ''));
    } else {
      byteStr = textEncoder.encode(str);
    }
  }
  return documents.parser.util.getDecoder(charset).decode(byteStr);
}, decodeWords:function(str) {
  var $jscomp$this = this;
  return (str || '').toString().replace(/(=\?([^?]+)\?[Bb]\?[^?]*\?=)\s*(?==\?([^?]+)\?[Bb]\?[^?]*\?=)/g, function(match, left, chLeft, chRight) {
    if (chLeft === chRight) {
      return left + '__\x00JOIN\x00__';
    }
    return match;
  }).replace(/(=\?([^?]+)\?[Qq]\?[^?]*\?=)\s*(?==\?([^?]+)\?[Qq]\?[^?]*\?=)/g, function(match, left, chLeft, chRight) {
    if (chLeft === chRight) {
      return left + '__\x00JOIN\x00__';
    }
    return match;
  }).replace(/(\?=)?__\x00JOIN\x00__(=\?([^?]+)\?[QqBb]\?)?/g, '').replace(/(=\?[^?]+\?[QqBb]\?[^?]*\?=)\s+(?==\?[^?]+\?[QqBb]\?[^?]*\?=)/g, '$1').replace(/=\?([\w_\-*]+)\?([QqBb])\?([^?]*)\?=/g, function(m, charset, encoding, text) {
    return $jscomp$this.decodeWord(charset, encoding, text);
  });
}, addressParser:function(str, options) {
  var $jscomp$this = this;
  options = options || {};
  var tokenizer = Ext.create('documents.parser.tokenizer', {str:str});
  var tokens = tokenizer.tokenize();
  var addresses = [];
  var address = [];
  var parsedAddresses = [];
  tokens.forEach(function(token) {
    if (token.type === 'operator' && (token.value === ',' || token.value === ';')) {
      if (address.length) {
        addresses.push(address);
      }
      address = [];
    } else {
      address.push(token);
    }
  });
  if (address.length) {
    addresses.push(address);
  }
  addresses.forEach(function(address) {
    address = $jscomp$this._handleAddress(address);
    if (address.length) {
      parsedAddresses = parsedAddresses.concat(address);
    }
  });
  if (options.flatten) {
    var addresses$24 = [];
    var walkAddressList = function(list) {
      list.forEach(function(address) {
        if (address.group) {
          return walkAddressList(address.group);
        } else {
          addresses$24.push(address);
        }
      });
    };
    walkAddressList(parsedAddresses);
    return addresses$24;
  }
  return parsedAddresses;
}, blobToArrayBuffer:function(blob) {
  var fr;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$generator$context.nextAddress == 1) {
      if (!('arrayBuffer' in blob)) {
        return $jscomp$generator$context.jumpTo(2);
      }
      return $jscomp$generator$context.yield(blob.arrayBuffer(), 3);
    }
    if ($jscomp$generator$context.nextAddress != 2) {
      return $jscomp$generator$context['return']($jscomp$generator$context.yieldResult);
    }
    fr = new FileReader;
    return $jscomp$generator$context['return'](new Promise(function(resolve, reject) {
      fr.onload = function(e) {
        resolve(e.target.result);
      };
      fr.onerror = function(e) {
        reject(fr.error);
      };
      fr.readAsArrayBuffer(blob);
    }));
  });
}, htmlEntities:{'\x26AElig':'Æ', '\x26AElig;':'Æ', '\x26AMP':'\x26', '\x26AMP;':'\x26', '\x26Aacute':'Á', '\x26Aacute;':'Á', '\x26Abreve;':'Ă', '\x26Acirc':'Â', '\x26Acirc;':'Â', '\x26Acy;':'А', '\x26Afr;':'\ud835\udd04', '\x26Agrave':'À', '\x26Agrave;':'À', '\x26Alpha;':'Α', '\x26Amacr;':'Ā', '\x26And;':'⩓', '\x26Aogon;':'Ą', '\x26Aopf;':'\ud835\udd38', '\x26ApplyFunction;':'⁡', '\x26Aring':'Å', '\x26Aring;':'Å', '\x26Ascr;':'\ud835\udc9c', '\x26Assign;':'≔', '\x26Atilde':'Ã', '\x26Atilde;':'Ã', 
'\x26Auml':'Ä', '\x26Auml;':'Ä', '\x26Backslash;':'∖', '\x26Barv;':'⫧', '\x26Barwed;':'⌆', '\x26Bcy;':'Б', '\x26Because;':'∵', '\x26Bernoullis;':'ℬ', '\x26Beta;':'Β', '\x26Bfr;':'\ud835\udd05', '\x26Bopf;':'\ud835\udd39', '\x26Breve;':'˘', '\x26Bscr;':'ℬ', '\x26Bumpeq;':'≎', '\x26CHcy;':'Ч', '\x26COPY':'©', '\x26COPY;':'©', '\x26Cacute;':'Ć', '\x26Cap;':'⋒', '\x26CapitalDifferentialD;':'ⅅ', '\x26Cayleys;':'ℭ', '\x26Ccaron;':'Č', '\x26Ccedil':'Ç', '\x26Ccedil;':'Ç', '\x26Ccirc;':'Ĉ', '\x26Cconint;':'∰', 
'\x26Cdot;':'Ċ', '\x26Cedilla;':'¸', '\x26CenterDot;':'·', '\x26Cfr;':'ℭ', '\x26Chi;':'Χ', '\x26CircleDot;':'⊙', '\x26CircleMinus;':'⊖', '\x26CirclePlus;':'⊕', '\x26CircleTimes;':'⊗', '\x26ClockwiseContourIntegral;':'∲', '\x26CloseCurlyDoubleQuote;':'”', '\x26CloseCurlyQuote;':'’', '\x26Colon;':'∷', '\x26Colone;':'⩴', '\x26Congruent;':'≡', '\x26Conint;':'∯', '\x26ContourIntegral;':'∮', '\x26Copf;':'ℂ', '\x26Coproduct;':'∐', '\x26CounterClockwiseContourIntegral;':'∳', '\x26Cross;':'⨯', '\x26Cscr;':'\ud835\udc9e', 
'\x26Cup;':'⋓', '\x26CupCap;':'≍', '\x26DD;':'ⅅ', '\x26DDotrahd;':'⤑', '\x26DJcy;':'Ђ', '\x26DScy;':'Ѕ', '\x26DZcy;':'Џ', '\x26Dagger;':'‡', '\x26Darr;':'↡', '\x26Dashv;':'⫤', '\x26Dcaron;':'Ď', '\x26Dcy;':'Д', '\x26Del;':'∇', '\x26Delta;':'Δ', '\x26Dfr;':'\ud835\udd07', '\x26DiacriticalAcute;':'´', '\x26DiacriticalDot;':'˙', '\x26DiacriticalDoubleAcute;':'˝', '\x26DiacriticalGrave;':'`', '\x26DiacriticalTilde;':'˜', '\x26Diamond;':'⋄', '\x26DifferentialD;':'ⅆ', '\x26Dopf;':'\ud835\udd3b', '\x26Dot;':'¨', 
'\x26DotDot;':'⃜', '\x26DotEqual;':'≐', '\x26DoubleContourIntegral;':'∯', '\x26DoubleDot;':'¨', '\x26DoubleDownArrow;':'⇓', '\x26DoubleLeftArrow;':'⇐', '\x26DoubleLeftRightArrow;':'⇔', '\x26DoubleLeftTee;':'⫤', '\x26DoubleLongLeftArrow;':'⟸', '\x26DoubleLongLeftRightArrow;':'⟺', '\x26DoubleLongRightArrow;':'⟹', '\x26DoubleRightArrow;':'⇒', '\x26DoubleRightTee;':'⊨', '\x26DoubleUpArrow;':'⇑', '\x26DoubleUpDownArrow;':'⇕', '\x26DoubleVerticalBar;':'∥', '\x26DownArrow;':'↓', '\x26DownArrowBar;':'⤓', 
'\x26DownArrowUpArrow;':'⇵', '\x26DownBreve;':'̑', '\x26DownLeftRightVector;':'⥐', '\x26DownLeftTeeVector;':'⥞', '\x26DownLeftVector;':'↽', '\x26DownLeftVectorBar;':'⥖', '\x26DownRightTeeVector;':'⥟', '\x26DownRightVector;':'⇁', '\x26DownRightVectorBar;':'⥗', '\x26DownTee;':'⊤', '\x26DownTeeArrow;':'↧', '\x26Downarrow;':'⇓', '\x26Dscr;':'\ud835\udc9f', '\x26Dstrok;':'Đ', '\x26ENG;':'Ŋ', '\x26ETH':'Ð', '\x26ETH;':'Ð', '\x26Eacute':'É', '\x26Eacute;':'É', '\x26Ecaron;':'Ě', '\x26Ecirc':'Ê', '\x26Ecirc;':'Ê', 
'\x26Ecy;':'Э', '\x26Edot;':'Ė', '\x26Efr;':'\ud835\udd08', '\x26Egrave':'È', '\x26Egrave;':'È', '\x26Element;':'∈', '\x26Emacr;':'Ē', '\x26EmptySmallSquare;':'◻', '\x26EmptyVerySmallSquare;':'▫', '\x26Eogon;':'Ę', '\x26Eopf;':'\ud835\udd3c', '\x26Epsilon;':'Ε', '\x26Equal;':'⩵', '\x26EqualTilde;':'≂', '\x26Equilibrium;':'⇌', '\x26Escr;':'ℰ', '\x26Esim;':'⩳', '\x26Eta;':'Η', '\x26Euml':'Ë', '\x26Euml;':'Ë', '\x26Exists;':'∃', '\x26ExponentialE;':'ⅇ', '\x26Fcy;':'Ф', '\x26Ffr;':'\ud835\udd09', '\x26FilledSmallSquare;':'◼', 
'\x26FilledVerySmallSquare;':'▪', '\x26Fopf;':'\ud835\udd3d', '\x26ForAll;':'∀', '\x26Fouriertrf;':'ℱ', '\x26Fscr;':'ℱ', '\x26GJcy;':'Ѓ', '\x26GT':'\x3e', '\x26GT;':'\x3e', '\x26Gamma;':'Γ', '\x26Gammad;':'Ϝ', '\x26Gbreve;':'Ğ', '\x26Gcedil;':'Ģ', '\x26Gcirc;':'Ĝ', '\x26Gcy;':'Г', '\x26Gdot;':'Ġ', '\x26Gfr;':'\ud835\udd0a', '\x26Gg;':'⋙', '\x26Gopf;':'\ud835\udd3e', '\x26GreaterEqual;':'≥', '\x26GreaterEqualLess;':'⋛', '\x26GreaterFullEqual;':'≧', '\x26GreaterGreater;':'⪢', '\x26GreaterLess;':'≷', 
'\x26GreaterSlantEqual;':'⩾', '\x26GreaterTilde;':'≳', '\x26Gscr;':'\ud835\udca2', '\x26Gt;':'≫', '\x26HARDcy;':'Ъ', '\x26Hacek;':'ˇ', '\x26Hat;':'^', '\x26Hcirc;':'Ĥ', '\x26Hfr;':'ℌ', '\x26HilbertSpace;':'ℋ', '\x26Hopf;':'ℍ', '\x26HorizontalLine;':'─', '\x26Hscr;':'ℋ', '\x26Hstrok;':'Ħ', '\x26HumpDownHump;':'≎', '\x26HumpEqual;':'≏', '\x26IEcy;':'Е', '\x26IJlig;':'Ĳ', '\x26IOcy;':'Ё', '\x26Iacute':'Í', '\x26Iacute;':'Í', '\x26Icirc':'Î', '\x26Icirc;':'Î', '\x26Icy;':'И', '\x26Idot;':'İ', '\x26Ifr;':'ℑ', 
'\x26Igrave':'Ì', '\x26Igrave;':'Ì', '\x26Im;':'ℑ', '\x26Imacr;':'Ī', '\x26ImaginaryI;':'ⅈ', '\x26Implies;':'⇒', '\x26Int;':'∬', '\x26Integral;':'∫', '\x26Intersection;':'⋂', '\x26InvisibleComma;':'⁣', '\x26InvisibleTimes;':'⁢', '\x26Iogon;':'Į', '\x26Iopf;':'\ud835\udd40', '\x26Iota;':'Ι', '\x26Iscr;':'ℐ', '\x26Itilde;':'Ĩ', '\x26Iukcy;':'І', '\x26Iuml':'Ï', '\x26Iuml;':'Ï', '\x26Jcirc;':'Ĵ', '\x26Jcy;':'Й', '\x26Jfr;':'\ud835\udd0d', '\x26Jopf;':'\ud835\udd41', '\x26Jscr;':'\ud835\udca5', '\x26Jsercy;':'Ј', 
'\x26Jukcy;':'Є', '\x26KHcy;':'Х', '\x26KJcy;':'Ќ', '\x26Kappa;':'Κ', '\x26Kcedil;':'Ķ', '\x26Kcy;':'К', '\x26Kfr;':'\ud835\udd0e', '\x26Kopf;':'\ud835\udd42', '\x26Kscr;':'\ud835\udca6', '\x26LJcy;':'Љ', '\x26LT':'\x3c', '\x26LT;':'\x3c', '\x26Lacute;':'Ĺ', '\x26Lambda;':'Λ', '\x26Lang;':'⟪', '\x26Laplacetrf;':'ℒ', '\x26Larr;':'↞', '\x26Lcaron;':'Ľ', '\x26Lcedil;':'Ļ', '\x26Lcy;':'Л', '\x26LeftAngleBracket;':'⟨', '\x26LeftArrow;':'←', '\x26LeftArrowBar;':'⇤', '\x26LeftArrowRightArrow;':'⇆', '\x26LeftCeiling;':'⌈', 
'\x26LeftDoubleBracket;':'⟦', '\x26LeftDownTeeVector;':'⥡', '\x26LeftDownVector;':'⇃', '\x26LeftDownVectorBar;':'⥙', '\x26LeftFloor;':'⌊', '\x26LeftRightArrow;':'↔', '\x26LeftRightVector;':'⥎', '\x26LeftTee;':'⊣', '\x26LeftTeeArrow;':'↤', '\x26LeftTeeVector;':'⥚', '\x26LeftTriangle;':'⊲', '\x26LeftTriangleBar;':'⧏', '\x26LeftTriangleEqual;':'⊴', '\x26LeftUpDownVector;':'⥑', '\x26LeftUpTeeVector;':'⥠', '\x26LeftUpVector;':'↿', '\x26LeftUpVectorBar;':'⥘', '\x26LeftVector;':'↼', '\x26LeftVectorBar;':'⥒', 
'\x26Leftarrow;':'⇐', '\x26Leftrightarrow;':'⇔', '\x26LessEqualGreater;':'⋚', '\x26LessFullEqual;':'≦', '\x26LessGreater;':'≶', '\x26LessLess;':'⪡', '\x26LessSlantEqual;':'⩽', '\x26LessTilde;':'≲', '\x26Lfr;':'\ud835\udd0f', '\x26Ll;':'⋘', '\x26Lleftarrow;':'⇚', '\x26Lmidot;':'Ŀ', '\x26LongLeftArrow;':'⟵', '\x26LongLeftRightArrow;':'⟷', '\x26LongRightArrow;':'⟶', '\x26Longleftarrow;':'⟸', '\x26Longleftrightarrow;':'⟺', '\x26Longrightarrow;':'⟹', '\x26Lopf;':'\ud835\udd43', '\x26LowerLeftArrow;':'↙', 
'\x26LowerRightArrow;':'↘', '\x26Lscr;':'ℒ', '\x26Lsh;':'↰', '\x26Lstrok;':'Ł', '\x26Lt;':'≪', '\x26Map;':'⤅', '\x26Mcy;':'М', '\x26MediumSpace;':' ', '\x26Mellintrf;':'ℳ', '\x26Mfr;':'\ud835\udd10', '\x26MinusPlus;':'∓', '\x26Mopf;':'\ud835\udd44', '\x26Mscr;':'ℳ', '\x26Mu;':'Μ', '\x26NJcy;':'Њ', '\x26Nacute;':'Ń', '\x26Ncaron;':'Ň', '\x26Ncedil;':'Ņ', '\x26Ncy;':'Н', '\x26NegativeMediumSpace;':'​', '\x26NegativeThickSpace;':'​', '\x26NegativeThinSpace;':'​', '\x26NegativeVeryThinSpace;':'​', '\x26NestedGreaterGreater;':'≫', 
'\x26NestedLessLess;':'≪', '\x26NewLine;':'\n', '\x26Nfr;':'\ud835\udd11', '\x26NoBreak;':'⁠', '\x26NonBreakingSpace;':' ', '\x26Nopf;':'ℕ', '\x26Not;':'⫬', '\x26NotCongruent;':'≢', '\x26NotCupCap;':'≭', '\x26NotDoubleVerticalBar;':'∦', '\x26NotElement;':'∉', '\x26NotEqual;':'≠', '\x26NotEqualTilde;':'≂̸', '\x26NotExists;':'∄', '\x26NotGreater;':'≯', '\x26NotGreaterEqual;':'≱', '\x26NotGreaterFullEqual;':'≧̸', '\x26NotGreaterGreater;':'≫̸', '\x26NotGreaterLess;':'≹', '\x26NotGreaterSlantEqual;':'⩾̸', 
'\x26NotGreaterTilde;':'≵', '\x26NotHumpDownHump;':'≎̸', '\x26NotHumpEqual;':'≏̸', '\x26NotLeftTriangle;':'⋪', '\x26NotLeftTriangleBar;':'⧏̸', '\x26NotLeftTriangleEqual;':'⋬', '\x26NotLess;':'≮', '\x26NotLessEqual;':'≰', '\x26NotLessGreater;':'≸', '\x26NotLessLess;':'≪̸', '\x26NotLessSlantEqual;':'⩽̸', '\x26NotLessTilde;':'≴', '\x26NotNestedGreaterGreater;':'⪢̸', '\x26NotNestedLessLess;':'⪡̸', '\x26NotPrecedes;':'⊀', '\x26NotPrecedesEqual;':'⪯̸', '\x26NotPrecedesSlantEqual;':'⋠', '\x26NotReverseElement;':'∌', 
'\x26NotRightTriangle;':'⋫', '\x26NotRightTriangleBar;':'⧐̸', '\x26NotRightTriangleEqual;':'⋭', '\x26NotSquareSubset;':'⊏̸', '\x26NotSquareSubsetEqual;':'⋢', '\x26NotSquareSuperset;':'⊐̸', '\x26NotSquareSupersetEqual;':'⋣', '\x26NotSubset;':'⊂⃒', '\x26NotSubsetEqual;':'⊈', '\x26NotSucceeds;':'⊁', '\x26NotSucceedsEqual;':'⪰̸', '\x26NotSucceedsSlantEqual;':'⋡', '\x26NotSucceedsTilde;':'≿̸', '\x26NotSuperset;':'⊃⃒', '\x26NotSupersetEqual;':'⊉', '\x26NotTilde;':'≁', '\x26NotTildeEqual;':'≄', '\x26NotTildeFullEqual;':'≇', 
'\x26NotTildeTilde;':'≉', '\x26NotVerticalBar;':'∤', '\x26Nscr;':'\ud835\udca9', '\x26Ntilde':'Ñ', '\x26Ntilde;':'Ñ', '\x26Nu;':'Ν', '\x26OElig;':'Œ', '\x26Oacute':'Ó', '\x26Oacute;':'Ó', '\x26Ocirc':'Ô', '\x26Ocirc;':'Ô', '\x26Ocy;':'О', '\x26Odblac;':'Ő', '\x26Ofr;':'\ud835\udd12', '\x26Ograve':'Ò', '\x26Ograve;':'Ò', '\x26Omacr;':'Ō', '\x26Omega;':'Ω', '\x26Omicron;':'Ο', '\x26Oopf;':'\ud835\udd46', '\x26OpenCurlyDoubleQuote;':'“', '\x26OpenCurlyQuote;':'‘', '\x26Or;':'⩔', '\x26Oscr;':'\ud835\udcaa', 
'\x26Oslash':'Ø', '\x26Oslash;':'Ø', '\x26Otilde':'Õ', '\x26Otilde;':'Õ', '\x26Otimes;':'⨷', '\x26Ouml':'Ö', '\x26Ouml;':'Ö', '\x26OverBar;':'‾', '\x26OverBrace;':'⏞', '\x26OverBracket;':'⎴', '\x26OverParenthesis;':'⏜', '\x26PartialD;':'∂', '\x26Pcy;':'П', '\x26Pfr;':'\ud835\udd13', '\x26Phi;':'Φ', '\x26Pi;':'Π', '\x26PlusMinus;':'±', '\x26Poincareplane;':'ℌ', '\x26Popf;':'ℙ', '\x26Pr;':'⪻', '\x26Precedes;':'≺', '\x26PrecedesEqual;':'⪯', '\x26PrecedesSlantEqual;':'≼', '\x26PrecedesTilde;':'≾', '\x26Prime;':'″', 
'\x26Product;':'∏', '\x26Proportion;':'∷', '\x26Proportional;':'∝', '\x26Pscr;':'\ud835\udcab', '\x26Psi;':'Ψ', '\x26QUOT':'"', '\x26QUOT;':'"', '\x26Qfr;':'\ud835\udd14', '\x26Qopf;':'ℚ', '\x26Qscr;':'\ud835\udcac', '\x26RBarr;':'⤐', '\x26REG':'®', '\x26REG;':'®', '\x26Racute;':'Ŕ', '\x26Rang;':'⟫', '\x26Rarr;':'↠', '\x26Rarrtl;':'⤖', '\x26Rcaron;':'Ř', '\x26Rcedil;':'Ŗ', '\x26Rcy;':'Р', '\x26Re;':'ℜ', '\x26ReverseElement;':'∋', '\x26ReverseEquilibrium;':'⇋', '\x26ReverseUpEquilibrium;':'⥯', '\x26Rfr;':'ℜ', 
'\x26Rho;':'Ρ', '\x26RightAngleBracket;':'⟩', '\x26RightArrow;':'→', '\x26RightArrowBar;':'⇥', '\x26RightArrowLeftArrow;':'⇄', '\x26RightCeiling;':'⌉', '\x26RightDoubleBracket;':'⟧', '\x26RightDownTeeVector;':'⥝', '\x26RightDownVector;':'⇂', '\x26RightDownVectorBar;':'⥕', '\x26RightFloor;':'⌋', '\x26RightTee;':'⊢', '\x26RightTeeArrow;':'↦', '\x26RightTeeVector;':'⥛', '\x26RightTriangle;':'⊳', '\x26RightTriangleBar;':'⧐', '\x26RightTriangleEqual;':'⊵', '\x26RightUpDownVector;':'⥏', '\x26RightUpTeeVector;':'⥜', 
'\x26RightUpVector;':'↾', '\x26RightUpVectorBar;':'⥔', '\x26RightVector;':'⇀', '\x26RightVectorBar;':'⥓', '\x26Rightarrow;':'⇒', '\x26Ropf;':'ℝ', '\x26RoundImplies;':'⥰', '\x26Rrightarrow;':'⇛', '\x26Rscr;':'ℛ', '\x26Rsh;':'↱', '\x26RuleDelayed;':'⧴', '\x26SHCHcy;':'Щ', '\x26SHcy;':'Ш', '\x26SOFTcy;':'Ь', '\x26Sacute;':'Ś', '\x26Sc;':'⪼', '\x26Scaron;':'Š', '\x26Scedil;':'Ş', '\x26Scirc;':'Ŝ', '\x26Scy;':'С', '\x26Sfr;':'\ud835\udd16', '\x26ShortDownArrow;':'↓', '\x26ShortLeftArrow;':'←', '\x26ShortRightArrow;':'→', 
'\x26ShortUpArrow;':'↑', '\x26Sigma;':'Σ', '\x26SmallCircle;':'∘', '\x26Sopf;':'\ud835\udd4a', '\x26Sqrt;':'√', '\x26Square;':'□', '\x26SquareIntersection;':'⊓', '\x26SquareSubset;':'⊏', '\x26SquareSubsetEqual;':'⊑', '\x26SquareSuperset;':'⊐', '\x26SquareSupersetEqual;':'⊒', '\x26SquareUnion;':'⊔', '\x26Sscr;':'\ud835\udcae', '\x26Star;':'⋆', '\x26Sub;':'⋐', '\x26Subset;':'⋐', '\x26SubsetEqual;':'⊆', '\x26Succeeds;':'≻', '\x26SucceedsEqual;':'⪰', '\x26SucceedsSlantEqual;':'≽', '\x26SucceedsTilde;':'≿', 
'\x26SuchThat;':'∋', '\x26Sum;':'∑', '\x26Sup;':'⋑', '\x26Superset;':'⊃', '\x26SupersetEqual;':'⊇', '\x26Supset;':'⋑', '\x26THORN':'Þ', '\x26THORN;':'Þ', '\x26TRADE;':'™', '\x26TSHcy;':'Ћ', '\x26TScy;':'Ц', '\x26Tab;':'\t', '\x26Tau;':'Τ', '\x26Tcaron;':'Ť', '\x26Tcedil;':'Ţ', '\x26Tcy;':'Т', '\x26Tfr;':'\ud835\udd17', '\x26Therefore;':'∴', '\x26Theta;':'Θ', '\x26ThickSpace;':'  ', '\x26ThinSpace;':' ', '\x26Tilde;':'∼', '\x26TildeEqual;':'≃', '\x26TildeFullEqual;':'≅', '\x26TildeTilde;':'≈', '\x26Topf;':'\ud835\udd4b', 
'\x26TripleDot;':'⃛', '\x26Tscr;':'\ud835\udcaf', '\x26Tstrok;':'Ŧ', '\x26Uacute':'Ú', '\x26Uacute;':'Ú', '\x26Uarr;':'↟', '\x26Uarrocir;':'⥉', '\x26Ubrcy;':'Ў', '\x26Ubreve;':'Ŭ', '\x26Ucirc':'Û', '\x26Ucirc;':'Û', '\x26Ucy;':'У', '\x26Udblac;':'Ű', '\x26Ufr;':'\ud835\udd18', '\x26Ugrave':'Ù', '\x26Ugrave;':'Ù', '\x26Umacr;':'Ū', '\x26UnderBar;':'_', '\x26UnderBrace;':'⏟', '\x26UnderBracket;':'⎵', '\x26UnderParenthesis;':'⏝', '\x26Union;':'⋃', '\x26UnionPlus;':'⊎', '\x26Uogon;':'Ų', '\x26Uopf;':'\ud835\udd4c', 
'\x26UpArrow;':'↑', '\x26UpArrowBar;':'⤒', '\x26UpArrowDownArrow;':'⇅', '\x26UpDownArrow;':'↕', '\x26UpEquilibrium;':'⥮', '\x26UpTee;':'⊥', '\x26UpTeeArrow;':'↥', '\x26Uparrow;':'⇑', '\x26Updownarrow;':'⇕', '\x26UpperLeftArrow;':'↖', '\x26UpperRightArrow;':'↗', '\x26Upsi;':'ϒ', '\x26Upsilon;':'Υ', '\x26Uring;':'Ů', '\x26Uscr;':'\ud835\udcb0', '\x26Utilde;':'Ũ', '\x26Uuml':'Ü', '\x26Uuml;':'Ü', '\x26VDash;':'⊫', '\x26Vbar;':'⫫', '\x26Vcy;':'В', '\x26Vdash;':'⊩', '\x26Vdashl;':'⫦', '\x26Vee;':'⋁', 
'\x26Verbar;':'‖', '\x26Vert;':'‖', '\x26VerticalBar;':'∣', '\x26VerticalLine;':'|', '\x26VerticalSeparator;':'❘', '\x26VerticalTilde;':'≀', '\x26VeryThinSpace;':' ', '\x26Vfr;':'\ud835\udd19', '\x26Vopf;':'\ud835\udd4d', '\x26Vscr;':'\ud835\udcb1', '\x26Vvdash;':'⊪', '\x26Wcirc;':'Ŵ', '\x26Wedge;':'⋀', '\x26Wfr;':'\ud835\udd1a', '\x26Wopf;':'\ud835\udd4e', '\x26Wscr;':'\ud835\udcb2', '\x26Xfr;':'\ud835\udd1b', '\x26Xi;':'Ξ', '\x26Xopf;':'\ud835\udd4f', '\x26Xscr;':'\ud835\udcb3', '\x26YAcy;':'Я', 
'\x26YIcy;':'Ї', '\x26YUcy;':'Ю', '\x26Yacute':'Ý', '\x26Yacute;':'Ý', '\x26Ycirc;':'Ŷ', '\x26Ycy;':'Ы', '\x26Yfr;':'\ud835\udd1c', '\x26Yopf;':'\ud835\udd50', '\x26Yscr;':'\ud835\udcb4', '\x26Yuml;':'Ÿ', '\x26ZHcy;':'Ж', '\x26Zacute;':'Ź', '\x26Zcaron;':'Ž', '\x26Zcy;':'З', '\x26Zdot;':'Ż', '\x26ZeroWidthSpace;':'​', '\x26Zeta;':'Ζ', '\x26Zfr;':'ℨ', '\x26Zopf;':'ℤ', '\x26Zscr;':'\ud835\udcb5', '\x26aacute':'á', '\x26aacute;':'á', '\x26abreve;':'ă', '\x26ac;':'∾', '\x26acE;':'∾̳', '\x26acd;':'∿', 
'\x26acirc':'â', '\x26acirc;':'â', '\x26acute':'´', '\x26acute;':'´', '\x26acy;':'а', '\x26aelig':'æ', '\x26aelig;':'æ', '\x26af;':'⁡', '\x26afr;':'\ud835\udd1e', '\x26agrave':'à', '\x26agrave;':'à', '\x26alefsym;':'ℵ', '\x26aleph;':'ℵ', '\x26alpha;':'α', '\x26amacr;':'ā', '\x26amalg;':'⨿', '\x26amp':'\x26', '\x26amp;':'\x26', '\x26and;':'∧', '\x26andand;':'⩕', '\x26andd;':'⩜', '\x26andslope;':'⩘', '\x26andv;':'⩚', '\x26ang;':'∠', '\x26ange;':'⦤', '\x26angle;':'∠', '\x26angmsd;':'∡', '\x26angmsdaa;':'⦨', 
'\x26angmsdab;':'⦩', '\x26angmsdac;':'⦪', '\x26angmsdad;':'⦫', '\x26angmsdae;':'⦬', '\x26angmsdaf;':'⦭', '\x26angmsdag;':'⦮', '\x26angmsdah;':'⦯', '\x26angrt;':'∟', '\x26angrtvb;':'⊾', '\x26angrtvbd;':'⦝', '\x26angsph;':'∢', '\x26angst;':'Å', '\x26angzarr;':'⍼', '\x26aogon;':'ą', '\x26aopf;':'\ud835\udd52', '\x26ap;':'≈', '\x26apE;':'⩰', '\x26apacir;':'⩯', '\x26ape;':'≊', '\x26apid;':'≋', '\x26apos;':"'", '\x26approx;':'≈', '\x26approxeq;':'≊', '\x26aring':'å', '\x26aring;':'å', '\x26ascr;':'\ud835\udcb6', 
'\x26ast;':'*', '\x26asymp;':'≈', '\x26asympeq;':'≍', '\x26atilde':'ã', '\x26atilde;':'ã', '\x26auml':'ä', '\x26auml;':'ä', '\x26awconint;':'∳', '\x26awint;':'⨑', '\x26bNot;':'⫭', '\x26backcong;':'≌', '\x26backepsilon;':'϶', '\x26backprime;':'‵', '\x26backsim;':'∽', '\x26backsimeq;':'⋍', '\x26barvee;':'⊽', '\x26barwed;':'⌅', '\x26barwedge;':'⌅', '\x26bbrk;':'⎵', '\x26bbrktbrk;':'⎶', '\x26bcong;':'≌', '\x26bcy;':'б', '\x26bdquo;':'„', '\x26becaus;':'∵', '\x26because;':'∵', '\x26bemptyv;':'⦰', '\x26bepsi;':'϶', 
'\x26bernou;':'ℬ', '\x26beta;':'β', '\x26beth;':'ℶ', '\x26between;':'≬', '\x26bfr;':'\ud835\udd1f', '\x26bigcap;':'⋂', '\x26bigcirc;':'◯', '\x26bigcup;':'⋃', '\x26bigodot;':'⨀', '\x26bigoplus;':'⨁', '\x26bigotimes;':'⨂', '\x26bigsqcup;':'⨆', '\x26bigstar;':'★', '\x26bigtriangledown;':'▽', '\x26bigtriangleup;':'△', '\x26biguplus;':'⨄', '\x26bigvee;':'⋁', '\x26bigwedge;':'⋀', '\x26bkarow;':'⤍', '\x26blacklozenge;':'⧫', '\x26blacksquare;':'▪', '\x26blacktriangle;':'▴', '\x26blacktriangledown;':'▾', 
'\x26blacktriangleleft;':'◂', '\x26blacktriangleright;':'▸', '\x26blank;':'␣', '\x26blk12;':'▒', '\x26blk14;':'░', '\x26blk34;':'▓', '\x26block;':'█', '\x26bne;':'\x3d⃥', '\x26bnequiv;':'≡⃥', '\x26bnot;':'⌐', '\x26bopf;':'\ud835\udd53', '\x26bot;':'⊥', '\x26bottom;':'⊥', '\x26bowtie;':'⋈', '\x26boxDL;':'╗', '\x26boxDR;':'╔', '\x26boxDl;':'╖', '\x26boxDr;':'╓', '\x26boxH;':'═', '\x26boxHD;':'╦', '\x26boxHU;':'╩', '\x26boxHd;':'╤', '\x26boxHu;':'╧', '\x26boxUL;':'╝', '\x26boxUR;':'╚', '\x26boxUl;':'╜', 
'\x26boxUr;':'╙', '\x26boxV;':'║', '\x26boxVH;':'╬', '\x26boxVL;':'╣', '\x26boxVR;':'╠', '\x26boxVh;':'╫', '\x26boxVl;':'╢', '\x26boxVr;':'╟', '\x26boxbox;':'⧉', '\x26boxdL;':'╕', '\x26boxdR;':'╒', '\x26boxdl;':'┐', '\x26boxdr;':'┌', '\x26boxh;':'─', '\x26boxhD;':'╥', '\x26boxhU;':'╨', '\x26boxhd;':'┬', '\x26boxhu;':'┴', '\x26boxminus;':'⊟', '\x26boxplus;':'⊞', '\x26boxtimes;':'⊠', '\x26boxuL;':'╛', '\x26boxuR;':'╘', '\x26boxul;':'┘', '\x26boxur;':'└', '\x26boxv;':'│', '\x26boxvH;':'╪', '\x26boxvL;':'╡', 
'\x26boxvR;':'╞', '\x26boxvh;':'┼', '\x26boxvl;':'┤', '\x26boxvr;':'├', '\x26bprime;':'‵', '\x26breve;':'˘', '\x26brvbar':'¦', '\x26brvbar;':'¦', '\x26bscr;':'\ud835\udcb7', '\x26bsemi;':'⁏', '\x26bsim;':'∽', '\x26bsime;':'⋍', '\x26bsol;':'\\', '\x26bsolb;':'⧅', '\x26bsolhsub;':'⟈', '\x26bull;':'•', '\x26bullet;':'•', '\x26bump;':'≎', '\x26bumpE;':'⪮', '\x26bumpe;':'≏', '\x26bumpeq;':'≏', '\x26cacute;':'ć', '\x26cap;':'∩', '\x26capand;':'⩄', '\x26capbrcup;':'⩉', '\x26capcap;':'⩋', '\x26capcup;':'⩇', 
'\x26capdot;':'⩀', '\x26caps;':'∩︀', '\x26caret;':'⁁', '\x26caron;':'ˇ', '\x26ccaps;':'⩍', '\x26ccaron;':'č', '\x26ccedil':'ç', '\x26ccedil;':'ç', '\x26ccirc;':'ĉ', '\x26ccups;':'⩌', '\x26ccupssm;':'⩐', '\x26cdot;':'ċ', '\x26cedil':'¸', '\x26cedil;':'¸', '\x26cemptyv;':'⦲', '\x26cent':'¢', '\x26cent;':'¢', '\x26centerdot;':'·', '\x26cfr;':'\ud835\udd20', '\x26chcy;':'ч', '\x26check;':'✓', '\x26checkmark;':'✓', '\x26chi;':'χ', '\x26cir;':'○', '\x26cirE;':'⧃', '\x26circ;':'ˆ', '\x26circeq;':'≗', '\x26circlearrowleft;':'↺', 
'\x26circlearrowright;':'↻', '\x26circledR;':'®', '\x26circledS;':'Ⓢ', '\x26circledast;':'⊛', '\x26circledcirc;':'⊚', '\x26circleddash;':'⊝', '\x26cire;':'≗', '\x26cirfnint;':'⨐', '\x26cirmid;':'⫯', '\x26cirscir;':'⧂', '\x26clubs;':'♣', '\x26clubsuit;':'♣', '\x26colon;':':', '\x26colone;':'≔', '\x26coloneq;':'≔', '\x26comma;':',', '\x26commat;':'@', '\x26comp;':'∁', '\x26compfn;':'∘', '\x26complement;':'∁', '\x26complexes;':'ℂ', '\x26cong;':'≅', '\x26congdot;':'⩭', '\x26conint;':'∮', '\x26copf;':'\ud835\udd54', 
'\x26coprod;':'∐', '\x26copy':'©', '\x26copy;':'©', '\x26copysr;':'℗', '\x26crarr;':'↵', '\x26cross;':'✗', '\x26cscr;':'\ud835\udcb8', '\x26csub;':'⫏', '\x26csube;':'⫑', '\x26csup;':'⫐', '\x26csupe;':'⫒', '\x26ctdot;':'⋯', '\x26cudarrl;':'⤸', '\x26cudarrr;':'⤵', '\x26cuepr;':'⋞', '\x26cuesc;':'⋟', '\x26cularr;':'↶', '\x26cularrp;':'⤽', '\x26cup;':'∪', '\x26cupbrcap;':'⩈', '\x26cupcap;':'⩆', '\x26cupcup;':'⩊', '\x26cupdot;':'⊍', '\x26cupor;':'⩅', '\x26cups;':'∪︀', '\x26curarr;':'↷', '\x26curarrm;':'⤼', 
'\x26curlyeqprec;':'⋞', '\x26curlyeqsucc;':'⋟', '\x26curlyvee;':'⋎', '\x26curlywedge;':'⋏', '\x26curren':'¤', '\x26curren;':'¤', '\x26curvearrowleft;':'↶', '\x26curvearrowright;':'↷', '\x26cuvee;':'⋎', '\x26cuwed;':'⋏', '\x26cwconint;':'∲', '\x26cwint;':'∱', '\x26cylcty;':'⌭', '\x26dArr;':'⇓', '\x26dHar;':'⥥', '\x26dagger;':'†', '\x26daleth;':'ℸ', '\x26darr;':'↓', '\x26dash;':'‐', '\x26dashv;':'⊣', '\x26dbkarow;':'⤏', '\x26dblac;':'˝', '\x26dcaron;':'ď', '\x26dcy;':'д', '\x26dd;':'ⅆ', '\x26ddagger;':'‡', 
'\x26ddarr;':'⇊', '\x26ddotseq;':'⩷', '\x26deg':'°', '\x26deg;':'°', '\x26delta;':'δ', '\x26demptyv;':'⦱', '\x26dfisht;':'⥿', '\x26dfr;':'\ud835\udd21', '\x26dharl;':'⇃', '\x26dharr;':'⇂', '\x26diam;':'⋄', '\x26diamond;':'⋄', '\x26diamondsuit;':'♦', '\x26diams;':'♦', '\x26die;':'¨', '\x26digamma;':'ϝ', '\x26disin;':'⋲', '\x26div;':'÷', '\x26divide':'÷', '\x26divide;':'÷', '\x26divideontimes;':'⋇', '\x26divonx;':'⋇', '\x26djcy;':'ђ', '\x26dlcorn;':'⌞', '\x26dlcrop;':'⌍', '\x26dollar;':'$', '\x26dopf;':'\ud835\udd55', 
'\x26dot;':'˙', '\x26doteq;':'≐', '\x26doteqdot;':'≑', '\x26dotminus;':'∸', '\x26dotplus;':'∔', '\x26dotsquare;':'⊡', '\x26doublebarwedge;':'⌆', '\x26downarrow;':'↓', '\x26downdownarrows;':'⇊', '\x26downharpoonleft;':'⇃', '\x26downharpoonright;':'⇂', '\x26drbkarow;':'⤐', '\x26drcorn;':'⌟', '\x26drcrop;':'⌌', '\x26dscr;':'\ud835\udcb9', '\x26dscy;':'ѕ', '\x26dsol;':'⧶', '\x26dstrok;':'đ', '\x26dtdot;':'⋱', '\x26dtri;':'▿', '\x26dtrif;':'▾', '\x26duarr;':'⇵', '\x26duhar;':'⥯', '\x26dwangle;':'⦦', '\x26dzcy;':'џ', 
'\x26dzigrarr;':'⟿', '\x26eDDot;':'⩷', '\x26eDot;':'≑', '\x26eacute':'é', '\x26eacute;':'é', '\x26easter;':'⩮', '\x26ecaron;':'ě', '\x26ecir;':'≖', '\x26ecirc':'ê', '\x26ecirc;':'ê', '\x26ecolon;':'≕', '\x26ecy;':'э', '\x26edot;':'ė', '\x26ee;':'ⅇ', '\x26efDot;':'≒', '\x26efr;':'\ud835\udd22', '\x26eg;':'⪚', '\x26egrave':'è', '\x26egrave;':'è', '\x26egs;':'⪖', '\x26egsdot;':'⪘', '\x26el;':'⪙', '\x26elinters;':'⏧', '\x26ell;':'ℓ', '\x26els;':'⪕', '\x26elsdot;':'⪗', '\x26emacr;':'ē', '\x26empty;':'∅', 
'\x26emptyset;':'∅', '\x26emptyv;':'∅', '\x26emsp13;':' ', '\x26emsp14;':' ', '\x26emsp;':' ', '\x26eng;':'ŋ', '\x26ensp;':' ', '\x26eogon;':'ę', '\x26eopf;':'\ud835\udd56', '\x26epar;':'⋕', '\x26eparsl;':'⧣', '\x26eplus;':'⩱', '\x26epsi;':'ε', '\x26epsilon;':'ε', '\x26epsiv;':'ϵ', '\x26eqcirc;':'≖', '\x26eqcolon;':'≕', '\x26eqsim;':'≂', '\x26eqslantgtr;':'⪖', '\x26eqslantless;':'⪕', '\x26equals;':'\x3d', '\x26equest;':'≟', '\x26equiv;':'≡', '\x26equivDD;':'⩸', '\x26eqvparsl;':'⧥', '\x26erDot;':'≓', 
'\x26erarr;':'⥱', '\x26escr;':'ℯ', '\x26esdot;':'≐', '\x26esim;':'≂', '\x26eta;':'η', '\x26eth':'ð', '\x26eth;':'ð', '\x26euml':'ë', '\x26euml;':'ë', '\x26euro;':'€', '\x26excl;':'!', '\x26exist;':'∃', '\x26expectation;':'ℰ', '\x26exponentiale;':'ⅇ', '\x26fallingdotseq;':'≒', '\x26fcy;':'ф', '\x26female;':'♀', '\x26ffilig;':'ﬃ', '\x26fflig;':'ﬀ', '\x26ffllig;':'ﬄ', '\x26ffr;':'\ud835\udd23', '\x26filig;':'ﬁ', '\x26fjlig;':'fj', '\x26flat;':'♭', '\x26fllig;':'ﬂ', '\x26fltns;':'▱', '\x26fnof;':'ƒ', 
'\x26fopf;':'\ud835\udd57', '\x26forall;':'∀', '\x26fork;':'⋔', '\x26forkv;':'⫙', '\x26fpartint;':'⨍', '\x26frac12':'½', '\x26frac12;':'½', '\x26frac13;':'⅓', '\x26frac14':'¼', '\x26frac14;':'¼', '\x26frac15;':'⅕', '\x26frac16;':'⅙', '\x26frac18;':'⅛', '\x26frac23;':'⅔', '\x26frac25;':'⅖', '\x26frac34':'¾', '\x26frac34;':'¾', '\x26frac35;':'⅗', '\x26frac38;':'⅜', '\x26frac45;':'⅘', '\x26frac56;':'⅚', '\x26frac58;':'⅝', '\x26frac78;':'⅞', '\x26frasl;':'⁄', '\x26frown;':'⌢', '\x26fscr;':'\ud835\udcbb', 
'\x26gE;':'≧', '\x26gEl;':'⪌', '\x26gacute;':'ǵ', '\x26gamma;':'γ', '\x26gammad;':'ϝ', '\x26gap;':'⪆', '\x26gbreve;':'ğ', '\x26gcirc;':'ĝ', '\x26gcy;':'г', '\x26gdot;':'ġ', '\x26ge;':'≥', '\x26gel;':'⋛', '\x26geq;':'≥', '\x26geqq;':'≧', '\x26geqslant;':'⩾', '\x26ges;':'⩾', '\x26gescc;':'⪩', '\x26gesdot;':'⪀', '\x26gesdoto;':'⪂', '\x26gesdotol;':'⪄', '\x26gesl;':'⋛︀', '\x26gesles;':'⪔', '\x26gfr;':'\ud835\udd24', '\x26gg;':'≫', '\x26ggg;':'⋙', '\x26gimel;':'ℷ', '\x26gjcy;':'ѓ', '\x26gl;':'≷', '\x26glE;':'⪒', 
'\x26gla;':'⪥', '\x26glj;':'⪤', '\x26gnE;':'≩', '\x26gnap;':'⪊', '\x26gnapprox;':'⪊', '\x26gne;':'⪈', '\x26gneq;':'⪈', '\x26gneqq;':'≩', '\x26gnsim;':'⋧', '\x26gopf;':'\ud835\udd58', '\x26grave;':'`', '\x26gscr;':'ℊ', '\x26gsim;':'≳', '\x26gsime;':'⪎', '\x26gsiml;':'⪐', '\x26gt':'\x3e', '\x26gt;':'\x3e', '\x26gtcc;':'⪧', '\x26gtcir;':'⩺', '\x26gtdot;':'⋗', '\x26gtlPar;':'⦕', '\x26gtquest;':'⩼', '\x26gtrapprox;':'⪆', '\x26gtrarr;':'⥸', '\x26gtrdot;':'⋗', '\x26gtreqless;':'⋛', '\x26gtreqqless;':'⪌', 
'\x26gtrless;':'≷', '\x26gtrsim;':'≳', '\x26gvertneqq;':'≩︀', '\x26gvnE;':'≩︀', '\x26hArr;':'⇔', '\x26hairsp;':' ', '\x26half;':'½', '\x26hamilt;':'ℋ', '\x26hardcy;':'ъ', '\x26harr;':'↔', '\x26harrcir;':'⥈', '\x26harrw;':'↭', '\x26hbar;':'ℏ', '\x26hcirc;':'ĥ', '\x26hearts;':'♥', '\x26heartsuit;':'♥', '\x26hellip;':'…', '\x26hercon;':'⊹', '\x26hfr;':'\ud835\udd25', '\x26hksearow;':'⤥', '\x26hkswarow;':'⤦', '\x26hoarr;':'⇿', '\x26homtht;':'∻', '\x26hookleftarrow;':'↩', '\x26hookrightarrow;':'↪', '\x26hopf;':'\ud835\udd59', 
'\x26horbar;':'―', '\x26hscr;':'\ud835\udcbd', '\x26hslash;':'ℏ', '\x26hstrok;':'ħ', '\x26hybull;':'⁃', '\x26hyphen;':'‐', '\x26iacute':'í', '\x26iacute;':'í', '\x26ic;':'⁣', '\x26icirc':'î', '\x26icirc;':'î', '\x26icy;':'и', '\x26iecy;':'е', '\x26iexcl':'¡', '\x26iexcl;':'¡', '\x26iff;':'⇔', '\x26ifr;':'\ud835\udd26', '\x26igrave':'ì', '\x26igrave;':'ì', '\x26ii;':'ⅈ', '\x26iiiint;':'⨌', '\x26iiint;':'∭', '\x26iinfin;':'⧜', '\x26iiota;':'℩', '\x26ijlig;':'ĳ', '\x26imacr;':'ī', '\x26image;':'ℑ', 
'\x26imagline;':'ℐ', '\x26imagpart;':'ℑ', '\x26imath;':'ı', '\x26imof;':'⊷', '\x26imped;':'Ƶ', '\x26in;':'∈', '\x26incare;':'℅', '\x26infin;':'∞', '\x26infintie;':'⧝', '\x26inodot;':'ı', '\x26int;':'∫', '\x26intcal;':'⊺', '\x26integers;':'ℤ', '\x26intercal;':'⊺', '\x26intlarhk;':'⨗', '\x26intprod;':'⨼', '\x26iocy;':'ё', '\x26iogon;':'į', '\x26iopf;':'\ud835\udd5a', '\x26iota;':'ι', '\x26iprod;':'⨼', '\x26iquest':'¿', '\x26iquest;':'¿', '\x26iscr;':'\ud835\udcbe', '\x26isin;':'∈', '\x26isinE;':'⋹', 
'\x26isindot;':'⋵', '\x26isins;':'⋴', '\x26isinsv;':'⋳', '\x26isinv;':'∈', '\x26it;':'⁢', '\x26itilde;':'ĩ', '\x26iukcy;':'і', '\x26iuml':'ï', '\x26iuml;':'ï', '\x26jcirc;':'ĵ', '\x26jcy;':'й', '\x26jfr;':'\ud835\udd27', '\x26jmath;':'ȷ', '\x26jopf;':'\ud835\udd5b', '\x26jscr;':'\ud835\udcbf', '\x26jsercy;':'ј', '\x26jukcy;':'є', '\x26kappa;':'κ', '\x26kappav;':'ϰ', '\x26kcedil;':'ķ', '\x26kcy;':'к', '\x26kfr;':'\ud835\udd28', '\x26kgreen;':'ĸ', '\x26khcy;':'х', '\x26kjcy;':'ќ', '\x26kopf;':'\ud835\udd5c', 
'\x26kscr;':'\ud835\udcc0', '\x26lAarr;':'⇚', '\x26lArr;':'⇐', '\x26lAtail;':'⤛', '\x26lBarr;':'⤎', '\x26lE;':'≦', '\x26lEg;':'⪋', '\x26lHar;':'⥢', '\x26lacute;':'ĺ', '\x26laemptyv;':'⦴', '\x26lagran;':'ℒ', '\x26lambda;':'λ', '\x26lang;':'⟨', '\x26langd;':'⦑', '\x26langle;':'⟨', '\x26lap;':'⪅', '\x26laquo':'«', '\x26laquo;':'«', '\x26larr;':'←', '\x26larrb;':'⇤', '\x26larrbfs;':'⤟', '\x26larrfs;':'⤝', '\x26larrhk;':'↩', '\x26larrlp;':'↫', '\x26larrpl;':'⤹', '\x26larrsim;':'⥳', '\x26larrtl;':'↢', 
'\x26lat;':'⪫', '\x26latail;':'⤙', '\x26late;':'⪭', '\x26lates;':'⪭︀', '\x26lbarr;':'⤌', '\x26lbbrk;':'❲', '\x26lbrace;':'{', '\x26lbrack;':'[', '\x26lbrke;':'⦋', '\x26lbrksld;':'⦏', '\x26lbrkslu;':'⦍', '\x26lcaron;':'ľ', '\x26lcedil;':'ļ', '\x26lceil;':'⌈', '\x26lcub;':'{', '\x26lcy;':'л', '\x26ldca;':'⤶', '\x26ldquo;':'“', '\x26ldquor;':'„', '\x26ldrdhar;':'⥧', '\x26ldrushar;':'⥋', '\x26ldsh;':'↲', '\x26le;':'≤', '\x26leftarrow;':'←', '\x26leftarrowtail;':'↢', '\x26leftharpoondown;':'↽', '\x26leftharpoonup;':'↼', 
'\x26leftleftarrows;':'⇇', '\x26leftrightarrow;':'↔', '\x26leftrightarrows;':'⇆', '\x26leftrightharpoons;':'⇋', '\x26leftrightsquigarrow;':'↭', '\x26leftthreetimes;':'⋋', '\x26leg;':'⋚', '\x26leq;':'≤', '\x26leqq;':'≦', '\x26leqslant;':'⩽', '\x26les;':'⩽', '\x26lescc;':'⪨', '\x26lesdot;':'⩿', '\x26lesdoto;':'⪁', '\x26lesdotor;':'⪃', '\x26lesg;':'⋚︀', '\x26lesges;':'⪓', '\x26lessapprox;':'⪅', '\x26lessdot;':'⋖', '\x26lesseqgtr;':'⋚', '\x26lesseqqgtr;':'⪋', '\x26lessgtr;':'≶', '\x26lesssim;':'≲', '\x26lfisht;':'⥼', 
'\x26lfloor;':'⌊', '\x26lfr;':'\ud835\udd29', '\x26lg;':'≶', '\x26lgE;':'⪑', '\x26lhard;':'↽', '\x26lharu;':'↼', '\x26lharul;':'⥪', '\x26lhblk;':'▄', '\x26ljcy;':'љ', '\x26ll;':'≪', '\x26llarr;':'⇇', '\x26llcorner;':'⌞', '\x26llhard;':'⥫', '\x26lltri;':'◺', '\x26lmidot;':'ŀ', '\x26lmoust;':'⎰', '\x26lmoustache;':'⎰', '\x26lnE;':'≨', '\x26lnap;':'⪉', '\x26lnapprox;':'⪉', '\x26lne;':'⪇', '\x26lneq;':'⪇', '\x26lneqq;':'≨', '\x26lnsim;':'⋦', '\x26loang;':'⟬', '\x26loarr;':'⇽', '\x26lobrk;':'⟦', '\x26longleftarrow;':'⟵', 
'\x26longleftrightarrow;':'⟷', '\x26longmapsto;':'⟼', '\x26longrightarrow;':'⟶', '\x26looparrowleft;':'↫', '\x26looparrowright;':'↬', '\x26lopar;':'⦅', '\x26lopf;':'\ud835\udd5d', '\x26loplus;':'⨭', '\x26lotimes;':'⨴', '\x26lowast;':'∗', '\x26lowbar;':'_', '\x26loz;':'◊', '\x26lozenge;':'◊', '\x26lozf;':'⧫', '\x26lpar;':'(', '\x26lparlt;':'⦓', '\x26lrarr;':'⇆', '\x26lrcorner;':'⌟', '\x26lrhar;':'⇋', '\x26lrhard;':'⥭', '\x26lrm;':'‎', '\x26lrtri;':'⊿', '\x26lsaquo;':'‹', '\x26lscr;':'\ud835\udcc1', 
'\x26lsh;':'↰', '\x26lsim;':'≲', '\x26lsime;':'⪍', '\x26lsimg;':'⪏', '\x26lsqb;':'[', '\x26lsquo;':'‘', '\x26lsquor;':'‚', '\x26lstrok;':'ł', '\x26lt':'\x3c', '\x26lt;':'\x3c', '\x26ltcc;':'⪦', '\x26ltcir;':'⩹', '\x26ltdot;':'⋖', '\x26lthree;':'⋋', '\x26ltimes;':'⋉', '\x26ltlarr;':'⥶', '\x26ltquest;':'⩻', '\x26ltrPar;':'⦖', '\x26ltri;':'◃', '\x26ltrie;':'⊴', '\x26ltrif;':'◂', '\x26lurdshar;':'⥊', '\x26luruhar;':'⥦', '\x26lvertneqq;':'≨︀', '\x26lvnE;':'≨︀', '\x26mDDot;':'∺', '\x26macr':'¯', '\x26macr;':'¯', 
'\x26male;':'♂', '\x26malt;':'✠', '\x26maltese;':'✠', '\x26map;':'↦', '\x26mapsto;':'↦', '\x26mapstodown;':'↧', '\x26mapstoleft;':'↤', '\x26mapstoup;':'↥', '\x26marker;':'▮', '\x26mcomma;':'⨩', '\x26mcy;':'м', '\x26mdash;':'—', '\x26measuredangle;':'∡', '\x26mfr;':'\ud835\udd2a', '\x26mho;':'℧', '\x26micro':'µ', '\x26micro;':'µ', '\x26mid;':'∣', '\x26midast;':'*', '\x26midcir;':'⫰', '\x26middot':'·', '\x26middot;':'·', '\x26minus;':'−', '\x26minusb;':'⊟', '\x26minusd;':'∸', '\x26minusdu;':'⨪', '\x26mlcp;':'⫛', 
'\x26mldr;':'…', '\x26mnplus;':'∓', '\x26models;':'⊧', '\x26mopf;':'\ud835\udd5e', '\x26mp;':'∓', '\x26mscr;':'\ud835\udcc2', '\x26mstpos;':'∾', '\x26mu;':'μ', '\x26multimap;':'⊸', '\x26mumap;':'⊸', '\x26nGg;':'⋙̸', '\x26nGt;':'≫⃒', '\x26nGtv;':'≫̸', '\x26nLeftarrow;':'⇍', '\x26nLeftrightarrow;':'⇎', '\x26nLl;':'⋘̸', '\x26nLt;':'≪⃒', '\x26nLtv;':'≪̸', '\x26nRightarrow;':'⇏', '\x26nVDash;':'⊯', '\x26nVdash;':'⊮', '\x26nabla;':'∇', '\x26nacute;':'ń', '\x26nang;':'∠⃒', '\x26nap;':'≉', '\x26napE;':'⩰̸', 
'\x26napid;':'≋̸', '\x26napos;':'ŉ', '\x26napprox;':'≉', '\x26natur;':'♮', '\x26natural;':'♮', '\x26naturals;':'ℕ', '\x26nbsp':' ', '\x26nbsp;':' ', '\x26nbump;':'≎̸', '\x26nbumpe;':'≏̸', '\x26ncap;':'⩃', '\x26ncaron;':'ň', '\x26ncedil;':'ņ', '\x26ncong;':'≇', '\x26ncongdot;':'⩭̸', '\x26ncup;':'⩂', '\x26ncy;':'н', '\x26ndash;':'–', '\x26ne;':'≠', '\x26neArr;':'⇗', '\x26nearhk;':'⤤', '\x26nearr;':'↗', '\x26nearrow;':'↗', '\x26nedot;':'≐̸', '\x26nequiv;':'≢', '\x26nesear;':'⤨', '\x26nesim;':'≂̸', '\x26nexist;':'∄', 
'\x26nexists;':'∄', '\x26nfr;':'\ud835\udd2b', '\x26ngE;':'≧̸', '\x26nge;':'≱', '\x26ngeq;':'≱', '\x26ngeqq;':'≧̸', '\x26ngeqslant;':'⩾̸', '\x26nges;':'⩾̸', '\x26ngsim;':'≵', '\x26ngt;':'≯', '\x26ngtr;':'≯', '\x26nhArr;':'⇎', '\x26nharr;':'↮', '\x26nhpar;':'⫲', '\x26ni;':'∋', '\x26nis;':'⋼', '\x26nisd;':'⋺', '\x26niv;':'∋', '\x26njcy;':'њ', '\x26nlArr;':'⇍', '\x26nlE;':'≦̸', '\x26nlarr;':'↚', '\x26nldr;':'‥', '\x26nle;':'≰', '\x26nleftarrow;':'↚', '\x26nleftrightarrow;':'↮', '\x26nleq;':'≰', '\x26nleqq;':'≦̸', 
'\x26nleqslant;':'⩽̸', '\x26nles;':'⩽̸', '\x26nless;':'≮', '\x26nlsim;':'≴', '\x26nlt;':'≮', '\x26nltri;':'⋪', '\x26nltrie;':'⋬', '\x26nmid;':'∤', '\x26nopf;':'\ud835\udd5f', '\x26not':'¬', '\x26not;':'¬', '\x26notin;':'∉', '\x26notinE;':'⋹̸', '\x26notindot;':'⋵̸', '\x26notinva;':'∉', '\x26notinvb;':'⋷', '\x26notinvc;':'⋶', '\x26notni;':'∌', '\x26notniva;':'∌', '\x26notnivb;':'⋾', '\x26notnivc;':'⋽', '\x26npar;':'∦', '\x26nparallel;':'∦', '\x26nparsl;':'⫽⃥', '\x26npart;':'∂̸', '\x26npolint;':'⨔', 
'\x26npr;':'⊀', '\x26nprcue;':'⋠', '\x26npre;':'⪯̸', '\x26nprec;':'⊀', '\x26npreceq;':'⪯̸', '\x26nrArr;':'⇏', '\x26nrarr;':'↛', '\x26nrarrc;':'⤳̸', '\x26nrarrw;':'↝̸', '\x26nrightarrow;':'↛', '\x26nrtri;':'⋫', '\x26nrtrie;':'⋭', '\x26nsc;':'⊁', '\x26nsccue;':'⋡', '\x26nsce;':'⪰̸', '\x26nscr;':'\ud835\udcc3', '\x26nshortmid;':'∤', '\x26nshortparallel;':'∦', '\x26nsim;':'≁', '\x26nsime;':'≄', '\x26nsimeq;':'≄', '\x26nsmid;':'∤', '\x26nspar;':'∦', '\x26nsqsube;':'⋢', '\x26nsqsupe;':'⋣', '\x26nsub;':'⊄', 
'\x26nsubE;':'⫅̸', '\x26nsube;':'⊈', '\x26nsubset;':'⊂⃒', '\x26nsubseteq;':'⊈', '\x26nsubseteqq;':'⫅̸', '\x26nsucc;':'⊁', '\x26nsucceq;':'⪰̸', '\x26nsup;':'⊅', '\x26nsupE;':'⫆̸', '\x26nsupe;':'⊉', '\x26nsupset;':'⊃⃒', '\x26nsupseteq;':'⊉', '\x26nsupseteqq;':'⫆̸', '\x26ntgl;':'≹', '\x26ntilde':'ñ', '\x26ntilde;':'ñ', '\x26ntlg;':'≸', '\x26ntriangleleft;':'⋪', '\x26ntrianglelefteq;':'⋬', '\x26ntriangleright;':'⋫', '\x26ntrianglerighteq;':'⋭', '\x26nu;':'ν', '\x26num;':'#', '\x26numero;':'№', '\x26numsp;':' ', 
'\x26nvDash;':'⊭', '\x26nvHarr;':'⤄', '\x26nvap;':'≍⃒', '\x26nvdash;':'⊬', '\x26nvge;':'≥⃒', '\x26nvgt;':'\x3e⃒', '\x26nvinfin;':'⧞', '\x26nvlArr;':'⤂', '\x26nvle;':'≤⃒', '\x26nvlt;':'\x3c⃒', '\x26nvltrie;':'⊴⃒', '\x26nvrArr;':'⤃', '\x26nvrtrie;':'⊵⃒', '\x26nvsim;':'∼⃒', '\x26nwArr;':'⇖', '\x26nwarhk;':'⤣', '\x26nwarr;':'↖', '\x26nwarrow;':'↖', '\x26nwnear;':'⤧', '\x26oS;':'Ⓢ', '\x26oacute':'ó', '\x26oacute;':'ó', '\x26oast;':'⊛', '\x26ocir;':'⊚', '\x26ocirc':'ô', '\x26ocirc;':'ô', '\x26ocy;':'о', 
'\x26odash;':'⊝', '\x26odblac;':'ő', '\x26odiv;':'⨸', '\x26odot;':'⊙', '\x26odsold;':'⦼', '\x26oelig;':'œ', '\x26ofcir;':'⦿', '\x26ofr;':'\ud835\udd2c', '\x26ogon;':'˛', '\x26ograve':'ò', '\x26ograve;':'ò', '\x26ogt;':'⧁', '\x26ohbar;':'⦵', '\x26ohm;':'Ω', '\x26oint;':'∮', '\x26olarr;':'↺', '\x26olcir;':'⦾', '\x26olcross;':'⦻', '\x26oline;':'‾', '\x26olt;':'⧀', '\x26omacr;':'ō', '\x26omega;':'ω', '\x26omicron;':'ο', '\x26omid;':'⦶', '\x26ominus;':'⊖', '\x26oopf;':'\ud835\udd60', '\x26opar;':'⦷', 
'\x26operp;':'⦹', '\x26oplus;':'⊕', '\x26or;':'∨', '\x26orarr;':'↻', '\x26ord;':'⩝', '\x26order;':'ℴ', '\x26orderof;':'ℴ', '\x26ordf':'ª', '\x26ordf;':'ª', '\x26ordm':'º', '\x26ordm;':'º', '\x26origof;':'⊶', '\x26oror;':'⩖', '\x26orslope;':'⩗', '\x26orv;':'⩛', '\x26oscr;':'ℴ', '\x26oslash':'ø', '\x26oslash;':'ø', '\x26osol;':'⊘', '\x26otilde':'õ', '\x26otilde;':'õ', '\x26otimes;':'⊗', '\x26otimesas;':'⨶', '\x26ouml':'ö', '\x26ouml;':'ö', '\x26ovbar;':'⌽', '\x26par;':'∥', '\x26para':'¶', '\x26para;':'¶', 
'\x26parallel;':'∥', '\x26parsim;':'⫳', '\x26parsl;':'⫽', '\x26part;':'∂', '\x26pcy;':'п', '\x26percnt;':'%', '\x26period;':'.', '\x26permil;':'‰', '\x26perp;':'⊥', '\x26pertenk;':'‱', '\x26pfr;':'\ud835\udd2d', '\x26phi;':'φ', '\x26phiv;':'ϕ', '\x26phmmat;':'ℳ', '\x26phone;':'☎', '\x26pi;':'π', '\x26pitchfork;':'⋔', '\x26piv;':'ϖ', '\x26planck;':'ℏ', '\x26planckh;':'ℎ', '\x26plankv;':'ℏ', '\x26plus;':'+', '\x26plusacir;':'⨣', '\x26plusb;':'⊞', '\x26pluscir;':'⨢', '\x26plusdo;':'∔', '\x26plusdu;':'⨥', 
'\x26pluse;':'⩲', '\x26plusmn':'±', '\x26plusmn;':'±', '\x26plussim;':'⨦', '\x26plustwo;':'⨧', '\x26pm;':'±', '\x26pointint;':'⨕', '\x26popf;':'\ud835\udd61', '\x26pound':'£', '\x26pound;':'£', '\x26pr;':'≺', '\x26prE;':'⪳', '\x26prap;':'⪷', '\x26prcue;':'≼', '\x26pre;':'⪯', '\x26prec;':'≺', '\x26precapprox;':'⪷', '\x26preccurlyeq;':'≼', '\x26preceq;':'⪯', '\x26precnapprox;':'⪹', '\x26precneqq;':'⪵', '\x26precnsim;':'⋨', '\x26precsim;':'≾', '\x26prime;':'′', '\x26primes;':'ℙ', '\x26prnE;':'⪵', '\x26prnap;':'⪹', 
'\x26prnsim;':'⋨', '\x26prod;':'∏', '\x26profalar;':'⌮', '\x26profline;':'⌒', '\x26profsurf;':'⌓', '\x26prop;':'∝', '\x26propto;':'∝', '\x26prsim;':'≾', '\x26prurel;':'⊰', '\x26pscr;':'\ud835\udcc5', '\x26psi;':'ψ', '\x26puncsp;':' ', '\x26qfr;':'\ud835\udd2e', '\x26qint;':'⨌', '\x26qopf;':'\ud835\udd62', '\x26qprime;':'⁗', '\x26qscr;':'\ud835\udcc6', '\x26quaternions;':'ℍ', '\x26quatint;':'⨖', '\x26quest;':'?', '\x26questeq;':'≟', '\x26quot':'"', '\x26quot;':'"', '\x26rAarr;':'⇛', '\x26rArr;':'⇒', 
'\x26rAtail;':'⤜', '\x26rBarr;':'⤏', '\x26rHar;':'⥤', '\x26race;':'∽̱', '\x26racute;':'ŕ', '\x26radic;':'√', '\x26raemptyv;':'⦳', '\x26rang;':'⟩', '\x26rangd;':'⦒', '\x26range;':'⦥', '\x26rangle;':'⟩', '\x26raquo':'»', '\x26raquo;':'»', '\x26rarr;':'→', '\x26rarrap;':'⥵', '\x26rarrb;':'⇥', '\x26rarrbfs;':'⤠', '\x26rarrc;':'⤳', '\x26rarrfs;':'⤞', '\x26rarrhk;':'↪', '\x26rarrlp;':'↬', '\x26rarrpl;':'⥅', '\x26rarrsim;':'⥴', '\x26rarrtl;':'↣', '\x26rarrw;':'↝', '\x26ratail;':'⤚', '\x26ratio;':'∶', '\x26rationals;':'ℚ', 
'\x26rbarr;':'⤍', '\x26rbbrk;':'❳', '\x26rbrace;':'}', '\x26rbrack;':']', '\x26rbrke;':'⦌', '\x26rbrksld;':'⦎', '\x26rbrkslu;':'⦐', '\x26rcaron;':'ř', '\x26rcedil;':'ŗ', '\x26rceil;':'⌉', '\x26rcub;':'}', '\x26rcy;':'р', '\x26rdca;':'⤷', '\x26rdldhar;':'⥩', '\x26rdquo;':'”', '\x26rdquor;':'”', '\x26rdsh;':'↳', '\x26real;':'ℜ', '\x26realine;':'ℛ', '\x26realpart;':'ℜ', '\x26reals;':'ℝ', '\x26rect;':'▭', '\x26reg':'®', '\x26reg;':'®', '\x26rfisht;':'⥽', '\x26rfloor;':'⌋', '\x26rfr;':'\ud835\udd2f', 
'\x26rhard;':'⇁', '\x26rharu;':'⇀', '\x26rharul;':'⥬', '\x26rho;':'ρ', '\x26rhov;':'ϱ', '\x26rightarrow;':'→', '\x26rightarrowtail;':'↣', '\x26rightharpoondown;':'⇁', '\x26rightharpoonup;':'⇀', '\x26rightleftarrows;':'⇄', '\x26rightleftharpoons;':'⇌', '\x26rightrightarrows;':'⇉', '\x26rightsquigarrow;':'↝', '\x26rightthreetimes;':'⋌', '\x26ring;':'˚', '\x26risingdotseq;':'≓', '\x26rlarr;':'⇄', '\x26rlhar;':'⇌', '\x26rlm;':'‏', '\x26rmoust;':'⎱', '\x26rmoustache;':'⎱', '\x26rnmid;':'⫮', '\x26roang;':'⟭', 
'\x26roarr;':'⇾', '\x26robrk;':'⟧', '\x26ropar;':'⦆', '\x26ropf;':'\ud835\udd63', '\x26roplus;':'⨮', '\x26rotimes;':'⨵', '\x26rpar;':')', '\x26rpargt;':'⦔', '\x26rppolint;':'⨒', '\x26rrarr;':'⇉', '\x26rsaquo;':'›', '\x26rscr;':'\ud835\udcc7', '\x26rsh;':'↱', '\x26rsqb;':']', '\x26rsquo;':'’', '\x26rsquor;':'’', '\x26rthree;':'⋌', '\x26rtimes;':'⋊', '\x26rtri;':'▹', '\x26rtrie;':'⊵', '\x26rtrif;':'▸', '\x26rtriltri;':'⧎', '\x26ruluhar;':'⥨', '\x26rx;':'℞', '\x26sacute;':'ś', '\x26sbquo;':'‚', '\x26sc;':'≻', 
'\x26scE;':'⪴', '\x26scap;':'⪸', '\x26scaron;':'š', '\x26sccue;':'≽', '\x26sce;':'⪰', '\x26scedil;':'ş', '\x26scirc;':'ŝ', '\x26scnE;':'⪶', '\x26scnap;':'⪺', '\x26scnsim;':'⋩', '\x26scpolint;':'⨓', '\x26scsim;':'≿', '\x26scy;':'с', '\x26sdot;':'⋅', '\x26sdotb;':'⊡', '\x26sdote;':'⩦', '\x26seArr;':'⇘', '\x26searhk;':'⤥', '\x26searr;':'↘', '\x26searrow;':'↘', '\x26sect':'§', '\x26sect;':'§', '\x26semi;':';', '\x26seswar;':'⤩', '\x26setminus;':'∖', '\x26setmn;':'∖', '\x26sext;':'✶', '\x26sfr;':'\ud835\udd30', 
'\x26sfrown;':'⌢', '\x26sharp;':'♯', '\x26shchcy;':'щ', '\x26shcy;':'ш', '\x26shortmid;':'∣', '\x26shortparallel;':'∥', '\x26shy':'­', '\x26shy;':'­', '\x26sigma;':'σ', '\x26sigmaf;':'ς', '\x26sigmav;':'ς', '\x26sim;':'∼', '\x26simdot;':'⩪', '\x26sime;':'≃', '\x26simeq;':'≃', '\x26simg;':'⪞', '\x26simgE;':'⪠', '\x26siml;':'⪝', '\x26simlE;':'⪟', '\x26simne;':'≆', '\x26simplus;':'⨤', '\x26simrarr;':'⥲', '\x26slarr;':'←', '\x26smallsetminus;':'∖', '\x26smashp;':'⨳', '\x26smeparsl;':'⧤', '\x26smid;':'∣', 
'\x26smile;':'⌣', '\x26smt;':'⪪', '\x26smte;':'⪬', '\x26smtes;':'⪬︀', '\x26softcy;':'ь', '\x26sol;':'/', '\x26solb;':'⧄', '\x26solbar;':'⌿', '\x26sopf;':'\ud835\udd64', '\x26spades;':'♠', '\x26spadesuit;':'♠', '\x26spar;':'∥', '\x26sqcap;':'⊓', '\x26sqcaps;':'⊓︀', '\x26sqcup;':'⊔', '\x26sqcups;':'⊔︀', '\x26sqsub;':'⊏', '\x26sqsube;':'⊑', '\x26sqsubset;':'⊏', '\x26sqsubseteq;':'⊑', '\x26sqsup;':'⊐', '\x26sqsupe;':'⊒', '\x26sqsupset;':'⊐', '\x26sqsupseteq;':'⊒', '\x26squ;':'□', '\x26square;':'□', '\x26squarf;':'▪', 
'\x26squf;':'▪', '\x26srarr;':'→', '\x26sscr;':'\ud835\udcc8', '\x26ssetmn;':'∖', '\x26ssmile;':'⌣', '\x26sstarf;':'⋆', '\x26star;':'☆', '\x26starf;':'★', '\x26straightepsilon;':'ϵ', '\x26straightphi;':'ϕ', '\x26strns;':'¯', '\x26sub;':'⊂', '\x26subE;':'⫅', '\x26subdot;':'⪽', '\x26sube;':'⊆', '\x26subedot;':'⫃', '\x26submult;':'⫁', '\x26subnE;':'⫋', '\x26subne;':'⊊', '\x26subplus;':'⪿', '\x26subrarr;':'⥹', '\x26subset;':'⊂', '\x26subseteq;':'⊆', '\x26subseteqq;':'⫅', '\x26subsetneq;':'⊊', '\x26subsetneqq;':'⫋', 
'\x26subsim;':'⫇', '\x26subsub;':'⫕', '\x26subsup;':'⫓', '\x26succ;':'≻', '\x26succapprox;':'⪸', '\x26succcurlyeq;':'≽', '\x26succeq;':'⪰', '\x26succnapprox;':'⪺', '\x26succneqq;':'⪶', '\x26succnsim;':'⋩', '\x26succsim;':'≿', '\x26sum;':'∑', '\x26sung;':'♪', '\x26sup1':'¹', '\x26sup1;':'¹', '\x26sup2':'²', '\x26sup2;':'²', '\x26sup3':'³', '\x26sup3;':'³', '\x26sup;':'⊃', '\x26supE;':'⫆', '\x26supdot;':'⪾', '\x26supdsub;':'⫘', '\x26supe;':'⊇', '\x26supedot;':'⫄', '\x26suphsol;':'⟉', '\x26suphsub;':'⫗', 
'\x26suplarr;':'⥻', '\x26supmult;':'⫂', '\x26supnE;':'⫌', '\x26supne;':'⊋', '\x26supplus;':'⫀', '\x26supset;':'⊃', '\x26supseteq;':'⊇', '\x26supseteqq;':'⫆', '\x26supsetneq;':'⊋', '\x26supsetneqq;':'⫌', '\x26supsim;':'⫈', '\x26supsub;':'⫔', '\x26supsup;':'⫖', '\x26swArr;':'⇙', '\x26swarhk;':'⤦', '\x26swarr;':'↙', '\x26swarrow;':'↙', '\x26swnwar;':'⤪', '\x26szlig':'ß', '\x26szlig;':'ß', '\x26target;':'⌖', '\x26tau;':'τ', '\x26tbrk;':'⎴', '\x26tcaron;':'ť', '\x26tcedil;':'ţ', '\x26tcy;':'т', '\x26tdot;':'⃛', 
'\x26telrec;':'⌕', '\x26tfr;':'\ud835\udd31', '\x26there4;':'∴', '\x26therefore;':'∴', '\x26theta;':'θ', '\x26thetasym;':'ϑ', '\x26thetav;':'ϑ', '\x26thickapprox;':'≈', '\x26thicksim;':'∼', '\x26thinsp;':' ', '\x26thkap;':'≈', '\x26thksim;':'∼', '\x26thorn':'þ', '\x26thorn;':'þ', '\x26tilde;':'˜', '\x26times':'×', '\x26times;':'×', '\x26timesb;':'⊠', '\x26timesbar;':'⨱', '\x26timesd;':'⨰', '\x26tint;':'∭', '\x26toea;':'⤨', '\x26top;':'⊤', '\x26topbot;':'⌶', '\x26topcir;':'⫱', '\x26topf;':'\ud835\udd65', 
'\x26topfork;':'⫚', '\x26tosa;':'⤩', '\x26tprime;':'‴', '\x26trade;':'™', '\x26triangle;':'▵', '\x26triangledown;':'▿', '\x26triangleleft;':'◃', '\x26trianglelefteq;':'⊴', '\x26triangleq;':'≜', '\x26triangleright;':'▹', '\x26trianglerighteq;':'⊵', '\x26tridot;':'◬', '\x26trie;':'≜', '\x26triminus;':'⨺', '\x26triplus;':'⨹', '\x26trisb;':'⧍', '\x26tritime;':'⨻', '\x26trpezium;':'⏢', '\x26tscr;':'\ud835\udcc9', '\x26tscy;':'ц', '\x26tshcy;':'ћ', '\x26tstrok;':'ŧ', '\x26twixt;':'≬', '\x26twoheadleftarrow;':'↞', 
'\x26twoheadrightarrow;':'↠', '\x26uArr;':'⇑', '\x26uHar;':'⥣', '\x26uacute':'ú', '\x26uacute;':'ú', '\x26uarr;':'↑', '\x26ubrcy;':'ў', '\x26ubreve;':'ŭ', '\x26ucirc':'û', '\x26ucirc;':'û', '\x26ucy;':'у', '\x26udarr;':'⇅', '\x26udblac;':'ű', '\x26udhar;':'⥮', '\x26ufisht;':'⥾', '\x26ufr;':'\ud835\udd32', '\x26ugrave':'ù', '\x26ugrave;':'ù', '\x26uharl;':'↿', '\x26uharr;':'↾', '\x26uhblk;':'▀', '\x26ulcorn;':'⌜', '\x26ulcorner;':'⌜', '\x26ulcrop;':'⌏', '\x26ultri;':'◸', '\x26umacr;':'ū', '\x26uml':'¨', 
'\x26uml;':'¨', '\x26uogon;':'ų', '\x26uopf;':'\ud835\udd66', '\x26uparrow;':'↑', '\x26updownarrow;':'↕', '\x26upharpoonleft;':'↿', '\x26upharpoonright;':'↾', '\x26uplus;':'⊎', '\x26upsi;':'υ', '\x26upsih;':'ϒ', '\x26upsilon;':'υ', '\x26upuparrows;':'⇈', '\x26urcorn;':'⌝', '\x26urcorner;':'⌝', '\x26urcrop;':'⌎', '\x26uring;':'ů', '\x26urtri;':'◹', '\x26uscr;':'\ud835\udcca', '\x26utdot;':'⋰', '\x26utilde;':'ũ', '\x26utri;':'▵', '\x26utrif;':'▴', '\x26uuarr;':'⇈', '\x26uuml':'ü', '\x26uuml;':'ü', 
'\x26uwangle;':'⦧', '\x26vArr;':'⇕', '\x26vBar;':'⫨', '\x26vBarv;':'⫩', '\x26vDash;':'⊨', '\x26vangrt;':'⦜', '\x26varepsilon;':'ϵ', '\x26varkappa;':'ϰ', '\x26varnothing;':'∅', '\x26varphi;':'ϕ', '\x26varpi;':'ϖ', '\x26varpropto;':'∝', '\x26varr;':'↕', '\x26varrho;':'ϱ', '\x26varsigma;':'ς', '\x26varsubsetneq;':'⊊︀', '\x26varsubsetneqq;':'⫋︀', '\x26varsupsetneq;':'⊋︀', '\x26varsupsetneqq;':'⫌︀', '\x26vartheta;':'ϑ', '\x26vartriangleleft;':'⊲', '\x26vartriangleright;':'⊳', '\x26vcy;':'в', '\x26vdash;':'⊢', 
'\x26vee;':'∨', '\x26veebar;':'⊻', '\x26veeeq;':'≚', '\x26vellip;':'⋮', '\x26verbar;':'|', '\x26vert;':'|', '\x26vfr;':'\ud835\udd33', '\x26vltri;':'⊲', '\x26vnsub;':'⊂⃒', '\x26vnsup;':'⊃⃒', '\x26vopf;':'\ud835\udd67', '\x26vprop;':'∝', '\x26vrtri;':'⊳', '\x26vscr;':'\ud835\udccb', '\x26vsubnE;':'⫋︀', '\x26vsubne;':'⊊︀', '\x26vsupnE;':'⫌︀', '\x26vsupne;':'⊋︀', '\x26vzigzag;':'⦚', '\x26wcirc;':'ŵ', '\x26wedbar;':'⩟', '\x26wedge;':'∧', '\x26wedgeq;':'≙', '\x26weierp;':'℘', '\x26wfr;':'\ud835\udd34', 
'\x26wopf;':'\ud835\udd68', '\x26wp;':'℘', '\x26wr;':'≀', '\x26wreath;':'≀', '\x26wscr;':'\ud835\udccc', '\x26xcap;':'⋂', '\x26xcirc;':'◯', '\x26xcup;':'⋃', '\x26xdtri;':'▽', '\x26xfr;':'\ud835\udd35', '\x26xhArr;':'⟺', '\x26xharr;':'⟷', '\x26xi;':'ξ', '\x26xlArr;':'⟸', '\x26xlarr;':'⟵', '\x26xmap;':'⟼', '\x26xnis;':'⋻', '\x26xodot;':'⨀', '\x26xopf;':'\ud835\udd69', '\x26xoplus;':'⨁', '\x26xotime;':'⨂', '\x26xrArr;':'⟹', '\x26xrarr;':'⟶', '\x26xscr;':'\ud835\udccd', '\x26xsqcup;':'⨆', '\x26xuplus;':'⨄', 
'\x26xutri;':'△', '\x26xvee;':'⋁', '\x26xwedge;':'⋀', '\x26yacute':'ý', '\x26yacute;':'ý', '\x26yacy;':'я', '\x26ycirc;':'ŷ', '\x26ycy;':'ы', '\x26yen':'¥', '\x26yen;':'¥', '\x26yfr;':'\ud835\udd36', '\x26yicy;':'ї', '\x26yopf;':'\ud835\udd6a', '\x26yscr;':'\ud835\udcce', '\x26yucy;':'ю', '\x26yuml':'ÿ', '\x26yuml;':'ÿ', '\x26zacute;':'ź', '\x26zcaron;':'ž', '\x26zcy;':'з', '\x26zdot;':'ż', '\x26zeetrf;':'ℨ', '\x26zeta;':'ζ', '\x26zfr;':'\ud835\udd37', '\x26zhcy;':'ж', '\x26zigrarr;':'⇝', '\x26zopf;':'\ud835\udd6b', 
'\x26zscr;':'\ud835\udccf', '\x26zwj;':'‍', '\x26zwnj;':'‌'}});
Ext.define('documents.parser.mime', {requires:['documents.parser.util'], htmlToText:function(str) {
  str = str.replace(/\r?\n/g, '').replace(/<!\-\-.*?\-\->/gi, ' ').replace(/<br\b[^>]*>/gi, '\n').replace(/<\/?(p|div|table|tr|td|th)\b[^>]*>/gi, '\n\n').replace(/<script\b[^>]*>.*?<\/script\b[^>]*>/gi, ' ').replace(/^.*<body\b[^>]*>/i, '').replace(/^.*<\/head\b[^>]*>/i, '').replace(/^.*<!doctype\b[^>]*>/i, '').replace(/<\/body\b[^>]*>.*$/i, '').replace(/<\/html\b[^>]*>.*$/i, '').replace(/<a\b[^>]*href\s*=\s*["']?([^\s"']+)[^>]*>/gi, ' ($1) ').replace(/<\/?(span|em|i|strong|b|u|a)\b[^>]*>/gi, '').replace(/<li\b[^>]*>[\n\u0001\s]*/gi, 
  '* ').replace(/<hr\b[^>]*>/g, '\n-------------\n').replace(/<[^>]*>/g, ' ').replace(/\u0001/g, '\n').replace(/[ \t]+/g, ' ').replace(/^\s+$/gm, '').replace(/\n\n+/g, '\n\n').replace(/^\n+/, '\n').replace(/\n+$/, '\n');
  str = this.decodeHTMLEntities(str);
  return str;
}, textToHtml:function(str) {
  var html = this.escapeHtml(str).replace(/\n/g, '\x3cbr /\x3e');
  return '\x3cdiv\x3e' + html + '\x3c/div\x3e';
}, escapeHtml:function(str) {
  return str.trim().replace(/[<>"'?&]/g, function(c) {
    var hex = c.charCodeAt(0).toString(16);
    if (hex.length < 2) {
      hex = '0' + hex;
    }
    return '\x26#x' + hex.toUpperCase() + ';';
  });
}, decodeHTMLEntities:function(str) {
  return str.replace(/&(#\d+|#x[a-f0-9]+|[a-z]+\d*);?/gi, function(match, entity) {
    if (typeof documents.parser.util.htmlEntities[match] === 'string') {
      return documents.parser.util.htmlEntities[match];
    }
    if (entity.charAt(0) !== '#' || match.charAt(match.length - 1) !== ';') {
      return match;
    }
    var codePoint;
    if (entity.charAt(1) === 'x') {
      codePoint = parseInt(entity.substr(2), 16);
    } else {
      codePoint = parseInt(entity.substr(1), 10);
    }
    var output = '';
    if (codePoint >= 55296 && codePoint <= 57343 || codePoint > 1114111) {
      return '�';
    }
    if (codePoint > 65535) {
      codePoint -= 65536;
      output += String.fromCharCode(codePoint >>> 10 & 1023 | 55296);
      codePoint = 56320 | codePoint & 1023;
    }
    output += String.fromCharCode(codePoint);
    return output;
  });
}, constructor:function(config) {
  this.textEncoder = new TextEncoder;
  this.root = this.currentNode = Ext.create('documents.parser.node', {postalMime:this});
  this.boundaries = [];
  this.textContent = {};
  this.attachments = [];
  this.started = false;
  this.callParent(arguments);
}, finalize:function() {
  var $jscomp$async$this = this;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    return $jscomp$generator$context.yield($jscomp$async$this.root.finalize(), 0);
  });
}, processLine:function(line, isFinal) {
  var $jscomp$async$this = this;
  var boundaries;
  var i$25;
  var boundary;
  var isTerminator;
  var boudaryMatches;
  var i$26;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    switch($jscomp$generator$context.nextAddress) {
      case 1:
        boundaries = $jscomp$async$this.boundaries;
        if (!(boundaries.length && line.length > 2 && line[0] === 45 && line[1] === 45)) {
          $jscomp$generator$context.jumpTo(2);
          break;
        }
        i$25 = boundaries.length - 1;
      case 3:
        if (!(i$25 >= 0)) {
          $jscomp$generator$context.jumpTo(2);
          break;
        }
        boundary = boundaries[i$25];
        if (line.length !== boundary.value.length + 2 && line.length !== boundary.value.length + 4) {
          $jscomp$generator$context.jumpTo(4);
          break;
        }
        isTerminator = line.length === boundary.value.length + 4;
        if (isTerminator && (line[line.length - 2] !== 45 || line[line.length - 1] !== 45)) {
          $jscomp$generator$context.jumpTo(4);
          break;
        }
        boudaryMatches = true;
        for (i$26 = 0; i$26 < boundary.value.length; i$26++) {
          if (line[i$26 + 2] !== boundary.value[i$26]) {
            boudaryMatches = false;
            break;
          }
        }
        if (!boudaryMatches) {
          $jscomp$generator$context.jumpTo(4);
          break;
        }
        if (isTerminator) {
          return $jscomp$generator$context.yield(boundary.node.finalize(), 9);
        }
        return $jscomp$generator$context.yield(boundary.node.finalizeChildNodes(), 8);
      case 8:
        $jscomp$async$this.currentNode = Ext.create('documents.parser.node', {postalMime:$jscomp$async$this, parentNode:boundary.node});
        $jscomp$generator$context.jumpTo(7);
        break;
      case 9:
        $jscomp$async$this.currentNode = boundary.node.parentNode || $jscomp$async$this.root;
      case 7:
        if (isFinal) {
          return $jscomp$generator$context['return']($jscomp$async$this.finalize());
        }
        return $jscomp$generator$context['return']();
      case 4:
        i$25--;
        $jscomp$generator$context.jumpTo(3);
        break;
      case 2:
        $jscomp$async$this.currentNode.feed(line);
        if (isFinal) {
          return $jscomp$generator$context['return']($jscomp$async$this.finalize());
        }
        $jscomp$generator$context.jumpToEnd();
    }
  });
}, readLine:function() {
  var $jscomp$this = this;
  var startPos = this.readPos;
  var endPos = this.readPos;
  var res = function() {
    return {bytes:new Uint8Array($jscomp$this.buf, startPos, endPos - startPos), done:$jscomp$this.readPos >= $jscomp$this.av.length};
  };
  while (this.readPos < this.av.length) {
    var c = this.av[this.readPos++];
    if (c !== 13 && c !== 10) {
      endPos = this.readPos;
    }
    if (c === 10) {
      return res();
    }
  }
  return res();
}, processNodeTree:function() {
  var $jscomp$this = this;
  var textContent = {};
  var textTypes = new Set;
  var textMap = this.textMap = new Map;
  var walk = function(node, alternative, related) {
    alternative = alternative || false;
    related = related || false;
    if (!node.contentType.multipart) {
      if (node.contentType.parsed.value === 'message/rfc822' && node.contentDisposition.parsed.value !== 'attachment') {
        var subParser = Ext.create('documents.parser.mime');
        node.subMessage = subParser.parse(node.content);
        if (!textMap.has(node)) {
          textMap.set(node, {});
        }
        var textEntry = textMap.get(node);
        if (node.subMessage.text || !node.subMessage.html) {
          textEntry.plain = textEntry.plain || [];
          textEntry.plain.push({type:'subMessage', value:node.subMessage});
          textTypes.add('plain');
        }
        if (node.subMessage.html) {
          textEntry.html = textEntry.html || [];
          textEntry.html.push({type:'subMessage', value:node.subMessage});
          textTypes.add('html');
        }
        if (subParser.textMap) {
          subParser.textMap.forEach(function(subTextEntry, subTextNode) {
            textMap.set(subTextNode, subTextEntry);
          });
        }
        for (var $jscomp$iter$1 = $jscomp.makeIterator(node.subMessage.attachments || []), $jscomp$key$attachment = $jscomp$iter$1.next(); !$jscomp$key$attachment.done; $jscomp$key$attachment = $jscomp$iter$1.next()) {
          var attachment = $jscomp$key$attachment.value;
          {
            $jscomp$this.attachments.push(attachment);
          }
        }
      } else {
        if ((/^text\//i.test(node.contentType.parsed.value) || node.contentType.parsed.value === 'message/delivery-status') && node.contentDisposition.parsed.value !== 'attachment') {
          var textType = node.contentType.parsed.value.substr(node.contentType.parsed.value.indexOf('/') + 1);
          if (node.contentType.parsed.value === 'message/delivery-status') {
            textType = 'plain';
          }
          var selectorNode = alternative || node;
          if (!textMap.has(selectorNode)) {
            textMap.set(selectorNode, {});
          }
          var textEntry$27 = textMap.get(selectorNode);
          textEntry$27[textType] = textEntry$27[textType] || [];
          textEntry$27[textType].push({type:'text', value:node.getTextContent()});
          textTypes.add(textType);
        } else {
          if (node.content) {
            var filename = node.contentDisposition.parsed.params.filename || node.contentType.parsed.params.name || null;
            var attachment$28 = {filename:documents.parser.util.decodeWords(filename), mimeType:node.contentType.parsed.value, disposition:node.contentDisposition.parsed.value || null};
            if (related && node.contentId) {
              attachment$28.related = true;
            }
            if (node.contentId) {
              attachment$28.contentId = node.contentId;
            }
            attachment$28.content = node.content;
            $jscomp$this.attachments.push(attachment$28);
          }
        }
      }
    } else {
      if (node.contentType.multipart === 'alternative') {
        alternative = node;
      } else {
        if (node.contentType.multipart === 'related') {
          related = node;
        }
      }
    }
    for (var $jscomp$iter$2 = $jscomp.makeIterator(node.childNodes), $jscomp$key$childNode = $jscomp$iter$2.next(); !$jscomp$key$childNode.done; $jscomp$key$childNode = $jscomp$iter$2.next()) {
      var childNode = $jscomp$key$childNode.value;
      {
        walk(childNode, alternative, related);
      }
    }
  };
  walk(this.root, false, []);
  textMap.forEach(function(mapEntry) {
    textTypes.forEach(function(textType) {
      if (!textContent[textType]) {
        textContent[textType] = [];
      }
      if (mapEntry[textType]) {
        mapEntry[textType].forEach(function(textEntry) {
          switch(textEntry.type) {
            case 'text':
              textContent[textType].push(textEntry.value);
              break;
            case 'subMessage':
              {
                switch(textType) {
                  case 'html':
                    textContent[textType].push(documents.parser.util.formatHtmlHeader(textEntry.value));
                    break;
                  case 'plain':
                    textContent[textType].push(documents.parser.util.formatTextHeader(textEntry.value));
                    break;
                }
              }
              break;
          }
        });
      } else {
        var alternativeType;
        switch(textType) {
          case 'html':
            alternativeType = 'plain';
            break;
          case 'plain':
            alternativeType = 'html';
            break;
        }
        (mapEntry[alternativeType] || []).forEach(function(textEntry) {
          switch(textEntry.type) {
            case 'text':
              switch(textType) {
                case 'html':
                  textContent[textType].push(documents.parser.util.textToHtml(textEntry.value));
                  break;
                case 'plain':
                  textContent[textType].push(documents.parser.util.htmlToText(textEntry.value));
                  break;
              }break;
            case 'subMessage':
              {
                switch(textType) {
                  case 'html':
                    textContent[textType].push(documents.parser.util.formatHtmlHeader(textEntry.value));
                    break;
                  case 'plain':
                    textContent[textType].push(documents.parser.util.formatTextHeader(textEntry.value));
                    break;
                }
              }
              break;
          }
        });
      }
    });
  });
  Object.keys(textContent).forEach(function(textType) {
    textContent[textType] = textContent[textType].join('\n');
  });
  this.textContent = textContent;
}, parse:function(buf) {
  var $jscomp$async$this = this;
  var line;
  var message;
  var $jscomp$loop$44;
  var $jscomp$iter$3, $jscomp$key$key;
  var addressHeader;
  var addresses;
  var $jscomp$loop$45;
  var $jscomp$iter$4, $jscomp$key$key;
  var addressHeader$30;
  var addresses$31;
  var $jscomp$loop$46;
  var $jscomp$iter$5, $jscomp$key$key;
  var addressHeaders;
  var $jscomp$loop$47;
  var $jscomp$iter$6, $jscomp$key$key;
  var header;
  var dateHeader;
  var date;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$generator$context.nextAddress == 1) {
      if ($jscomp$async$this.started) {
        throw new Error('Can not reuse parser, create a new PostalMime object');
      }
      $jscomp$async$this.started = true;
      buf = buf || ArrayBuffer(0);
      if (typeof buf === 'string') {
        buf = $jscomp$async$this.textEncoder.encode(buf);
      }
      if (buf instanceof Blob || Object.prototype.toString.call(buf) === '[object Blob]') {
        buf = blobToArrayBuffer(buf);
      }
      if (buf.buffer instanceof ArrayBuffer) {
        buf = (new Uint8Array(buf)).buffer;
      }
      $jscomp$async$this.buf = buf;
      $jscomp$async$this.av = new Uint8Array(buf);
      $jscomp$async$this.readPos = 0;
    }
    if ($jscomp$generator$context.nextAddress != 5) {
      if (!($jscomp$async$this.readPos < $jscomp$async$this.av.length)) {
        return $jscomp$generator$context.yield($jscomp$async$this.processNodeTree(), 5);
      }
      line = $jscomp$async$this.readLine();
      return $jscomp$generator$context.yield($jscomp$async$this.processLine(line.bytes, line.done), 2);
    }
    message = {headers:$jscomp$async$this.root.headers.map(function(entry) {
      return {key:entry.key, value:entry.value};
    }).reverse()};
    $jscomp$loop$44 = {};
    $jscomp$iter$3 = $jscomp.makeIterator(['from', 'sender', 'reply-to']), $jscomp$key$key = $jscomp$iter$3.next();
    for (; !$jscomp$key$key.done; $jscomp$loop$44 = {key:$jscomp$loop$44.key}, $jscomp$key$key = $jscomp$iter$3.next()) {
      $jscomp$loop$44.key = $jscomp$key$key.value;
      {
        addressHeader = $jscomp$async$this.root.headers.find(function($jscomp$loop$44) {
          return function(line) {
            return line.key === $jscomp$loop$44.key;
          };
        }($jscomp$loop$44));
        if (addressHeader && addressHeader.value) {
          addresses = documents.parser.util.addressParser(addressHeader.value);
          if (addresses && addresses.length) {
            message[$jscomp$loop$44.key.replace(/\-(.)/g, function(o, c) {
              return c.toUpperCase();
            })] = addresses[0];
          }
        }
      }
    }
    $jscomp$loop$45 = {};
    $jscomp$iter$4 = $jscomp.makeIterator(['delivered-to', 'return-path']), $jscomp$key$key = $jscomp$iter$4.next();
    for (; !$jscomp$key$key.done; $jscomp$loop$45 = {key$29:$jscomp$loop$45.key$29}, $jscomp$key$key = $jscomp$iter$4.next()) {
      $jscomp$loop$45.key$29 = $jscomp$key$key.value;
      {
        addressHeader$30 = $jscomp$async$this.root.headers.find(function($jscomp$loop$45) {
          return function(line) {
            return line.key === $jscomp$loop$45.key$29;
          };
        }($jscomp$loop$45));
        if (addressHeader$30 && addressHeader$30.value) {
          addresses$31 = documents.parser.util.addressParser(addressHeader$30.value);
          if (addresses$31 && addresses$31.length && addresses$31[0].address) {
            message[$jscomp$loop$45.key$29.replace(/\-(.)/g, function(o, c) {
              return c.toUpperCase();
            })] = addresses$31[0].address;
          }
        }
      }
    }
    $jscomp$loop$46 = {};
    $jscomp$iter$5 = $jscomp.makeIterator(['to', 'cc', 'bcc']), $jscomp$key$key = $jscomp$iter$5.next();
    for (; !$jscomp$key$key.done; $jscomp$loop$46 = {key$32:$jscomp$loop$46.key$32, addresses$33:$jscomp$loop$46.addresses$33}, $jscomp$key$key = $jscomp$iter$5.next()) {
      $jscomp$loop$46.key$32 = $jscomp$key$key.value;
      {
        addressHeaders = $jscomp$async$this.root.headers.filter(function($jscomp$loop$46) {
          return function(line) {
            return line.key === $jscomp$loop$46.key$32;
          };
        }($jscomp$loop$46));
        $jscomp$loop$46.addresses$33 = [];
        addressHeaders.filter(function(entry) {
          return entry && entry.value;
        }).map(function(entry) {
          return documents.parser.util.addressParser(entry.value);
        }).forEach(function($jscomp$loop$46) {
          return function(parsed) {
            return $jscomp$loop$46.addresses$33 = $jscomp$loop$46.addresses$33.concat(parsed || []);
          };
        }($jscomp$loop$46));
        if ($jscomp$loop$46.addresses$33 && $jscomp$loop$46.addresses$33.length) {
          message[$jscomp$loop$46.key$32] = $jscomp$loop$46.addresses$33;
        }
      }
    }
    $jscomp$loop$47 = {};
    $jscomp$iter$6 = $jscomp.makeIterator(['subject', 'message-id', 'in-reply-to', 'references']), $jscomp$key$key = $jscomp$iter$6.next();
    for (; !$jscomp$key$key.done; $jscomp$loop$47 = {key$34:$jscomp$loop$47.key$34}, $jscomp$key$key = $jscomp$iter$6.next()) {
      $jscomp$loop$47.key$34 = $jscomp$key$key.value;
      {
        header = $jscomp$async$this.root.headers.find(function($jscomp$loop$47) {
          return function(line) {
            return line.key === $jscomp$loop$47.key$34;
          };
        }($jscomp$loop$47));
        if (header && header.value) {
          message[$jscomp$loop$47.key$34.replace(/\-(.)/g, function(o, c) {
            return c.toUpperCase();
          })] = documents.parser.util.decodeWords(header.value);
        }
      }
    }
    dateHeader = $jscomp$async$this.root.headers.find(function(line) {
      return line.key === 'date';
    });
    if (dateHeader) {
      date = new Date(dateHeader.value);
      if (!date || date.toString() === 'Invalid Date') {
        date = dateHeader.value;
      } else {
        date = date.toISOString();
      }
      message.date = date;
    }
    if ($jscomp$async$this.textContent && $jscomp$async$this.textContent.html) {
      message.html = $jscomp$async$this.textContent.html;
    }
    if ($jscomp$async$this.textContent && $jscomp$async$this.textContent.plain) {
      message.text = $jscomp$async$this.textContent.plain;
    }
    message.attachments = $jscomp$async$this.attachments;
    return $jscomp$generator$context['return'](message);
  });
}});
Ext.define('documents.parser.node', {constructor:function(opts) {
  opts = opts || {};
  this.postalMime = opts.postalMime;
  this.root = !!opts.parentNode;
  this.childNodes = [];
  if (opts.parentNode) {
    opts.parentNode.childNodes.push(this);
  }
  this.state = 'header';
  this.headerLines = [];
  this.decoders = new Map;
  this.contentType = {value:'text/plain'};
  this.contentTransferEncoding = {value:'8bit'};
  this.contentDisposition = {value:''};
  this.headers = [];
  this.contentDecoder = false;
  this.textEncoder = new TextEncoder;
  this.callParent();
}, setupContentDecoder:function(transferEncoding) {
  if (/base64/i.test(transferEncoding)) {
    this.contentDecoder = Ext.create('documents.parser.base64Decoder');
  } else {
    if (/quoted-printable/i.test(transferEncoding)) {
      this.contentDecoder = Ext.create('documents.parser.answerDecoder', {decoder:documents.parser.util.getDecoder(this.contentType.parsed.params.charset)});
    } else {
      this.contentDecoder = Ext.create('documents.parser.chunkDecoder');
    }
  }
}, finalize:function() {
  var $jscomp$async$this = this;
  var boundaries;
  var i$35;
  var boundary;
  var JSCompiler_temp_const$jscomp$48;
  var JSCompiler_temp$jscomp$49;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    switch($jscomp$generator$context.nextAddress) {
      case 1:
        if ($jscomp$async$this.state === 'finished') {
          return $jscomp$generator$context['return']();
        }
        if ($jscomp$async$this.state === 'header') {
          $jscomp$async$this.processHeaders();
        }
        boundaries = $jscomp$async$this.postalMime.boundaries;
        for (i$35 = boundaries.length - 1; i$35 >= 0; i$35--) {
          boundary = boundaries[i$35];
          if (boundary.node === $jscomp$async$this) {
            boundaries.splice(i$35, 1);
            break;
          }
        }
        return $jscomp$generator$context.yield($jscomp$async$this.finalizeChildNodes(), 2);
      case 2:
        JSCompiler_temp_const$jscomp$48 = $jscomp$async$this;
        if (!$jscomp$async$this.contentDecoder) {
          JSCompiler_temp$jscomp$49 = null;
          $jscomp$generator$context.jumpTo(3);
          break;
        }
        return $jscomp$generator$context.yield($jscomp$async$this.contentDecoder.finalize(), 4);
      case 4:
        JSCompiler_temp$jscomp$49 = $jscomp$generator$context.yieldResult;
      case 3:
        JSCompiler_temp_const$jscomp$48.content = JSCompiler_temp$jscomp$49;
        $jscomp$async$this.state = 'finished';
        $jscomp$generator$context.jumpToEnd();
    }
  });
}, finalizeChildNodes:function() {
  var $jscomp$async$this = this;
  var $jscomp$iter$7, $jscomp$key$childNode;
  var childNode;
  return $jscomp.asyncExecutePromiseGeneratorProgram(function($jscomp$generator$context) {
    if ($jscomp$generator$context.nextAddress == 1) {
      $jscomp$iter$7 = $jscomp.makeIterator($jscomp$async$this.childNodes), $jscomp$key$childNode = $jscomp$iter$7.next();
    }
    if ($jscomp$generator$context.nextAddress != 3) {
      if (!!$jscomp$key$childNode.done) {
        return $jscomp$generator$context.jumpTo(0);
      }
      childNode = $jscomp$key$childNode.value;
      return $jscomp$generator$context.yield(childNode.finalize(), 3);
    }
    $jscomp$key$childNode = $jscomp$iter$7.next();
    return $jscomp$generator$context.jumpTo(2);
  });
}, parseStructuredHeader:function(str) {
  var response = {value:false, params:{}};
  var key = false;
  var value = '';
  var stage = 'value';
  var quote = false;
  var escaped = false;
  var chr;
  for (var i$36 = 0, len = str.length; i$36 < len; i$36++) {
    chr = str.charAt(i$36);
    switch(stage) {
      case 'key':
        if (chr === '\x3d') {
          key = value.trim().toLowerCase();
          stage = 'value';
          value = '';
          break;
        }
        value += chr;
        break;
      case 'value':
        if (escaped) {
          value += chr;
        } else {
          if (chr === '\\') {
            escaped = true;
            continue;
          } else {
            if (quote && chr === quote) {
              quote = false;
            } else {
              if (!quote && chr === '"') {
                quote = chr;
              } else {
                if (!quote && chr === ';') {
                  if (key === false) {
                    response.value = value.trim();
                  } else {
                    response.params[key] = value.trim();
                  }
                  stage = 'key';
                  value = '';
                } else {
                  value += chr;
                }
              }
            }
          }
        }
        escaped = false;
        break;
    }
  }
  value = value.trim();
  if (stage === 'value') {
    if (key === false) {
      response.value = value;
    } else {
      response.params[key] = value;
    }
  } else {
    if (value) {
      response.params[value.toLowerCase()] = '';
    }
  }
  if (response.value) {
    response.value = response.value.toLowerCase();
  }
  documents.parser.util.decodeParameterValueContinuations(response);
  return response;
}, decodeFlowedText:function(str, delSp) {
  return str.split(/\r?\n/).reduce(function(previousValue, currentValue) {
    if (/ $/.test(previousValue) && !/(^|\n)-- $/.test(previousValue)) {
      if (delSp) {
        return previousValue.slice(0, -1) + currentValue;
      } else {
        return previousValue + currentValue;
      }
    } else {
      return previousValue + '\n' + currentValue;
    }
  }).replace(/^ /gm, '');
}, getTextContent:function() {
  if (!this.content) {
    return '';
  }
  var str = documents.parser.util.getDecoder(this.contentType.parsed.params.charset).decode(this.content);
  if (/^flowed$/i.test(this.contentType.parsed.params.format)) {
    str = this.decodeFlowedText(str, /^yes$/i.test(this.contentType.parsed.params.delsp));
  }
  return str;
}, processHeaders:function() {
  for (var i$37 = this.headerLines.length - 1; i$37 >= 0; i$37--) {
    var line = this.headerLines[i$37];
    if (i$37 && /^\s/.test(line)) {
      this.headerLines[i$37 - 1] += '\n' + line;
      this.headerLines.splice(i$37, 1);
    } else {
      line = line.replace(/\s+/g, ' ');
      var sep = line.indexOf(':');
      var key = sep < 0 ? line.trim() : line.substr(0, sep).trim();
      var value = sep < 0 ? '' : line.substr(sep + 1).trim();
      this.headers.push({key:key.toLowerCase(), originalKey:key, value:value});
      switch(key.toLowerCase()) {
        case 'content-type':
          this.contentType = {value:value, parsed:{}};
          break;
        case 'content-transfer-encoding':
          this.contentTransferEncoding = {value:value, parsed:{}};
          break;
        case 'content-disposition':
          this.contentDisposition = {value:value, parsed:{}};
          break;
        case 'content-id':
          this.contentId = value;
          break;
      }
    }
  }
  this.contentType.parsed = this.parseStructuredHeader(this.contentType.value);
  this.contentType.multipart = /^multipart\//i.test(this.contentType.parsed.value) ? this.contentType.parsed.value.substr(this.contentType.parsed.value.indexOf('/') + 1) : false;
  if (this.contentType.multipart && this.contentType.parsed.params.boundary) {
    this.postalMime.boundaries.push({value:this.textEncoder.encode(this.contentType.parsed.params.boundary), node:this});
  }
  this.contentDisposition.parsed = this.parseStructuredHeader(this.contentDisposition.value);
  this.contentTransferEncoding.encoding = this.contentTransferEncoding.value.toLowerCase().split(/[^\w-]/).shift();
  this.setupContentDecoder(this.contentTransferEncoding.encoding);
}, feed:function(line) {
  switch(this.state) {
    case 'header':
      if (!line.length) {
        this.state = 'body';
        return this.processHeaders();
      }
      this.headerLines.push(documents.parser.util.getDecoder().decode(line));
      break;
    case 'body':
      {
        this.contentDecoder.update(line);
      }
  }
}});
Ext.define('documents.parser.tokenizer', {constructor:function(opts) {
  this.str = (opts.str || '').toString();
  this.operatorCurrent = '';
  this.operatorExpecting = '';
  this.node = null;
  this.escaped = false;
  this.list = [];
  this.operators = {'"':'"', '(':')', '\x3c':'\x3e', ',':'', ':':';', ';':''};
  this.callParent(arguments);
}, tokenize:function() {
  var chr, list = [];
  for (var i$38 = 0, len = this.str.length; i$38 < len; i$38++) {
    chr = this.str.charAt(i$38);
    this.checkChar(chr);
  }
  this.list.forEach(function(node) {
    node.value = (node.value || '').toString().trim();
    if (node.value) {
      list.push(node);
    }
  });
  return list;
}, checkChar:function(chr) {
  if (this.escaped) {
  } else {
    if (chr === this.operatorExpecting) {
      this.node = {type:'operator', value:chr};
      this.list.push(this.node);
      this.node = null;
      this.operatorExpecting = '';
      this.escaped = false;
      return;
    } else {
      if (!this.operatorExpecting && chr in this.operators) {
        this.node = {type:'operator', value:chr};
        this.list.push(this.node);
        this.node = null;
        this.operatorExpecting = this.operators[chr];
        this.escaped = false;
        return;
      } else {
        if (['"', "'"].includes(this.operatorExpecting) && chr === '\\') {
          this.escaped = true;
          return;
        }
      }
    }
  }
  if (!this.node) {
    this.node = {type:'text', value:''};
    this.list.push(this.node);
  }
  if (chr === '\n') {
    chr = ' ';
  }
  if (chr.charCodeAt(0) >= 33 || [' ', '\t'].includes(chr)) {
    this.node.value += chr;
  }
  this.escaped = false;
}});
Ext.define('documents.plugins.DocumentsList', {extend:'Ext.panel.Panel', alias:'widget.bpcDocsListPlugin', mixins:['documents.components.bpc.mixins.BpcPluginMixin'], layout:'fit', moduleId:undefined, documentLinkContext:undefined, title:undefined, tagFilter:undefined, tagsExclusive:undefined, attachedDocs:[], initComponent:function() {
  var $jscomp$this = this;
  this.iconCls = documents.Support.icon('fa-hdd');
  this.moduleId = this.getValueFromPluginConfiguration('moduleId');
  this.tagsExclusive = this.getValueFromPluginConfiguration('tagsExclusive') || false;
  this.tagFilter = this.getValueFromPluginConfiguration('tags') || [];
  this.baseTitle = this.getValueFromPluginConfiguration('title') || 'Documents';
  this.monitorActionsEnabled = this.isMonitor() && (this.getValueFromPluginConfiguration('actionsEnabled') === true || false);
  this.showUploadBarTop = this.getValueFromPluginConfiguration('showUploadBarTop');
  if (this.showUploadBarTop !== false) {
    this.showUploadBarTop = true;
  }
  this.showUploadButton = this.getValueFromPluginConfiguration('showUploadButton');
  if (!this.moduleId) {
    this.items = [{xtype:'container', html:'wrong config'}];
    this.callParent(arguments);
    return;
  }
  var pluginConfig = {};
  var targetInstanceId = this.getValueFromPluginConfiguration('instanceId', true);
  if (targetInstanceId) {
    var module = BpcCommon.Api.getBaseModuleIdById(targetInstanceId);
    if (module) {
      var refType = 'assetId', refValue = null, attachedDocs = [];
      var tagFilter = this.getValueFromPluginConfiguration('tags');
      tagFilter = Ext.isString(tagFilter) && tagFilter.length > 0 ? [tagFilter] : Ext.isArray(tagFilter) && tagFilter.length ? tagFilter : undefined;
      if (module === 'vam') {
        var assetConfig = this.getValueFromPluginConfiguration('assetConfig', true);
        refType = this.getValueFromPluginConfiguration('assetType') || 'assetId';
        if (assetConfig && 'asset' in assetConfig && assetConfig.asset) {
          var value = assetConfig.asset.get(refType);
          refValue = value ? value : null;
        }
      } else {
        if (this.isMonitor()) {
          var r = this.getValueFromPluginConfiguration('record', true);
          refType = BpcCommon.Api.getSetting(targetInstanceId, 'column_id') || 'PROCESSID';
          refValue = r.get(refType);
          var documentIdColumns = this.getValueFromPluginConfiguration('bpcDocumentIdColumns');
          if (documentIdColumns && Ext.isString(documentIdColumns)) {
            documentIdColumns = [documentIdColumns];
          }
          var documentIdsObj = documents.Support.getDocumentIdsFromColumns(documentIdColumns, r);
          var temp = [];
          Ext.iterate(documentIdsObj, function(col, docIdsArray) {
            temp = temp.concat(docIdsArray);
          });
          attachedDocs = temp;
        }
      }
      var columns = this.getValueFromPluginConfiguration('columns');
      pluginConfig.attachedDocs = attachedDocs;
      pluginConfig.showUploadButton = this.showUploadButton;
      pluginConfig.plugin = this;
      pluginConfig.tagFilter = tagFilter;
      pluginConfig.columns = columns;
      pluginConfig.filter = {instanceId:Ext.isString(targetInstanceId) ? targetInstanceId : String(targetInstanceId), moduleId:BpcCommon.Api.getBaseModuleIdById(targetInstanceId), referenceType:refType, referenceValue:refValue};
      this.pluginConfig = pluginConfig;
      this.docs = Ext.create({xtype:'bpcDocsGrid', moduleId:this.moduleId, pluginConfig:pluginConfig});
      this.docs.store.on('prefetch', this.onPrefetch.bind(this));
      this.docs.scopeGrid(pluginConfig);
      this.docs.changeFilters({instanceId:this.moduleId, filter:JSON.stringify([pluginConfig.filter])});
      this.items = [this.docs];
    }
  }
  this.countInBraces = true;
  this.callParent(arguments);
  this.updateTitle();
  this.addActions();
  documents.Api.getTags({instanceId:this.moduleId}).then(function(json) {
    $jscomp$this.docs.tags = json || [];
  })['finally'](function() {
    $jscomp$this.loadData();
  });
}, addActions:function() {
  if (this.monitorActionsEnabled) {
    this.getMonitorActionPanel();
    this.addActionsToMonitor([this.getActionButtonCfg('UPLOAD', 'fa-upload', 'UPLOAD_TOOLTIP', this.docs.createDocument.bind(this.docs))]);
  }
}, loadData:function() {
  this.docs.loadData();
}, onPrefetch:function(s, records, success) {
  this.updateTitle(undefined, success ? s.getTotalCount() : 0);
}, updateDocuments:function(json) {
  var objectToUpdate = {links:[this.pluginConfig.filter], documentTitle:json.filename};
  if (this.pluginConfig.tagFilter) {
    objectToUpdate.tags = this.pluginConfig.tagFilter;
  }
  return documents.Support.newRequest(documents.Support.buildDocumentUrl(json.id), undefined, objectToUpdate, 'PUT');
}});
Ext.define('documents.plugins.DocumentsListInterface', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'BPC Documents - List', COMPONENT_XTYPE:'bpcDocsListPlugin', PLUGIN_ID:'bpc-documents-list'}});
Ext.define('documents.plugins.player.MediaPlayerPlugin', {extend:'Ext.button.Button', alias:'widget.bpcDocumentsMediaPlayer', mixins:['documents.components.bpc.mixins.BpcPluginMixin', 'documents.components.bpc.mixins.Component'], layout:'fit', getStorageObject:function() {
  var loginName = BpcCommon.Api.getUserName();
  var globalVolume = this.getGlobalVolume();
  var $jscomp$destructuring$var34 = this;
  var serverUUID = $jscomp$destructuring$var34.serverUUID;
  return {loginName:loginName, serverUUID:serverUUID, values:{globalVolume:globalVolume}};
}, readyToStore:false, localStorageKey:'bpcDocumentsMediaSettings', serverUUID:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.iconCls = this.faIcon('fa-volume');
  this.tooltip = this.TR('GLOBAL_VOLUME_CONTROL');
  this.volumeField = Ext.create({xtype:'numberfield', value:0, readOnly:true, width:50, hideLabel:true, listeners:{change:function(f, newVal) {
    if (newVal === 0) {
      $jscomp$this.setIconCls($jscomp$this.faIcon('volume-slash'));
    } else {
      if (newVal === 100) {
        $jscomp$this.setIconCls($jscomp$this.faIcon('volume-up'));
      } else {
        $jscomp$this.setIconCls($jscomp$this.faIcon('volume'));
      }
    }
    $jscomp$this.storeLocalStorageData();
  }}});
  this.volumeSlider = Ext.create(Ext.apply({}, this.getHorizontalSlider(), {listeners:{change:function(c, newValue) {
    $jscomp$this.volumeField.setValue(newValue);
  }}}));
  this.createControls();
  this.handler = function() {
    $jscomp$this.showControls();
  };
  this.callParent(arguments);
  documents.Api.getBpcUUID().then(function(id) {
    $jscomp$this.readyToStore = true;
    $jscomp$this.serverUUID = id;
    $jscomp$this.applyLocalStorageData();
  });
}, storeLocalStorageData:function() {
  if (this.readyToStore) {
    documents.util.localStorage.set(this.localStorageKey, this.getStorageObject());
  }
}, applyLocalStorageData:function() {
  var $jscomp$destructuring$var35 = documents.util.localStorage.get(this.localStorageKey) || {};
  var values = $jscomp$destructuring$var35.values;
  var loginName = $jscomp$destructuring$var35.loginName;
  var serverUUID = $jscomp$destructuring$var35.serverUUID;
  loginName = loginName || '';
  if (loginName === BpcCommon.Api.getUserName() && this.serverUUID === serverUUID) {
    var $jscomp$destructuring$var36 = values || {};
    var globalVolume = $jscomp$destructuring$var36.globalVolume;
    if (typeof globalVolume === 'undefined') {
      globalVolume = 50;
    }
    this.volumeField.setValue(globalVolume);
    this.volumeSlider.setValue(globalVolume);
  }
}, createControls:function() {
  var $jscomp$this = this;
  var volumeDownButton = Ext.create({xtype:'button', iconCls:this.faIcon('volume-down'), handler:function() {
    var value = $jscomp$this.volumeSlider.getValue();
    value = value - 10;
    $jscomp$this.volumeSlider.setValue(value);
  }});
  var volumeUpButton = Ext.create({xtype:'button', iconCls:this.faIcon('volume-up'), handler:function() {
    var value = $jscomp$this.volumeSlider.getValue();
    value = value + 10;
    $jscomp$this.volumeSlider.setValue(value);
  }});
  this.bpcAudioControls = Ext.create({xtype:'window', layout:'fit', width:600, header:false, padding:'5 5 5 5', closeAction:'hide', items:[{xtype:'panel', layout:{type:'vbox', align:'stretch'}, items:[{xtype:'container', flex:1, layout:{type:'hbox', aligm:'stretch'}, items:[volumeDownButton, this.volumeSlider, volumeUpButton, this.volumeField]}], dockedItems:[{xtype:'toolbar', dock:'bottom', padding:'3 3 3 3', items:[{xtype:'button', text:this.TR('MUTE'), ui:'secondary', mutedValue:-1, muted:false, 
  handler:function(b) {
    if (b.muted) {
      $jscomp$this.volumeSlider.setValue(b.mutedValue);
      b.mutedValue = -1;
      b.muted = false;
    } else {
      b.mutedValue = $jscomp$this.volumeSlider.getValue();
      $jscomp$this.volumeSlider.setValue(0);
      b.muted = true;
    }
  }, iconCls:this.faIcon('volume-mute')}, {xtype:'tbfill'}, {text:this.TR('CLOSE'), ui:'primary', handler:function() {
    $jscomp$this.bpcAudioControls.close();
  }}]}]}]});
}, showControls:function() {
  if (this.bpcAudioControls.isVisible()) {
    this.bpcAudioControls.hide();
  } else {
    this.bpcAudioControls.show();
    this.bpcAudioControls.alignTo(this, 'tl-bl', [-100, 0]);
  }
}, getHorizontalSlider:function() {
  return {xtype:'slider', flex:1, value:50, increment:10, minValue:0, maxValue:100, vertical:false};
}, getGlobalVolume:function() {
  return this.volumeField.getValue();
}});
Ext.define('documents.plugins.player.PlayerInterface', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'BPC Documents - Media Player', COMPONENT_XTYPE:'bpcDocumentsMediaPlayer', PLUGIN_ID:'bpc-documents-player'}});
Ext.define('documents.util.BpcSupport', {singleton:true, getVersionFromPackage:function(packageName) {
  packageName = packageName === undefined ? 'BpcCommon' : packageName;
  var $jscomp$destructuring$var37 = Ext.manifest;
  var packages = $jscomp$destructuring$var37.packages;
  packages = packages || {};
  var common = packages[packageName] || {};
  var $jscomp$destructuring$var38 = common;
  var version = $jscomp$destructuring$var38.version;
  return version || '';
}, isBpcVersion:function(targetVersion) {
  targetVersion = targetVersion === undefined ? 4 : targetVersion;
  try {
    var version = this.getVersionFromPackage();
    var parts = version.split('.');
    return parseInt(parts[0]) === targetVersion;
  } catch (e) {
    return false;
  }
}, versionStartsWith:function(versionPrefix) {
  versionPrefix = versionPrefix === undefined ? '4.1' : versionPrefix;
  return this.getVersionFromPackage().startsWith(versionPrefix);
}});
Ext.define('documents.util.localStorage', {singleton:true, set:function(key, value) {
  value = value === undefined ? '' : value;
  if (typeof value !== 'string') {
    value = JSON.stringify(value);
  }
  window.localStorage.setItem(key, value);
}, get:function(key) {
  var value = window.localStorage.getItem(key);
  return typeof value !== 'undefined' ? JSON.parse(value) : value;
}});
Ext.define('documents.util.ModuleJump', {singleton:true, jumpToReference:function(moduleId, instanceId, referenceValue) {
  var payload = undefined;
  if (referenceValue) {
    if (moduleId === 'monitor') {
      payload = {storeState:{filters:[{property:BpcCommon.Api.getSetting(instanceId, 'column_id'), value:referenceValue}]}};
    }
    if (moduleId === 'vam') {
      payload = {assets:[referenceValue]};
    }
  }
  BpcCommon.Api.showModule(instanceId, false, payload);
}, jumpByItem:function(linkItem) {
  var $jscomp$destructuring$var39 = linkItem;
  var moduleId = $jscomp$destructuring$var39.moduleId;
  var instanceId = $jscomp$destructuring$var39.instanceId;
  var referenceValue = $jscomp$destructuring$var39.referenceValue;
  this.jumpToReference(moduleId, instanceId, referenceValue);
}});
Ext.define('documents.util.Screenshot', {singleton:true, getScreenshot:function(id) {
  id = id || this.getId();
  return html2canvas(document.querySelector('#' + id)).then(function(canvas) {
    return canvas.toDataURL('image/jpeg', 1);
  });
}, openScreenshotInWindow:function(id) {
  this.getScreenshot(id).then(function(dataUrl) {
    documents.Support.getWindowCmp('Screnshot', 'x-fal fa-camera', 0.4, 0.4, {xtype:'panel', html:'\x3cdiv style\x3d"width: 100%; height: 100%"\x3e\x3cimg src\x3d"' + dataUrl + '" alt\x3d"Your screenshot"\x3e\x3c/div\x3e'});
  });
}});
Ext.define('documents.view.Main', {extend:'Ext.panel.Panel', alias:'widget.bpcDocsMain', mixins:['documents.components.bpc.mixins.Component'], header:false, layout:{type:'hbox', align:'stretch'}, moduleCfg:undefined, style:'background-color:#f1f1f1;', viewModel:{data:{selectedOrganisation:undefined}, store:{filteredOrganisations:undefined}}, controller:'documentsMainController', loadData:function() {
  this.grid.loadData();
}, onComboFilterChange:function(c, newVal, oldVal) {
  var p = this.grid.getFilters();
  if (newVal) {
    p[c.filterKey] = newVal;
  } else {
    if (p[c.filterKey]) {
      delete p[c.filterKey];
    }
  }
  this.filterRemoveButton.setDisabled(Ext.Object.isEmpty(p));
  this.grid.changeFilters(p, true);
}, instanceLoaded:false, canRead:false, canWrite:false, isAdmin:false, initComponent:function() {
  var $jscomp$this = this;
  this.handleModuleRights();
  if (!this.canRead && !this.canWrite) {
    this.items = [{xtype:'container', flex:1, html:'PERMISSION DENIED'}];
  } else {
    this.instanceLoaded = true;
    var keepDeletedDocs = BpcCommon.Api.getSetting(this.moduleId, 'keepDeletedDocuments');
    var showTrash = keepDeletedDocs === true;
    if (!this.isAdmin && showTrash) {
      showTrash = false;
    }
    var adminButtons = [{handler:function(btn) {
      var detailContainer = Ext.create({xtype:'bpcDocsTagsGrid', moduleId:$jscomp$this.moduleId, header:false});
      var window = documents.Support.getWindowCmp(detailContainer.getTitle(), 'fa-tags', 0.8, 0.8, detailContainer);
    }, text:this.TR('TAGS'), tooltip:this.TR('MANAGE_TAGS'), iconCls:this.faIcon('fa-tags'), hidden:!this.isAdmin}, {handler:function(btn) {
      var detailContainer = Ext.create({xtype:'docsTrash', moduleId:$jscomp$this.moduleId, header:false});
      var window = documents.Support.getWindowCmp(detailContainer.getTitle(), 'fa-trash', 0.8, 0.8, detailContainer);
    }, text:this.TR('BIN'), tooltip:this.TR('MANAGE_BIN'), iconCls:this.faIcon('fa-trash'), hidden:!showTrash}];
    var moduleHeaderOptions = BpcCommon.Api.getSetting(this.moduleId, 'moduleHeaderOptions') || {};
    var $jscomp$destructuring$var40 = moduleHeaderOptions;
    var allowAdministration = $jscomp$destructuring$var40.allowAdministration;
    var moduleHeaderAdminButtons = allowAdministration ? adminButtons : [];
    this.bpcModuleHeader = Ext.create({xtype:'container', layout:{type:'hbox', align:'stretch'}, items:[{xtype:'toolbar', flex:1, items:moduleHeaderAdminButtons.concat([{xtype:'tbfill'}, {handler:function(btn) {
      return $jscomp$this.loadData();
    }, tooltip:this.TR('REFRESH'), iconCls:this.faIcon('fa-sync')}])}]});
    var defaults = {moduleId:this.moduleId, flex:1, moduleInstance:this};
    this.grid = Ext.create(Ext.apply({}, {xtype:'bpcDocsGrid', flex:5}, defaults));
    this.fileTypesCombo = Ext.create({xtype:'combo', store:Ext.create('Ext.data.Store', {data:[]}), emptyText:documents.Support.TR('FILE_FILTER'), valueField:'name', displayField:'display', listConfig:{itemTpl:'\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width: 80%; ; text-overflow: ellipsis;white-space: nowrap;"\x3e{display}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;text-align:right;width: 20%;white-space: nowrap;"\x3e{count}\x3c/span\x3e\x3c/div\x3e'}, width:200, filterKey:'fileType', 
    listeners:{change:this.onComboFilterChange.bind(this)}});
    var fields = BpcCommon.Api.getSetting(this.moduleId, 'customFields') || {};
    var customFieldNames = Object.keys(fields);
    var customFieldName = undefined;
    var emptyText = '';
    if (customFieldNames.length) {
      var lang = BpcCommon.Api.getSetting('_core', 'language');
      customFieldName = customFieldNames[0];
      emptyText = fields[customFieldName].text[lang];
    }
    this.customMetaField = Ext.create({xtype:'combo', store:Ext.create('Ext.data.Store', {data:[]}), emptyText:emptyText, valueField:'name', displayField:'display', hidden:customFieldNames.length === 0, listConfig:{itemTpl:'\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width: 80%; ; text-overflow: ellipsis;white-space: nowrap;"\x3e{display}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;text-align:right;width: 20%;white-space: nowrap;"\x3e{count}\x3c/span\x3e\x3c/div\x3e'}, width:200, 
    filterProperty:customFieldNames[0], filterKey:'meta', listeners:{change:this.onComboFilterChange.bind(this)}});
    this.tagsStore = Ext.create('Ext.data.Store', {proxy:{type:'ajax', url:documents.Support.basePath + 'tags', reader:{type:'json', rootProperty:'data', totalProperty:'total'}, extraParams:{instanceId:this.moduleId, limit:2000}, listeners:{exception:function(p, r) {
    }}}, listeners:{load:function(s, records, success) {
      var tags = success ? records.map(function(r) {
        return {id:r.get('id'), name:r.get('name'), color:r.get('color'), record:r};
      }) : [];
      documents.Cache.tags[$jscomp$this.moduleId] = {createTime:new Date, success:success, tags:tags};
      $jscomp$this.grid.updateTags(tags);
    }}, autoLoad:false});
    this.tagsCombo = Ext.create({xtype:'combo', store:this.tagsStore, emptyText:documents.Support.TR('TAG_FILTER'), valueField:'id', displayField:'name', queryMode:'local', width:200, filterKey:'tags', listConfig:{itemTpl:'\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width: 80%; text-overflow: ellipsis; white-space: nowrap;"\x3e{name}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;text-align:right;width: 20%;white-space: nowrap;"\x3e{count}\x3c/span\x3e\x3c/div\x3e'}, listeners:{change:this.onComboFilterChange.bind(this)}});
    var hideReferenceFilters = BpcCommon.Api.getSetting(this.moduleId, 'hideReferenceFilters');
    this.linksCombo = Ext.create({xtype:'combo', store:Ext.create('Ext.data.Store', {data:[]}), margin:'0 1 0 1', hidden:hideReferenceFilters, emptyText:documents.Support.TR('APPLICATION_FILTER'), valueField:'name', displayField:'display', filterKey:'sourceInstanceId', listConfig:{itemTpl:'\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width: 80%; text-overflow: ellipsis; white-space: nowrap;"\x3e{display}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;text-align:right;width: 20%;white-space: nowrap;"\x3e{count}\x3c/span\x3e\x3c/div\x3e'}, 
    width:200, listeners:{change:this.onComboFilterChange.bind(this)}});
    this.linkRefsCombo = Ext.create({xtype:'combo', hidden:hideReferenceFilters, store:Ext.create('Ext.data.Store', {data:[]}), margin:'0 1 0 1', emptyText:documents.Support.TR('ITEM_FILTER'), valueField:'name', displayField:'name', listConfig:{itemTpl:'\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width: 80%; text-overflow: ellipsis; white-space: nowrap;"\x3e{name}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;text-align:right;width: 20%;white-space: nowrap;"\x3e{count}\x3c/span\x3e\x3c/div\x3e'}, 
    width:200, filterKey:'referenceValue', listeners:{change:this.onComboFilterChange.bind(this)}});
    var hasCustomField = (customFieldName || '').length > 0;
    this.grid.store.on('prefetch', function(s, records, successful, op) {
      if (successful) {
        var json = documents.Support.getResponseObject(op.getResponse());
        var $jscomp$destructuring$var41 = json || {};
        var links = $jscomp$destructuring$var41.links;
        var types$39 = $jscomp$destructuring$var41.types;
        var linkRefs = $jscomp$destructuring$var41.linkRefs;
        var tags$40 = $jscomp$destructuring$var41.tags;
        links = links || [];
        types$39 = types$39 || [];
        linkRefs = linkRefs || [];
        $jscomp$this.tagsStore.load({scope:$jscomp$this, callback:function(r, op, success) {
          if (tags$40) {
            $jscomp$this.tagsCombo.store.each(function(r) {
              var id = r.getId();
              var match = tags$40.filter(function(t) {
                return t.name === id;
              });
              if (match.length) {
                r.set('count', match[0].count);
              }
            });
          }
        }});
        if (links) {
          $jscomp$this.linksCombo.store.removeAll();
          $jscomp$this.linksCombo.store.add(links.map(function(item) {
            item.display = item.name ? BpcCommon.Api.getSetting(item.name, 'module_name') : item.name;
            return item;
          }));
        }
        if (hasCustomField && json[customFieldName]) {
          $jscomp$this.customMetaField.store.removeAll();
          var fieldData = json[customFieldName] || [];
          $jscomp$this.customMetaField.store.add(fieldData.map(function(item) {
            item.display = item.name;
            return item;
          }));
        }
        if (types$39) {
          $jscomp$this.fileTypesCombo.store.removeAll();
          $jscomp$this.fileTypesCombo.store.add(types$39.map(function(item) {
            if (item.name) {
              var p = item.name.split('/');
              if (p.length === 2) {
                item.type = p[0];
                item.shortName = p[1];
              }
            }
            item.display = item.shortName ? item.shortName.toUpperCase() : item.name || '';
            return item;
          }));
        }
        if (linkRefs) {
          $jscomp$this.linkRefsCombo.store.removeAll();
          $jscomp$this.linkRefsCombo.store.add(linkRefs);
        }
      }
    });
    this.filterRemoveButton = Ext.create({xtype:'button', iconCls:documents.Support.icon('fa-filter'), disabled:true, ui:'secondary', handler:function(b) {
      $jscomp$this.grid.removeAllFilters();
    }});
    this.grid.onFilterRemoved = function() {
      $jscomp$this.query('combo[filterKey]').map(function(c) {
        c.suspendEvents();
        c.reset();
        c.resumeEvents();
      });
      $jscomp$this.filterRemoveButton.setDisabled(true);
      $jscomp$this.grid.changeFilters([], true);
    };
    var config = BpcCommon.Api.getSetting(this.moduleId, 'mainArea') || {};
    var $jscomp$destructuring$var42 = config;
    var deleteButtonHidden = $jscomp$destructuring$var42.deleteButtonHidden;
    deleteButtonHidden = deleteButtonHidden || false;
    this.deleteButton = Ext.create(Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-trash', 'DELETE'), {disabled:true, hidden:deleteButtonHidden || !this.canWrite, handler:function(b) {
      return $jscomp$this.grid.deleteRecords();
    }}));
    this.grid.on('selectionchange', function(selModel, recs) {
      $jscomp$this.deleteButton.setDisabled(recs.length === 0);
    });
    this.topContainer = Ext.create({xtype:'container', dock:'top', layout:{type:'vbox', align:'stretch'}, padding:'0 0 0 0', items:[{xtype:'toolbar', layout:{type:'hbox', align:'stretch'}, padding:'0 0 3 0', items:[Ext.apply(documents.Utils.ui.getButtonCfg(true, 'fa-plus', 'UPLOAD'), {hidden:!this.canWrite, handler:function() {
      return $jscomp$this.grid.createDocument();
    }}), {xtype:'tbfill'}, this.deleteButton]}, {xtype:'toolbar', layout:{type:'hbox', align:'stretch'}, items:[this.filterRemoveButton, this.tagsCombo, this.linksCombo, this.linkRefsCombo, {xtype:'tbfill'}, this.customMetaField, this.fileTypesCombo]}]});
    this.wrapper = Ext.create({dockedItems:this.topContainer, xtype:'panel', layout:'fit', header:false, flex:5, items:[this.grid]});
    this.rightPanel = Ext.create(Ext.apply({}, {xtype:'bpcDocsRight', flex:3}, defaults));
    this.items = [this.wrapper, {xtype:'splitter'}, this.rightPanel];
  }
  var theSpecial = BpcCommon.Api.getSetting(this.moduleId, 'hasCustomComponents');
  this.callParent(arguments);
  this.updateInstance(this.moduleCfg || {});
  if (theSpecial) {
    this.applicationToolbarLeft = {xtype:'toolbar', flex:1, controller:{someConfig:true}, items:[{xtype:'theGatewayOrganisationComboBox', flex:1, style:{paddingRight:'5px'}}], viewModel:{parent:this.getViewModel()}};
  }
}, updateInstance:function(state) {
  if (this.instanceLoaded) {
    var $jscomp$destructuring$var43 = state || {};
    var id = $jscomp$destructuring$var43.id;
    if (id) {
      var params = this.grid.getFilters();
      params.documentIds = id;
      this.grid.changeFilters(params, true);
    }
  }
}});
Ext.define('documents.view.MainController', {extend:'Ext.app.ViewController', alias:'controller.documentsMainController', init:function() {
  var moduleId = this.getView().moduleId;
  var theSpecial = BpcCommon.Api.getSetting(moduleId, 'hasCustomComponents');
  if (theSpecial) {
    try {
      var restrictedToRole = undefined;
      try {
        restrictedToRole = BpcCommon.Api.getSetting(moduleId, ncggateway.asset.ConfigKey.restrictToOrganisationRole);
      } catch (e) {
        restrictedToRole = undefined;
      }
      var filteredStore = Ext.create('Ext.data.ChainedStore', {source:'theGatewayOrganisations'});
      if (restrictedToRole) {
        filteredStore.addFilter([{property:'role', value:restrictedToRole, operator:'\x3d'}]);
      }
      var vm = this.getViewModel();
      vm.set('filteredOrganisations', filteredStore);
      vm.set('global.activeOrganisation', filteredStore.getCount() === 1 ? filteredStore.first() : null);
    } catch (e$41) {
    }
  }
}});
Ext.define('documents.view.moduleInstance.Right', {extend:'Ext.container.Container', alias:'widget.bpcDocsRight', layout:{type:'vbox', align:'stretch'}, documentId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.grid = Ext.create({xtype:'bpcDocsMetaGrid', moduleId:this.moduleId});
  var listenerPath = documents.Support.getIsListenerPath(this.moduleId);
  var config = BpcCommon.Api.getSetting(this.moduleId, 'previewAreaRight') || {};
  var $jscomp$destructuring$var44 = config;
  var buttonsHidden = $jscomp$destructuring$var44.buttonsHidden;
  var changeLogHidden = $jscomp$destructuring$var44.changeLogHidden;
  buttonsHidden = buttonsHidden || false;
  changeLogHidden = changeLogHidden || false;
  this.actionsToolbar = Ext.create({xtype:'toolbar', layout:{type:'hbox', align:'stretch'}, margin:'0 0 0 0', hidden:buttonsHidden, style:'border: 1px solid #eee;', items:[Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-download', 'DOWNLOAD'), {handler:function() {
    documents.Support.downloadFile($jscomp$this.documentId);
  }}), Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-clone', 'COPY_DOWNLOAD_LINK'), {handler:this.onCloneClick.bind(this), hidden:true}), Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-edit', 'EDIT_DOCUMENT'), {hidden:!documents.Support.canWrite(this.moduleId), handler:this.onEditClick.bind(this)}), Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-cogs', 'RUN_ACTIONS'), {menu:[{iconCls:documents.Support.icon('fa-solar-system'), text:documents.Support.TR('SEND_TO_INUBIT'), path:listenerPath, 
  disabled:!Ext.isDefined(listenerPath), handler:function(menuItem) {
    documents.Api.sendToInubit($jscomp$this.documentId, menuItem.path, $jscomp$this.moduleId);
  }}, {iconCls:documents.Support.icon('fa-qrcode'), text:documents.Support.TR('QRCODE_LINK'), handler:this.onQrCodeClick.bind(this)}], disabled:!Ext.isDefined(listenerPath)}), {xtype:'tbfill'}, Ext.apply(documents.Utils.ui.getButtonCfg(false, 'fa-trash', 'DELETE'), {hidden:!documents.Support.canWrite(this.moduleId), handler:this.onDeleteClick.bind(this)})].map(function(b) {
    b.margin = '0 1 0 1';
    b.disabled = true;
    return b;
  })});
  this.filePreviewPanel = Ext.create({xtype:'panel', title:documents.Support.TR('PREVIEW'), iconCls:documents.Support.icon('fa-search'), layout:'fit'});
  this.fileHistoryPanel = Ext.create({xtype:'bpcDocsFileHistoryGrid', documentId:this.documentId, hidden:changeLogHidden});
  this.previewPanel = Ext.create({xtype:'tabpanel', header:false, layout:'fit', flex:2, style:'border: 1px solid #eee;', itemId:'docsPreview', items:[this.filePreviewPanel, this.fileHistoryPanel]});
  var topItems = [this.actionsToolbar, this.previewPanel, this.grid];
  this.items = [{xtype:'container', flex:1, layout:{type:'vbox', align:'stretch'}, items:topItems}].map(function(i) {
    i.moduleInstance = $jscomp$this.moduleInstance;
    return i;
  });
  this.callParent(arguments);
  this.resetPanel();
}, refreshGui:function() {
  if (this.documentId) {
    this.loadId(this.documentId, true);
  }
}, loadId:function(id, force) {
  var $jscomp$this = this;
  force = force === undefined ? false : force;
  if (force) {
    this.resetPanel();
  }
  documents.Support.getDocument(id, this.moduleId, force).then(function(fullDocumentObject) {
    $jscomp$this.updateActiveDocumentCmp(fullDocumentObject);
  }, function(error) {
    BpcCommon.Api.showNotification('Sorry, please be patient, something went wrong. Sorry for any inconviences.');
  });
}, updateActiveDocumentCmp:function(fullDocumentObject) {
  var update = Ext.isDefined(fullDocumentObject);
  this.filePreviewPanel.removeAll();
  this.actionsToolbar.query('button').map(function(b) {
    return b.setDisabled(!update);
  });
  if (update) {
    if (fullDocumentObject.record.get('id') !== this.documentId) {
      this.record = fullDocumentObject.record;
      this.grid.lastRecord = fullDocumentObject.record;
      this.documentId = this.record.get('id');
      this.grid.updateWithRecord(this.record);
      this.filePreviewPanel.add(documents.Support.getViewerComponentByDocument(fullDocumentObject));
      this.fileHistoryPanel.changeDocumentId(this.documentId);
    }
    return;
  }
  this.documentId = undefined;
  this.filePreviewPanel.add({xtype:'container', layout:'center', style:'display: flex;align-items: center;justify-content: center;opacity:0.7;', html:'\x3cdiv style\x3d"text-align: center;"\x3e' + BpcCommon.util.Icons.getSpanForIconCls('fa-file', '#000', 50) + '\x3cbr\x3e\x3cbr\x3e' + documents.Support.TR('PREVIEW') + '\x3c/div\x3e'});
  this.grid.resetInit();
}, resetPanel:function() {
  this.updateActiveDocumentCmp();
}, onCloneClick:function(b) {
  if (this.documentId) {
    if (documents.Support.copyToClipBoard(documents.Api.getDownloadUrl(this.documentId))) {
      BpcCommon.Api.showNotification('URL: ' + url + 'has been copied to your clipboard');
    }
  }
}, onQrCodeClick:function(b) {
  if (this.documentId) {
    var url$42 = documents.Api.getDownloadUrl(this.documentId);
    Ext.create('Ext.window.Window', {title:'QR Code Link To Document', height:document.body.clientHeight * 0.5, width:document.body.clientWidth * 0.5, layout:'fit', items:[{xtype:'panel', html:'\x3cdiv id\x3d"qrcode"\x3e\x3c/div\x3e', listeners:{afterrender:function(c) {
      new QRCode(document.getElementById('qrcode'), url$42);
    }}, bbar:[{xtype:'button', iconCls:documents.Support.icon('fa-browser'), text:'Content: ' + url$42}]}]}).show();
  }
}, onDeleteClick:function(b) {
  var $jscomp$this = this;
  if (this.documentId) {
    documents.Support.deleteById(documents.Support.buildDocumentUrl(this.documentId)).then(function(success) {
      if (success) {
        $jscomp$this.updateActiveDocumentCmp();
        $jscomp$this.moduleInstance.loadData();
      }
    });
  }
}, onEditClick:function(b) {
  if (this.documentId) {
    var w = documents.Support.openDetailEditWindow(this.record, this.moduleId);
    if (this.moduleInstance && this.moduleInstance.grid && this.moduleInstance.grid.handleOpenWindow) {
      this.moduleInstance.grid.handleOpenWindow(w);
    }
  }
}});
Ext.define('documents.view.document.Base', {extend:'Ext.panel.Panel', mixins:['documents.components.bpc.mixins.Component'], header:false, layout:{type:'vbox', align:'stretch'}, scrollable:true, createCloseButton:function() {
  var $jscomp$this = this;
  var cfg = Ext.apply({}, documents.Utils.ui.getButtonCfg(false, 'fa-times', 'CLOSE'));
  return Ext.create(Ext.apply(cfg, {handler:function() {
    $jscomp$this.doCloseWindow();
  }}));
}, spacer:function() {
  return {xtype:'tbspacer', width:3};
}, doCloseWindow:function() {
  this.up('window').close();
}, doHideWindow:function() {
  this.up('window').hide();
}, doShowWindow:function() {
  this.up('window').show();
}, toBackground:function() {
  this.up('window').toBack();
}, finishAction:function(b) {
  var buttonsState = this.onUploaded ? !this.onUploaded() : false;
  b.setDisabled(buttonsState);
  if (this.autoCloseCheckbox.getValue() === true) {
    this.doCloseWindow();
  }
}, showToast:function(title, message, width, cmp) {
  title = title || documents.Support.TR('UPLOAD');
  message = message || documents.Support.TR('UPLOAD_SUCCESS');
  cmp = cmp || this;
  documents.Utils.toast(title, message, width, cmp);
}});
Ext.define('documents.view.document.Create', {extend:'documents.view.document.Base', alias:'widget.bpcDocsCreate', moduleId:undefined, defaultFileProperties:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.layout = 'card';
  this.documentStorage = BpcCommon.Api.getSetting(this.moduleId, 'documentStorage');
  this.uploadPanel = Ext.create(Ext.apply({}, {xtype:'bpcDocsUploadGridPanel', flex:1, moduleId:this.moduleId, defaultFileProperties:this.defaultFileProperties || {}}));
  this.uploadButton = Ext.create(documents.Utils.ui.simpleButton(true, 'fa-upload', 'UPLOAD', {handler:this.onUploadClick.bind(this)}));
  this.uploadPanel.store.on({destroyable:true, add:function(s) {
    $jscomp$this.onStoreDataChanged();
  }, remove:function(s) {
    $jscomp$this.onStoreDataChanged();
  }});
  this.createFolderButton = Ext.create(documents.Utils.ui.simpleButton(false, 'fa-folder', 'CREATE_FOLDER', {hidden:this.documentStorage !== documents.Utils.FAST_STORAGE_ADAPTER_ID(), handler:function() {
    return $jscomp$this.uploadPanel.addFolderItem();
  }}));
  this.textEditor = Ext.create({title:documents.Support.TR('NEW_DOCUMENT'), xtype:'docsTextEditor', readOnly:false});
  this.languageSelector = Ext.create('Ext.form.ComboBox', {fieldLabel:'SELECT_LANGUAGE', store:Ext.create('Ext.data.Store', {data:[{'mime':'text/plain', 'editorLanguage':'text'}, {'mime':'text/css', 'editorLanguage':'css'}, {'mime':'application/javascript', 'editorLanguage':'javascript'}, {'mime':'application/json', 'editorLanguage':'json'}, {'mime':'text/xml', 'editorLanguage':'xml'}, {'mime':'application/edifact', 'editorLanguage':'edifact'}]}), queryMode:'local', displayField:'editorLanguage', 
  valueField:'mime', value:'text/plain', listeners:{change:function(f) {
    var selection = f.getSelection();
    $jscomp$this.textEditor.setAceMode(selection.get('editorLanguage'));
  }}});
  this.createFilePanel = Ext.create({xtype:'panel', header:false, layout:'fit', items:[this.textEditor], bbar:[{xtype:'button', ui:'secondary', text:documents.Support.TR('CANCEL'), handler:function() {
    $jscomp$this.setMainView($jscomp$this.uploadPanel);
    $jscomp$this.createFolderButton.setDisabled(false);
    $jscomp$this.createFileButton.setDisabled(false);
    $jscomp$this.uploadButton.setDisabled(false);
    $jscomp$this.textEditor.setValue('');
  }}, {xtype:'tbfill'}, this.languageSelector, {xtype:'button', ui:'primary', text:documents.Support.TR('ADD'), handler:function() {
    var base64string = $jscomp$this.textEditor.getValue(true);
    var mime = $jscomp$this.languageSelector.getSelection().get('mime');
    var documentObject = {base64string:base64string};
    var doc = {mime:mime, documentObject:documentObject};
    $jscomp$this.uploadPanel.addRawItem(doc);
    $jscomp$this.setMainView($jscomp$this.uploadPanel);
    $jscomp$this.createFolderButton.setDisabled(false);
    $jscomp$this.createFileButton.setDisabled(false);
    $jscomp$this.uploadButton.setDisabled(false);
    $jscomp$this.textEditor.setValue('');
  }}]});
  this.createFileButton = Ext.create(documents.Utils.ui.simpleButton(false, 'fa-file', 'CREATE_FILE', {hidden:this.documentStorage !== documents.Utils.FAST_STORAGE_ADAPTER_ID(), handler:function() {
    return $jscomp$this.doCreateFile();
  }}));
  this.autoCloseCheckbox = Ext.create({xtype:'checkbox', value:BpcCommon.Api.getSetting(this.moduleId, 'upload_autoCloseWindow'), boxLabel:documents.Support.TR('AUTO_CLOSE_WINDOW')});
  this.dockedItems = [{xtype:'toolbar', dock:'bottom', padding:'3 3 3 3', items:[this.createFolderButton, this.spacer(), this.createFileButton, {xtype:'tbfill'}, this.autoCloseCheckbox, this.spacer(), this.createCloseButton(), this.spacer(), this.uploadButton]}];
  this.items = [];
  this.callParent(arguments);
  this.setMainView(this.uploadPanel);
}, onOwnerCtBoxReady:function() {
  if (this.uploadPanel.readyToUpload === false) {
    this.doHideWindow();
  }
}, onUploaded:function() {
  return this.uploadPanel.hasUploads();
}, onStoreDataChanged:function() {
  this.uploadButton.setDisabled(!this.uploadPanel.hasUploads());
}, onUploadClick:function(b) {
  var $jscomp$this = this;
  b.setDisabled(true);
  this.setLoading();
  this.uploadPanel.uploadAll().then(function(data) {
    var $jscomp$destructuring$var45 = data || {};
    var uploads = $jscomp$destructuring$var45.uploads;
    uploads = uploads || 0;
    if (uploads > 0) {
      $jscomp$this.showToast();
    }
  })['finally'](function() {
    $jscomp$this.setLoading(false);
    $jscomp$this.finishAction(b);
  });
}, doCreateFile:function() {
  this.createFolderButton.setDisabled(true);
  this.createFileButton.setDisabled(true);
  this.uploadButton.setDisabled(true);
  this.setMainView(this.createFilePanel);
}, setMainView:function(cmp) {
  this.getLayout().setActiveItem(cmp);
}});
Ext.define('documents.view.document.Edit', {extend:'documents.view.document.Base', alias:'widget.bpcDocsEdit', documentId:undefined, record:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.documentId = this.record.getId();
  this.documentTitleField = this.createDocumentsField('documentTitle');
  this.descriptionField = this.createDocumentsField('description');
  this.tagField = Ext.create({xtype:'toolbar', layout:{type:'hbox', align:'stretch'}, padding:'5 5 5 5', items:[], defaultType:'bpcDocsTag', listeners:{add:this.updateTags.bind(this), remove:this.updateTags.bind(this)}});
  this.tagsCombo = Ext.create({xtype:'combo', anchor:'90%', fieldLabel:documents.Support.TR('TAGS'), store:Ext.create('Ext.data.Store', {data:documents.Cache.getTags(this.moduleId) || []}), queryMode:'local', emptyText:documents.Support.TR('ACTION_ADD_TAGS'), valueField:'id', displayField:'name', width:200, filterKey:'tag', listeners:{select:function(c, r) {
    var id = r.get('id');
    if ($jscomp$this.tags.indexOf(id) !== -1) {
      documents.Utils.toast('TAG_EXISTS', 'TAG_NOT_ADDED_ALREADY_EXISTS', undefined, $jscomp$this);
      c.setValue(undefined);
      return;
    }
    $jscomp$this.addDisplayTag(r.get('name'), id, r.getData());
    c.setValue(undefined);
  }}});
  this.tags = [];
  var allowFileChange = BpcCommon.Api.getSetting(this.moduleId, 'upload_allowFileChange');
  var fileFieldHidden = allowFileChange === false;
  this.fileField = Ext.create({xtype:'docsFileUpload', fieldLabel:documents.Support.TR('DOC_FILE'), commitedUpload:true, emptyText:documents.Support.TR('EMPTYTEXT_FILE_FIELD'), anchor:'90%', moduleId:this.moduleId, documentId:this.documentId, hidden:fileFieldHidden});
  this.fieldset = Ext.create({xtype:'fieldset', title:documents.Support.TR('EDIT_DOCUMENT'), items:[this.documentTitleField, this.descriptionField, this.tagsCombo, this.tagField, this.fileField]});
  var isAdmin = documents.Api.isDocumentsAdmin();
  var readOnly = !isAdmin;
  this.idField = Ext.create({xtype:'documentsTextField', trKey:'DOC_ID', readOnly:!isAdmin, value:this.record.getId(), hidden:true});
  this.filenameField = this.createDocumentsField('filename', {readOnly:readOnly});
  this.mimeTypeField = this.createDocumentsField('mime', {readOnly:readOnly});
  this.additionalFields = Ext.create({xtype:'fieldset', title:documents.Support.TR('EDIT_DOCUMENT_EXTENDED'), items:[this.idField, this.filenameField, this.mimeTypeField]});
  this.items = [this.fieldset, this.additionalFields];
  this.updateButton = Ext.create(Ext.apply(documents.Utils.ui.getButtonCfg(true, 'fa-edit', 'CHANGE'), {handler:function() {
    var doc = {documentTitle:$jscomp$this.documentTitleField.getValue(), description:$jscomp$this.descriptionField.getValue(), tags:$jscomp$this.getVisibleTagIds()};
    if (isAdmin) {
      doc['filename'] = $jscomp$this.filenameField.getValue();
    }
    $jscomp$this.doEdit(doc);
  }}));
  this.autoCloseCheckbox = Ext.create({xtype:'checkbox', value:BpcCommon.Api.getSetting(this.moduleId, 'upload_autoCloseWindow'), boxLabel:documents.Support.TR('AUTO_CLOSE_WINDOW')});
  this.dockedItems = [{xtype:'toolbar', dock:'bottom', padding:'3 3 3 3', items:[{xtype:'tbfill'}, this.autoCloseCheckbox, this.spacer(), this.createCloseButton(), this.spacer(), this.updateButton]}];
  this.callParent(arguments);
  var tagsIds = this.record.get('tags') || [];
  if (tagsIds.length) {
    var tags$43 = documents.Cache.getTags(this.moduleId) || [];
    tagsIds.map(function(id) {
      var match = tags$43.filter(function(i) {
        return i.id === id;
      });
      $jscomp$this.addDisplayTag(match.length ? match[0].name : '-1', id, match.length ? match[0] : undefined);
    });
  }
}, createDocumentsField:function(key, cfg) {
  key = key === undefined ? '' : key;
  cfg = cfg === undefined ? {} : cfg;
  return Ext.create(Ext.apply({xtype:'documentsTextField', trKey:'DOC_' + (key || '').toUpperCase(), value:this.record.get(key)}, cfg));
}, getVisibleTagIds:function() {
  return this.tagField.query('bpcDocsTag').map(function(c) {
    return c.getTagId();
  });
}, updateTags:function() {
  this.tags = this.getVisibleTagIds();
}, doEdit:function(data) {
  var $jscomp$this = this;
  data = data === undefined ? {} : data;
  this.updateButton.disable();
  this.setLoading();
  var promises = [];
  promises.push(documents.Api.updateDocument(this.documentId, data));
  var value = this.fileField.getValue() || '';
  var fileChanged = value.length;
  if (fileChanged) {
    promises.push(this.fileField.doUploadFile().then(function(r) {
      return r.json();
    }));
  }
  Promise.all(promises).then(function(array) {
    if (fileChanged) {
      $jscomp$this.fileField.reset();
    }
    $jscomp$this.showToast($jscomp$this.TR('EDIT'), $jscomp$this.TR('EDIT_SUCCESS'));
  })['finally'](function() {
    $jscomp$this.setLoading(false);
    $jscomp$this.finishAction($jscomp$this.updateButton);
  });
}, addDisplayTag:function(name, id, tagCfg) {
  this.tagField.add({_tagName:name, _tagId:id, _tagCfg:tagCfg});
}});
